﻿using System;
using System.Collections.Generic;
using System.Text;

namespace at.uba.uwwtd2008.DataAccess
{
    class DataFormatConverter
    {
        System.Globalization.CultureInfo cultureInfo = System.Globalization.CultureInfo.CurrentCulture;
        Encoding fileEncoding = null;

        public DataFormatConverter(string culture)
        {
            if (culture == null) culture = System.Globalization.CultureInfo.CurrentCulture.Name;

            cultureInfo = new System.Globalization.CultureInfo(culture);
            fileEncoding = Encoding.GetEncoding(cultureInfo.TextInfo.ANSICodePage);
        }

        public Encoding FileEncoding {
            get { return fileEncoding; }
        }

        public object parseDate(string str)
        {
            if ((str == null) || (str.Length < 1)) return DBNull.Value;

            // workaround for bulgaria (they have a strange " ?." behind each date)
            if (str.EndsWith(" ?."))
            {
                str = str.Substring(0, str.Length - 3);
            }
            return DateTime.Parse(str, cultureInfo.DateTimeFormat);
        }

        public string formatDateLong(DateTime dt)
        {
            return dt.ToString(cultureInfo.DateTimeFormat);
        }

        public string formatDateShort(DateTime dt)
        {
            return dt.ToString(cultureInfo.DateTimeFormat);
        }

        public string format(double d)
        {
            return d.ToString(cultureInfo.NumberFormat);
        }

        public string format(long l)
        {
            return l.ToString(cultureInfo.NumberFormat);
        }

        public string format(decimal d)
        {
            return d.ToString(cultureInfo.NumberFormat);
        }

        public object importStringFieldNullable(string []parts, int items) {
            if (items >= parts.Length) return DBNull.Value;
            return parts[items];
        }

        public object importInt32(string str)
        {
            if ((str == null) || (str.Length < 1)) return DBNull.Value;
            return Int32.Parse(str, cultureInfo.NumberFormat);
        }
        public object importInt64(string str)
        {
            if ((str == null) || (str.Length<1)) return DBNull.Value;
            return Int64.Parse(str, cultureInfo.NumberFormat);
        }
        public object importDouble(string str)
        {
            if ((str == null) || (str.Length < 1)) return DBNull.Value;
            return Double.Parse(str, cultureInfo.NumberFormat);
        }
        public object importDecimal(string str)
        {
            if ((str == null) || (str.Length < 1)) return DBNull.Value;
            try
            {
                return Decimal.Parse(str, cultureInfo.NumberFormat);
            }
            catch (FormatException)
            { // as a fallback - try to import the numbers as double
                object obj = importDouble(str);
                if (obj is double)
                {
                    double d = (double)obj;
                    return (decimal)d;
                }
                return obj;
            }
        }
       
        

    }
}
