﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Data;
using System.IO;
using at.uba.uwwtd2008.xml;


namespace at.uba.uwwtd2008.DataAccess
{
    struct AggloBigcity 
    {
        public string refAggloID;
        public object repID;
        public string bigID;
    }

    class InterfaceAgglomerations : InterfaceBase
    {
        private string[] aggFldNames = {"aggState", "repCode", "aggCode", "aggName", "aggNUTS", "aggLatitude", 
	        "aggLongitude", "aggGenerated", "bigID", "aggCalculation", "aggChanges", "aggChangesComment", 
	        "aggPeriodOver", "aggC1", "aggMethodC1", "aggC2", "aggMethodC2", "aggPercWithoutTreatment", "aggMethodWithoutTreatment", 
	        "aggPercPrimTreatment", "aggPercSecTreatment", "aggPercStringentTreatment", "aggHaveRegistrationSystem", 
	        "aggExistMaintenancePlan", "aggPressureTest", "aggVideoInspections", "aggOtherMeasures", "aggExplanationOther", 
	        "aggSewageNetwork", "aggBestTechnicalKnowledge", "aggDilutionRates", "aggCapacity", "aggAccOverflows", 
	        "aggAccOverflowNumber", "aggSewerOverflows_m3", "aggSewerOverflows_pe", "aggRemarks", "aggCollectedThroughIAS", "exportDate"};

        public InterfaceAgglomerations(IDbConnection conn, IDbTransaction trans, DataFormatConverter conv) : base(conn, trans, conv)
        {
            
        }

        private IDbCommand CreateInsertBigCityCommand() 
        {
            string strSQL = "INSERT INTO T_BigCity (bigCityID, bigCountryCode, bigCity) VALUES (?, ?, ?);";

            IDbCommand comInsBigCity = dataFactory.CreateCommand(strSQL, conn, trans);
            comInsBigCity.Parameters.Add(dataFactory.CreateParameter("@bigCityID", DbType.String, 32, "bigCityID", null));
            comInsBigCity.Parameters.Add(dataFactory.CreateParameter("@bigCountryCode", DbType.String, 4, "bigCountryCode", null));
            comInsBigCity.Parameters.Add(dataFactory.CreateParameter("@bigCity", DbType.String, 255, "bigCity", null));

            return comInsBigCity;
        }

        private IDbCommand CreateUpdateAggloCommand() 
        {
            string strSQL = "UPDATE T_Agglomerations set bigID = ? WHERE aggCode = ? and repID = ?;";

            IDbCommand comUpdtBigCity = dataFactory.CreateCommand(strSQL, conn, trans);
            comUpdtBigCity.Parameters.Add(dataFactory.CreateParameter("@bigID", DbType.Int32, 0, "bigID", null));
            comUpdtBigCity.Parameters.Add(dataFactory.CreateParameter("@aggCode", DbType.String, 32, "aggCode", null));
            comUpdtBigCity.Parameters.Add(dataFactory.CreateParameter("@repID", DbType.Int32, 0, "repID", null));

            return comUpdtBigCity;
        }

        private IDbCommand CreateSelectCommand()
        {
            string strSQL = "SELECT aggState, repID, aggCode, aggName, aggNUTS, aggLatitude, aggLongitude, " +
             "aggGenerated, bigID, aggCalculation, aggChanges, aggChangesComment, aggPeriodOver, aggC1, aggMethodC1, " +
             "aggC2, aggMethodC2, aggPercWithoutTreatment, aggMethodWithoutTreatment, aggPercPrimTreatment, aggPercSecTreatment, " +
             "aggPercStringentTreatment, aggHaveRegistrationSystem, aggExistMaintenancePlan, aggPressureTest, aggVideoInspections, " +
             "aggOtherMeasures, aggExplanationOther, aggSewageNetwork, aggBestTechnicalKnowledge, aggDilutionRates, aggCapacity, " +
             "aggAccOverflows, aggAccOverflowNumber, aggSewerOverflows_m3, aggSewerOverflows_pe, aggRemarks, aggCollectedThroughIAS, " +
             "aggForecast FROM T_Agglomerations WHERE repID = ?";

            IDbCommand comSellAgglomeration = dataFactory.CreateCommand(strSQL, conn, trans);

            comSellAgglomeration.Parameters.Add(dataFactory.CreateParameter("@repID", DbType.Int32, 0, "repID", null));
            return comSellAgglomeration;
        }

        private IDbCommand CreateCommand()
        {
		    string strSQL = "INSERT INTO T_Agglomerations (aggState, repID, aggCode, aggName, aggNUTS, aggLatitude, aggLongitude, " +
		     "aggGenerated, bigID, aggCalculation, aggChanges, aggChangesComment, aggPeriodOver, aggC1, aggMethodC1, " +
		     "aggC2, aggMethodC2, aggPercWithoutTreatment, aggMethodWithoutTreatment, aggPercPrimTreatment, aggPercSecTreatment, " +
		     "aggPercStringentTreatment, aggHaveRegistrationSystem, aggExistMaintenancePlan, aggPressureTest, aggVideoInspections, " +
		     "aggOtherMeasures, aggExplanationOther, aggSewageNetwork, aggBestTechnicalKnowledge, aggDilutionRates, aggCapacity, " +
		     "aggAccOverflows, aggAccOverflowNumber, aggSewerOverflows_m3, aggSewerOverflows_pe, aggRemarks, aggCollectedThroughIAS, aggForecast) " +
		     "VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?); ";

            IDbCommand comInsAgglomeration = dataFactory.CreateCommand(strSQL, conn, trans);
		    comInsAgglomeration.Parameters.Add(dataFactory.CreateParameter("@aggState", DbType.Int32, 0, "aggState", null));
		    comInsAgglomeration.Parameters.Add(dataFactory.CreateParameter("@repID", DbType.Int32, 0, "repID", null));
            comInsAgglomeration.Parameters.Add(dataFactory.CreateParameter("@aggCode", DbType.String, 32, "aggCode", null));
            comInsAgglomeration.Parameters.Add(dataFactory.CreateParameter("@aggName", DbType.String, 255, "aggName", null));
		    comInsAgglomeration.Parameters.Add(dataFactory.CreateParameter("@aggNUTS", DbType.Int32, 0, "aggNUTS", null));
		    comInsAgglomeration.Parameters.Add(dataFactory.CreateParameter("@aggLatitude", DbType.String, 20, "aggLatitude", null));
            comInsAgglomeration.Parameters.Add(dataFactory.CreateParameter("@aggLongitude", DbType.String, 20, "aggLongitude", null));
		    comInsAgglomeration.Parameters.Add(dataFactory.CreateParameter("@aggGenerated", DbType.Int32, 0, "aggGenerated", null));
		    comInsAgglomeration.Parameters.Add(dataFactory.CreateParameter("@bigID", DbType.Int32, 0, "bigID", null));
		    comInsAgglomeration.Parameters.Add(dataFactory.CreateParameter("@aggCalculation", DbType.String, 4096, "aggCalculation", null));
		    comInsAgglomeration.Parameters.Add(dataFactory.CreateParameter("@aggChanges", DbType.Int32, 0, "aggChanges", null));
		    comInsAgglomeration.Parameters.Add(dataFactory.CreateParameter("@aggChangesComment", DbType.String, 4096, "aggChangesComment", null));
		    comInsAgglomeration.Parameters.Add(dataFactory.CreateParameter("@aggPeriodOver", DbType.Date, 0, "aggPeriodOver", null));
		    comInsAgglomeration.Parameters.Add(dataFactory.CreateParameter("@aggC1", DbType.Double, 0, "aggC1", null));
		    comInsAgglomeration.Parameters.Add(dataFactory.CreateParameter("@aggMethodC1", DbType.Int32, 0, "aggMethodC1", null));
		    comInsAgglomeration.Parameters.Add(dataFactory.CreateParameter("@aggC2", DbType.Double, 0, "aggC2", null));
		    comInsAgglomeration.Parameters.Add(dataFactory.CreateParameter("@aggMethodC2", DbType.Int32, 0, "aggMethodC2", null));
		    comInsAgglomeration.Parameters.Add(dataFactory.CreateParameter("@aggPercWithoutTreatment", DbType.Double, 0, "aggPercWithoutTreatment", null));
		    comInsAgglomeration.Parameters.Add(dataFactory.CreateParameter("@aggMethodWithoutTreatment", DbType.Int32, 0, "aggMethodWithoutTreatment", null));
		    comInsAgglomeration.Parameters.Add(dataFactory.CreateParameter("@aggPercPrimTreatment", DbType.Double, 0, "aggPercPrimTreatment", null));
		    comInsAgglomeration.Parameters.Add(dataFactory.CreateParameter("@aggPercSecTreatment", DbType.Double, 0, "aggPercSecTreatment", null));
		    comInsAgglomeration.Parameters.Add(dataFactory.CreateParameter("@aggPercStringentTreatment", DbType.Double, 0, "aggPercStringentTreatment", null));
		    comInsAgglomeration.Parameters.Add(dataFactory.CreateParameter("@aggHaveRegistrationSystem", DbType.Int32, 0, "aggHaveRegistrationSystem", null));
		    comInsAgglomeration.Parameters.Add(dataFactory.CreateParameter("@aggExistMaintenancePlan", DbType.Int32, 0, "aggExistMaintenancePlan", null));
		    comInsAgglomeration.Parameters.Add(dataFactory.CreateParameter("@aggPressureTest", DbType.Boolean, 0, "aggPressureTest", null));
            comInsAgglomeration.Parameters.Add(dataFactory.CreateParameter("@aggVideoInspections", DbType.Boolean, 0, "aggVideoInspections", null));
            comInsAgglomeration.Parameters.Add(dataFactory.CreateParameter("@aggOtherMeasures", DbType.Boolean, 0, "aggOtherMeasures", null));
		    comInsAgglomeration.Parameters.Add(dataFactory.CreateParameter("@aggExplanationOther", DbType.String, 4096, "aggExplanationOther", null));
		    comInsAgglomeration.Parameters.Add(dataFactory.CreateParameter("@aggSewageNetwork", DbType.Int32, 0, "aggSewageNetwork", null));
		    comInsAgglomeration.Parameters.Add(dataFactory.CreateParameter("@aggBestTechnicalKnowledge", DbType.Int32, 0, "aggBestTechnicalKnowledge", null));
            comInsAgglomeration.Parameters.Add(dataFactory.CreateParameter("@aggDilutionRates", DbType.Boolean, 0, "aggDilutionRates", null));
            comInsAgglomeration.Parameters.Add(dataFactory.CreateParameter("@aggCapacity", DbType.Boolean, 0, "aggCapacity", null));
            comInsAgglomeration.Parameters.Add(dataFactory.CreateParameter("@aggAccOverflows", DbType.Boolean, 0, "aggAccOverflows", null));
		    comInsAgglomeration.Parameters.Add(dataFactory.CreateParameter("@aggAccOverflowNumber", DbType.Int32, 0, "aggAccOverflowNumber", null));
		    comInsAgglomeration.Parameters.Add(dataFactory.CreateParameter("@aggSewerOverflows_m3", DbType.Int32, 0, "aggSewerOverflows_m3", null));
		    comInsAgglomeration.Parameters.Add(dataFactory.CreateParameter("@aggSewerOverflows_pe", DbType.Int32, 0, "aggSewerOverflows_pe", null));
		    comInsAgglomeration.Parameters.Add(dataFactory.CreateParameter("@aggRemarks", DbType.String, 4096, "aggRemarks", null));
		    comInsAgglomeration.Parameters.Add(dataFactory.CreateParameter("@aggCollectedThroughIAS", DbType.Date, 0, "aggCollectedThroughIAS", null));
            comInsAgglomeration.Parameters.Add(dataFactory.CreateParameter("@aggForecast", DbType.Date, 0, "aggForecast", null));
            return comInsAgglomeration;
        }

        public override void importData(StreamReader rdr, DataRow repPeriod)
        {
            bool hadErrors = false;
            System.Collections.ArrayList lstAggloToBigCity = new System.Collections.ArrayList();
            System.Collections.ArrayList lstUniqAggCodes = new System.Collections.ArrayList();
            IDbCommand cmdBigCity = dataFactory.CreateCommand("SELECT bigID FROM T_BigCity WHERE bigCityID = ?;", conn, trans);
            IDataParameter parBigCity = dataFactory.CreateParameter("@bigCityID", DbType.String, 32, "bigCityID", null);
            cmdBigCity.Parameters.Add(parBigCity);

            logInfo("========================================================================");
            logInfo("  Importing Agglomerations");
            logInfo("========================================================================");
            string line;
            long lineNo = 0;
            while ((line = rdr.ReadLine()) != null) {
                try
                {
                    lineNo++;
                    line = line.TrimEnd();
                    if (line.Length < 1) continue; // ignore empty lines
                    hadErrors = false;
                    string[] parts = line.Split(new string[] { "\t" }, StringSplitOptions.None);

                    if (parts[0].Equals("")) continue; // ignore empty lines
                    IDbCommand cmd = CreateCommand();
                    for (int idx = 0; idx < parts.Length; idx++)
                    {
                        if (idx >= aggFldNames.Length)
                        {
                            // do error reporting ("too many items")
                            continue;
                        }
                        string strFeldName = aggFldNames[idx];
                        if (strFeldName.Equals("exportDate")) // this is nothing to import
                            continue;
                        try
                        {
                            object objTemp = parts[idx];
                            if (!checkQAACRule(objTemp, strFeldName, repPeriod, true, "line: " + lineNo))
                                hadErrors = true;
                            if (parts[idx].Length > 0)
                            {
                                //blnRefValue = true;
                                switch (strFeldName)
                                {
                                    case "repCode":
                                        objTemp = dataFactory.getRepIDforReportCode(parts[idx]);
                                        // if (objTemp == null)  do error reporting
                                        setParameterValue(cmd, idx, objTemp);
                                        break;
                                    case "aggState":
                                        objTemp = dataFactory.getLovRowByKey("STAT", parts[idx])["lovID"];
                                        // if (objTemp == null) do error reporting
                                        setParameterValue(cmd, idx, objTemp);
                                        break;
                                    case "aggNUTS":
                                        objTemp = dataFactory.getNUTSIdForNUTSCode(parts[idx]);
                                        // if (objTemp == null) do error reporting
                                        setParameterValue(cmd, idx, objTemp);
                                        break;
                                    case "bigID":
                                        parBigCity.Value = parts[idx];
                                        objTemp = cmdBigCity.ExecuteScalar();
                                        // if (objTemp == null) do error reporting
                                        setParameterValue(cmd, idx, objTemp);
                                        if ((parts[idx] != null) && (parts[idx].Length > 0))
                                        {
                                            if ((objTemp is DBNull) || (objTemp == null))
                                            {
                                                // there is a new BigCity
                                                AggloBigcity bca = new AggloBigcity();
                                                bca.bigID = parts[idx];
                                                bca.repID = dataFactory.getRepIDforReportCode(parts[Array.IndexOf(aggFldNames, "repCode")]);
                                                bca.refAggloID = parts[Array.IndexOf(aggFldNames, "aggCode")];

                                                lstAggloToBigCity.Add(bca);
                                            }
                                        }
                                        break;
                                    case "aggChanges":
                                        objTemp = dataFactory.getLovRowByKey("YN", parts[idx])["lovID"];
                                        // if (objTemp == null) do error reporting
                                        setParameterValue(cmd, idx, objTemp);
                                        break;
                                    case "aggChangesComment":
                                        string strAggChanges = parts[Array.IndexOf(aggFldNames, "aggChanges")];
                                        if ((strAggChanges != null) && (strAggChanges.Equals("1")))
                                        {
                                            string strErrorText = "";
                                            if (!checkFieldMandatoryPresence(parts[idx], strFeldName, ref strErrorText, "aggChanges is set"))
                                            {
                                                logError("line " + lineNo + ": " + strErrorText);
                                                hadErrors = true;
                                            }
                                        }
                                        setParameterValue(cmd, idx, parts[idx]);
                                        break;
                                    case "aggMethodC1":
                                        objTemp = dataFactory.getLovRowByKey("COLLDET", parts[idx])["lovID"];
                                        // if (objTemp == null) do error reporting
                                        setParameterValue(cmd, idx, objTemp);
                                        break;
                                    case "aggMethodC2":
                                        objTemp = dataFactory.getLovRowByKey("COLLDET", parts[idx])["lovID"];
                                        // if (objTemp == null) do error reporting
                                        setParameterValue(cmd, idx, objTemp);
                                        break;
                                    case "aggMethodWithoutTreatment":
                                        objTemp = dataFactory.getLovRowByKey("COLLDET", parts[idx])["lovID"];
                                        // if (objTemp == null) do error reporting
                                        setParameterValue(cmd, idx, objTemp);
                                        break;
                                    case "aggHaveRegistrationSystem":
                                        objTemp = dataFactory.getLovRowByKey("YN", parts[idx])["lovID"];
                                        // if (objTemp == null) do error reporting
                                        setParameterValue(cmd, idx, objTemp);
                                        break;
                                    case "aggExistMaintenancePlan":
                                        objTemp = dataFactory.getLovRowByKey("YN", parts[idx])["lovID"];
                                        // if (objTemp == null) do error reporting
                                        setParameterValue(cmd, idx, objTemp);
                                        break;
                                    case "aggSewageNetwork":
                                        objTemp = dataFactory.getLovRowByKey("SEWANET", parts[idx])["lovID"];
                                        // if (objTemp == null) do error reporting
                                        setParameterValue(cmd, idx, objTemp);
                                        break;
                                    case "aggBestTechnicalKnowledge":
                                        objTemp = dataFactory.getLovRowByKey("YN", parts[idx])["lovID"];
                                        // if (objTemp == null) do error reporting
                                        setParameterValue(cmd, idx, objTemp);
                                        break;
                                    case "aggCode":
                                        if (lstUniqAggCodes.Contains(parts[idx]))
                                        {
                                            logError("line: " + lineNo + " - The AggloID " + parts[idx] + " cannot be used twice");
                                            hadErrors = true;
                                        }
                                        else
                                        {
                                            setParameterValue(cmd, idx, parts[idx]);
                                        }
                                        break;
                                    case "aggGenerated":
                                        setParameterValue(cmd, idx, parts[idx]);
                                        if ((parts[idx] != null) && (parts[idx].Length > 0))
                                        {
                                            string errorString = "";
                                            if (!checkFieldMandatoryPresence(parts[Array.IndexOf(aggFldNames, "aggCalculation")], "aggCalculation", ref errorString))
                                            {
                                                logError("line: " + lineNo + " - " + errorString + " - aggGenerated is set");
                                                hadErrors = true;
                                            }
                                        }
                                        break;
                                    default:
                                        setParameterValue(cmd, idx, parts[idx]);
                                        break;
                                }
                            }
                            else
                            {
                                setParameterValue(cmd, idx, null);
                            }
                        }
                        catch (Exception ex)
                        {
                            logError("line: " + lineNo + ": Exception: " + ex.Message + ";");
                            hadErrors = true;
                        }
                    }

                    cmd.ExecuteNonQuery();
                    if (!hadErrors)
                        reportSuccess();
                    else
                        reportFail();
                }
                catch (Exception ex)
                {
                    logError("line: " + lineNo + ": Exception: " + ex.Message + ";");
                    reportFail();
                }
            }

            // check if there are bigCities to import
            foreach (AggloBigcity bca in lstAggloToBigCity)
            {
                try
                {
                    // get the reporter
                    parBigCity.Value = bca.bigID;
                    object curBC = cmdBigCity.ExecuteScalar();
                    if ((curBC == null) || (curBC is DBNull))
                    {
                        // get the Agglos name ...
                        IDbCommand cmd = dataFactory.CreateCommand("SELECT aggName from T_AGGLOMERATIONS WHERE aggCode = ? and repID = ?", conn, trans);
                        IDbDataParameter parAggCode = dataFactory.CreateParameter("@aggCode", DbType.String, 32, "aggCode", null);
                        IDbDataParameter parRepID = dataFactory.CreateParameter("@repID", DbType.Int32, 0, "repID", null);
                        cmd.Parameters.Add(parAggCode);
                        cmd.Parameters.Add(parRepID);

                        parAggCode.Value = bca.bigID;
                        parRepID.Value = bca.repID;
                        DataRow rpt = dataFactory.getRepPeridodforRepID(bca.repID).GetParentRow("T_ReporterT_ReportPeriod");
                        object countryCode = rpt["rptMStateKey"];
                        IDbCommand cmdIns = CreateInsertBigCityCommand();
                        ((IDbDataParameter)cmdIns.Parameters[0]).Value = bca.bigID; // bigCityID
                        ((IDbDataParameter)cmdIns.Parameters[1]).Value = rpt["rptMStateKey"]; // country code
                        object bigCityName = cmd.ExecuteScalar();
                        ((IDbDataParameter)cmdIns.Parameters[2]).Value = bigCityName; // bigCity
                        // Check if the BigCity could be found ....
                        if ((bigCityName == null) || (bigCityName is DBNull))
                        {
                            logError("The BigCity " + bca.bigID + " cannot be found");
                            logger.reportFail("BigCities");
                            continue;
                        }
                        cmdIns.ExecuteNonQuery();

                        curBC = cmdBigCity.ExecuteScalar();
                    }

                    IDbCommand cmdAggloUpdate = CreateUpdateAggloCommand();
                    ((IDbDataParameter)cmdAggloUpdate.Parameters[0]).Value = curBC;
                    ((IDbDataParameter)cmdAggloUpdate.Parameters[0]).Value = bca.refAggloID;
                    ((IDbDataParameter)cmdAggloUpdate.Parameters[0]).Value = bca.repID;
                    cmdAggloUpdate.ExecuteNonQuery();
                } // try
                catch (Exception ex)
                {
                    logError("Failed to update BigCities (" + bca.bigID + "): " + ex.Message + "\n" + ex.StackTrace);
                    logger.reportFail("BigCities");
                }
            }
        }

        public override void exportData(StreamWriter wr, DataRow repPeriod)
        {
            IDbCommand cmdBigCity = dataFactory.CreateCommand("SELECT bigCityID FROM T_BigCity WHERE bigID = ?;", conn, trans);
            IDataParameter parBigCity = dataFactory.CreateParameter("@bigID", DbType.Int32, 0, "bigID", null);
            cmdBigCity.Parameters.Add(parBigCity);

            logInfo("========================================================================");
            logInfo("  Exporting Agglomerations");
            logInfo("========================================================================");


            string[] parts = new string[aggFldNames.Length];
            IDbCommand cmd = CreateSelectCommand();
            setParameterValue(cmd, 0, repPeriod["repID"]);
            IDataReader rdr = cmd.ExecuteReader();
            try
            {
                while (rdr.Read())
                {
                    bool hasErrors = false;
                    try
                    {
                        for (int i = 0; i < aggFldNames.Length; i++)
                        {
                            string strFieldName = aggFldNames[i];
                            if (strFieldName.Equals("exportDate")) continue;
                            if (!checkQAACRule(rdr[i], strFieldName, repPeriod, false, null))
                                hasErrors = true;

                            switch (strFieldName)
                            {
                                case "repCode":
                                    parts[i] = getReaderValueString(repPeriod["repCode"]);
                                    break;
                                case "aggState":
                                    parts[i] = dataFactory.getLovRowByID(rdr[i])["lovKey"] as string;
                                    break;
                                case "aggNUTS":
                                    parts[i] = dataFactory.getNutsCode(rdr[i]);
                                    break;
                                case "bigID":
                                    if ((rdr[i] == null) || (rdr[i] is DBNull))
                                        parts[i] = "";
                                    else
                                    {
                                        parBigCity.Value = rdr[i];
                                        parts[i] = cmdBigCity.ExecuteScalar() as string;
                                    }
                                    break;
                                case "aggChanges":
                                    if ((rdr[i] == null) || (rdr[i] is DBNull))
                                        parts[i] = "";
                                    else
                                        parts[i] = dataFactory.getLovRowByID(rdr[i])["lovKey"] as string;
                                    break;
                                case "aggChangesComment":
                                    string strAggChanges = parts[Array.IndexOf(aggFldNames, "aggChanges")];
                                    if ((strAggChanges != null) && (strAggChanges.Equals("1")))
                                    {
                                        string strErrorText = "";
                                        if (!checkFieldMandatoryPresence(rdr[i], "aggChangesComment", ref strErrorText, "aggChanges is set"))
                                        {
                                            logError(strErrorText);
                                            hasErrors = true;
                                        }
                                    }
                                    parts[i] = getReaderValueString(rdr[i]);
                                    break;
                                case "aggLongitude":
                                case "aggLatitude":
                                    parts[i] = getReaderValueString(DataAccessFactory.parseDBDecimal(rdr[i]));
                                    break;
                                case "aggMethodC1":
                                    if ((rdr[i] == null) || (rdr[i] is DBNull))
                                        parts[i] = "";
                                    else
                                        parts[i] = dataFactory.getLovRowByID(rdr[i])["lovKey"] as string;
                                    break;
                                case "aggMethodC2":
                                    if ((rdr[i] == null) || (rdr[i] is DBNull))
                                        parts[i] = "";
                                    else
                                        parts[i] = dataFactory.getLovRowByID(rdr[i])["lovKey"] as string;
                                    break;
                                case "aggMethodWithoutTreatment":
                                    if ((rdr[i] == null) || (rdr[i] is DBNull))
                                        parts[i] = "";
                                    else
                                        parts[i] = dataFactory.getLovRowByID(rdr[i])["lovKey"] as string;
                                    break;
                                case "aggHaveRegistrationSystem":
                                    if ((rdr[i] == null) || (rdr[i] is DBNull))
                                        parts[i] = "";
                                    else
                                        parts[i] = dataFactory.getLovRowByID(rdr[i])["lovKey"] as string;
                                    break;
                                case "aggExistMaintenancePlan":
                                    if ((rdr[i] == null) || (rdr[i] is DBNull))
                                        parts[i] = "";
                                    else
                                        parts[i] = dataFactory.getLovRowByID(rdr[i])["lovKey"] as string;
                                    break;
                                case "aggSewageNetwork":
                                    if ((rdr[i] == null) || (rdr[i] is DBNull))
                                        parts[i] = "";
                                    else
                                        parts[i] = dataFactory.getLovRowByID(rdr[i])["lovKey"] as string;
                                    break;
                                case "aggBestTechnicalKnowledge":
                                    if ((rdr[i] == null) || (rdr[i] is DBNull))
                                        parts[i] = "";
                                    else
                                        parts[i] = dataFactory.getLovRowByID(rdr[i])["lovKey"] as string;
                                    break;
                                case "aggGenerated":
                                    if ((rdr[i] != null) && !(rdr[i] is DBNull))
                                    {
                                        string errorString = "";
                                        if (!checkFieldMandatoryPresence(rdr[Array.IndexOf(aggFldNames, "aggCalculation")], "aggCalculation", ref errorString))
                                        {
                                            logError(errorString + " - aggGenerated is set");
                                            hasErrors = true;
                                        }
                                    }
                                    parts[i] = getReaderValueString(rdr[i]);
                                    break;
                                default:
                                    parts[i] = getReaderValueString(rdr[i]);
                                    break;
                            }
                        }

                        parts[parts.Length - 1] = conv.formatDateLong(DateTime.Now);
                        wr.WriteLine(string.Join("\t", parts));
                        if (!hasErrors)
                            reportSuccess();
                        else
                            reportFail();
                    }
                    catch (Exception ex)
                    {
                        logError("Exception: " + ex.Message + ";");
                        reportFail();
                    }
                    
                }
            }
            finally
            {
                rdr.Close();
            }
        }

        public override void exportXML(UWWTD_DataUWWTD_Report report, DataRow repPeriod)
        {
            List<UWWTD_DataUWWTD_ReportAgglomeration> dataList = new List<UWWTD_DataUWWTD_ReportAgglomeration>();

            IDbCommand cmd = CreateSelectCommand();
            IDbCommand comAgglo = dataFactory.CreateCommand("SELECT aggCode FROM T_Agglomerations WHERE aggID = ?;", conn, trans);
            IDataParameter parAggID = dataFactory.CreateParameter("@aggID", DbType.String, 32, "aggID", null);
            comAgglo.Parameters.Add(parAggID);

            logInfo("========================================================================");
            logInfo("  Exporting Agglomerations for ReportPeriod " + repPeriod["repCode"]);
            logInfo("========================================================================");

            setParameterValue(cmd, "@repID", repPeriod["repID"]);
            IDataReader rdr = cmd.ExecuteReader();
            while (rdr.Read())
            {
                bool hasErrors = false;
                try
                {
                    for(int j = 0; j < aggFldNames.Length; j++)
                    {
                        if (aggFldNames[j].Equals("exportDate")) continue;
                        if (!checkQAACRule(rdr[j], aggFldNames[j], repPeriod, false, (rdr["aggCode"] as string) != null ? "AggID: " + rdr["aggCode"] as string : null))
                            hasErrors = true;
                    }
                    UWWTD_DataUWWTD_ReportAgglomeration agglo = new UWWTD_DataUWWTD_ReportAgglomeration();
                    agglo.aggCode = rdr["aggCode"] as string;

                    agglo.aggAccOverflowNumberSpecified = !(rdr["aggAccOverflowNumber"] is DBNull);
                    if (agglo.aggAccOverflowNumberSpecified)
                        agglo.aggAccOverflowNumber = long.Parse(rdr["aggAccOverflowNumber"].ToString());
                    agglo.aggAccOverflowsSpecified = !(rdr["aggAccOverflows"] is DBNull);
                    if (agglo.aggAccOverflowsSpecified)
                        agglo.aggAccOverflows = (bool)rdr["aggAccOverflows"];
                    agglo.aggBestTechnicalKnowledge = dataFactory.getLovRowByID(rdr["aggBestTechnicalKnowledge"])["lovKey"] as string;

                    agglo.aggC1Specified = !(rdr["aggC1"] is DBNull);
                    if (agglo.aggC1Specified)
                        agglo.aggC1 = (long)double.Parse(rdr["aggC1"].ToString());

                    agglo.aggC2Specified = !(rdr["aggC2"] is DBNull);
                    if (agglo.aggC2Specified)
                        agglo.aggC2 = (long)double.Parse(rdr["aggC2"].ToString());

                    agglo.aggCalculation = rdr["aggCalculation"] as string;

                    agglo.aggCapacitySpecified = !(rdr["aggCapacity"] is DBNull);
                    if (agglo.aggCapacitySpecified)
                        agglo.aggCapacity = (bool)rdr["aggCapacity"];

                    agglo.aggChanges = dataFactory.getLovRowByID(rdr["aggChanges"])["lovKey"] as string;
                    agglo.aggChangesComment = rdr["aggChangesComment"] as string;
                    if ((agglo.aggChanges != null) && (agglo.aggChanges.Equals("1")))
                    {
                        string errorText = "";
                        if (!checkFieldMandatoryPresence(rdr["aggChangesComment"], "aggChangesComment", ref errorText, "aggChanges is set"))
                        {
                            logError("AggID " + agglo.aggCode + " - " + errorText);
                            hasErrors = true;
                        }
                    }
                    agglo.aggDilutionRatesSpecified = !(rdr["aggDilutionRates"] is DBNull);
                    if (agglo.aggDilutionRatesSpecified)
                        agglo.aggDilutionRates = (bool)rdr["aggDilutionRates"];

                    agglo.aggExistMaintenancePlan = dataFactory.getLovRowByID(rdr["aggExistMaintenancePlan"])["lovKey"] as string;
                    agglo.aggExplanationOther = rdr["aggExplanationOther"] as string;
                    agglo.aggForecastSpecified = !(rdr["aggForecast"] is DBNull);
                    if (agglo.aggForecastSpecified)
                        agglo.aggForecast = (DateTime)rdr["aggForecast"];

                    if (rdr["aggGenerated"] is DBNull)
                        agglo.aggGenerated = 0;
                    else
                    {
                        agglo.aggGenerated = long.Parse(rdr["aggGenerated"].ToString());
                        string errorString = "";
                        if (!checkFieldMandatoryPresence(rdr["aggCalculation"], "aggCalculation", ref errorString))
                        {
                            logError(((agglo.aggCode) != null ? ("AggID: " + agglo.aggCode + " - ") : "") + errorString + "aggGenerated is set");
                            hasErrors = true;
                        }
                        
                    }
                    agglo.aggHaveRegistrationSystem = dataFactory.getLovRowByID(rdr["aggHaveRegistrationSystem"])["lovKey"] as string;
                    agglo.aggLatitudeSpecified = !(rdr["aggLatitude"] is DBNull);
                    if (agglo.aggLatitudeSpecified)
                        agglo.aggLatitude = (decimal) DataAccessFactory.parseDBDecimal(rdr["aggLatitude"]);
                    agglo.aggLongitudeSpecified = !((rdr["aggLongitude"] is DBNull));
                    if (agglo.aggLongitudeSpecified)
                        agglo.aggLongitude = (decimal) DataAccessFactory.parseDBDecimal(rdr["aggLongitude"]);
                    
                    agglo.aggMethodC1 = dataFactory.getLovRowByID(rdr["aggMethodC1"])["lovKey"] as string;
                    agglo.aggMethodC2 = dataFactory.getLovRowByID(rdr["aggMethodC2"])["lovKey"] as string;
                    agglo.aggMethodWithoutTreatment = dataFactory.getLovRowByID(rdr["aggMethodWithoutTreatment"])["lovKey"] as string;

                    agglo.aggName = rdr["aggName"] as string;
                    if (rdr["aggNUTS"] is DBNull)
                        agglo.aggNUTS = null;
                    else
                    {
                        agglo.aggNUTS = dataFactory.getNutsCode(rdr["aggNUTS"]);
                        if ((agglo.aggNUTS == null) || (agglo.aggNUTS.Length < 1))
                        {
                            logError("AggID " + agglo.aggCode + " - nutsID " + rdr["aggNUTS"] + " could not be found");
                            agglo.aggNUTS = null;
                            hasErrors = true;
                        }
                    }

                    agglo.aggOtherMeasuresSpecified = !(rdr["aggOtherMeasures"] is DBNull);
                    if (agglo.aggOtherMeasuresSpecified)
                        agglo.aggOtherMeasures = (bool)rdr["aggOtherMeasures"];
                    agglo.aggPercPrimTreatmentSpecified = !(rdr["aggPercPrimTreatment"] is DBNull);
                    if (agglo.aggPercPrimTreatmentSpecified)
                        agglo.aggPercPrimTreatment = convertDecimal(rdr["aggPercPrimTreatment"]);
                    agglo.aggPercSecTreatmentSpecified = !(rdr["aggPercSecTreatment"] is DBNull);
                    if (agglo.aggPercSecTreatmentSpecified)
                        agglo.aggPercSecTreatment = convertDecimal(rdr["aggPercSecTreatment"]);
                    agglo.aggPercStringentTreatmentSpecified = !(rdr["aggPercStringentTreatment"] is DBNull);
                    if (agglo.aggPercStringentTreatmentSpecified)
                        agglo.aggPercStringentTreatment = convertDecimal(rdr["aggPercStringentTreatment"]);
                    agglo.aggPercWithoutTreatmentSpecified = !(rdr["aggPercWithoutTreatment"] is DBNull);
                    if (agglo.aggPercWithoutTreatmentSpecified)
                        agglo.aggPercWithoutTreatment = convertDecimal(rdr["aggPercWithoutTreatment"]);
                    if (rdr["aggPeriodOver"] is DBNull)
                        agglo.aggPeriodOver = DateTime.MinValue; // TODO: Was checked by QA/QC - but we have to give it something ....
                    else
                        agglo.aggPeriodOver = (DateTime)rdr["aggPeriodOver"];
                    agglo.aggPercWithoutTreatmentSpecified = !(rdr["aggPercWithoutTreatment"] is DBNull);
                    if (agglo.aggPercWithoutTreatmentSpecified)
                        agglo.aggPercWithoutTreatment = convertDecimal(rdr["aggPercWithoutTreatment"]);
                    agglo.aggPressureTest = (bool)rdr["aggPressureTest"];
                    agglo.aggPressureTestSpecified = !(rdr["aggPressureTest"] is DBNull);
                    if (agglo.aggPressureTestSpecified)
                        agglo.aggPressureTest = (bool)rdr["aggPressureTest"];
                    agglo.aggRemarks = rdr["aggRemarks"] as string;
                    agglo.aggSewageNetwork = dataFactory.getLovRowByID(rdr["aggSewageNetwork"])["lovKey"] as string;
                    agglo.aggSewerOverflows_m3Specified = !(rdr["aggSewerOverflows_m3"] is DBNull);
                    if (agglo.aggSewerOverflows_m3Specified)
                        agglo.aggSewerOverflows_m3 = long.Parse(rdr["aggSewerOverflows_m3"].ToString());
                    agglo.aggSewerOverflows_peSpecified = !(rdr["aggSewerOverflows_pe"] is DBNull);
                    if (agglo.aggSewerOverflows_peSpecified)
                        agglo.aggSewerOverflows_pe = long.Parse(rdr["aggSewerOverflows_pe"].ToString());

                    agglo.aggState = dataFactory.getLovRowByID(rdr["aggState"])["lovKey"] as string;
                    agglo.aggVideoInspectionsSpecified = !(rdr["aggVideoInspections"] is DBNull);
                    if (agglo.aggVideoInspectionsSpecified)
                        agglo.aggVideoInspections = (bool)rdr["aggVideoInspections"];
                    if (rdr["bigID"] is DBNull)
                        agglo.bigID = null;
                    else
                        agglo.bigID = (dataFactory.CreateCommand("SELECT bigCityID FROM T_BIGCITY WHERE bigID = " + rdr["bigID"], conn).ExecuteScalar()) as string;
                    dataList.Add(agglo);
                    if (hasErrors)
                        reportFail();
                    else
                        reportSuccess();

                }
                catch (Exception ex)
                {
                    logError("Exception: " + ex.ToString());
                    reportFail();
                }
            }
            report.Agglomerations = dataList.ToArray();
        }

        public override void importXML(object collection, DataRow repPeriod)
        {
            UWWTD_DataUWWTD_ReportAgglomeration[] agglomerations = collection as UWWTD_DataUWWTD_ReportAgglomeration[];
            System.Collections.ArrayList lstUniqCodes = new System.Collections.ArrayList();

            IDbCommand cmdBigCity = dataFactory.CreateCommand("SELECT bigID FROM T_BigCity WHERE bigCityID = ?;", conn, trans);
            IDataParameter parBigCity = dataFactory.CreateParameter("@bigCityID", DbType.String, 32, "bigCityID", null);
            cmdBigCity.Parameters.Add(parBigCity);

            logInfo("========================================================================");
            logInfo("  Importing Agglomerations for ReportPeriod " + repPeriod["repCode"]);
            logInfo("========================================================================");

            IDbCommand cmd = CreateCommand();
            foreach (UWWTD_DataUWWTD_ReportAgglomeration agglomeration in agglomerations)
            {
                try
                {
                    if (lstUniqCodes.Contains(agglomeration.aggCode))
                    {
                        //strFCode = "INVKEY"
                        //strFeldName = "rcaCode"
                        //strWert = CStr(colRcaInp(intColCnt))
                        //Err.Raise(10000)
                        continue;
                    }
                    lstUniqCodes.Add(agglomeration.aggCode);
                    bool hasErrors = false;
                    for (int j = 0; j < aggFldNames.Length; j++)
                    {
                        try
                        {
                            if (aggFldNames[j].Equals("repCode") || aggFldNames[j].Equals("exportDate") || aggFldNames[j].Equals("aggCollectedThroughIAS")) continue;
                            object value = agglomeration.GetType().GetField(aggFldNames[j]).GetValue(agglomeration);
                            if (agglomeration.GetType().GetField(aggFldNames[j] + "Specified") != null)
                            {
                                if (!((bool)agglomeration.GetType().GetField(aggFldNames[j] + "Specified").GetValue(agglomeration))) value = null;
                            }
                            if (!checkQAACRule(value, aggFldNames[j], repPeriod, true, agglomeration.aggCode != null ? "aggCode: " + agglomeration.aggCode : null))
                                hasErrors = true;
                        }
                        catch (Exception ex)
                        {
                            logError("Error in " + aggFldNames[j] + ": " + ex.ToString());
                        }
                    }

                    string errorString = "";
                    if (!checkFieldMandatoryPresence(agglomeration.aggCalculation, "aggCalculation", ref errorString))
                    {
                        logError((agglomeration.aggCode != null ? "aggCode: " + agglomeration.aggCode + " - " : "") + errorString + "aggGenerated is set");
                        hasErrors = true;
                    }

                    int i = 0;
                    setParameterValue(cmd, i++, dataFactory.getLovRowByKey("STAT", agglomeration.aggState)["lovID"]);
                    setParameterValue(cmd, i++, repPeriod["repID"]);
                    setParameterValue(cmd, i++, agglomeration.aggCode);
                    setParameterValue(cmd, i++, agglomeration.aggName);
                    setParameterValue(cmd, i++, dataFactory.getNUTSIdForNUTSCode(agglomeration.aggNUTS));
                    if (agglomeration.aggLatitudeSpecified)
                        setParameterValue(cmd, i++, agglomeration.aggLatitude);
                    else
                        setParameterValue(cmd, i++, DBNull.Value);
                    if (agglomeration.aggLongitudeSpecified)
                        setParameterValue(cmd, i++, agglomeration.aggLongitude);
                    else
                        setParameterValue(cmd, i++, DBNull.Value);
                    setParameterValue(cmd, i++, agglomeration.aggGenerated);
                    if ((agglomeration.bigID != null) && (agglomeration.bigID.Length > 0))
                    {
                        parBigCity.Value = agglomeration.bigID;
                        object objTemp = cmdBigCity.ExecuteScalar();
                        setParameterValue(cmd, i++, objTemp);
                        if (objTemp == null)
                        {
                            logError("AggID: " + agglomeration.aggCode + " - Could not find BigCity Discharger " + agglomeration.bigID + ".");
                            hasErrors = true;
                        }
                    }
                    else
                        setParameterValue(cmd, i++, null);

                    setParameterValue(cmd, i++, agglomeration.aggCalculation);
                    setParameterValue(cmd, i++, dataFactory.getLovRowByKey("YN", agglomeration.aggChanges)["lovID"]);
                    if ((agglomeration.aggChanges != null) && (agglomeration.aggChanges.Equals("1")))
                    {
                        string strErrorText = "";
                        if (!checkFieldMandatoryPresence(agglomeration.aggChangesComment, "aggChangesComment", ref strErrorText, "aggChanges is set"))
                        {
                            logError("AggID: " + agglomeration.aggCode + " - " + strErrorText);
                            hasErrors = true;
                        }
                    }
                    setParameterValue(cmd, i++, agglomeration.aggChangesComment);
                    setParameterValue(cmd, i++, agglomeration.aggPeriodOver);
                    if (agglomeration.aggC1Specified)
                        setParameterValue(cmd, i++, agglomeration.aggC1);
                    else
                        setParameterValue(cmd, i++, DBNull.Value);
                    setParameterValue(cmd, i++, dataFactory.getLovRowByKey("COLLDET", agglomeration.aggMethodC1)["lovID"]);
                    if (agglomeration.aggC2Specified)
                        setParameterValue(cmd, i++, agglomeration.aggC2);
                    else
                        setParameterValue(cmd, i++, DBNull.Value);
                    setParameterValue(cmd, i++, dataFactory.getLovRowByKey("COLLDET", agglomeration.aggMethodC2)["lovID"]);
                    if (agglomeration.aggPercWithoutTreatmentSpecified)
                        setParameterValue(cmd, i++, agglomeration.aggPercWithoutTreatment);
                    else
                        setParameterValue(cmd, i++, DBNull.Value);
                    setParameterValue(cmd, i++, dataFactory.getLovRowByKey("COLLDET", agglomeration.aggMethodWithoutTreatment)["lovID"]);
                    if (agglomeration.aggPercPrimTreatmentSpecified)
                        setParameterValue(cmd, i++, agglomeration.aggPercPrimTreatment);
                    else
                        setParameterValue(cmd, i++, DBNull.Value);
                    if (agglomeration.aggPercSecTreatmentSpecified)
                        setParameterValue(cmd, i++, agglomeration.aggPercSecTreatment);
                    else
                        setParameterValue(cmd, i++, DBNull.Value);
                    if (agglomeration.aggPercStringentTreatmentSpecified)
                        setParameterValue(cmd, i++, agglomeration.aggPercStringentTreatment);
                    else
                        setParameterValue(cmd, i++, DBNull.Value);
                    setParameterValue(cmd, i++, agglomeration.aggHaveRegistrationSystem != null ? dataFactory.getLovRowByKey("YN", agglomeration.aggHaveRegistrationSystem)["lovID"] : DBNull.Value);
                    setParameterValue(cmd, i++, agglomeration.aggExistMaintenancePlan != null ? dataFactory.getLovRowByKey("YN", agglomeration.aggExistMaintenancePlan)["lovID"] : DBNull.Value);
                    if (agglomeration.aggPressureTestSpecified)
                        setParameterValue(cmd, i++, agglomeration.aggPressureTest);
                    else
                        setParameterValue(cmd, i++, DBNull.Value);
                    if (agglomeration.aggVideoInspectionsSpecified)
                        setParameterValue(cmd, i++, agglomeration.aggVideoInspections);
                    else
                        setParameterValue(cmd, i++, DBNull.Value);
                    if (agglomeration.aggOtherMeasuresSpecified)
                        setParameterValue(cmd, i++, agglomeration.aggOtherMeasures);
                    else
                        setParameterValue(cmd, i++, DBNull.Value);
                    setParameterValue(cmd, i++, agglomeration.aggExplanationOther);
                    setParameterValue(cmd, i++, agglomeration.aggSewageNetwork != null ? dataFactory.getLovRowByKey("SEWANET", agglomeration.aggSewageNetwork)["lovID"] : DBNull.Value);
                    setParameterValue(cmd, i++, agglomeration.aggBestTechnicalKnowledge != null ? dataFactory.getLovRowByKey("YN", agglomeration.aggBestTechnicalKnowledge)["lovID"] : DBNull.Value);
                    if (agglomeration.aggDilutionRatesSpecified)
                        setParameterValue(cmd, i++, agglomeration.aggDilutionRates);
                    else
                        setParameterValue(cmd, i++, DBNull.Value);
                    if (agglomeration.aggCapacitySpecified)
                        setParameterValue(cmd, i++, agglomeration.aggCapacity);
                    else
                        setParameterValue(cmd, i++, DBNull.Value);
                    if (agglomeration.aggAccOverflowsSpecified)
                        setParameterValue(cmd, i++, agglomeration.aggAccOverflows);
                    else
                        setParameterValue(cmd, i++, DBNull.Value);
                    if (agglomeration.aggAccOverflowNumberSpecified)
                        setParameterValue(cmd, i++, agglomeration.aggAccOverflowNumber);
                    else
                        setParameterValue(cmd, i++, DBNull.Value);
                    if (agglomeration.aggSewerOverflows_m3Specified)
                        setParameterValue(cmd, i++, agglomeration.aggSewerOverflows_m3);
                    else
                        setParameterValue(cmd, i++, DBNull.Value);
                    if (agglomeration.aggSewerOverflows_peSpecified)
                        setParameterValue(cmd, i++, agglomeration.aggSewerOverflows_pe);
                    else
                        setParameterValue(cmd, i++, DBNull.Value);
                    setParameterValue(cmd, i++, agglomeration.aggRemarks);
                    setParameterValue(cmd, i++, null); //aggCollectedThroughIAS
                    if (agglomeration.aggForecastSpecified)
                        setParameterValue(cmd, i++, agglomeration.aggForecast);
                    else
                        setParameterValue(cmd, i++, null);

                    cmd.ExecuteNonQuery();
                    if (hasErrors)
                        reportFail();
                    else
                        reportSuccess();
                }
                catch (Exception ex)
                {
                    logError("Exception: " + ex.Message + ";");
                    reportFail();
                }
            }
        }

        public override bool checkQAACRule(object value, string fieldName, DataRow repPeriod, bool isImport, string lineInfo)
        {
            bool retVal = true;
            string errorText = "";
            string infoText = "";

            switch (fieldName)
            {
                case "aggName":
                    if (!checkOptionalPresence(value, fieldName, ref infoText))
                        break;

                    if ((value is string) && (((string)value).Length < 1)) {
                        errorText = fieldName + " is too short";
                        retVal = false;
                    }
                    break;
                case "aggNUTS":
                    if (!checkOptionalPresence(value, fieldName, ref infoText))
                        break;

                    if (isImport) {
                        object obj = dataFactory.getNUTSIdForNUTSCode((string)value);
                        if ((obj == null) || (obj is DBNull)) {
                            errorText = fieldName + " contains invalid NUTS-Code \"" + value + "\".";
                            retVal = false;
                            break;
                        }
                    } 
                    break;
                case "aggState":
                    if (!checkFieldMandatoryPresence(value, fieldName, ref errorText))
                    {
                        retVal = false;
                        break;
                    }
                    if (value is string)
                    {
                        if (dataFactory.getLovRowByKey("STAT", value as string)["lovID"] is DBNull)
                        {
                            retVal = false;
                            errorText = fieldName + " not set to a correct value (" + value + ")";
                        }
                    }
                    else
                        if (!dataFactory.getLovRowByID(value)["lovName"].Equals("STAT"))
                        {
                            retVal = false;
                            errorText = fieldName + " not set to a correct value (" + value + ")";
                        }
                    break;
                case "aggCode":
                    if (!checkFieldMandatoryPresence(value, fieldName, ref errorText))
                    {
                        retVal = false;
                        break;
                    }
                    string rptMemberState = repPeriod.GetParentRow("T_ReporterT_ReportPeriod").GetParentRow("T_Lov_T_Reporter")["lovKey"] as string;
                    if (!((value as string).StartsWith(rptMemberState)))
                    {
                        retVal = false;
                        errorText = fieldName + " (" + value + ") does not start with " + rptMemberState;
                        break;
                    }
                    break;
                case "aggGenerated":
                case "aggPeriodOver":
                    if (!checkFieldMandatoryPresence(value, fieldName, ref errorText))
                    {
                        retVal = false;
                        break;
                    }
                    break;
                case "aggSewageNetwork":
                    if (!checkOptionalPresence(value, fieldName, ref errorText))
                    {
                        break;
                    }

                    if (!checkLOVValue(value, fieldName, "SEWANET", ref errorText, isImport, null))
                    {
                        retVal = false;
                    }
                    break;
                case "aggMethodC1":
                case "aggMethodC2":
                case "aggMethodWithoutTreatment":
                    if (!checkOptionalPresence(value, fieldName, ref errorText))
                    {
                        break;
                    }
                    if (isImport)
                    {
                        if (!(value is string))
                        {
                            errorText = fieldName + " error in program execution - on import this item should be Text";
                            retVal = false;
                            break;
                        }

                        if (dataFactory.getLovRowByKey("COLLDET", value as string)["lovID"] is DBNull)
                        {
                            retVal = false;
                            errorText = fieldName + " not set to a correct value (" + value + ")";
                        }
                    }
                    else
                    {
                        if (!dataFactory.getLovRowByID(value)["lovName"].Equals("COLLDET"))
                        {
                            retVal = false;
                            errorText = fieldName + " not set to a correct value (" + value + ")";
                            break;
                        }
                    }
                    break;
                case "aggLatitude":
                case "aggLongitude":
                    checkOptionalPresence(value, fieldName, ref infoText);
                    break;
                default:
                    break;
            }
            if (!retVal)
            {
                if (lineInfo != null)
                {
                    errorText = lineInfo + " - " + errorText;
                }

                logError(errorText);
            }

            if (infoText.Length > 0) {
                if (lineInfo != null) {
                    infoText = lineInfo + " - " + infoText;
                }

                logInfo(infoText);
            }

            return retVal;
        }
    }
}
