﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Data;
using System.IO;
using at.uba.uwwtd2008.xml;

namespace at.uba.uwwtd2008.DataAccess
{
    class InterfaceBigCities : InterfaceBase
    {
        public InterfaceBigCities(IDbConnection conn, IDbTransaction trans, DataFormatConverter conv)
            : base(conn, trans, conv)
        {
            
        }

        private IDbCommand CreateInsertBigCityCommand() 
        {
            string strSQL = "INSERT INTO T_BigCity (bigCityID, bigCountryCode, bigCity) VALUES (?, ?, ?);";

            IDbCommand comInsBigCity = dataFactory.CreateCommand(strSQL, conn, trans);
            comInsBigCity.Parameters.Add(dataFactory.CreateParameter("@bigCityID", DbType.String, 32, "bigCityID", null));
            comInsBigCity.Parameters.Add(dataFactory.CreateParameter("@bigCountryCode", DbType.String, 4, "bigCountryCode", null));
            comInsBigCity.Parameters.Add(dataFactory.CreateParameter("@bigCity", DbType.String, 255, "bigCity", null));

            return comInsBigCity;
        }

        public override void importData(StreamReader rdr, DataRow repPeriod)
        {
            throw new NotImplementedException();
        }

        public override void exportData(StreamWriter wr, DataRow repPeriod)
        {
            throw new NotImplementedException();
        }

        public override void exportXML(UWWTD_DataUWWTD_Report report, DataRow repPeriod)
        {
            List<UWWTD_DataUWWTD_ReportBigCityDischarger> dataList = new List<UWWTD_DataUWWTD_ReportBigCityDischarger>();

            IDbCommand cmd = dataFactory.CreateCommand("SELECT bigCityID, bigCountryCode, bigCity FROM T_BigCity WHERE bigCountryCode = ?;", conn, trans);
            IDataParameter parCmd = dataFactory.CreateParameter("@bigCountryCode", DbType.String, 32, "bigCountryCode", null);
            cmd.Parameters.Add(parCmd);

            logInfo("========================================================================");
            logInfo("  Exporting BigCityDischargers for ReportPeriod " + repPeriod["repCode"]);
            logInfo("========================================================================");

            setParameterValue(cmd, "@bigCountryCode", repPeriod.GetParentRow("T_ReporterT_ReportPeriod")["rptMStateKey"]);
            IDataReader rdr = cmd.ExecuteReader();
            while (rdr.Read())
            {
                bool hasErrors = false;
                try
                {
                    foreach (string fieldName in new string[] { "bigCityID", "bigCountryCode", "bigCity" })
                    {
                        if (!checkQAACRule(rdr[fieldName], fieldName, repPeriod, false, null))
                            hasErrors = true;
                    }
            UWWTD_DataUWWTD_ReportBigCityDischarger city = new UWWTD_DataUWWTD_ReportBigCityDischarger();
            city.bigCityID = rdr["bigCityID"] as string;
            city.bigCountryCode = rdr["bigCountryCode"] as string;
            city.bigCity = rdr["bigCity"] as string;
            dataList.Add(city);
            if (hasErrors)
                reportFail();
            else
                reportSuccess();

                }
                catch (Exception ex)
                {
                    logError("Exception: " + ex.ToString());
                    reportFail();
                }
            }
            report.BigCityDischargers = dataList.ToArray();
        }

        public override void importXML(object collection, DataRow repPeriod)
        {
            IDbCommand cmd = CreateInsertBigCityCommand();

            IDbCommand comBigLookup = dataFactory.CreateCommand("SELECT COUNT(bigID) FROM T_BIGCITY WHERE bigCityID = ? and bigCountryCode = ?;", conn, trans);
            comBigLookup.Parameters.Add(dataFactory.CreateParameter("@bigCityID", DbType.String, 32, "bigCityID", null));
            comBigLookup.Parameters.Add(dataFactory.CreateParameter("@bigCountryCode", DbType.String, 4, "bigCountryCode", null));

            logInfo("========================================================================");
            logInfo("  Importing BigCityDischargers for ReportPeriod " + repPeriod["repCode"]);
            logInfo("========================================================================");

            UWWTD_DataUWWTD_ReportBigCityDischarger[] bigCities = collection as UWWTD_DataUWWTD_ReportBigCityDischarger[];
            foreach (UWWTD_DataUWWTD_ReportBigCityDischarger bigCity in bigCities)
            {
                setParameterValue(comBigLookup, 0, bigCity.bigCityID);
                setParameterValue(comBigLookup, 1, bigCity.bigCountryCode);

                if (UWWXMLWriter.convertInt64(comBigLookup.ExecuteScalar()) < 1)
                {
                    setParameterValue(cmd, 0, bigCity.bigCityID);
                    setParameterValue(cmd, 1, bigCity.bigCountryCode);
                    setParameterValue(cmd, 2, bigCity.bigCity);
                    cmd.ExecuteNonQuery();
                }
            }
        }

        public override bool checkQAACRule(object value, string fieldName, DataRow repPeriod, bool isImport, string lineInfo)
        {
            bool retVal = true;
            string errorText = "";
            string rptMemberState = "";

            switch (fieldName)
            {
                case "bigCityID":
                    if (!checkFieldMandatoryPresence(value, fieldName, ref errorText))
                    {
                        retVal = false;
                        break;
                    }
                    rptMemberState = repPeriod.GetParentRow("T_ReporterT_ReportPeriod").GetParentRow("T_Lov_T_Reporter")["lovKey"] as string;
                    if (!((value as string).StartsWith(rptMemberState)))
                    {
                        retVal = false;
                        errorText = fieldName + " (" + value + ") does not start with " + rptMemberState;
                        break;
                    }
                    break;
                case "bigCountryCode":
                    if (!checkFieldMandatoryPresence(value, fieldName, ref errorText))
                    {
                        retVal = false;
                        break;
                    }
                    rptMemberState = repPeriod.GetParentRow("T_ReporterT_ReportPeriod").GetParentRow("T_Lov_T_Reporter")["lovKey"] as string;
                    if (!((value as string).Equals(rptMemberState)))
                    {
                        retVal = false;
                        errorText = fieldName + " (" + value + ") does not start with " + rptMemberState;
                        break;
                    }
                    break;
                case "bigCity":
                    if (!checkFieldMandatoryPresence(value, fieldName, ref errorText))
                    {
                        retVal = false;
                        break;
                    }
                    break;
                default:
                    break;
            }

            if (!retVal)
            {
                if (lineInfo != null)
                {
                    errorText = lineInfo + " - " + errorText;
                }

                logError(errorText);
            }

            return retVal;
        }
    }
}
