﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Data;
using System.IO;
using at.uba.uwwtd2008.xml;

namespace at.uba.uwwtd2008.DataAccess
{
    class InterfaceDischargePoint : InterfaceBase
    {
        private string []dcpFldNames = {"dcpState", "repCode", "uwwID", "dcpCode", "dcpName", "dcpNuts", 
            "dcpLatitude", "dcpLongitude", "dcpWaterBodyType", "dcpIrrigation", "dcpTypeOfReceivingArea", 
            "rcaID", "dcpSurfaceWaters", "dcpWaterbodyID", "dcpNotAffect", "dcpMSProvide", "dcpCOMAccept", 
            "dcpGroundWater", "dcpReceivingWater", "dcpWFDSubUnit", "dcpWFDRBD", "dcpRemarks", "exportDate", 
            "dcpWFDRBDReferenceDate", "dcpWaterBodyReferenceDate", "dcpGroundWaterReferenceDate", 
            "dcpReceivingWaterReferenceDate", "dcpWFDSubUnitReferenceDate"};

        public InterfaceDischargePoint(IDbConnection conn, IDbTransaction trans, DataFormatConverter conv)
            : base(conn, trans, conv)
        {
        }

        private IDbCommand CreateSelectCommand()
        {
            string strSQL = "SELECT dcpState, repID, uwwID, dcpCode, dcpName, dcpNuts, dcpLatitude, dcpLongitude, " +
             "dcpWaterBodyType, dcpIrrigation, dcpTypeOfReceivingArea, rcaID, dcpSurfaceWaters, dcpWaterbodyID, dcpNotAffect, " +
             "dcpMSProvide, dcpCOMAccept, dcpGroundWater, dcpReceivingWater, dcpWFDSubUnit, dcpWFDRBD, dcpRemarks, " +
             "dcpWFDRBDReferenceDate, dcpWaterBodyReferenceDate, dcpGroundWaterReferenceDate, dcpReceivingWaterReferenceDate, dcpWFDSubUnitReferenceDate FROM " +
             "T_DischargePoints WHERE repID = ?;";

            IDbCommand comSelCmd = dataFactory.CreateCommand(strSQL, conn, trans);

            comSelCmd.Parameters.Add(dataFactory.CreateParameter("@repID", DbType.Int32, 0, "repID", null));
            return comSelCmd;
        }

        private IDbCommand CreateCommand()
        {
		    string strSQL = "INSERT INTO T_DischargePoints ( dcpState, repID, uwwID, dcpCode, dcpName, dcpNuts, dcpLatitude, dcpLongitude, " +
		     "dcpWaterBodyType, dcpIrrigation, dcpTypeOfReceivingArea, rcaID, dcpSurfaceWaters, dcpWaterbodyID, dcpNotAffect, " +
             "dcpMSProvide, dcpCOMAccept, dcpGroundWater, dcpReceivingWater, dcpWFDSubUnit, dcpWFDRBD, dcpRemarks, dcpWFDRBDReferenceDate, "+
             "dcpWaterBodyReferenceDate, dcpGroundWaterReferenceDate, dcpReceivingWaterReferenceDate, dcpWFDSubUnitReferenceDate ) " +
		     "VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?); ";

            IDbCommand comInsCmd = dataFactory.CreateCommand(strSQL, conn, trans);

		    comInsCmd.Parameters.Add(dataFactory.CreateParameter("@dcpState", DbType.Int64, 0, "dcpState", null));
		    comInsCmd.Parameters.Add(dataFactory.CreateParameter("@repID", DbType.Int64, 0, "repID", null));
		    comInsCmd.Parameters.Add(dataFactory.CreateParameter("@uwwID", DbType.Int64, 0, "uwwID", null));
            comInsCmd.Parameters.Add(dataFactory.CreateParameter("@dcpCode", DbType.String, 32, "dcpCode", null));
            comInsCmd.Parameters.Add(dataFactory.CreateParameter("@dcpName", DbType.String, 255, "dcpName", null));
		    comInsCmd.Parameters.Add(dataFactory.CreateParameter("@dcpNuts", DbType.Int64, 0, "dcpNuts", null));
		    comInsCmd.Parameters.Add(dataFactory.CreateParameter("@dcpLatitude", DbType.String, 20, "dcpLatitude", null));
            comInsCmd.Parameters.Add(dataFactory.CreateParameter("@dcpLongitude", DbType.String, 20, "dcpLongitude", null));
		    comInsCmd.Parameters.Add(dataFactory.CreateParameter("@dcpWaterBodyType", DbType.Int64, 0, "dcpWaterBodyType", null));
		    comInsCmd.Parameters.Add(dataFactory.CreateParameter("@dcpIrrigation", DbType.Int64, 0, "dcpIrrigation", null));
		    comInsCmd.Parameters.Add(dataFactory.CreateParameter("@dcpTypeOfReceivingArea", DbType.Int64, 0, "dcpTypeOfReceivingArea", null));
		    comInsCmd.Parameters.Add(dataFactory.CreateParameter("@rcaID", DbType.Int64, 0, "rcaID", null));
		    comInsCmd.Parameters.Add(dataFactory.CreateParameter("@dcpSurfaceWaters", DbType.Int64, 0, "dcpSurfaceWaters", null));
            comInsCmd.Parameters.Add(dataFactory.CreateParameter("@dcpWaterbodyID", DbType.String, 64, "dcpWaterbodyID", null));
		    comInsCmd.Parameters.Add(dataFactory.CreateParameter("@dcpNotAffect", DbType.Int64, 0, "dcpNotAffect", null));
		    comInsCmd.Parameters.Add(dataFactory.CreateParameter("@dcpMSProvide", DbType.Int64, 0, "dcpMSProvide", null));
		    comInsCmd.Parameters.Add(dataFactory.CreateParameter("@dcpCOMAccept", DbType.Int64, 0, "dcpCOMAccept", null));
		    comInsCmd.Parameters.Add(dataFactory.CreateParameter("@dcpGroundWater", DbType.String, 64, "dcpGroundWater", null));
		    comInsCmd.Parameters.Add(dataFactory.CreateParameter("@dcpReceivingWater", DbType.String, 64, "dcpReceivingWater", null));
		    comInsCmd.Parameters.Add(dataFactory.CreateParameter("@dcpWFDSubUnit", DbType.String, 64, "dcpWFDSubUnit", null));
		    comInsCmd.Parameters.Add(dataFactory.CreateParameter("@dcpWFDRBD", DbType.String, 64, "dcpWFDRBD", null));
		    comInsCmd.Parameters.Add(dataFactory.CreateParameter("@dcpRemarks", DbType.String, 4096, "dcpRemarks", null));
            comInsCmd.Parameters.Add(dataFactory.CreateParameter("@dcpWFDRBDReferenceDate", DbType.Date, 0, "dcpWFDRBDReferenceDate", null));
            comInsCmd.Parameters.Add(dataFactory.CreateParameter("@dcpWaterBodyReferenceDate", DbType.Date, 0, "dcpWaterBodyReferenceDate", null));
            comInsCmd.Parameters.Add(dataFactory.CreateParameter("@dcpGroundWaterReferenceDate", DbType.Date, 0, "dcpGroundWaterReferenceDate", null));
            comInsCmd.Parameters.Add(dataFactory.CreateParameter("@dcpReceivingWaterReferenceDate", DbType.Date, 0, "dcpReceivingWaterReferenceDate", null));
            comInsCmd.Parameters.Add(dataFactory.CreateParameter("@dcpWFDSubUnitReferenceDate", DbType.Date, 0, "dcpWFDSubUnitReferenceDate", null));

            return comInsCmd;
        }

        public override void importData(StreamReader rdr, DataRow repPeriod)
        {
            System.Collections.ArrayList lstUniqAggCodes = new System.Collections.ArrayList();
            IDbCommand comUWWTP = dataFactory.CreateCommand("SELECT uwwID FROM T_UWWTPS WHERE uwwCode = ?;", conn, trans);
            IDataParameter parUWWTP = dataFactory.CreateParameter("@uwwCode", DbType.String, 32, "uwwCode", null);
            comUWWTP.Parameters.Add(parUWWTP);
            IDbCommand comReceivingArea = dataFactory.CreateCommand("SELECT rcaID FROM T_ReceivingAreas WHERE rcaCode = ?;", conn, trans);
            IDataParameter parReceivingArea = dataFactory.CreateParameter("@rcaCode", DbType.String, 32, "rcaCode", null);
            comReceivingArea.Parameters.Add(parReceivingArea);

            logInfo("========================================================================");
            logInfo("  Importing Discharge Points");
            logInfo("========================================================================");
            string line;
            long lineNo = 0;
            while ((line = rdr.ReadLine()) != null)
            {
                bool hadError = false;
                lineNo++;
                line = line.TrimEnd();
                if (line.Length < 1) continue; // ignore empty lines
                string[] parts = line.Split(new string[] { "\t" }, StringSplitOptions.None);

                try
                {
                    IDbCommand cmd = CreateCommand();
                    int j = 0;
                    for (int idx = 0; idx < parts.Length; idx++)
                    {
                        if (idx >= dcpFldNames.Length)
                        {
                            // do error reporting ("too many items")
                            continue;
                        }
                        string strFeldName = dcpFldNames[idx];
                        if (strFeldName.Equals("exportDate")) // this is nothing to import
                            continue;
                        object objTemp = parts[idx];
                        if (!checkQAACRule(objTemp, strFeldName, repPeriod, true, "line: " + lineNo))
                            hadError = true;
                        if (parts[idx].Length > 0)
                        {
                            //blnRefValue = true;
                            switch (strFeldName)
                            {
                                case "repCode":
                                    objTemp = dataFactory.getRepIDforReportCode(parts[idx]);
                                    // if (objTemp == null)  do error reporting
                                    setParameterValue(cmd, j, objTemp);
                                    break;
                                case "dcpState":
                                    objTemp = dataFactory.getLovRowByKey("STAT", parts[idx])["lovID"];
                                    // if (objTemp == null) do error reporting
                                    setParameterValue(cmd, j, objTemp);
                                    break;
                                case "uwwID":
                                    parUWWTP.Value = parts[idx];
                                    objTemp = comUWWTP.ExecuteScalar();
                                    // if (objTemp == null) do error reporting
                                    setParameterValue(cmd, j, objTemp);
                                    break;
                                case "dcpNuts":
                                    objTemp = dataFactory.getNUTSIdForNUTSCode(parts[idx]);
                                    // if (objTemp == null) do error reporting
                                    setParameterValue(cmd, j, objTemp);
                                    break;
                                case "dcpWaterBodyType":
                                    string wattype = "WATTYP2";
                                    if (parts[Array.IndexOf(dcpFldNames, "dcpSurfaceWaters")].Equals("1"))
                                        wattype = "WATTYP1";
                                    DataRow lovRow = dataFactory.getLovRowByKey(wattype, parts[idx]);
                                    if (lovRow == null)
                                    {
                                        // do error reporting
                                    }
                                    else
                                    {
                                        objTemp = lovRow["lovID"];
                                        setParameterValue(cmd, j, objTemp);
                                    }
                                    break;
                                case "dcpIrrigation":
                                    objTemp = dataFactory.getLovRowByKey("LAND", parts[idx])["lovID"];
                                    // if (objTemp == null) do error reporting
                                    setParameterValue(cmd, j, objTemp);
                                    break;
                                case "dcpTypeOfReceivingArea":
                                    if ((objTemp as string).Equals("CSA"))
                                        objTemp = "SA";
                                    objTemp = dataFactory.getLovRowByKey("RATYP", objTemp as string)["lovID"];
                                    // if (objTemp == null) do error reporting
                                    setParameterValue(cmd, j, objTemp);
                                    break;
                                case "rcaID":
                                    parReceivingArea.Value = parts[idx];
                                    if (parts[idx].Length < 1)
                                    {
                                        setParameterValue(cmd, j, null);
                                        break;
                                    }
                                    objTemp = comReceivingArea.ExecuteScalar();
                                    if (objTemp == null)
                                    {
                                        logError("Receiving Area " + parts[idx] + " could not be found");
                                        hadError = true;
                                    }
                                    setParameterValue(cmd, j, objTemp);
                                    break;
                                case "dcpSurfaceWaters":
                                    objTemp = dataFactory.getLovRowByKey("YN", parts[idx])["lovID"];
                                    // if (objTemp == null) do error reporting
                                    setParameterValue(cmd, j, objTemp);
                                    break;
                                case "dcpNotAffect":
                                    objTemp = dataFactory.getLovRowByKey("YN", parts[idx])["lovID"];
                                    // if (objTemp == null) do error reporting
                                    setParameterValue(cmd, j, objTemp);
                                    break;
                                case "dcpMSProvide":
                                    objTemp = dataFactory.getLovRowByKey("YN", parts[idx])["lovID"];
                                    // if (objTemp == null) do error reporting
                                    setParameterValue(cmd, j, objTemp);
                                    break;
                                case "dcpCOMAccept":
                                    objTemp = dataFactory.getLovRowByKey("YN", parts[idx])["lovID"];
                                    // if (objTemp == null) do error reporting
                                    setParameterValue(cmd, j, objTemp);
                                    break;
                                case "dcpCode":
                                    if (lstUniqAggCodes.Contains(parts[idx]))
                                    {
                                        //strFCode = "INVKEY"
                                        //strFeldName = "rcaCode"
                                        //strWert = CStr(colRcaInp(intColCnt))
                                        //Err.Raise(10000)
                                        hadError = true;
                                    }
                                    else
                                    {
                                        setParameterValue(cmd, j, parts[idx]);
                                        lstUniqAggCodes.Add(parts[idx]);
                                    }
                                    break;
                                default:
                                    setParameterValue(cmd, j, parts[idx]);
                                    break;
                            }
                        }
                        else
                        {
                            setParameterValue(cmd, j, null);
                        }
                        j++;
                    }

                    cmd.ExecuteNonQuery();
                    if (!hadError)
                        reportSuccess();
                    else
                        reportFail();
                }
                catch (Exception ex) {
                    logError("Failed to do import - "+ex.Message);
                    reportFail();
                    logError(ex.StackTrace);
                }
            }
        }

        public override void exportXML(UWWTD_DataUWWTD_Report report, DataRow repPeriod)
        {
            List<UWWTD_DataUWWTD_ReportDischargePoint> dataList = new List<UWWTD_DataUWWTD_ReportDischargePoint>();

            IDbCommand cmd = CreateSelectCommand();
            IDbCommand comAgglo = dataFactory.CreateCommand("SELECT aggCode FROM T_Agglomerations WHERE aggID = ?;", conn, trans);
            IDataParameter parAggID = dataFactory.CreateParameter("@aggID", DbType.String, 32, "aggID", null);
            comAgglo.Parameters.Add(parAggID);

            logInfo("========================================================================");
            logInfo("  Exporting DischargePoints for ReportPeriod " + repPeriod["repCode"]);
            logInfo("========================================================================");

            setParameterValue(cmd, "@repID", repPeriod["repID"]);
            IDataReader rdr = cmd.ExecuteReader();
            while (rdr.Read())
            {
                bool hasErrors = false;
                try
                {
                    int fieldNo = 0;
                    for (int j = 0; j < dcpFldNames.Length; j++)
                    {
                        if (dcpFldNames[j].Equals("exportDate")) continue;
                        if (!checkQAACRule(rdr[fieldNo], dcpFldNames[j], repPeriod, false, null))
                            hasErrors = true;
                        fieldNo++;
                    }
                    UWWTD_DataUWWTD_ReportDischargePoint dp = new UWWTD_DataUWWTD_ReportDischargePoint();
                    dp.dcpState = dataFactory.getLovRowByID(rdr["dcpState"])["lovKey"] as string;
                    if (!(rdr["uwwID"] is DBNull))
                    {
                        dp.uwwID = (dataFactory.CreateCommand("SELECT uwwCode FROM T_UWWTPS WHERE uwwID = " + rdr["uwwID"], conn).ExecuteScalar()) as string;
                    }
                    dp.dcpCode = rdr["dcpCode"] as string;
                    dp.dcpName = rdr["dcpName"] as string;
                    if (rdr["dcpNuts"] is DBNull)
                        dp.dcpNUTS = null;
                    else
                        dp.dcpNUTS = dataFactory.getNutsCode(rdr["dcpNuts"]);
                    dp.dcpLatitudeSpecified = !(rdr["dcpLatitude"] is DBNull);
                    if (dp.dcpLatitudeSpecified)
                        dp.dcpLatitude = (decimal) DataAccessFactory.parseDBDecimal(rdr["dcpLatitude"]);
                    dp.dcpLongitudeSpecified = !((rdr["dcpLongitude"] is DBNull));
                    if (dp.dcpLongitudeSpecified)
                        dp.dcpLongitude = (decimal) DataAccessFactory.parseDBDecimal(rdr["dcpLongitude"]);

                    if (rdr["dcpWaterBodyType"] is DBNull)
                        dp.dcpWaterBodyType = null;
                    else
                        dp.dcpWaterBodyType = dataFactory.getLovRowByID(rdr["dcpWaterBodyType"])["lovKey"] as string;

                    if (rdr["dcpIrrigation"] is DBNull)
                        dp.dcpIrrigation = null;
                    else
                        dp.dcpIrrigation = dataFactory.getLovRowByID(rdr["dcpIrrigation"])["lovKey"] as string;
                    if (rdr["dcpTypeOfReceivingArea"] is DBNull)
                        dp.dcpTypeOfReceivingArea = null;
                    else
                        dp.dcpTypeOfReceivingArea = dataFactory.getLovRowByID(rdr["dcpTypeOfReceivingArea"])["lovKey"] as string;

                    if (rdr["rcaID"] is DBNull)
                        dp.rcaID = null;
                    else
                        dp.rcaID = (dataFactory.CreateCommand("SELECT rcaCode FROM T_ReceivingAreas WHERE rcaID = " + rdr["rcaID"], conn).ExecuteScalar()) as string;

                    if (rdr["dcpWaterbodyID"] is DBNull)
                        dp.dcpWaterbodyID = null;
                    else
                        dp.dcpWaterbodyID = rdr["dcpWaterbodyID"] as string;
                    if (rdr["dcpSurfaceWaters"] is DBNull)
                        dp.dcpSurfaceWaters = null;
                    else
                        dp.dcpSurfaceWaters = dataFactory.getLovRowByID(rdr["dcpSurfaceWaters"])["lovKey"] as string;
                    if (rdr["dcpNotAffect"] is DBNull)
                        dp.dcpNotAffect = null;
                    else
                        dp.dcpNotAffect = dataFactory.getLovRowByID(rdr["dcpNotAffect"])["lovKey"] as string;
                    if (rdr["dcpMSProvide"] is DBNull)
                        dp.dcpMSProvide = null;
                    else
                        dp.dcpMSProvide = dataFactory.getLovRowByID(rdr["dcpMSProvide"])["lovKey"] as string;
                    if (rdr["dcpCOMAccept"] is DBNull)
                        dp.dcpCOMAccept = null;
                    else
                        dp.dcpCOMAccept = dataFactory.getLovRowByID(rdr["dcpCOMAccept"])["lovKey"] as string;
                    dp.dcpGroundWater = rdr["dcpGroundWater"] as string;
                    dp.dcpReceivingWater = rdr["dcpReceivingWater"] as string;
                    dp.dcpWFDSubUnit = rdr["dcpWFDSubUnit"] as string;
                    dp.dcpWFDRBD = rdr["dcpWFDRBD"] as string;
                    dp.dcpRemarks = rdr["dcpRemarks"] as string;
                    dp.dcpWFDRBDReferenceDateSpecified = rdr["dcpWFDRBDReferenceDate"] is DateTime;
                    if (dp.dcpWFDRBDReferenceDateSpecified)
                    {
                        dp.dcpWFDRBDReferenceDate = (DateTime)rdr["dcpWFDRBDReferenceDate"];
                    }
                    dp.dcpWaterBodyReferenceDateSpecified = rdr["dcpWaterBodyReferenceDate"] is DateTime;
                    if (dp.dcpWaterBodyReferenceDateSpecified)
                    {
                        dp.dcpWaterBodyReferenceDate = (DateTime)rdr["dcpWaterBodyReferenceDate"];
                    }
                    dp.dcpGroundWaterReferenceDateSpecified = rdr["dcpGroundWaterReferenceDate"] is DateTime;
                    if (dp.dcpGroundWaterReferenceDateSpecified)
                    {
                        dp.dcpGroundWaterReferenceDate = (DateTime)rdr["dcpGroundWaterReferenceDate"];
                    }
                    dp.dcpReceivingWaterReferenceDateSpecified = rdr["dcpReceivingWaterReferenceDate"] is DateTime;
                    if (dp.dcpReceivingWaterReferenceDateSpecified)
                    {
                        dp.dcpReceivingWaterReferenceDate = (DateTime)rdr["dcpReceivingWaterReferenceDate"];
                    }
                    dp.dcpWFDSubUnitReferenceDateSpecified = rdr["dcpWFDSubUnitReferenceDate"] is DateTime;
                    if (dp.dcpWFDSubUnitReferenceDateSpecified)
                    {
                        dp.dcpWFDSubUnitReferenceDate = (DateTime)rdr["dcpWFDSubUnitReferenceDate"];
                    }
                    dataList.Add(dp);
                    if (hasErrors)
                        reportFail();
                    else
                        reportSuccess();
                    
                }
                catch (Exception ex)
                {
                    logError("Exception: " + ex.ToString());
                    reportFail();
                }
            }
            report.DischargePoints = dataList.ToArray();
        }

        public override void exportData(StreamWriter wr, DataRow repPeriod)
        {
            IDbCommand comUWWTP = dataFactory.CreateCommand("SELECT uwwCode FROM T_UWWTPS WHERE uwwID = ?;", conn, trans);
            IDataParameter parUWWTP = dataFactory.CreateParameter("@uwwCode", DbType.Int32, 0, "uwwID", null);
            comUWWTP.Parameters.Add(parUWWTP);
            IDbCommand comReceivingArea = dataFactory.CreateCommand("SELECT rcaCode FROM T_ReceivingAreas WHERE rcaID = ?;", conn, trans);
            IDataParameter parReceivingArea = dataFactory.CreateParameter("@rcaCode", DbType.Int32, 0, "rcaID", null);
            comReceivingArea.Parameters.Add(parReceivingArea);

            logInfo("========================================================================");
            logInfo("  Exporting DischargePoints");
            logInfo("========================================================================");

            string[] parts = new string[dcpFldNames.Length];
            IDbCommand cmd = CreateSelectCommand();
            setParameterValue(cmd, 0, repPeriod["repID"]);
            IDataReader rdr = cmd.ExecuteReader();
            try
            {
                while (rdr.Read())
                {
                    bool hasErrors = false;
                    int j = 0;
                    for (int i = 0; i < dcpFldNames.Length; i++)
                    {
                        string strFieldName = dcpFldNames[i];
                        if (strFieldName.Equals("exportDate")) continue;
                        if ((!strFieldName.Equals("repCode")) && (!strFieldName.Equals("rptMStateKey")))
                            if (!checkQAACRule(rdr[strFieldName], strFieldName, repPeriod, false, null))
                                hasErrors = true;

                        switch (strFieldName)
                        {
                            case "repCode":
                                parts[i] = getReaderValueString(repPeriod["repCode"]);
                                break;
                            case "dcpState":
                                parts[i] = getReaderValueString(dataFactory.getLovRowByID(rdr[j])["lovKey"]);
                                break;
                            case "dcpNuts":
                                parts[i] = dataFactory.getNutsCode(rdr[j]);
                                break;
                            case "uwwID":
                                parUWWTP.Value = rdr[j];
                                parts[i] = getReaderValueString(comUWWTP.ExecuteScalar());
                                break;
                            case "dcpLongitude":
                            case "dcpLatitude":
                                parts[i] = getReaderValueString(DataAccessFactory.parseDBDecimal(rdr[j]));
                                break;
                            case "dcpWaterBodyType":
                                parts[i] = getReaderValueString(dataFactory.getLovRowByID(rdr[j])["lovKey"]);
                                break;
                            case "dcpIrrigation":
                                parts[i] = getReaderValueString(dataFactory.getLovRowByID(rdr[j])["lovKey"]);
                                break;
                            case "dcpTypeOfReceivingArea":
                                parts[i] = getReaderValueString(dataFactory.getLovRowByID(rdr[j])["lovKey"]);
                                break;
                            case "rcaID":
                                parReceivingArea.Value = rdr[j];
                                parts[i] = getReaderValueString(comReceivingArea.ExecuteScalar());
                                if ((parts[i] == null) || (parts[i].Length<1))
                                    logger.reportFail("DCPWithoutRA");
                                break;
                            case "dcpSurfaceWaters":
                                parts[i] = getReaderValueString(dataFactory.getLovRowByID(rdr[j])["lovKey"]);
                                break;
                            case "dcpNotAffect":
                                parts[i] = getReaderValueString(dataFactory.getLovRowByID(rdr[j])["lovKey"]);
                                break;
                            case "dcpMSProvide":
                                parts[i] = getReaderValueString(dataFactory.getLovRowByID(rdr[j])["lovKey"]);
                                break;
                            case "dcpCOMAccept":
                                parts[i] = getReaderValueString(dataFactory.getLovRowByID(rdr[j])["lovKey"]);
                                break;
                            default:
                                parts[i] = getReaderValueString(rdr[j]);
                                break;
                        }
                        j++;
                    }
                    parts[Array.IndexOf(dcpFldNames, "exportDate")] = conv.formatDateLong(DateTime.Now);
                    wr.WriteLine(string.Join("\t", parts));

                    if (!hasErrors)
                        reportSuccess();
                    else
                        reportFail();

                }
            }
            finally
            {
                rdr.Close();
            }
        }

        public override void importXML(object collection, DataRow repPeriod)
        {
            UWWTD_DataUWWTD_ReportDischargePoint[] dischargePoints = collection as UWWTD_DataUWWTD_ReportDischargePoint[];
            System.Collections.ArrayList lstUniqCodes = new System.Collections.ArrayList();
            bool hasErrors = false;
            object objTemp;

            IDbCommand cmdGetUwwId = dataFactory.CreateCommand("SELECT uwwID from T_UWWTPS WHERE repID = ? and uwwCode = ?;", conn, trans);
            cmdGetUwwId.Parameters.Add(dataFactory.CreateParameter("@repID", DbType.Int64, 0, "repID", repPeriod["repID"]));
            cmdGetUwwId.Parameters.Add(dataFactory.CreateParameter("@uwwCode", DbType.String, 32, "uwwCode", null));

            IDbCommand cmdGetRcaId = dataFactory.CreateCommand("SELECT rcaID from T_ReceivingAreas WHERE repID = ? and rcaCode = ?;", conn, trans);
            cmdGetRcaId.Parameters.Add(dataFactory.CreateParameter("@repID", DbType.Int64, 0, "repID", repPeriod["repID"]));
            cmdGetRcaId.Parameters.Add(dataFactory.CreateParameter("@rcaCode", DbType.String, 32, "rcaCode", null));

            logInfo("========================================================================");
            logInfo("  Importing Dischargepoints");
            logInfo("========================================================================");
            IDbCommand cmd = CreateCommand();
            foreach (UWWTD_DataUWWTD_ReportDischargePoint dischargePoint in dischargePoints)
            {
                try
                {
                    hasErrors = false;
                    foreach (string strFeldName in dcpFldNames)
                    {
                        try
                        {
                            if (strFeldName.Equals("repCode") || strFeldName.Equals("exportDate")) continue;
                            string strXMLFieldName = strFeldName;
                            if (strXMLFieldName.Equals("dcpNuts")) strXMLFieldName = "dcpNUTS";
                            object value = dischargePoint.GetType().GetField(strXMLFieldName).GetValue(dischargePoint);
                            if (dischargePoint.GetType().GetField(strXMLFieldName + "Specified") != null)
                            {
                                if (!((bool)dischargePoint.GetType().GetField(strXMLFieldName + "Specified").GetValue(dischargePoint))) value = null;
                            }
                            if (!checkQAACRule(value, strXMLFieldName, repPeriod, true, null))
                                hasErrors = true;
                        }
                        catch (Exception ex)
                        {
                            logError("Error in " + strFeldName + ": " + ex.ToString());
                        }

                    }
                    int i = 0;
                    setParameterValue(cmd, i++, dataFactory.getLovRowByKey("STAT", dischargePoint.dcpState)["lovID"]);
                    setParameterValue(cmd, i++, repPeriod["repID"]);
                    setParameterValue(cmdGetUwwId, 1, dischargePoint.uwwID);
                    object uwwID = cmdGetUwwId.ExecuteScalar();
                    if (uwwID == null)
                    {
                        logError("uwwID \"" + dischargePoint.uwwID + "\" could not be found");
                        hasErrors = true;
                    }
                    setParameterValue(cmd, i++, uwwID);
                    setParameterValue(cmd, i++, dischargePoint.dcpCode);
                    setParameterValue(cmd, i++, dischargePoint.dcpName);
                    setParameterValue(cmd, i++, dataFactory.getNUTSIdForNUTSCode(dischargePoint.dcpNUTS));
                    if (dischargePoint.dcpLatitudeSpecified)
                        setParameterValue(cmd, i++, dischargePoint.dcpLatitude);
                    else
                        setParameterValue(cmd, i++, null);
                    if (dischargePoint.dcpLongitudeSpecified)
                        setParameterValue(cmd, i++, dischargePoint.dcpLongitude);
                    else
                        setParameterValue(cmd, i++, null);
                    string wattype = "WATTYP2";
                    if ((dischargePoint.dcpSurfaceWaters != null) && (dischargePoint.dcpSurfaceWaters.Equals("1")))
                        wattype = "WATTYP1";
                    DataRow lovRow = dataFactory.getLovRowByKey(wattype, dischargePoint.dcpWaterBodyType);
                    if (lovRow == null)
                    {
                        logError("lov-entry for " + wattype + " \"" + dischargePoint.dcpWaterBodyType + "\" could not be found");
                        hasErrors = true;
                        setParameterValue(cmd, i++, null);
                    }
                    else
                    {
                        setParameterValue(cmd, i++, lovRow["lovID"]);
                    }

                    if (dischargePoint.dcpIrrigation != null)
                        setParameterValue(cmd, i++, dataFactory.getLovRowByKey("LAND", dischargePoint.dcpIrrigation)["lovID"]);
                    else
                        setParameterValue(cmd, i++, null);

                    objTemp = dischargePoint.dcpTypeOfReceivingArea;
                    if ((objTemp as string).Equals("CSA"))
                        objTemp = "SA";
                    objTemp = dataFactory.getLovRowByKey("RATYP", objTemp as string)["lovID"];
                    setParameterValue(cmd, i++, objTemp);
                    object rcaID = null;
                    if ((dischargePoint.rcaID == null) || (dischargePoint.rcaID.Length < 1))
                    {
                        // allready checked in QA/QC
                        setParameterValue(cmd, i++, null);
                    }
                    else
                    {
                        setParameterValue(cmdGetRcaId, 1, dischargePoint.rcaID);
                        rcaID = cmdGetRcaId.ExecuteScalar();
                        if ((rcaID == null) || (rcaID is DBNull))
                        {
                            logError("rcaID \"" + dischargePoint.rcaID + "\" could not be found");
                            hasErrors = true;
                        }
                        setParameterValue(cmd, i++, rcaID);
                    }
                    objTemp = dataFactory.getLovRowByKey("YN", dischargePoint.dcpSurfaceWaters)["lovID"];
                    setParameterValue(cmd, i++, objTemp);
                    setParameterValue(cmd, i++, dischargePoint.dcpWaterbodyID);
                    objTemp = dataFactory.getLovRowByKey("YN", dischargePoint.dcpNotAffect)["lovID"];
                    setParameterValue(cmd, i++, objTemp);
                    objTemp = dataFactory.getLovRowByKey("YN", dischargePoint.dcpMSProvide)["lovID"];
                    setParameterValue(cmd, i++, objTemp);
                    objTemp = dataFactory.getLovRowByKey("YN", dischargePoint.dcpCOMAccept)["lovID"];
                    setParameterValue(cmd, i++, objTemp);
                    setParameterValue(cmd, i++, dischargePoint.dcpGroundWater);
                    setParameterValue(cmd, i++, dischargePoint.dcpReceivingWater);
                    setParameterValue(cmd, i++, dischargePoint.dcpWFDSubUnit);
                    setParameterValue(cmd, i++, dischargePoint.dcpWFDRBD);
                    setParameterValue(cmd, i++, dischargePoint.dcpRemarks);
                    if (dischargePoint.dcpWFDRBDReferenceDateSpecified)
                        setParameterValue(cmd, i++, dischargePoint.dcpWFDRBDReferenceDate);
                    else
                        setParameterValue(cmd, i++, null);
                    if (dischargePoint.dcpWaterBodyReferenceDateSpecified)
                        setParameterValue(cmd, i++, dischargePoint.dcpWaterBodyReferenceDate);
                    else
                        setParameterValue(cmd, i++, null);
                    if (dischargePoint.dcpGroundWaterReferenceDateSpecified)
                        setParameterValue(cmd, i++, dischargePoint.dcpGroundWaterReferenceDate);
                    else
                        setParameterValue(cmd, i++, null);
                    if (dischargePoint.dcpReceivingWaterReferenceDateSpecified)
                        setParameterValue(cmd, i++, dischargePoint.dcpReceivingWaterReferenceDate);
                    else
                        setParameterValue(cmd, i++, null);
                    if (dischargePoint.dcpWFDSubUnitReferenceDateSpecified)
                        setParameterValue(cmd, i++, dischargePoint.dcpWFDSubUnitReferenceDate);
                    else
                        setParameterValue(cmd, i++, null);
                    cmd.ExecuteNonQuery();
                    if (!hasErrors)
                        reportSuccess();
                    else
                        reportFail();
                }
                catch (Exception ex)
                {
                    logError("Failed to import data - " + ex.Message);
                    logInfo(ex.StackTrace);
                    reportFail();
                }
            }
        }

        public override bool checkQAACRule(object value, string fieldName, DataRow repPeriod, bool isImport, string lineInfo)
        {
            bool retVal = true;
            string errorText = "";
            string infoText = "";

            switch (fieldName)
            {
                case "dcpState":
                    if(!checkFieldMandatoryPresence(value, fieldName, ref errorText)) {
                        retVal = false;
                        break;
                    }
                    if (value is string)
                    {
                        if (dataFactory.getLovRowByKey("STAT", value as string)["lovID"] is DBNull)
                        {
                            retVal = false;
                            errorText = fieldName + " not set to a correct value (" + value + ")";
                        }
                    }
                    else
                        if ((!value.Equals(dataFactory.getLovRowByKey("STAT", "0")["lovID"])) &&
                            (!value.Equals(dataFactory.getLovRowByKey("STAT", "1")["lovID"])))
                        {
                            retVal = false;
                            errorText = fieldName + " not set to a correct value (" + value + ")";
                        }
                    break;
                case "uwwID":
                    if (!checkFieldMandatoryPresence(value, fieldName, ref errorText))
                    {
                        retVal = false;
                        break;
                    }
                    break;
                case "dcpCode":
                    if (!checkFieldMandatoryPresence(value, fieldName, ref errorText))
                    {
                        retVal = false;
                        break;
                    }
                    if (value is string)
                    {
                        string memberstate = repPeriod.GetParentRow("T_ReporterT_ReportPeriod")["rptMStateKey"] as string;
                        if (!((string) value).StartsWith(memberstate))
                        {
                            retVal = false;
                            errorText = fieldName + " for this reporter should start with " + memberstate;
                            break;
                        }
                    }
                    // no more export Checks
                    break;
                case "rcaID":
                    if (!checkOptionalPresence(value, fieldName, ref infoText))
                        break;

                    if (value is string)
                    {
                        string memberstate = repPeriod.GetParentRow("T_ReporterT_ReportPeriod")["rptMStateKey"] as string;
                        if (!((string)value).StartsWith(memberstate))
                        {
                            infoText = fieldName + " should start with " + memberstate;
                            // retVal = false; // since this is an optional field, do not set this as an error
                            break;
                        }
                    }
                    break;
                case "dcpWaterBodyType":
                    if (!checkFieldMandatoryPresence(value, fieldName, ref errorText))
                    {
                        retVal = false;
                        break;
                    }
                    if (value is string)
                    {
                        string strVal = (string)value;
                        if (!(strVal.Equals("CW") ||
                            strVal.Equals("ES") ||
                            strVal.Equals("FW") ||
                            strVal.Equals("LC") ||
                            strVal.Equals("LF")))
                        {

                            retVal = false;
                            errorText = fieldName + " has wrong Value " + strVal + "; should be one of CW, ES, FW, LC or LF";
                        }
                    }
                    break;
                case "dcpLatitude":
                case "dcpLongitude":
                    checkOptionalPresence(value, fieldName, ref infoText);
                    break;
                case "dcpTypeOfReceivingArea":
                    if (checkOptionalPresence(value, fieldName, ref infoText)) {
                        infoText = ""; // do not output anything if field is missing
                        if (!checkLOVValue(value, fieldName, "RATYP", ref errorText, isImport, null))
                            retVal = false;
                    }
                    break;
                case "dcpIrrigation":
                    if (!checkOptionalPresence(value, fieldName, ref infoText)) {
                        infoText = ""; // do not output anything if field is missing
                        break;
                    }
                    if (!checkLOVValue(value, fieldName, "LAND", ref errorText, isImport, null)) {
                        retVal = false;
                    }
                    break;
                case "rcaOverlappingSA":
                case "dcpSurfaceWaters":
                    if (!checkOptionalPresence(value, fieldName, ref infoText))
                    {
                        infoText = ""; // do not output anything if field is missing
                        break;
                    }
                    if (!checkLOVValue(value, fieldName, "YN", ref errorText, isImport, null)) {
                        retVal = false;
                    }
                    break;
                //case "dcpName":
                //case "dcpWaterbodyID":
                //case "dcpWaterBodyReferenceDate":
                //case "dcpNotAffect":
                //case "dcpMSProvide":
                //case "dcpCOMAccept":
                //case "dcpGroundWater":
                //case "dcpGroundWaterReferenceDate":
                //case "dcpReceivingWater":
                //case "dcpReceivingWaterReferenceDate":
                //case "dcpWFDSubUnit":
                //case "dcpWFDSubUnitReferenceDate":
                //case "dcpWFDRBD":
                //case "dcpWFDRBDReferenceDate":
                //case "dcpRemarks":
                //    if (!checkOptionalPresence(value, fieldName, ref infoText))
                //    {
                //        break;
                //    }
                //    break;
                default:
                    break;
            }

            if (!retVal)
            {
                if (lineInfo != null)
                {
                    errorText = lineInfo + " - " + errorText;
                }

                logError(errorText);
            }

            if (infoText.Length > 0) {
                if (lineInfo != null) {
                    infoText = lineInfo + " - " + infoText;
                }

                logInfo(infoText);
            }

            return retVal;
        }
    }
}
