﻿using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Data;
using at.uba.uwwtd2008.xml;

namespace at.uba.uwwtd2008.DataAccess
{
    class InterfaceIndustries : InterfaceBase
    {
        private string[] indFldNames = {"repCode", "mslID", "indCodePlant", "indNamePlant", "indBranch", 
	        "indOrganicLoad", "indConditions", "indDateCompiliance", "exportDate"};

        DataRow reporter;

        public InterfaceIndustries(IDbConnection conn, IDbTransaction trans, DataFormatConverter conv, DataRow reporter)
            : base(conn, trans, conv)
        {
            this.reporter = reporter;
        }

        private IDbCommand CreateSelectCommand()
        {
            string strSQL = "SELECT repID, mslID, indCodePlant, indNamePlant, indBranch, indOrganicLoad, " +
                "indConditions, indDateCompiliance, indState FROM T_Industries WHERE repID = ?; ";

            IDbCommand comSelCmd = dataFactory.CreateCommand(strSQL, conn, trans);

            comSelCmd.Parameters.Add(dataFactory.CreateParameter("@repID", DbType.Int32, 0, "repID", null));
            return comSelCmd;
        }

        private IDbCommand CreateCommand()
        {
            string strSQL = "INSERT INTO T_Industries (repID, mslID, indCodePlant, indNamePlant, indBranch, indOrganicLoad, " +
                "indConditions, indDateCompiliance, indState) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?); ";

            IDbCommand comInsCmd = dataFactory.CreateCommand(strSQL, conn, trans);

            comInsCmd.Parameters.Add(dataFactory.CreateParameter("@repID", DbType.Int64, 0, "repID", null));
            comInsCmd.Parameters.Add(dataFactory.CreateParameter("@mslID", DbType.Int64, 0, "mslID", null));
            comInsCmd.Parameters.Add(dataFactory.CreateParameter("@indCodePlant", DbType.String, 32, "indCodePlant", null));
            comInsCmd.Parameters.Add(dataFactory.CreateParameter("@indNamePlant", DbType.String, 255, "indNamePlant", null));
            comInsCmd.Parameters.Add(dataFactory.CreateParameter("@indBranch", DbType.Int64, 0, "indBranch", null));
            comInsCmd.Parameters.Add(dataFactory.CreateParameter("@indOrganicload", DbType.Int64, 0, "indOrganicload", null));
            comInsCmd.Parameters.Add(dataFactory.CreateParameter("@indConditions", DbType.Boolean, 0, "indConditions", null));
            comInsCmd.Parameters.Add(dataFactory.CreateParameter("@indDateCompiliance", DbType.Date, 0, "indDateCompiliance", null));
            comInsCmd.Parameters.Add(dataFactory.CreateParameter("@indState", DbType.Int64, 0, "indState", dataFactory.getLovRowByKey("STAT", "1")["lovID"]));

            return comInsCmd;
        }

        public override void importData(StreamReader rdr, DataRow repPeriod)
        {
            System.Collections.ArrayList lstUniqAggCodes = new System.Collections.ArrayList();

            logInfo("========================================================================");
            logInfo("  Importing Industries");
            logInfo("========================================================================");
            string line;
            long lineNo = 0;
            while ((line = rdr.ReadLine()) != null)
            {
                bool hadErrors = false;
                lineNo++;
                try
                {
                    line = line.TrimEnd();
                    if (line.Length < 1) continue; // ignore empty lines
                    string[] parts = line.Split(new string[] { "\t" }, StringSplitOptions.None);

                    IDbCommand cmd = CreateCommand();
                    for (int idx = 0; idx < parts.Length; idx++)
                    {
                        if (idx >= indFldNames.Length)
                        {
                            // do error reporting ("too many items")
                            continue;
                        }
                        string strFeldName = indFldNames[idx];
                        if (strFeldName.Equals("exportDate")) // this is nothing to import
                            continue;
                        object objTemp = parts[idx];
                        if (!checkQAACRule(objTemp, strFeldName, repPeriod, true, "line: " + lineNo))
                            hadErrors = true;
                        if (parts[idx].Length > 0)
                        {
                            //blnRefValue = true;
                            switch (strFeldName)
                            {
                                case "repCode":
                                    objTemp = dataFactory.getRepIDforReportCode(parts[idx]);
                                    // if (objTemp == null)  do error reporting
                                    setParameterValue(cmd, idx, objTemp);
                                    break;
                                case "mslID":
                                    //objTemp = dataFactory.getLovRowByKey("MS", parts[idx]);
                                    // if (objTemp == null)  do error reporting
                                    setParameterValue(cmd, idx, reporter["rptMState"]); // special treatment
                                    break;
                                case "indBranch":
                                    objTemp = dataFactory.getLovRowByKey("BRANCH", parts[idx])["lovID"];
                                    // if (objTemp == null)  do error reporting
                                    setParameterValue(cmd, idx, objTemp);
                                    break;
                                default:
                                    setParameterValue(cmd, idx, parts[idx]);
                                    break;
                            }
                        }
                        else
                        {
                            setParameterValue(cmd, idx, null);
                        }
                    }

                    cmd.ExecuteNonQuery();
                    if (hadErrors)
                        reportFail();
                    else
                        reportSuccess();
                }
                catch (Exception ex)
                {
                    logError("Exception " + ex.Message + ";");
                    reportFail();
                }
            }
        }

        public override void exportData(StreamWriter wr, DataRow repPeriod)
        {
            logInfo("========================================================================");
            logInfo("  Exporting Industries");
            logInfo("========================================================================");

            string[] parts = new string[indFldNames.Length];
            IDbCommand cmd = CreateSelectCommand();
            setParameterValue(cmd, 0, repPeriod["repID"]);
            IDataReader rdr = cmd.ExecuteReader();
            try
            {
                while (rdr.Read())
                {
                    try
                    {
                        bool hasErrors = false;
                        for (int i = 0; i < indFldNames.Length; i++)
                        {
                            string strFieldName = indFldNames[i];
                            if (strFieldName.Equals("exportDate")) continue;
                            if (!strFieldName.Equals("repCode"))
                            {
                                if (!checkQAACRule(rdr[strFieldName], strFieldName, repPeriod, false, null))
                                    hasErrors = true;
                            }
                            switch (strFieldName)
                            {
                                case "repCode":
                                    parts[i] = getReaderValueString(repPeriod["repCode"]);
                                    break;
                                case "mslID":
                                    parts[i] = dataFactory.getLovRowByID(repPeriod.GetParentRow("T_ReporterT_ReportPeriod")["rptMState"])["lovKey"] as string;
                                    break;
                                case "indBranch":
                                    if ((rdr[i] == null) || (rdr[i] is DBNull))
                                        parts[i] = "";
                                    else
                                        parts[i] = dataFactory.getLovRowByID(rdr[i])["lovKey"] as string;
                                    break;
                                case "indState":
                                    if ((rdr[i] == null) || (rdr[i] is DBNull))
                                        parts[i] = "";
                                    else
                                        parts[i] = dataFactory.getLovRowByID(rdr[i])["lovKey"] as string;
                                    break;
                                default:
                                    parts[i] = getReaderValueString(rdr[i]);
                                    break;
                            }
                        }
                        parts[parts.Length - 1] = conv.formatDateLong(DateTime.Now);
                        wr.WriteLine(string.Join("\t", parts));

                        if (hasErrors)
                            reportFail();
                        else
                            reportSuccess();
                    }
                    catch (Exception ex)
                    {
                        logError("Exception " + ex.Message + "\n" + ex.StackTrace);
                        reportFail();
                    }

                }
            }
            finally
            {
                rdr.Close();
            }
        }

        public override void importXML(object collection, DataRow repPeriod)
        {
            UWWTD_DataUWWTD_ReportIndustry[] industries = collection as UWWTD_DataUWWTD_ReportIndustry[];
            
            bool hasErrors = false;

            logInfo("========================================================================");
            logInfo("  Importing Industries");
            logInfo("========================================================================");

            IDbCommand cmd = CreateCommand();
            foreach (UWWTD_DataUWWTD_ReportIndustry industry in industries)
            {
                try
                {
                    hasErrors = false;
                    foreach (string strFeldName in indFldNames)
                    {
                        try
                        {
                            if (strFeldName.Equals("repCode") || strFeldName.Equals("exportDate") || strFeldName.Equals("mslID")) continue;
                            object value = industry.GetType().GetField(strFeldName).GetValue(industry);
                            if (industry.GetType().GetField(strFeldName + "Specified") != null)
                            {
                                if (!((bool)industry.GetType().GetField(strFeldName + "Specified").GetValue(industry))) value = null;
                            }
                            if (!checkQAACRule(value, strFeldName, repPeriod, true, null))
                                hasErrors = true;
                        }
                        catch (Exception ex)
                        {
                            logError("Exception at Field: " + strFeldName + " - " + ex.ToString());
                        }
                    }
                    int i = 0;
                    setParameterValue(cmd, i++, repPeriod["repID"]);
                    setParameterValue(cmd, i++, repPeriod.GetParentRow("T_ReporterT_ReportPeriod")["rptMState"]);
                    setParameterValue(cmd, i++, industry.indCodePlant);
                    setParameterValue(cmd, i++, industry.indNamePlant);
                    setParameterValue(cmd, i++, dataFactory.getLovRowByKey("BRANCH", industry.indBranch)["lovID"]);
                    if (industry.indOrganicLoadSpecified)
                        setParameterValue(cmd, i++, industry.indOrganicLoad);
                    else
                        setParameterValue(cmd, i++, null);
                    if (industry.indConditionsSpecified)
                        setParameterValue(cmd, i++, industry.indConditions);
                    else
                        setParameterValue(cmd, i++, null);
                    if (industry.indDateCompilianceSpecified)
                        setParameterValue(cmd, i++, industry.indDateCompiliance);
                    else
                        setParameterValue(cmd, i++, null);
                    setParameterValue(cmd, i++, dataFactory.getLovRowByKey("STAT", industry.indState)["lovID"]);
                    cmd.ExecuteNonQuery();
                    if (!hasErrors)
                        reportSuccess();
                    else
                        reportFail();
                }
                catch (Exception ex)
                {
                    logError("Exception " + ex.Message + "\n" + ex.StackTrace);
                    reportFail();
                }
            }
        
        }

        public override void exportXML(UWWTD_DataUWWTD_Report report, DataRow repPeriod)
        {
            List<UWWTD_DataUWWTD_ReportIndustry> dataList = new List<UWWTD_DataUWWTD_ReportIndustry>();
            IDbCommand cmd = CreateSelectCommand();
            logInfo("========================================================================");
            logInfo("  Exporting Industries for ReportPeriod");
            logInfo("========================================================================");

            setParameterValue(cmd, "@repID", repPeriod["repID"]);
            IDataReader rdr = cmd.ExecuteReader();
            while (rdr.Read())
            {
                bool hasErrors = false;
                try
                {
                    for (int j = 0; j < indFldNames.Length; j++)
                    {
                        if (indFldNames[j].Equals("exportDate")) continue;
                        if (!checkQAACRule(rdr[j], indFldNames[j], repPeriod, false, null))
                            hasErrors = true;
                    }
                    UWWTD_DataUWWTD_ReportIndustry data = new UWWTD_DataUWWTD_ReportIndustry();
                    data.indState = dataFactory.getLovRowByID(rdr["indState"])["lovKey"] as string;
                    data.indCodePlant = rdr["indCodePlant"] as string;
                    data.indNamePlant = rdr["indNamePlant"] as string;
                    if (rdr["indBranch"] is DBNull)
                        data.indBranch = null;
                    else
                        data.indBranch = dataFactory.getLovRowByID(rdr["indBranch"])["lovKey"] as string;
                    if (rdr["indOrganicLoad"] is DBNull)
                        data.indOrganicLoadSpecified = false;
                    else
                    {
                        data.indOrganicLoadSpecified = true;
                        data.indOrganicLoad = long.Parse(rdr["indOrganicLoad"].ToString());
                    }
                    if (rdr["indConditions"] is DBNull)
                        data.indConditionsSpecified = false;
                    else
                    {
                        data.indConditionsSpecified = true;
                        data.indConditions = (bool)rdr["indConditions"];
                    }
                    if (rdr["indDateCompiliance"] is DBNull)
                        data.indDateCompilianceSpecified = false;
                    else
                    {
                        data.indDateCompilianceSpecified = true;
                        data.indDateCompiliance = (DateTime)rdr["indDateCompiliance"];
                    }
                    if (rdr["indState"] is DBNull)
                        data.indState = null;
                    else
                        data.indState = dataFactory.getLovRowByID(rdr["indState"])["lovKey"] as string;

                    dataList.Add(data);
                    if (hasErrors)
                        reportFail();
                    else
                        reportSuccess();
                }
                catch (Exception ex)
                {
                    logError("Exception: " + ex.ToString());
                    reportFail();
                }
            }
            report.Industries = dataList.ToArray();
        }

        public override bool checkQAACRule(object value, string fieldName, DataRow repPeriod, bool isImport, string lineInfo)
        {
            bool retVal = true;
            string errorText = "";

            switch (fieldName)
            {
                case "indState":
                    if (!checkFieldMandatoryPresence(value, fieldName, ref errorText))
                    {
                        retVal = false;
                        break;
                    }
                    if (!checkLOVValue(value, fieldName, "STAT", ref errorText, isImport, null))
                        retVal = false;
                    break;
                case "indCodePlant":
                    if (!checkFieldMandatoryPresence(value, fieldName, ref errorText))
                    {
                        retVal = false;
                        break;
                    }
                    string rptMemberState = repPeriod.GetParentRow("T_ReporterT_ReportPeriod").GetParentRow("T_Lov_T_Reporter")["lovKey"] as string;
                    if (!((value as string).StartsWith(rptMemberState)))
                    {
                        retVal = false;
                        errorText = fieldName + " (" + value + ") does not start with " + rptMemberState;
                        break;
                    }
                    break;
                case "indBranch":
                    if (!checkOptionalPresence(value, fieldName, ref errorText))
                    {
                        break;
                    }
                    if (!checkLOVValue(value, fieldName, "BRANCH", ref errorText, isImport, null))
                        retVal = false;
                    break;
                default:
                    break;
            }

            if (!retVal)
            {
                if (lineInfo != null)
                {
                    errorText = lineInfo + " - " + errorText;
                }

                logError(errorText);
            }

            return retVal;
        }
    }
}
