﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Data;
using System.IO;
using at.uba.uwwtd2008.xml;


namespace at.uba.uwwtd2008.DataAccess
{
    class InterfaceMemberState : InterfaceBase
    {
        private string []mslFldNames = {"rptMStateKey", "repCode", "mslSludgeProduction", "mslDischargePipelines", 
	        "mslDischargeShips", "mslDischargeOthers", "mslReuseSoilAgriculture", "mslReuseOthers", "mslDisposalLandfill", 
            "mslDisposalIncineration", "mslDisposalOthers", "mslRemarks", "mslWWReusePerc", "mslWWReuseAgri", 
            "mslWWReuseInd", "mslWWReuseOther", "mslWWReuseExplain", "exportDate"};

        DataRow reporter;

        public InterfaceMemberState(IDbConnection conn, IDbTransaction trans, DataFormatConverter conv, DataRow reporter)
            : base(conn, trans, conv)
        {
            this.reporter = reporter;
        }

        private IDbCommand CreateSelectCommand()
        {
            string strSQL = "SELECT mslMS, repID, mslSludgeProduction, mslDischargePipelines, " +
                 "mslDischargeShips, mslDischargeOthers, mslReuseSoilAgriculture, mslReuseOthers, mslDisposalLandfill, " +
                 "mslDisposalIncineration, mslDisposalOthers, mslRemarks, mslWWReusePerc, " +
                 "mslWWReuseAgri, mslWWReuseInd, mslWWReuseOther, mslWWReuseExplain FROM T_MSLevel WHERE repID = ?; ";

            IDbCommand comSelCmd = dataFactory.CreateCommand(strSQL, conn, trans);

            comSelCmd.Parameters.Add(dataFactory.CreateParameter("@repID", DbType.Int32, 0, "repID", null));
            return comSelCmd;
        }

        private IDbCommand CreateCommand()
        {
		    string strSQL ="INSERT INTO T_MSLevel (mslMS, repID, mslSludgeProduction, mslDischargePipelines, " +
		         "mslDischargeShips, mslDischargeOthers, mslReuseSoilAgriculture, mslReuseOthers, mslDisposalLandfill, " +
		         "mslDisposalIncineration, mslDisposalOthers, mslRemarks, mslWWReusePerc, " +
		         "mslWWReuseAgri, mslWWReuseInd, mslWWReuseOther, mslWWReuseExplain) " +
		         "VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?); ";

            IDbCommand comInsCmd = dataFactory.CreateCommand(strSQL, conn, trans);

		    comInsCmd.Parameters.Add(dataFactory.CreateParameter("@rptMStateKey", DbType.Int64, 0, "mslMS", null));
		    comInsCmd.Parameters.Add(dataFactory.CreateParameter("@repID", DbType.Int64, 0, "repID", null));
		    comInsCmd.Parameters.Add(dataFactory.CreateParameter("@mslSludgeProduction", DbType.Double, 0, "mslSludgeProduction", null));
		    comInsCmd.Parameters.Add(dataFactory.CreateParameter("@mslDischargePipelines", DbType.Double, 0, "mslDischargePipelines", null));
		    comInsCmd.Parameters.Add(dataFactory.CreateParameter("@mslDischargeShips", DbType.Double, 0, "mslDischargeShips", null));
		    comInsCmd.Parameters.Add(dataFactory.CreateParameter("@mslDischargeOthers", DbType.Double, 0, "mslDischargeOthers", null));
		    comInsCmd.Parameters.Add(dataFactory.CreateParameter("@mslReuseSoilAgriculture", DbType.Double, 0, "mslReuseSoilAgriculture", null));
		    comInsCmd.Parameters.Add(dataFactory.CreateParameter("@mslReuseOthers", DbType.Double, 0, "mslReuseOthers", null));
		    comInsCmd.Parameters.Add(dataFactory.CreateParameter("@mslDisposalLandfill", DbType.Double, 0, "mslDisposalLandfill", null));
		    comInsCmd.Parameters.Add(dataFactory.CreateParameter("@mslDisposalIncineration", DbType.Double, 0, "mslDisposalIncineration", null));
		    comInsCmd.Parameters.Add(dataFactory.CreateParameter("@mslDisposalOthers", DbType.Double, 0, "mslDisposalOthers", null));
		    comInsCmd.Parameters.Add(dataFactory.CreateParameter("@mslRemarks", DbType.String, 255, "mslRemarks", null));
		    comInsCmd.Parameters.Add(dataFactory.CreateParameter("@mslWWReusePerc", DbType.Int64, 0, "mslWWReusePerc", null));
		    comInsCmd.Parameters.Add(dataFactory.CreateParameter("@mslWWReuseAgri", DbType.Boolean, 0, "mslWWReuseAgri", null));
		    comInsCmd.Parameters.Add(dataFactory.CreateParameter("@mslWWReuseInd", DbType.Boolean, 0, "mslWWReuseInd", null));
		    comInsCmd.Parameters.Add(dataFactory.CreateParameter("@mslWWReuseOther", DbType.Boolean, 0, "mslWWReuseOther", null));
		    comInsCmd.Parameters.Add(dataFactory.CreateParameter("@mslWWReuseExplain", DbType.String, 50, "mslWWReuseExplain", null));
            return comInsCmd;
        }

        public override void importData(StreamReader rdr, DataRow repPeriod)
        {
            System.Collections.ArrayList lstUniqAggCodes = new System.Collections.ArrayList();
            IDbCommand comUWWTP = dataFactory.CreateCommand("SELECT uwwID FROM T_UWWTPS WHERE uwwCode = ?;", conn, trans);
            IDataParameter parUWWTP = dataFactory.CreateParameter("@uwwCode", DbType.String, 32, "uwwCode", null);
            comUWWTP.Parameters.Add(parUWWTP);
            IDbCommand comReceivingArea = dataFactory.CreateCommand("SELECT rcaID FROM T_ReceivingAreas WHERE rcaCode = ?;", conn, trans);
            IDataParameter parReceivingArea = dataFactory.CreateParameter("@rcaCode", DbType.String, 32, "rcaCode", null);
            comReceivingArea.Parameters.Add(parReceivingArea);

            logInfo("========================================================================");
            logInfo("  Importing Memberstate Level Data");
            logInfo("========================================================================");
            string line;
            long lineNo = 0;
            while ((line = rdr.ReadLine()) != null)
            {
                lineNo++;
                bool hadError = false;
                try
                {
                    line = line.TrimEnd();
                    if (line.Length < 1) continue; // ignore empty lines
                    string[] parts = line.Split(new string[] { "\t" }, StringSplitOptions.None);

                    IDbCommand cmd = CreateCommand();
                    for (int idx = 0; idx < parts.Length; idx++)
                    {
                        if (idx >= mslFldNames.Length)
                        {
                            // do error reporting ("too many items")
                            continue;
                        }
                        string strFeldName = mslFldNames[idx];
                        if (strFeldName.Equals("exportDate")) // this is nothing to import
                            continue;
                        object objTemp = parts[idx];
                        if (!checkQAACRule(objTemp, strFeldName, repPeriod, true, "line: " + lineNo))
                            hadError = true;
                        if (parts[idx].Length > 0)
                        {
                            //blnRefValue = true;
                            switch (strFeldName)
                            {
                                case "repCode":
                                    objTemp = dataFactory.getRepIDforReportCode(parts[idx]);
                                    // if (objTemp == null)  do error reporting
                                    setParameterValue(cmd, idx, objTemp);
                                    break;
                                case "rptMStateKey":
                                    //objTemp = dataFactory.getLovRowByKey("MS", parts[idx]);
                                    // if (objTemp == null)  do error reporting
                                    setParameterValue(cmd, idx, reporter["rptMState"]); // special treatment
                                    break;
                                default:
                                    setParameterValue(cmd, idx, parts[idx]);
                                    break;
                            }
                        }
                        else
                        {
                            setParameterValue(cmd, idx, null);
                        }
                    }

                    cmd.ExecuteNonQuery();
                    if (hadError)
                        reportFail();
                    else 
                        reportSuccess();
                }
                catch (Exception ex)
                {
                    reportFail();
                    logError("Exception: " + ex.Message + ";");
                }
            }
        }

        public override void exportData(StreamWriter wr, DataRow repPeriod)
        {
            logInfo("========================================================================");
            logInfo("  Exporting Memberstate Level Data");
            logInfo("========================================================================");
            string[] parts = new string[mslFldNames.Length];
            IDbCommand cmd = CreateSelectCommand();
            setParameterValue(cmd, 0, repPeriod["repID"]);
            IDataReader rdr = cmd.ExecuteReader();
            try
            {
                while (rdr.Read())
                {
                    try
                    {
                        bool hasErrors = false;
                        for (int i = 0; i < mslFldNames.Length; i++)
                        {
                            string strFieldName = mslFldNames[i];
                            if (strFieldName.Equals("exportDate")) continue;
                            if ((!strFieldName.Equals("repCode")) && (!strFieldName.Equals("rptMStateKey")))
                                if (!checkQAACRule(rdr[strFieldName], strFieldName, repPeriod, false, null))
                                    hasErrors = true;

                            switch (strFieldName)
                            {
                                case "repCode":
                                    parts[i] = getReaderValueString(repPeriod["repCode"]);
                                    break;
                                case "rptMStateKey":
                                    parts[i] = dataFactory.getLovRowByID(repPeriod.GetParentRow("T_ReporterT_ReportPeriod")["rptMState"])["lovKey"] as string;
                                    break;
                                default:
                                    parts[i] = getReaderValueString(rdr[i]);
                                    break;
                            }
                        }
                        parts[parts.Length - 1] = conv.formatDateLong(DateTime.Now);
                        wr.WriteLine(string.Join("\t", parts));

                        if (!hasErrors)
                            reportSuccess();
                        else
                            reportFail();
                    }
                    catch (Exception ex)
                    {
                        logError("Exception: " + ex.ToString());
                        reportFail();
                    }

                }
            }
            finally
            {
                rdr.Close();
            }
        }

        public override void exportXML(UWWTD_DataUWWTD_Report report, DataRow repPeriod)
        {
            List<UWWTD_DataUWWTD_ReportMSLevel> dataList = new List<UWWTD_DataUWWTD_ReportMSLevel>();
            int count = 0;

            IDbCommand cmd = CreateSelectCommand();
            IDbCommand cmdRca = dataFactory.CreateCommand("select rcaCode from T_ReceivingAreas where rcaID = ?", conn);
            cmdRca.Parameters.Add(dataFactory.CreateParameter("@rcaID", DbType.Int32, 0, "rcaID", DBNull.Value));

            logInfo("========================================================================");
            logInfo("  Exporting Memberstate Level for ReportPeriod " + repPeriod["repCode"]);
            logInfo("========================================================================");

            setParameterValue(cmd, "@repID", repPeriod["repID"]);
            IDataReader rdr = cmd.ExecuteReader();
            while (rdr.Read())
            {
                count++;
                bool hasErrors = false;
                try
                {
                    for (int j = 0; j < mslFldNames.Length; j++)
                    {
                        if (mslFldNames[j].Equals("exportDate")) continue;
                        if (!checkQAACRule(rdr[j], mslFldNames[j], repPeriod, false, null))
                            hasErrors = true;
                    }
                    UWWTD_DataUWWTD_ReportMSLevel data = new UWWTD_DataUWWTD_ReportMSLevel();
                    if (rdr["mslSludgeProduction"] is DBNull)
                        data.mslSludgeProductionSpecified = false;
                    else
                    {
                        data.mslSludgeProductionSpecified = true;
                        data.mslSludgeProduction = long.Parse(rdr["mslSludgeProduction"].ToString());
                    }
                    if (rdr["mslDischargePipelines"] is DBNull)
                        data.mslDischargePipelinesSpecified = false;
                    else
                    {
                        data.mslDischargePipelinesSpecified = true;
                        data.mslDischargePipelines = convertDecimal(rdr["mslDischargePipelines"]);
                    }
                    if (rdr["mslDischargeShips"] is DBNull)
                        data.mslDischargeShipsSpecified = false;
                    else
                    {
                        data.mslDischargeShipsSpecified = true;
                        data.mslDischargeShips = convertDecimal(rdr["mslDischargeShips"]);
                    }
                    if (rdr["mslDischargeOthers"] is DBNull)
                        data.mslDischargeOthersSpecified = false;
                    else
                    {
                        data.mslDischargeOthersSpecified = true;
                        data.mslDischargeOthers = convertDecimal(rdr["mslDischargeOthers"]);
                    }
                    if (rdr["mslReuseSoilAgriculture"] is DBNull)
                        data.mslReuseSoilAgricultureSpecified = false;
                    else
                    {
                        data.mslReuseSoilAgricultureSpecified = true;
                        data.mslReuseSoilAgriculture = convertDecimal(rdr["mslReuseSoilAgriculture"]);
                    }
                    if (rdr["mslReuseOthers"] is DBNull)
                        data.mslReuseOthersSpecified = false;
                    else
                    {
                        data.mslReuseOthersSpecified = true;
                        data.mslReuseOthers = convertDecimal(rdr["mslReuseOthers"]);
                    }
                    if (rdr["mslDisposalLandfill"] is DBNull)
                        data.mslDisposalLandfillSpecified = false;
                    else
                    {
                        data.mslDisposalLandfillSpecified = true;
                        data.mslDisposalLandfill = convertDecimal(rdr["mslDisposalLandfill"]);
                    }
                    if (rdr["mslDisposalIncineration"] is DBNull)
                        data.mslDisposalIncinerationSpecified = false;
                    else
                    {
                        data.mslDisposalIncinerationSpecified = true;
                        data.mslDisposalIncineration = convertDecimal(rdr["mslDisposalIncineration"]);
                    }
                    if (rdr["mslDisposalOthers"] is DBNull)
                        data.mslDisposalOthersSpecified = false;
                    else
                    {
                        data.mslDisposalOthersSpecified = true;
                        data.mslDisposalOthers = convertDecimal(rdr["mslDisposalOthers"]);
                    }

                    data.mslRemarks = rdr["mslRemarks"] as String;

                    if (rdr["mslWWReusePerc"] is DBNull)
                        data.mslWWReusePercSpecified = false;
                    else
                    {
                        data.mslWWReusePercSpecified = true;
                        data.mslWWReusePerc = long.Parse(rdr["mslWWReusePerc"].ToString());
                    }

                    if (rdr["mslWWReuseAgri"] is DBNull)
                        data.mslWWReuseAgriSpecified = false;
                    else
                    {
                        data.mslWWReuseAgriSpecified = true;
                        data.mslWWReuseAgri = (bool)rdr["mslWWReuseAgri"];
                    }
                    if (rdr["mslWWReuseInd"] is DBNull)
                        data.mslWWReuseIndSpecified = false;
                    else
                    {
                        data.mslWWReuseIndSpecified = true;
                        data.mslWWReuseInd = (bool)rdr["mslWWReuseInd"];
                    }
                    if (rdr["mslWWReuseOther"] is DBNull)
                        data.mslWWReuseOtherSpecified = false;
                    else
                    {
                        data.mslWWReuseOtherSpecified = true;
                        data.mslWWReuseOther = (bool)rdr["mslWWReuseOther"];
                    }

                    data.mslWWReuseExplain = rdr["mslWWReuseExplain"] as String;
                    dataList.Add(data);
                    if (hasErrors)
                        reportFail();
                    else
                        reportSuccess();

                }
                catch (Exception ex)
                {
                    logError("Exception: " + ex.ToString());
                    reportFail();
                }
            }
            if (dataList.Count > 0)
                report.MSLevel = dataList.ToArray()[0];

            if (count < 1)
            {
                logError("No Memberstate Level Data Present");
                reportFail();
            } // if
        }

        public override void importXML(object collection, DataRow repPeriod)
        {
            UWWTD_DataUWWTD_ReportMSLevel memberstate = collection as UWWTD_DataUWWTD_ReportMSLevel;
            bool hasErrors = false;

            logInfo("========================================================================");
            logInfo("  Importing Memberstate Level Data");
            logInfo("========================================================================");

            if (memberstate == null)
            {
                logError("No Memberstate Level Data available!!");
                reportFail();
                return;
            }

            IDbCommand cmd = CreateCommand();
            try
            {
                hasErrors = false;
                foreach (string strFeldName in mslFldNames)
                {
                    if (strFeldName.Equals("repCode") || strFeldName.Equals("exportDate") || strFeldName.Equals("rptMStateKey")) continue;
                    object value = memberstate.GetType().GetField(strFeldName).GetValue(memberstate);
                    if (memberstate.GetType().GetField(strFeldName + "Specified") != null)
                    {
                        if (!((bool)memberstate.GetType().GetField(strFeldName + "Specified").GetValue(memberstate))) value = null;
                    }
                    if (!checkQAACRule(value, strFeldName, repPeriod, true, null))
                        hasErrors = true;
                }
                int i = 0;
                setParameterValue(cmd, i++, repPeriod.GetParentRow("T_ReporterT_ReportPeriod")["rptMState"]);
                setParameterValue(cmd, i++, repPeriod["repID"]);
                if (memberstate.mslSludgeProductionSpecified)
                    setParameterValue(cmd, i++, memberstate.mslSludgeProduction);
                else
                    setParameterValue(cmd, i++, null);
                if (memberstate.mslDischargePipelinesSpecified)
                    setParameterValue(cmd, i++, memberstate.mslDischargePipelines);
                else
                    setParameterValue(cmd, i++, null);
                if (memberstate.mslDischargeShipsSpecified)
                    setParameterValue(cmd, i++, memberstate.mslDischargeShips);
                else
                    setParameterValue(cmd, i++, null);
                if (memberstate.mslDischargeOthersSpecified)
                    setParameterValue(cmd, i++, memberstate.mslDischargeOthers);
                else
                    setParameterValue(cmd, i++, null);
                if (memberstate.mslReuseSoilAgricultureSpecified)
                    setParameterValue(cmd, i++, memberstate.mslReuseSoilAgriculture);
                else
                    setParameterValue(cmd, i++, null);
                if (memberstate.mslReuseOthersSpecified)
                    setParameterValue(cmd, i++, memberstate.mslReuseOthers);
                else
                    setParameterValue(cmd, i++, null);
                if (memberstate.mslDisposalLandfillSpecified)
                    setParameterValue(cmd, i++, memberstate.mslDisposalLandfill);
                else
                    setParameterValue(cmd, i++, null);
                if (memberstate.mslDisposalIncinerationSpecified)
                    setParameterValue(cmd, i++, memberstate.mslDisposalIncineration);
                else
                    setParameterValue(cmd, i++, null);
                if (memberstate.mslDisposalOthersSpecified)
                    setParameterValue(cmd, i++, memberstate.mslDisposalOthers);
                else
                    setParameterValue(cmd, i++, null);
                setParameterValue(cmd, i++, memberstate.mslRemarks);
                if (memberstate.mslWWReusePercSpecified)
                    setParameterValue(cmd, i++, memberstate.mslWWReusePerc);
                else
                    setParameterValue(cmd, i++, null);
                if (memberstate.mslWWReuseAgriSpecified)
                    setParameterValue(cmd, i++, memberstate.mslWWReuseAgri);
                else
                    setParameterValue(cmd, i++, null);
                if (memberstate.mslWWReuseIndSpecified)
                    setParameterValue(cmd, i++, memberstate.mslWWReuseInd);
                else
                    setParameterValue(cmd, i++, null);
                if (memberstate.mslWWReuseOtherSpecified)
                    setParameterValue(cmd, i++, memberstate.mslWWReuseOther);
                else
                    setParameterValue(cmd, i++, null);
                setParameterValue(cmd, i++, memberstate.mslWWReuseExplain);
                cmd.ExecuteNonQuery();
                if (!hasErrors)
                {
                    reportSuccess();
                }
                else
                {
                    reportFail();
                }
            }
            catch (Exception ex)
            {
                logError("Exception: " + ex.Message + ";");
                reportFail();
            }
        }

        public override bool checkQAACRule(object value, string fieldName, DataRow repPeriod, bool isImport, string lineInfo)
        {
            bool retVal = true;
            string errorText = "";
            string infoText = "";

            switch (fieldName)
            {
                case "mslSludgeProduction":
                    if (!checkOptionalPresence(value, fieldName, ref infoText))
                    {
                        infoText = fieldName + " is not present";
                        break;
                    }
                    break;
                default:
                    break;
            }
            if (!retVal)
            {
                if (lineInfo != null)
                {
                    errorText = lineInfo + " - " + errorText;
                }

                logError(errorText);
            }

            if (infoText.Length > 0)
            {
                if (lineInfo != null)
                {
                    infoText = lineInfo + " - " + infoText;
                }

                logInfo(infoText);
            }

            return retVal;
        }
    }
}
