﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Data;
using System.IO;
using System.Collections;
using at.uba.uwwtd2008.xml;

namespace at.uba.uwwtd2008.DataAccess
{
    class InterfaceReceivingAreas : InterfaceBase
    {
        private string[] rcaFldNames = {"repCode", "rcaState", "rcaCode", "rcaName", "rcaType", "rcaSensitiveArea", "rcaDateArt58", "rcaParameterN",
             "rcaParameterP", "rcaParameterOther", "rcaDateDesignation", "rcaANitro", "rcaAPhos", "rcaB", "rcaC", "rcaCRelevantDirective",
             "rcaMorphology", "rcaHydrologie", "rcaHydraulic", "rcaAbsenceRisk", "rcaDateArt54", "rcaPlants", "rcaPlantsCapacity",
             "rcaNIncomingMeasured", "rcaNIncomingCalculated", "rcaNIncomingEstimated", "rcaPIncomingMeasured", "rcaPIncomingCalculated",
             "rcaPIncomingEstimated", "rcaNDischargedMeasured", "rcaNDischargedCalculated", "rcaNDischargedEstimated", "rcaPDischargedMeasured",
             "rcaPDischargedCalculated", "rcaPDischargedEstimated", "rcaGIS", "rcaCIDOtherDirective", "rcaCDateOtherDirective", "rcaRemarks",
             "rcaArt54Applied", "exportDate"};

        public InterfaceReceivingAreas(IDbConnection conn, IDbTransaction trans, DataFormatConverter conv)
            : base(conn, trans, conv)
        {
        }

        private IDbCommand CreateSelectCommand()
        {
            string strSQL = "SELECT repID, rcaState, rcaCode, rcaName, rcaType, rcaSensitiveArea, rcaDateArt58, " +
             "rcaParameterN, rcaParameterP, rcaParameterOther, rcaDateDesignation, rcaANitro, rcaAPhos, rcaB, rcaC, rcaCRelevantDirective, " +
             "rcaMorphology, rcaHydrologie, rcaHydraulic, rcaAbsenceRisk, rcaDateArt54, rcaPlants, rcaPlantsCapacity, rcaNIncomingMeasured, " +
             "rcaNIncomingCalculated, rcaNIncomingEstimated, rcaPIncomingMeasured, rcaPIncomingCalculated, rcaPIncomingEstimated, " +
             "rcaNDischargedMeasured, rcaNDischargedCalculated, rcaNDischargedEstimated, rcaPDischargedMeasured, rcaPDischargedCalculated, " +
             "rcaPDischargedEstimated, rcaGIS, rcaCIDOtherDirective, rcaCDateOtherDirective, rcaRemarks, rcaArt54Applied FROM T_ReceivingAreas WHERE repID = ?; ";

            IDbCommand comSelCmd = dataFactory.CreateCommand(strSQL, conn, trans);

            comSelCmd.Parameters.Add(dataFactory.CreateParameter("@repID", DbType.Int32, 0, "repID", null));
            return comSelCmd;
        }

        private IDbCommand CreateUpdateSesitiveAreaCmd()
        {
            string strSQL = "UPDATE T_ReceivingAreas set rcaSensitiveArea = ? where rcaCode = ? and repID = ?";
            IDbCommand comUpdtCmd = dataFactory.CreateCommand(strSQL, conn, trans);
            comUpdtCmd.Parameters.Add(dataFactory.CreateParameter("@rcaSensitiveArea", DbType.Int64, 0, "rcaSensitiveArea", null));
            comUpdtCmd.Parameters.Add(dataFactory.CreateParameter("@rcaCode", DbType.String, 0, "rcaCode", null));
            comUpdtCmd.Parameters.Add(dataFactory.CreateParameter("@repID", DbType.Int64, 0, "repID", null));
            return comUpdtCmd;
        }

        private IDbCommand CreateGetRcaIDCmd()
        {
            string strSQL = "SELECT rcaID from T_ReceivingAreas where rcaCode = ? and repID = ?";
            IDbCommand comSelID = dataFactory.CreateCommand(strSQL, conn, trans);
            comSelID.Parameters.Add(dataFactory.CreateParameter("@rcaCode", DbType.String, 0, "rcaCode", null));
            comSelID.Parameters.Add(dataFactory.CreateParameter("@repID", DbType.Int64, 0, "repID", null));
            return comSelID;
        }

        private IDbCommand CreateCommand()
        {
            string strSQL = "INSERT INTO T_ReceivingAreas ( repID, rcaState, rcaCode, rcaName, rcaType, rcaSensitiveArea, rcaDateArt58, " +
             "rcaParameterN, rcaParameterP, rcaParameterOther, rcaDateDesignation, rcaANitro, rcaAPhos, rcaB, rcaC, rcaCRelevantDirective, " +
             "rcaMorphology, rcaHydrologie, rcaHydraulic, rcaAbsenceRisk, rcaDateArt54, rcaPlants, rcaPlantsCapacity, rcaNIncomingMeasured, " +
             "rcaNIncomingCalculated, rcaNIncomingEstimated, rcaPIncomingMeasured, rcaPIncomingCalculated, rcaPIncomingEstimated, " +
             "rcaNDischargedMeasured, rcaNDischargedCalculated, rcaNDischargedEstimated, rcaPDischargedMeasured, rcaPDischargedCalculated, " +
             "rcaPDischargedEstimated, rcaGIS, rcaCIDOtherDirective, rcaCDateOtherDirective, rcaRemarks, rcaArt54Applied ) " +
             "VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ? ); ";

            IDbCommand comInsCmd = dataFactory.CreateCommand(strSQL, conn, trans);
            comInsCmd.Parameters.Add(dataFactory.CreateParameter("@repID", DbType.Int64, 0, "repID", null));
            comInsCmd.Parameters.Add(dataFactory.CreateParameter("@rcaState", DbType.Int64, 0, "rcaState", null));
            comInsCmd.Parameters.Add(dataFactory.CreateParameter("@rcaCode", DbType.String, 32, "rcaCode", null));
            comInsCmd.Parameters.Add(dataFactory.CreateParameter("@rcaName", DbType.String, 255, "rcaName", null));
            comInsCmd.Parameters.Add(dataFactory.CreateParameter("@rcaType", DbType.Int64, 0, "rcaType", null));
            comInsCmd.Parameters.Add(dataFactory.CreateParameter("@rcaSensitiveArea", DbType.Int64, 0, "rcaSensitiveArea", null));
            comInsCmd.Parameters.Add(dataFactory.CreateParameter("@rcaDateArt58", DbType.Date, 0, "rcaDateArt58", null));
            comInsCmd.Parameters.Add(dataFactory.CreateParameter("@rcaParameterN", DbType.Boolean, 0, "rcaParameterN", null));
            comInsCmd.Parameters.Add(dataFactory.CreateParameter("@rcaParameterP", DbType.Boolean, 0, "rcaParameterP", null));
            comInsCmd.Parameters.Add(dataFactory.CreateParameter("@rcaParameterOther", DbType.Boolean, 0, "rcaParameterOther", null));
            comInsCmd.Parameters.Add(dataFactory.CreateParameter("@rcaDateDesignation", DbType.Date, 0, "rcaDateDesignation", null));
            comInsCmd.Parameters.Add(dataFactory.CreateParameter("@rcaANitro", DbType.Boolean, 0, "rcaANitro", null));
            comInsCmd.Parameters.Add(dataFactory.CreateParameter("@rcaAPhos", DbType.Boolean, 0, "rcaAPhos", null));
            comInsCmd.Parameters.Add(dataFactory.CreateParameter("@rcaB", DbType.Boolean, 0, "rcaB", null));
            comInsCmd.Parameters.Add(dataFactory.CreateParameter("@rcaC", DbType.Boolean, 0, "rcaC", null));
            comInsCmd.Parameters.Add(dataFactory.CreateParameter("@rcaCRelevantDirective", DbType.String, 4096, "rcaCRelevantDirective", null));
            comInsCmd.Parameters.Add(dataFactory.CreateParameter("@rcaMorphology", DbType.Boolean, 0, "rcaMorphology", null));
            comInsCmd.Parameters.Add(dataFactory.CreateParameter("@rcaHydrologie", DbType.Boolean, 0, "rcaHydrologie", null));
            comInsCmd.Parameters.Add(dataFactory.CreateParameter("@rcaHydraulic", DbType.Boolean, 0, "rcaHydraulic", null));
            comInsCmd.Parameters.Add(dataFactory.CreateParameter("@rcaAbsenceRisk", DbType.Boolean, 0, "rcaAbsenceRisk", null));
            comInsCmd.Parameters.Add(dataFactory.CreateParameter("@rcaDateArt54", DbType.Date, 0, "rcaDateArt54", null));
            comInsCmd.Parameters.Add(dataFactory.CreateParameter("@rcaPlants", DbType.Int64, 0, "rcaPlants", null));
            comInsCmd.Parameters.Add(dataFactory.CreateParameter("@rcaPlantsCapacity", DbType.Int64, 0, "rcaPlantsCapacity", null));
            comInsCmd.Parameters.Add(dataFactory.CreateParameter("@rcaNIncomingMeasured", DbType.Double, 0, "rcaNIncomingMeasured", null));
            comInsCmd.Parameters.Add(dataFactory.CreateParameter("@rcaNIncomingCalculated", DbType.Double, 0, "rcaNIncomingCalculated", null));
            comInsCmd.Parameters.Add(dataFactory.CreateParameter("@rcaNIncomingEstimated", DbType.Double, 0, "rcaNIncomingEstimated", null));
            comInsCmd.Parameters.Add(dataFactory.CreateParameter("@rcaPIncomingMeasured", DbType.Double, 0, "rcaPIncomingMeasured", null));
            comInsCmd.Parameters.Add(dataFactory.CreateParameter("@rcaPIncomingCalculated", DbType.Double, 0, "rcaPIncomingCalculated", null));
            comInsCmd.Parameters.Add(dataFactory.CreateParameter("@rcaPIncomingEstimated", DbType.Double, 0, "rcaPIncomingEstimated", null));
            comInsCmd.Parameters.Add(dataFactory.CreateParameter("@rcaNDischargedMeasured", DbType.Double, 0, "rcaNDischargedMeasured", null));
            comInsCmd.Parameters.Add(dataFactory.CreateParameter("@rcaNDischargedCalculated", DbType.Double, 0, "rcaNDischargedCalculated", null));
            comInsCmd.Parameters.Add(dataFactory.CreateParameter("@rcaNDischargedEstimated", DbType.Double, 0, "rcaNDischargedEstimated", null));
            comInsCmd.Parameters.Add(dataFactory.CreateParameter("@rcaPDischargedMeasured", DbType.Double, 0, "rcaPDischargedMeasured", null));
            comInsCmd.Parameters.Add(dataFactory.CreateParameter("@rcaPDischargedCalculated", DbType.Double, 0, "rcaPDischargedCalculated", null));
            comInsCmd.Parameters.Add(dataFactory.CreateParameter("@rcaPDischargedEstimated", DbType.Double, 0, "rcaPDischargedEstimated", null));
            comInsCmd.Parameters.Add(dataFactory.CreateParameter("@rcaGIS", DbType.String, 128, "rcaGIS", null));
            comInsCmd.Parameters.Add(dataFactory.CreateParameter("@rcaCIDOtherDirective", DbType.String, 128, "rcaCIDOtherDirective", null));
            comInsCmd.Parameters.Add(dataFactory.CreateParameter("@rcaCDateOtherDirective", DbType.Date, 0, "rcaCDateOtherDirective", null));
            comInsCmd.Parameters.Add(dataFactory.CreateParameter("@rcaRemarks", DbType.String, 4096, "rcaRemarks", null));
            comInsCmd.Parameters.Add(dataFactory.CreateParameter("@rcaArt54Applied", DbType.Boolean, 0, "rcaArt54Applied", null));
            return comInsCmd;
        }

        public override void importData(StreamReader rdr, DataRow repPeriod)
        {
            System.Collections.ArrayList lstUniqAggCodes = new System.Collections.ArrayList();
            Hashtable sensitiveAreaRefs = new Hashtable();
            ArrayList catchments = new ArrayList();
            bool hasErrors = false;
            int lineNo=0;

            logInfo("========================================================================");
            logInfo("  Importing Receiving Areas");
            logInfo("========================================================================");
            string line;
            while ((line = rdr.ReadLine()) != null)
            {
                lineNo++;
                try
                {
                    hasErrors = false;
                    line = line.TrimEnd(); // do not remove whitespaces from front
                    if (line.Length < 1) continue; // ignore empty lines
                    string[] parts = line.Split(new string[] { "\t" }, StringSplitOptions.None);

                    object objCurRep = null;
                    IDbCommand cmd = CreateCommand();
                    for (int idx = 0; idx < parts.Length; idx++)
                    {
                        if (idx >= rcaFldNames.Length)
                        {
                            logError("Too many Items on line #" + lineNo);
                            // do error reporting ("too many items")
                            hasErrors = true;

                            break;
                        }
                        string strFeldName = rcaFldNames[idx];
                        if (strFeldName.Equals("exportDate")) // this is nothing to import
                            continue;
                        object objTemp = parts[idx];
                        if (!checkQAACRule(objTemp, strFeldName, repPeriod, true, "line: " + lineNo))
                            hasErrors = true;
                        if (parts[idx].Length > 0)
                        {
                            //blnRefValue = true;
                            switch (strFeldName)
                            {
                                case "repCode":
                                    objCurRep = dataFactory.getRepIDforReportCode(parts[idx]);
                                    // if (objTemp == null)  do error reporting
                                    setParameterValue(cmd, idx, objCurRep);
                                    break;
                                case "rcaState":
                                    objTemp = dataFactory.getLovRowByKey("STAT", parts[idx])["lovID"];
                                    // if (objTemp == null) do error reporting
                                    setParameterValue(cmd, idx, objTemp);
                                    break;
                                case "rcaType":
                                    objTemp = dataFactory.getLovRowByKey("RATYP", parts[idx])["lovID"];
                                    setParameterValue(cmd, idx, objTemp);
                                    if (parts[idx].Equals("CSA"))
                                        catchments.Add(parts[Array.IndexOf(rcaFldNames, "rcaCode")]);
                                    break;
                                case "rcaCode":
                                    if (lstUniqAggCodes.Contains(parts[idx]))
                                    {
                                        logError("Receiving Area " + parts[idx] + " is not unique");
                                        hasErrors = true;
                                    }
                                    else
                                    {
                                        setParameterValue(cmd, idx, parts[idx]);
                                    }
                                    break;
                                case "rcaSensitiveArea":
                                    setParameterValue(cmd, idx, null);
                                    RcaRef rcaRef = new RcaRef();
                                    rcaRef.repID = objCurRep;
                                    rcaRef.rcaCode = parts[idx];
                                    sensitiveAreaRefs[rcaRef] = parts[Array.IndexOf(rcaFldNames, "rcaCode")];
                                    break;
                                default:
                                    setParameterValue(cmd, idx, parts[idx]);
                                    break;
                            }
                        }
                        else
                        {
                            setParameterValue(cmd, idx, null);
                        }
                    }

                    // post-check this line
                    bool rcaANitro = !(((IDbDataParameter)cmd.Parameters["@rcaANitro"]).Value is DBNull) && ((bool) (((IDbDataParameter)cmd.Parameters["@rcaANitro"]).Value));
                    bool rcaAPhos = !(((IDbDataParameter)cmd.Parameters["@rcaAPhos"]).Value is DBNull) && ((bool) (((IDbDataParameter)cmd.Parameters["@rcaAPhos"]).Value));
                    bool rcaB = !(((IDbDataParameter)cmd.Parameters["@rcaB"]).Value is DBNull) && ((bool) (((IDbDataParameter)cmd.Parameters["@rcaB"]).Value));
                    bool rcaC = !(((IDbDataParameter)cmd.Parameters["@rcaC"]).Value is DBNull) && ((bool) (((IDbDataParameter)cmd.Parameters["@rcaC"]).Value));
                    bool type5854 = dataFactory.getRATYPLovValues().Select("lovKey='A54'")[0]["lovID"] == cmd.Parameters["@rcaType"];

                    if (!(rcaANitro || rcaAPhos || rcaB || rcaC))
                    {
                        if (type5854)
                        {
                            logError("RcaID " + ((IDbDataParameter)cmd.Parameters["@rcaCode"]).Value + ": No designation criteria specified");
                            hasErrors = true;
                        }
                    }
                    if (rcaC)
                    {
                        string errorText = "";
                        if (!((((IDbDataParameter)cmd.Parameters["@rcaCDateOtherDirective"]).Value) is DBNull))
                        {
                            logError("RcaID " + ((IDbDataParameter)cmd.Parameters["@rcaCode"]).Value + ": rcaCDateOtherSpecified is Master Data - rcaC is Set");
                            hasErrors = true;
                        }
                        if (!checkFieldMandatoryPresence(((IDbDataParameter)cmd.Parameters["@rcaCIDOtherDirective"]).Value, "rcaCIDOtherDirective", ref errorText, "rcaC is Set"))
                        {
                            logError("RcaID " + ((IDbDataParameter)cmd.Parameters["@rcaCode"]).Value + ": " + errorText);
                            hasErrors = true;
                        }
                        if (!checkFieldMandatoryPresence(((IDbDataParameter)cmd.Parameters["@rcaCRelevantDirective"]).Value, "rcaCRelevantDirective", ref errorText, "rcaC is Set"))
                        {
                            logError("RcaID " + ((IDbDataParameter)cmd.Parameters["@rcaCode"]).Value + ": " + errorText);
                            hasErrors = true;
                        }
                    }

                    cmd.ExecuteNonQuery();
                    if (hasErrors)
                        reportFail();
                    else
                        reportSuccess();
                }
                catch (Exception ex)
                {
                    logError("Exception: " + ex.Message + ";");
                    reportFail();
                }
            }

            // execute post-run to update Sensitive Areas List
            foreach (object key in sensitiveAreaRefs.Keys)
            {
                RcaRef rcaRef = (RcaRef)key;
                try
                {
                    // obtain the rcaID for the sensitive Area
                    IDbCommand cmdGetRcaID = CreateGetRcaIDCmd();
                    setParameterValue(cmdGetRcaID, 0, rcaRef.rcaCode);
                    setParameterValue(cmdGetRcaID, 1, rcaRef.repID);
                    object rcaID = cmdGetRcaID.ExecuteScalar();
                    if ((rcaID == null) || (rcaID is DBNull))
                    {
                        // this is an error - sensitive Area was not found .....
                        continue;
                    }
                    IDbCommand cmdUpdtRca = CreateUpdateSesitiveAreaCmd();
                    setParameterValue(cmdUpdtRca, 0, rcaID);
                    setParameterValue(cmdUpdtRca, 1, sensitiveAreaRefs[key]);
                    setParameterValue(cmdUpdtRca, 2, rcaRef.repID);
                    cmdUpdtRca.ExecuteNonQuery();
                    catchments.Remove(sensitiveAreaRefs[key]);
                }
                catch (Exception ex)
                {
                    logError("Exception while trying to connect Sensitive Area " + rcaRef.rcaCode + ": " + ex.Message + "\n" + ex.StackTrace);
                }
                    
            }

            logger.getCounter("RAWithoutCatchment").SetFailedCount(catchments.Count);
        }

        public override void exportData(StreamWriter wr, DataRow repPeriod)
        {
            IDbCommand cmdSensitiveArea = dataFactory.CreateCommand("SELECT rcaCode FROM T_ReceivingAreas WHERE rcaID = ?;", conn, trans);
            IDataParameter parSensitiveArea = dataFactory.CreateParameter("@rcaID", DbType.Int32, 0, "rcaID", null);
            cmdSensitiveArea.Parameters.Add(parSensitiveArea);

            logInfo("========================================================================");
            logInfo("  Exporting Receiving Areas");
            logInfo("========================================================================");

            string[] parts = new string[rcaFldNames.Length];
            IDbCommand cmd = CreateSelectCommand();
            setParameterValue(cmd, 0, repPeriod["repID"]);
            IDataReader rdr = cmd.ExecuteReader();
            try
            {
                while (rdr.Read())
                {
                    bool hasErrors = false;
                    try
                    {
                        for (int i = 0; i < rcaFldNames.Length; i++)
                        {
                            string strFieldName = rcaFldNames[i];
                            if (strFieldName.Equals("exportDate")) continue;
                            if (!strFieldName.Equals("repCode"))
                            {
                                if (!checkQAACRule(rdr[strFieldName], strFieldName, repPeriod, false, "rca : " + parts[Array.IndexOf(rcaFldNames, "rcaCode")]))
                                    hasErrors = true;
                            }

                            switch (strFieldName)
                            {
                                case "repCode":
                                    parts[i] = getReaderValueString(repPeriod["repCode"]);
                                    break;
                                case "rcaState":
                                    parts[i] = dataFactory.getLovRowByID(rdr[i])["lovKey"] as string;
                                    break;
                                case "rcaType":
                                    parts[i] = dataFactory.getLovRowByID(rdr[i])["lovKey"] as string;
                                    break;
                                case "rcaSensitiveArea":
                                    if ((rdr[i] == null) || (rdr[i] is DBNull))
                                    {
                                        parts[i] = getReaderValueString(null);
                                        if (parts[Array.IndexOf(rcaFldNames, "rcaType")].Equals("CSA"))
                                            logger.reportFail("RAWithoutCatchment");
                                    }
                                    else
                                    {
                                        parSensitiveArea.Value = rdr[i];
                                        parts[i] = getReaderValueString(cmdSensitiveArea.ExecuteScalar());
                                    }
                                    break;
                                default:
                                    parts[i] = getReaderValueString(rdr[i]);
                                    break;
                            }
                        }

                        // post-check this line
                        bool rcaANitro = !(rdr["rcaANitro"] is DBNull) && ((bool)(rdr["rcaANitro"]));
                        bool rcaAPhos = !(rdr["rcaAPhos"] is DBNull) && ((bool)(rdr["rcaAPhos"]));
                        bool rcaB = !(rdr["rcaB"] is DBNull) && ((bool)(rdr["rcaB"]));
                        bool rcaC = !(rdr["rcaC"] is DBNull) && ((bool)(rdr["rcaC"]));
                        bool type5854 = dataFactory.getRATYPLovValues().Select("lovKey='A54'")[0]["lovID"] == rdr["rcaType"];

                        if (!(rcaANitro || rcaAPhos || rcaB || rcaC))
                        {
                            if (!type5854)
                            {
                                logError("RcaID " + rdr["rcaCode"] + ": No designation criteria specified");
                                hasErrors = true;
                            }
                        }
                        if (rcaC)
                        {
                            string errorText = "";
                            if (!(rdr["rcaCDateOtherDirective"] is DBNull))
                            {
                                logError("RcaID " + rdr["rcaCode"] + ": rcaCDateOtherSpecified is Master Data - rcaC is Set");
                                hasErrors = true;
                            }
                            if (!checkFieldMandatoryPresence(rdr["rcaCIDOtherDirective"], "rcaCIDOtherDirective", ref errorText, "rcaC is Set"))
                            {
                                logError("RcaID " + rdr["rcaCode"] + ": " + errorText);
                                hasErrors = true;
                            }
                            if (!checkFieldMandatoryPresence(rdr["rcaCRelevantDirective"], "rcaCRelevantDirective", ref errorText, "rcaC is Set"))
                            {
                                logError("RcaID " + rdr["rcaCode"] + ": " + errorText);
                                hasErrors = true;
                            }
                        }
                        parts[parts.Length - 1] = conv.formatDateLong(DateTime.Now);
                        wr.WriteLine(string.Join("\t", parts));

                        if (!hasErrors)
                            reportSuccess();
                        else
                            reportFail();
                    }
                    catch (Exception ex)
                    {
                        logError("Exception: " + ex.Message + ";");
                        reportFail();
                    }
                }
            }
            finally
            {
                rdr.Close();
            }
        }

        public override void exportXML(UWWTD_DataUWWTD_Report report, DataRow repPeriod)
        {
            List<UWWTD_DataUWWTD_ReportReceivingArea> dataList = new List<UWWTD_DataUWWTD_ReportReceivingArea>();

            IDbCommand cmd = CreateSelectCommand();
            IDbCommand cmdRca = dataFactory.CreateCommand("select rcaCode from T_ReceivingAreas where rcaID = ?", conn);
            cmdRca.Parameters.Add(dataFactory.CreateParameter("@rcaID", DbType.Int32, 0, "rcaID", DBNull.Value));

            logInfo("========================================================================");
            logInfo("  Exporting Receiving Areas for ReportPeriod " + repPeriod["repCode"]);
            logInfo("========================================================================");

            setParameterValue(cmd, "@repID", repPeriod["repID"]);
            IDataReader rdr = cmd.ExecuteReader();
            while (rdr.Read())
            {
                bool hasErrors = false;
                try
                {
                    for (int j = 0; j < rcaFldNames.Length; j++)
                    {
                        if (rcaFldNames[j].Equals("exportDate")) continue;
                        if (!checkQAACRule(rdr[j], rcaFldNames[j], repPeriod, false, null))
                            hasErrors = true;
                    }
                    UWWTD_DataUWWTD_ReportReceivingArea data = new UWWTD_DataUWWTD_ReportReceivingArea();
                    data.rcaState = dataFactory.getLovRowByID(rdr["rcaState"])["lovKey"] as string;
                    data.rcaCode = rdr["rcaCode"] as string;
                    data.rcaName = rdr["rcaName"] as string;
                    if (rdr["rcaType"] is DBNull)
                        data.rcaType = null;
                    else
                        data.rcaType = dataFactory.getLovRowByID(rdr["rcaType"])["lovKey"] as string;

                    if (rdr["rcaSensitiveArea"] is DBNull)
                        data.rcaSensitiveArea = null;
                    else
                    {
                        //data.rcaSensitiveArea = dataFactory.getLovRowByID(rdr["rcaSensitiveArea"])["lovKey"] as string;
                        ((IDbDataParameter)cmdRca.Parameters[0]).Value = rdr["rcaSensitiveArea"];
                        data.rcaSensitiveArea = cmdRca.ExecuteScalar() as string;
                        cmd.Dispose();
                    }

                    data.rcaDateArt58Specified = true;
                    if (rdr["rcaDateArt58"] is DBNull)
                        data.rcaDateArt58Specified = false;
                    else
                        data.rcaDateArt58 = ((DateTime)rdr["rcaDateArt58"]);
                    data.rcaParameterPSpecified = true;
                    if (rdr["rcaParameterP"] is DBNull)
                        data.rcaParameterPSpecified = false;
                    else
                        data.rcaParameterP = ((bool)rdr["rcaParameterP"]);
                    data.rcaParameterNSpecified = true;
                    if (rdr["rcaParameterN"] is DBNull)
                        data.rcaParameterNSpecified = false;
                    else
                        data.rcaParameterN = ((bool)rdr["rcaParameterN"]);
                    data.rcaParameterOtherSpecified = true;
                    if (rdr["rcaParameterOther"] is DBNull)
                        data.rcaParameterOtherSpecified = false;
                    else
                        data.rcaParameterOther = ((bool)rdr["rcaParameterOther"]);
                    data.rcaDateDesignationSpecified = true;
                    if (rdr["rcaDateDesignation"] is DBNull)
                        data.rcaDateDesignationSpecified = false;
                    else
                        data.rcaDateDesignation = ((DateTime)rdr["rcaDateDesignation"]);
                    data.rcaANitroSpecified = true;
                    if (rdr["rcaANitro"] is DBNull)
                        data.rcaANitroSpecified = false;
                    else
                        data.rcaANitro = ((bool)rdr["rcaANitro"]);
                    data.rcaAPhosSpecified = true;
                    if (rdr["rcaAPhos"] is DBNull)
                        data.rcaAPhosSpecified = false;
                    else
                        data.rcaAPhos = ((bool)rdr["rcaAPhos"]);
                    data.rcaBSpecified = true;
                    if (rdr["rcaB"] is DBNull)
                        data.rcaBSpecified = false;
                    else
                        data.rcaB = ((bool)rdr["rcaB"]);
                    data.rcaCSpecified = true;
                    if (rdr["rcaC"] is DBNull)
                        data.rcaCSpecified = false;
                    else
                        data.rcaC = ((bool)rdr["rcaC"]);
                    data.rcaCRelevantDirective = rdr["rcaCRelevantDirective"] as string;
                    data.rcaMorphologySpecified = true;
                    if (rdr["rcaMorphology"] is DBNull)
                        data.rcaMorphologySpecified = false;
                    else
                        data.rcaMorphology = ((bool)rdr["rcaMorphology"]);
                    data.rcaHydrologieSpecified = true;
                    if (rdr["rcaHydrologie"] is DBNull)
                        data.rcaHydrologieSpecified = false;
                    else
                        data.rcaHydrologie = ((bool)rdr["rcaHydrologie"]);
                    data.rcaHydraulicSpecified = true;
                    if (rdr["rcaHydraulic"] is DBNull)
                        data.rcaHydraulicSpecified = false;
                    else
                        data.rcaHydraulic = ((bool)rdr["rcaHydraulic"]);
                    data.rcaAbsenceRiskSpecified = true;
                    if (rdr["rcaAbsenceRisk"] is DBNull)
                        data.rcaAbsenceRiskSpecified = false;
                    else
                        data.rcaAbsenceRisk = ((bool)rdr["rcaAbsenceRisk"]);
                    data.rcaDateArt54Specified = true;
                    if (rdr["rcaDateArt54"] is DBNull)
                        data.rcaDateArt54Specified = false;
                    else
                        data.rcaDateArt54 = ((DateTime)rdr["rcaDateArt54"]);

                    data.rcaPlantsSpecified = true;
                    if (rdr["rcaPlants"] is DBNull)
                        data.rcaPlantsSpecified = false;
                    else
                        data.rcaPlants = long.Parse(rdr["rcaPlants"].ToString());
                    data.rcaPlantsCapacitySpecified = true;
                    if (rdr["rcaPlantsCapacity"] is DBNull)
                        data.rcaPlantsCapacitySpecified = false;
                    else
                        data.rcaPlantsCapacity = long.Parse(rdr["rcaPlantsCapacity"].ToString());

                    data.rcaNIncomingMeasuredSpecified = true;
                    if (rdr["rcaNIncomingMeasured"] is DBNull)
                        data.rcaNIncomingMeasuredSpecified = false;
                    else
                        data.rcaNIncomingMeasured = convertDecimal(rdr["rcaNIncomingMeasured"]);
                    data.rcaNIncomingEstimatedSpecified = true;
                    if (rdr["rcaNIncomingEstimated"] is DBNull)
                        data.rcaNIncomingEstimatedSpecified = false;
                    else
                        data.rcaNIncomingEstimated = convertDecimal(rdr["rcaNIncomingEstimated"]);
                    data.rcaNIncomingCalculatedSpecified = true;
                    if (rdr["rcaNIncomingCalculated"] is DBNull)
                        data.rcaNIncomingCalculatedSpecified = false;
                    else
                        data.rcaNIncomingCalculated = convertDecimal(rdr["rcaNIncomingCalculated"]);
                    data.rcaPIncomingMeasuredSpecified = true;
                    if (rdr["rcaPIncomingMeasured"] is DBNull)
                        data.rcaPIncomingMeasuredSpecified = false;
                    else
                        data.rcaPIncomingMeasured = convertDecimal(rdr["rcaPIncomingMeasured"]);
                    data.rcaPIncomingEstimatedSpecified = true;
                    if (rdr["rcaPIncomingEstimated"] is DBNull)
                        data.rcaPIncomingEstimatedSpecified = false;
                    else
                        data.rcaPIncomingEstimated = convertDecimal(rdr["rcaPIncomingEstimated"]);
                    data.rcaPIncomingCalculatedSpecified = true;
                    if (rdr["rcaPIncomingCalculated"] is DBNull)
                        data.rcaPIncomingCalculatedSpecified = false;
                    else
                        data.rcaPIncomingCalculated = convertDecimal(rdr["rcaPIncomingCalculated"]);
                    data.rcaNDischargedMeasuredSpecified = true;
                    if (rdr["rcaNDischargedMeasured"] is DBNull)
                        data.rcaNDischargedMeasuredSpecified = false;
                    else
                        data.rcaNDischargedMeasured = convertDecimal(rdr["rcaNDischargedMeasured"]);
                    data.rcaNDischargedEstimatedSpecified = true;
                    if (rdr["rcaNDischargedEstimated"] is DBNull)
                        data.rcaNDischargedEstimatedSpecified = false;
                    else
                        data.rcaNDischargedEstimated = convertDecimal(rdr["rcaNDischargedEstimated"]);
                    data.rcaNDischargedCalculatedSpecified = true;
                    if (rdr["rcaNDischargedCalculated"] is DBNull)
                        data.rcaNDischargedCalculatedSpecified = false;
                    else
                        data.rcaNDischargedCalculated = convertDecimal(rdr["rcaNDischargedCalculated"]);
                    data.rcaPDischargedMeasuredSpecified = true;
                    if (rdr["rcaPDischargedMeasured"] is DBNull)
                        data.rcaPDischargedMeasuredSpecified = false;
                    else
                        data.rcaPDischargedMeasured = convertDecimal(rdr["rcaPDischargedMeasured"]);
                    data.rcaPDischargedEstimatedSpecified = true;
                    if (rdr["rcaPDischargedEstimated"] is DBNull)
                        data.rcaPDischargedEstimatedSpecified = false;
                    else
                        data.rcaPDischargedEstimated = convertDecimal(rdr["rcaPDischargedEstimated"]);
                    data.rcaPDischargedCalculatedSpecified = true;
                    if (rdr["rcaPDischargedCalculated"] is DBNull)
                        data.rcaPDischargedCalculatedSpecified = false;
                    else
                        data.rcaPDischargedCalculated = convertDecimal(rdr["rcaPDischargedCalculated"]);

                    data.rcaGIS = rdr["rcaGIS"] as string;
                    data.rcaCIDOtherDirective = rdr["rcaCIDOtherDirective"] as string;

                    data.rcaCDateOtherDirectiveSpecified = true;
                    if (rdr["rcaCDateOtherDirective"] is DBNull)
                        data.rcaCDateOtherDirectiveSpecified = false;
                    else
                        data.rcaCDateOtherDirective = ((DateTime)rdr["rcaCDateOtherDirective"]);

                    data.rcaRemarks = rdr["rcaRemarks"] as string;

                    data.rcaArt54AppliedSpecified = true;
                    if (rdr["rcaArt54Applied"] is DBNull)
                        data.rcaArt54AppliedSpecified = false;
                    else
                        data.rcaArt54Applied = ((bool)rdr["rcaArt54Applied"]);

                    if ((!(data.rcaANitro || data.rcaAPhos || data.rcaB || data.rcaC)) && !data.rcaType.Equals("A58"))
                    {
                        logError("RcaID " + data.rcaCode + ": No designation criteria specified");
                        hasErrors = true;
                    }
                    if (data.rcaC)
                    {
                        string errorText = "";
                        if (!data.rcaCDateOtherDirectiveSpecified)
                        {
                            logError("RcaID " + data.rcaCode + ": rcaCDateOtherSpecified is not present - rcaC is Set");
                            hasErrors = true;
                        }
                        if (!checkFieldMandatoryPresence(data.rcaCIDOtherDirective, "rcaCIDOtherDirective", ref errorText, "rcaC is Set"))
                        {
                            logError("RcaID " + data.rcaCode + ": " + errorText);
                            hasErrors = true;
                        }
                        if (!checkFieldMandatoryPresence(data.rcaCRelevantDirective, "rcaCRelevantDirective", ref errorText, "rcaC is Set"))
                        {
                            logError("RcaID " + data.rcaCode + ": " + errorText);
                            hasErrors = true;
                        }
                    }

                    dataList.Add(data);
                    if (hasErrors)
                        reportFail();
                    else
                        reportSuccess();

                }
                catch (Exception ex)
                {
                    logError("Exception: " + ex.ToString());
                    reportFail();
                }
            }
            report.ReceivingAreas = dataList.ToArray();
        }

        public override void importXML(object collection, DataRow repPeriod)
        {
            UWWTD_DataUWWTD_ReportReceivingArea[] receivingAreas = collection as UWWTD_DataUWWTD_ReportReceivingArea[];
            System.Collections.ArrayList lstUniqAggCodes = new System.Collections.ArrayList();
            IDbCommand cmdBigCity = dataFactory.CreateCommand("SELECT bigID FROM T_BigCity WHERE bigCityID = ?;", conn, trans);
            IDataParameter parBigCity = dataFactory.CreateParameter("@bigCityID", DbType.String, 32, "bigCityID", null);
            cmdBigCity.Parameters.Add(parBigCity);
            Hashtable sensitiveAreaRefs = new Hashtable();

            IDbCommand cmd = CreateCommand();

            logInfo("========================================================================");
            logInfo("  Importing Receiving Areas");
            logInfo("========================================================================");
            foreach (UWWTD_DataUWWTD_ReportReceivingArea receivingArea in receivingAreas)
            {
                bool hasErrors = false;
                try
                {
                    hasErrors = false;
                    foreach (string strFeldName in rcaFldNames)
                    {
                        if (strFeldName.Equals("repCode") || strFeldName.Equals("exportDate")) continue;
                        object value = receivingArea.GetType().GetField(strFeldName).GetValue(receivingArea);
                        if (receivingArea.GetType().GetField(strFeldName + "Specified") != null)
                        {
                            if (!((bool)receivingArea.GetType().GetField(strFeldName + "Specified").GetValue(receivingArea))) value = null;
                        }
                        if (!checkQAACRule(value, strFeldName, repPeriod, true, null))
                            hasErrors = true;
                    }

                    if (lstUniqAggCodes.Contains(receivingArea.rcaCode))
                    {
                        //strFCode = "INVKEY"
                        //strFeldName = "rcaCode"
                        //strWert = CStr(colRcaInp(intColCnt))
                        //Err.Raise(10000)
                        continue;
                    }
                    lstUniqAggCodes.Add(receivingArea.rcaCode);

                    int i = 0;
                    setParameterValue(cmd, i++, repPeriod["repID"]);
                    setParameterValue(cmd, i++, dataFactory.getLovRowByKey("STAT", receivingArea.rcaState)["lovID"]);
                    setParameterValue(cmd, i++, receivingArea.rcaCode);
                    setParameterValue(cmd, i++, receivingArea.rcaName);
                    setParameterValue(cmd, i++, dataFactory.getLovRowByKey("RATYP", receivingArea.rcaType)["lovID"]);
                    setParameterValue(cmd, i++, null); // sensitiveArea - will be handled later
                    setParameterValue(cmd, i++, receivingArea.rcaDateArt58Specified ? (object)receivingArea.rcaDateArt58 : null);
                    setParameterValue(cmd, i++, receivingArea.rcaParameterN);
                    setParameterValue(cmd, i++, receivingArea.rcaParameterP);
                    setParameterValue(cmd, i++, receivingArea.rcaParameterOther);
                    setParameterValue(cmd, i++, receivingArea.rcaDateDesignationSpecified ? (object)receivingArea.rcaDateDesignation : null);
                    setParameterValue(cmd, i++, receivingArea.rcaANitro);
                    setParameterValue(cmd, i++, receivingArea.rcaAPhos);
                    setParameterValue(cmd, i++, receivingArea.rcaB);
                    setParameterValue(cmd, i++, receivingArea.rcaC);
                    if (!(receivingArea.rcaANitro || receivingArea.rcaAPhos || receivingArea.rcaB || receivingArea.rcaC))
                    {
                        if ((receivingArea.rcaType != null) && (!receivingArea.rcaType.Equals("A58")))
                        logError("RcaID " + receivingArea.rcaCode + ": No designation criteria specified");
                        hasErrors = true;
                    }
                    if (receivingArea.rcaC)
                    {
                        string errorText = "";
                        if (!receivingArea.rcaCDateOtherDirectiveSpecified)
                        {
                            logError("RcaID " + receivingArea.rcaCode + ": rcaCDateOtherSpecified is not present - rcaC is Set");
                            hasErrors = true;
                        }
                        if (!checkFieldMandatoryPresence(receivingArea.rcaCIDOtherDirective, "rcaCIDOtherDirective", ref errorText, "rcaC is Set"))
                        {
                            logError("RcaID " + receivingArea.rcaCode + ": " + errorText);
                            hasErrors = true;
                        }
                        if (!checkFieldMandatoryPresence(receivingArea.rcaCRelevantDirective, "rcaCRelevantDirective", ref errorText, "rcaC is Set"))
                        {
                            logError("RcaID " + receivingArea.rcaCode + ": " + errorText);
                            hasErrors = true;
                        }
                    }
                    setParameterValue(cmd, i++, receivingArea.rcaCRelevantDirective);
                    setParameterValue(cmd, i++, receivingArea.rcaMorphology);
                    setParameterValue(cmd, i++, receivingArea.rcaHydrologie);
                    setParameterValue(cmd, i++, receivingArea.rcaHydraulic);
                    setParameterValue(cmd, i++, receivingArea.rcaAbsenceRisk);
                    setParameterValue(cmd, i++, receivingArea.rcaDateArt54Specified ? (object)receivingArea.rcaDateArt54 : null);
                    setParameterValue(cmd, i++, receivingArea.rcaPlants);
                    if (!receivingArea.rcaPlantsCapacitySpecified)
                        setParameterValue(cmd, i++, null);
                    else
                        setParameterValue(cmd, i++, receivingArea.rcaPlantsCapacity);
                    if (!receivingArea.rcaNIncomingMeasuredSpecified)
                        setParameterValue(cmd, i++, null);
                    else
                        setParameterValue(cmd, i++, (double)receivingArea.rcaNIncomingMeasured);
                    if (!receivingArea.rcaNIncomingCalculatedSpecified)
                        setParameterValue(cmd, i++, null);
                    else
                        setParameterValue(cmd, i++, (double)receivingArea.rcaNIncomingCalculated);
                    if (!receivingArea.rcaNIncomingEstimatedSpecified)
                        setParameterValue(cmd, i++, null);
                    else
                        setParameterValue(cmd, i++, (double)receivingArea.rcaNIncomingEstimated);
                    if (!receivingArea.rcaPlantsCapacitySpecified)
                        setParameterValue(cmd, i++, null);
                    else
                        if (!receivingArea.rcaPIncomingMeasuredSpecified)
                        setParameterValue(cmd, i++, null);
                    else
                        setParameterValue(cmd, i++, (double)receivingArea.rcaPIncomingMeasured);
                    if (!receivingArea.rcaPlantsCapacitySpecified)
                        setParameterValue(cmd, i++, null);
                    else
                        setParameterValue(cmd, i++, (double)receivingArea.rcaPIncomingCalculated);
                    if (!receivingArea.rcaPIncomingEstimatedSpecified)
                        setParameterValue(cmd, i++, null);
                    else
                        setParameterValue(cmd, i++, (double)receivingArea.rcaPIncomingEstimated);
                    if (!receivingArea.rcaNDischargedMeasuredSpecified)
                        setParameterValue(cmd, i++, null);
                    else
                        setParameterValue(cmd, i++, (double)receivingArea.rcaNDischargedMeasured);
                    if (!receivingArea.rcaNDischargedCalculatedSpecified)
                        setParameterValue(cmd, i++, null);
                    else
                        setParameterValue(cmd, i++, (double)receivingArea.rcaNDischargedCalculated);
                    if (!receivingArea.rcaNDischargedEstimatedSpecified)
                        setParameterValue(cmd, i++, null);
                    else
                        setParameterValue(cmd, i++, (double)receivingArea.rcaNDischargedEstimated);
                    if (!receivingArea.rcaPDischargedMeasuredSpecified)
                        setParameterValue(cmd, i++, null);
                    else
                        setParameterValue(cmd, i++, (double)receivingArea.rcaPDischargedMeasured);
                    if (!receivingArea.rcaPDischargedCalculatedSpecified)
                        setParameterValue(cmd, i++, null);
                    else
                        setParameterValue(cmd, i++, (double)receivingArea.rcaPDischargedCalculated);
                    if (!receivingArea.rcaPDischargedEstimatedSpecified)
                        setParameterValue(cmd, i++, null);
                    else
                        setParameterValue(cmd, i++, (double)receivingArea.rcaPDischargedEstimated);
                    setParameterValue(cmd, i++, receivingArea.rcaGIS);
                    setParameterValue(cmd, i++, receivingArea.rcaCIDOtherDirective);
                    setParameterValue(cmd, i++, receivingArea.rcaCDateOtherDirectiveSpecified ? (object)receivingArea.rcaCDateOtherDirective : null);
                    setParameterValue(cmd, i++, receivingArea.rcaRemarks);
                    setParameterValue(cmd, i++, receivingArea.rcaArt54AppliedSpecified ? (object)receivingArea.rcaArt54Applied : null);

                    if ((receivingArea.rcaSensitiveArea != null) &&
                        (receivingArea.rcaSensitiveArea.Length > 0))
                    {
                        RcaRef rcaRef = new RcaRef();
                        rcaRef.repID = repPeriod["repID"];
                        rcaRef.rcaCode = receivingArea.rcaSensitiveArea;
                        sensitiveAreaRefs[rcaRef] = receivingArea.rcaCode;
                    }

                    cmd.ExecuteNonQuery();
                    if (hasErrors)
                        reportFail();
                    else
                        reportSuccess();
                }
                catch (Exception ex)
                {
                    logError("Exception: " + ex.Message + ";");
                    reportFail();
                }
            }

            // execute post-run to update Sensitive Areas List
            foreach (object key in sensitiveAreaRefs.Keys)
            {
                RcaRef rcaRef = (RcaRef)key;
                // obtain the rcaID for the sensitive Area
                IDbCommand cmdGetRcaID = CreateGetRcaIDCmd();
                setParameterValue(cmdGetRcaID, 0, rcaRef.rcaCode);
                setParameterValue(cmdGetRcaID, 1, rcaRef.repID);
                object rcaID = cmdGetRcaID.ExecuteScalar();
                if ((rcaID == null) || (rcaID is DBNull))
                {
                    // this is an error - sensitive Area was not found .....
                    continue;
                }
                IDbCommand cmdUpdtRca = CreateUpdateSesitiveAreaCmd();
                setParameterValue(cmdUpdtRca, 0, rcaID);
                setParameterValue(cmdUpdtRca, 1, sensitiveAreaRefs[key]);
                setParameterValue(cmdUpdtRca, 2, rcaRef.repID);
                cmdUpdtRca.ExecuteNonQuery();
            }
        }

        public override bool checkQAACRule(object value, string fieldName, DataRow repPeriod, bool isImport, string lineInfo)
        {
            bool retVal = true;
            string errorText = "";

            switch (fieldName)
            {
                case "rcaState":
                    if (!checkFieldMandatoryPresence(value, fieldName, ref errorText))
                    {
                        retVal = false;
                        break;
                    }
                    if (!checkLOVValue(value, fieldName, "STAT", ref errorText, isImport, null))
                        retVal = false;
                    break;
                case "rcaCode":
                    if (!checkFieldMandatoryPresence(value, fieldName, ref errorText))
                    {
                        retVal = false;
                        break;
                    }
                    string rptMemberState = repPeriod.GetParentRow("T_ReporterT_ReportPeriod").GetParentRow("T_Lov_T_Reporter")["lovKey"] as string;
                    if (!((value as string).StartsWith(rptMemberState)))
                    {
                        retVal = false;
                        errorText = fieldName + " (" + value + ") does not start with " + rptMemberState;
                        break;
                    }
                    break;
                case "rcaType":
                    if (!checkOptionalPresence(value, fieldName, ref errorText))
                    {
                        break;
                    }
                    if (!checkLOVValue(value, fieldName, "RATYP", ref errorText, isImport, null))
                        retVal = false;
                    break;
                case "rcaName":
                    if (!checkFieldMandatoryPresence(value, fieldName, ref errorText))
                    {
                        retVal = false;
                        break;
                    }
                    break;
                default:
                    break;
            }

            if (!retVal)
            {
                if (lineInfo != null)
                {
                    errorText = lineInfo + " - " + errorText;
                }

                logError(errorText);
            }

            return retVal;
        }
    }

    public struct RcaRef
    {
        public object repID;
        public string rcaCode;
    }
}
