﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Data;
using System.IO;
using at.uba.uwwtd2008.xml;

namespace at.uba.uwwtd2008.DataAccess
{
    class InterfaceUwwAgglo : InterfaceBase
    {
        private string[] aucFldNames = {"repCode", "aucUwwCode", "aucUwwName", "aucAggCode", 
	        "aucAggName", "aucPercEnteringUWWTP", "aucMethodPercEnteringUWWTP", "aucPercC2T", "exportDate"};

        public InterfaceUwwAgglo(IDbConnection conn, IDbTransaction trans, DataFormatConverter conv)
            : base(conn, trans, conv)
        {
        }

        private IDbCommand CreateSelectCommand()
        {
            string strSQL = "SELECT repID, aucUwwCode, aucUwwName, aucAggCode, aucAggName, " +
             "aucPercEnteringUWWTP, aucMethodPercEnteringUWWTP, aucPercC2T FROM T_Uwwtp_Agglo WHERE repID = ?;";

            IDbCommand comSelCmd = dataFactory.CreateCommand(strSQL, conn, trans);

            comSelCmd.Parameters.Add(dataFactory.CreateParameter("@repID", DbType.Int64, 0, "repID", null));

            return comSelCmd;
        }

        private IDbCommand CreateCommand()
        {
            string strSQL = "INSERT INTO T_Uwwtp_Agglo (repID, aucUwwCode, aucUwwName, aucAggCode, aucAggName, " +
             "aucPercEnteringUWWTP, aucMethodPercEnteringUWWTP, aucPercC2T, uwwID, aggID) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?); ";

            IDbCommand comInsCmd = dataFactory.CreateCommand(strSQL, conn, trans);

            comInsCmd.Parameters.Add(dataFactory.CreateParameter("@repID", DbType.Int64, 0, "repID", null));
            comInsCmd.Parameters.Add(dataFactory.CreateParameter("@aucUwwCode", DbType.String, 32, "aucUwwCode", null));
            comInsCmd.Parameters.Add(dataFactory.CreateParameter("@aucUwwName", DbType.String, 255, "aucUwwName", null));
            comInsCmd.Parameters.Add(dataFactory.CreateParameter("@aucAggCode", DbType.String, 32, "aucAggCode", null));
            comInsCmd.Parameters.Add(dataFactory.CreateParameter("@aucAggName", DbType.String, 255, "aucAggName", null));
            comInsCmd.Parameters.Add(dataFactory.CreateParameter("@aucPercEnteringUWWTP", DbType.Double, 0, "aucPercEnteringUWWTP", null));
            comInsCmd.Parameters.Add(dataFactory.CreateParameter("@aucMethodPercEnteringUWWTP", DbType.Int64, 0, "aucMethodPercEnteringUWWTP", null));
            comInsCmd.Parameters.Add(dataFactory.CreateParameter("@aucPercC2T", DbType.Double, 0, "aucPercC2T", null));
            comInsCmd.Parameters.Add(dataFactory.CreateParameter("@uwwID", DbType.Int64, 0, "uwwID", null));
            comInsCmd.Parameters.Add(dataFactory.CreateParameter("@aggID", DbType.Int64, 0, "aggID", null));

            return comInsCmd;
        }


        public override void importData(StreamReader rdr, DataRow repPeriod)
        {
            System.Collections.ArrayList lstUniqAggCodes = new System.Collections.ArrayList();
            IDbCommand cmdUwwtp = dataFactory.CreateCommand("SELECT uwwID FROM T_UWWTPS WHERE uwwCode = ?;", conn, trans);
            IDataParameter parUwwtp = dataFactory.CreateParameter("@uwwCode", DbType.String, 32, "uwwCode", null);
            cmdUwwtp.Parameters.Add(parUwwtp);
            IDbCommand cmdAgglo = dataFactory.CreateCommand("SELECT aggID FROM T_Agglomerations WHERE aggCode = ?;", conn, trans);
            IDataParameter parAgglo = dataFactory.CreateParameter("@aggCode", DbType.String, 32, "aggCode", null);
            cmdAgglo.Parameters.Add(parAgglo);
            IDbCommand cmdUwwtpName = dataFactory.CreateCommand("SELECT uwwName FROM T_UWWTPS WHERE uwwCode = ?;", conn, trans);
            IDataParameter parUwwtpName = dataFactory.CreateParameter("@uwwCode", DbType.String, 32, "uwwCode", null);
            cmdUwwtpName.Parameters.Add(parUwwtpName);
            IDbCommand cmdAggloName = dataFactory.CreateCommand("SELECT aggName FROM T_Agglomerations WHERE aggCode = ?;", conn, trans);
            IDataParameter parAggloName = dataFactory.CreateParameter("@aggCode", DbType.String, 32, "aggCode", null);
            cmdAggloName.Parameters.Add(parAggloName);

            logInfo("========================================================================");
            logInfo("  Importing UwwAgglos");
            logInfo("========================================================================");
            bool hadError = false;
            long lineNo = 0;
            string line;
            while ((line = rdr.ReadLine()) != null)
            {
                hadError = false;
                try
                {
                    lineNo++;
                    line = line.TrimEnd();
                    if (line.Length < 1) continue; // ignore empty lines
                    string[] parts = line.Split(new string[] { "\t" }, StringSplitOptions.None);

                    IDbCommand cmd = CreateCommand();
                    for (int idx = 0; idx < parts.Length; idx++)
                    {
                        if (idx >= aucFldNames.Length)
                        {
                            // do error reporting ("too many items")
                            continue;
                        }
                        string strFeldName = aucFldNames[idx];
                        if (strFeldName.Equals("exportDate")) // this is nothing to import
                            continue;
                        if (parts[idx].Length > 0)
                        {
                            object objTemp = parts[idx];
                            if (!checkQAACRule(objTemp, strFeldName, repPeriod, true, null))
                                hadError = true;
                            switch (strFeldName)
                            {
                                case "repCode":
                                    objTemp = dataFactory.getRepIDforReportCode(parts[idx]);
                                    // if (objTemp == null)  do error reporting
                                    setParameterValue(cmd, idx, objTemp);
                                    break;
                                case "aucUwwCode":
                                    parUwwtp.Value = parts[idx];
                                    objTemp = cmdUwwtp.ExecuteScalar();
                                    // if (objTemp == null) do error reporting
                                    setParameterValue(cmd, idx, parts[idx]);
                                    setParameterValue(cmd, 8, objTemp);
                                    break;
                                case "aucAggCode":
                                    parAgglo.Value = parts[idx];
                                    objTemp = cmdAgglo.ExecuteScalar();
                                    // if (objTemp == null) do error reporting
                                    setParameterValue(cmd, idx, parts[idx]);
                                    setParameterValue(cmd, 9, objTemp);
                                    break;
                                case "aucUwwName":
                                    // parameter was initilized befoore
                                    //parUwwtpName.Value = parUwwtp.Value;
                                    //objTemp = cmdUwwtpName.ExecuteScalar();
                                    // if (objTemp == null) do error reporting
                                    setParameterValue(cmd, idx, parts[idx]);
                                    break;
                                case "aucAggName":
                                    // parameter was initilized befoore
                                    //parAggloName.Value = parAggloName.Value;
                                    //objTemp = cmdAggloName.ExecuteScalar();
                                    // if (objTemp == null) do error reporting
                                    setParameterValue(cmd, idx, parts[idx]);
                                    break;
                                case "aucMethodPercEnteringUWWTP":
                                    objTemp = null;
                                    if (!parts[idx].Equals(""))
                                    {
                                        objTemp = dataFactory.getLovRowByKey("COLLDET", parts[idx])["lovID"];
                                    }
                                    setParameterValue(cmd, idx, objTemp);
                                    break;
                                default:
                                    setParameterValue(cmd, idx, parts[idx]);
                                    break;
                            }
                        }
                        else
                        {
                            setParameterValue(cmd, idx, null);
                        }
                    }
                    cmd.ExecuteNonQuery();
                    if (hadError)
                        reportFail();
                    else
                        reportSuccess();
                }
                catch (Exception ex)
                {
                    logError("Exception: " + ex.Message);
                    reportFail();
                }

            }
        }

        public override void exportData(StreamWriter wr, DataRow repPeriod)
        {
            IDbCommand cmd = CreateSelectCommand();

            logInfo("========================================================================");
            logInfo("  Exporting UwwtpAgglos");
            logInfo("========================================================================");

            string[] parts = new string[aucFldNames.Length];

            setParameterValue(cmd, "@repID", repPeriod["repID"]);
            IDataReader rdr = cmd.ExecuteReader();
            try
            {
                while (rdr.Read())
                {
                    try
                    {
                        bool hasErrors = false;
                        for (int i = 0; i < aucFldNames.Length; i++)
                        {
                            string strFieldName = aucFldNames[i];
                            if (strFieldName.Equals("exportDate")) continue;
                            if ((!strFieldName.Equals("repCode")) && (!strFieldName.Equals("rptMStateKey")))
                                if (!checkQAACRule(rdr[strFieldName], strFieldName, repPeriod, false, "rep: " + repPeriod["repCode"]))
                                    hasErrors = true;

                            switch (strFieldName)
                            {
                                case "repCode":
                                    parts[i] = getReaderValueString(repPeriod["repCode"]);
                                    break;
                                case "aucMethodPercEnteringUWWTP":
                                    if (rdr[i] is DBNull)
                                        parts[i] = "";
                                    else
                                        parts[i] = (string) dataFactory.getLovRowByID(rdr[i])["lovKey"];
                                    break;
                                default:
                                    parts[i] = getReaderValueString(rdr[i]);
                                    break;
                            }
                        }
                        parts[parts.Length - 1] = conv.formatDateLong(DateTime.Now);
                        wr.WriteLine(string.Join("\t", parts));
                        if (hasErrors)
                            reportFail();
                        else
                            reportSuccess();
                    }
                    catch (Exception ex)
                    {
                        logError("Export UWWAGGLO " + rdr["aucUwwCode"] + "," + rdr["aucAggCode"] + " caused Exception: " + ex.ToString());
                    }
                }
            }
            finally
            {
                rdr.Close();
            }
        }

        public override void exportXML(UWWTD_DataUWWTD_Report report, DataRow repPeriod)
        {
            List<UWWTD_DataUWWTD_ReportUwwtpAgglo> dataList = new List<UWWTD_DataUWWTD_ReportUwwtpAgglo>();

            logInfo("========================================================================");
            logInfo("  Exporting UwwtpAgglos for ReportPeriod " + repPeriod["repCode"]);
            logInfo("========================================================================");

            IDbCommand cmd = CreateSelectCommand();
            setParameterValue(cmd, "@repID", repPeriod["repID"]);
            IDataReader rdr = cmd.ExecuteReader();
            while (rdr.Read())
            {
                bool hasErrors = false;
                try
                {
                    for (int j = 0; j < aucFldNames.Length; j++)
                    {
                        if (aucFldNames[j].Equals("exportDate")) continue;
                        if (!checkQAACRule(rdr[j], aucFldNames[j], repPeriod, false, null))
                            hasErrors = true;
                    }
                    UWWTD_DataUWWTD_ReportUwwtpAgglo data = new UWWTD_DataUWWTD_ReportUwwtpAgglo();
                    data.aucAggCode = rdr["aucAggCode"] as string;
                    data.aucAggName = rdr["aucAggName"] as string;
                    data.aucUwwCode = rdr["aucUwwCode"] as string;
                    data.aucUwwName = rdr["aucUwwName"] as string;

                    if (rdr["aucMethodPercEnteringUWWTP"] is DBNull)
                        data.aucMethodPercEnteringUWWTP = null;
                    else
                        data.aucMethodPercEnteringUWWTP = dataFactory.getLovRowByID(rdr["aucMethodPercEnteringUWWTP"])["lovKey"] as string;

                    data.aucPercEnteringUWWTPSpecified = true;
                    if (rdr["aucPercEnteringUWWTP"] is DBNull)
                        data.aucPercEnteringUWWTPSpecified = false;
                    else
                        data.aucPercEnteringUWWTP = convertDecimal(rdr["aucPercEnteringUWWTP"]);

                    data.aucPercC2TSpecified = true;
                    if (rdr["aucPercC2T"] is DBNull)
                        data.aucPercC2TSpecified = false;
                    else
                        data.aucPercC2T = convertDecimal(rdr["aucPercC2T"]);
                    dataList.Add(data);
                    if (hasErrors)
                        reportFail();
                    else
                        reportSuccess();

                }
                catch (Exception ex)
                {
                    logError("Exception: " + ex.ToString());
                    reportFail();
                }
            }
            report.UwwtpAgglos = dataList.ToArray();
        }

        public override void importXML(object collection, DataRow repPeriod)
        {
            UWWTD_DataUWWTD_ReportUwwtpAgglo[] uwwagglos = (UWWTD_DataUWWTD_ReportUwwtpAgglo[])collection;
            IDbCommand cmdUwwtp = dataFactory.CreateCommand("SELECT uwwID FROM T_UWWTPS WHERE uwwCode = ?;", conn, trans);
            IDataParameter parUwwtp = dataFactory.CreateParameter("@uwwCode", DbType.String, 32, "uwwCode", null);
            cmdUwwtp.Parameters.Add(parUwwtp);
            IDbCommand cmdAgglo = dataFactory.CreateCommand("SELECT aggID FROM T_Agglomerations WHERE aggCode = ?;", conn, trans);
            IDataParameter parAgglo = dataFactory.CreateParameter("@aggCode", DbType.String, 32, "aggCode", null);
            cmdAgglo.Parameters.Add(parAgglo);
            IDbCommand cmdUwwtpName = dataFactory.CreateCommand("SELECT uwwName FROM T_UWWTPS WHERE uwwCode = ?;", conn, trans);
            IDataParameter parUwwtpName = dataFactory.CreateParameter("@uwwCode", DbType.String, 32, "uwwCode", null);
            cmdUwwtpName.Parameters.Add(parUwwtpName);
            IDbCommand cmdAggloName = dataFactory.CreateCommand("SELECT aggName FROM T_Agglomerations WHERE aggCode = ?;", conn, trans);
            IDataParameter parAggloName = dataFactory.CreateParameter("@aggCode", DbType.String, 32, "aggCode", null);
            cmdAggloName.Parameters.Add(parAggloName);

            logInfo("========================================================================");
            logInfo("  Importing UwwtpAgglos");
            logInfo("========================================================================");

            bool hasErrors = false;
            IDbCommand cmd = CreateCommand();
            foreach (UWWTD_DataUWWTD_ReportUwwtpAgglo uwwagglo in uwwagglos)
            {
                try
                {
                    hasErrors = false;
                    foreach (string strFeldName in aucFldNames)
                    {
                        if (strFeldName.Equals("repCode") || strFeldName.Equals("exportDate")) continue;
                        object value = uwwagglo.GetType().GetField(strFeldName).GetValue(uwwagglo);
                        if (uwwagglo.GetType().GetField(strFeldName + "Specified") != null)
                        {
                            if (!((bool)uwwagglo.GetType().GetField(strFeldName + "Specified").GetValue(uwwagglo))) value = null;
                        }
                        if (!checkQAACRule(value, strFeldName, repPeriod, true, null))
                            hasErrors = true;
                    }
                    setParameterValue(cmd, "@repID", repPeriod["repID"]);
                    setParameterValue(cmd, "@aucUwwCode", uwwagglo.aucUwwCode);
                    setParameterValue(cmd, "@aucUwwName", uwwagglo.aucUwwName);
                    setParameterValue(cmd, "@aucAggCode", uwwagglo.aucAggCode);
                    setParameterValue(cmd, "@aucAggName", uwwagglo.aucAggName);
                    if (uwwagglo.aucPercEnteringUWWTPSpecified)
                        setParameterValue(cmd, "@aucPercEnteringUWWTP", uwwagglo.aucPercEnteringUWWTP);
                    else
                        setParameterValue(cmd, "@aucPercEnteringUWWTP", null);
                    if (uwwagglo.aucMethodPercEnteringUWWTP != null)
                        setParameterValue(cmd, "@aucMethodPercEnteringUWWTP", dataFactory.getLovRowByKey("COLLDET", uwwagglo.aucMethodPercEnteringUWWTP)["lovID"]);
                    else
                        setParameterValue(cmd, "@aucMethodPercEnteringUWWTP", DBNull.Value);
                    if (uwwagglo.aucPercC2TSpecified)
                        setParameterValue(cmd, "@aucPercC2T", uwwagglo.aucPercC2T);
                    else
                        setParameterValue(cmd, "@aucPercC2T", null);
                    parUwwtp.Value = uwwagglo.aucUwwCode;
                    if (uwwagglo.aucUwwCode != null)
                        setParameterValue(cmd, "@uwwID", cmdUwwtp.ExecuteScalar());
                    else
                        setParameterValue(cmd, "@uwwID", DBNull.Value);
                    parAgglo.Value = uwwagglo.aucAggCode;
                    if (uwwagglo.aucUwwCode != null)
                        setParameterValue(cmd, "@aggID", cmdAgglo.ExecuteScalar());
                    else
                        setParameterValue(cmd, "@aggID", DBNull.Value);

                    cmd.ExecuteNonQuery();
                    if (hasErrors)
                        reportFail();
                    else 
                        reportSuccess();
                }
                catch (Exception ex)
                {
                    reportFail();
                    logError("Error importing uwwAgglo for uwwID " + uwwagglo.aucUwwCode + " and Agglo " + uwwagglo.aucAggCode + " - " + ex.Message);
                }
            }
        }

        public override bool checkQAACRule(object value, string fieldName, DataRow repPeriod, bool isImport, string lineInfo)
        {
            bool retVal = true;
            string errorText = "";
            string rptMemberState = "";

            switch (fieldName)
            {
                case "aucUwwCode":
                    if (!checkFieldMandatoryPresence(value, fieldName, ref errorText))
                    {
                        retVal = false;
                        break;
                    }
                    rptMemberState = repPeriod.GetParentRow("T_ReporterT_ReportPeriod").GetParentRow("T_Lov_T_Reporter")["lovKey"] as string;
                    if (!((value as string).StartsWith(rptMemberState)))
                    {
                        retVal = false;
                        errorText = fieldName + " (" + value + ") does not start with " + rptMemberState;
                        break;
                    }
                    break;
                //case "aucUwwName":
                //    if (!checkFieldMandatoryPresence(value, fieldName, ref errorText))
                //    {
                //        retVal = false;
                //        break;
                //    }
                //    break;
                case "aucAggCode":
                    if (!checkFieldMandatoryPresence(value, fieldName, ref errorText))
                    {
                        retVal = false;
                        break;
                    }
                    rptMemberState = repPeriod.GetParentRow("T_ReporterT_ReportPeriod").GetParentRow("T_Lov_T_Reporter")["lovKey"] as string;
                    if (!((value as string).StartsWith(rptMemberState)))
                    {
                        retVal = false;
                        errorText = fieldName + " (" + value + ") does not start with " + rptMemberState;
                        break;
                    }
                    break;
                //case "aucAggName":
                //    if (!checkFieldMandatoryPresence(value, fieldName, ref errorText))
                //    {
                //        retVal = false;
                //        break;
                //    }
                //    break;
                case "aucMethodPercEnteringUWWTP":
                    if (!checkOptionalPresence(value, fieldName, ref errorText))
                    {
                        break;
                    }
                    if (!checkLOVValue(value, fieldName, "COLLDET", ref errorText, isImport, null))
                    {
                        retVal = false;
                    }
                    break;
                default:
                    break;
            }

            if (!retVal)
            {
                if (lineInfo != null)
                {
                    errorText = lineInfo + " - " + errorText;
                }

                logError(errorText);
            }

            return retVal;
        }
    }
}
