﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Xml.Serialization;
using System.IO;
using System.Windows.Forms;
using System.Collections;
using at.uba.uwwtd2008.DataAccess;
using at.uba.uwwtd2008.Properties;
using at.uba.uwwtd2008.Items;
using at.uba.uwwtd2008.Dialogs;

namespace at.uba.uwwtd2008.Dialogs
{
    public partial class CSVExportDialog : Form
    {
        DataAccessFactory dataFactory = DataAccessFactory.Instance;
        string currentOutput = null;
        Settings settings = Settings.Default;
        LoggingClass currentLogger;

        string[] files = { "Reporter.txt", "ReportPeriod.txt", "Contact.txt", "ReceivingAreas.txt", "Agglomerations.txt",
                             "Uwwtps.txt", "UwwtpAgglo.txt", "DischargePoints.txt", "MSLevel.txt", "Industries.txt"};


        public CSVExportDialog()
        {
            InitializeComponent();
            if ((settings.Last_Export_Dir != null) && (settings.Last_Export_Dir.Length > 0))
                currentOutput = settings.Last_Export_Dir;
            else
                currentOutput = ".";
            textBox1.Text = currentOutput;
        }

        private void OnBrowseClick(object sender, EventArgs e)
        {
            dlgExportFolder.SelectedPath = currentOutput;
            if (dlgExportFolder.ShowDialog(this) == DialogResult.OK)
            {
                textBox1.Text = dlgExportFolder.SelectedPath;
                settings.Last_Export_Dir = textBox1.Text;
                settings.Save();
            }
        }

        private void OnExportFileChanged(object sender, EventArgs e)
        {
            currentOutput = ((TextBox)sender).Text;
        }

        private void OnClose(object sender, EventArgs e)
        {
            this.Close();

        }

        private void OnFormLoad(object sender, EventArgs e)
        {
            IDbConnection conn = dataFactory.GetConnection();
            try
            {
                // load all the reporter relevant data
                dataFactory.FillDataAdapter(
                    dataFactory.CreateDataAdapter(
                        dataFactory.CreateCommand("SELECT * FROM T_LOV", conn)),
                    dataSet, "T_LOV");
                dataFactory.FillDataAdapter(
                    dataFactory.CreateDataAdapter(
                        dataFactory.CreateCommand("SELECT * FROM T_REPORTPERIOD", conn)),
                    dataSet, "T_REPORTPERIOD");
                dataFactory.FillDataAdapter(
                    dataFactory.CreateDataAdapter(
                        dataFactory.CreateCommand("SELECT * FROM T_REPORTER", conn)),
                    dataSet, "T_REPORTER");
                dataFactory.FillDataAdapter(
                    dataFactory.CreateDataAdapter(
                        dataFactory.CreateCommand("SELECT * FROM T_CONTACT", conn)),
                    dataSet, "T_CONTACT");
                dataFactory.FillDataAdapter(
                    dataFactory.CreateDataAdapter(
                        dataFactory.CreateCommand("SELECT * FROM T_NUTS", conn)),
                    dataSet, "T_NUTS");
                dataSet.Relations.Add("T_REPORTER_T_REPORTPERIOD",
                    dataSet.Tables["T_REPORTER"].Columns["rptID"],
                    dataSet.Tables["T_REPORTPERIOD"].Columns["rptID"]);
                dataSet.Relations.Add("T_LOV_T_REPORTER",
                    dataSet.Tables["T_LOV"].Columns["lovID"],
                    dataSet.Tables["T_REPORTER"].Columns["rptMState"]);
                dataSet.Relations.Add("T_CONTACT_T_REPORTPERIOD",
                    dataSet.Tables["T_REPORTPERIOD"].Columns["repID"],
                    dataSet.Tables["T_CONTACT"].Columns["repID"]);
            }
            finally
            {
                conn.Close();
            }

            DataTable tab = new DataTable();
            tab.Columns.Add("repID", dataSet.Tables["T_REPORTPERIOD"].Columns["repID"].DataType);
            tab.Columns.Add("display", typeof(string));

            foreach (DataRow reportperiod in dataSet.Tables["T_REPORTPERIOD"].Rows)
            {
                CheckBoxDisplayable t = new CheckBoxDisplayable();
                t.key=reportperiod["repID"];
                DataRow reporter = reportperiod.GetParentRow("T_REPORTER_T_REPORTPERIOD");
                t.display = reporter.GetParentRow("T_LOV_T_REPORTER")["lovWert"] + " - " + reportperiod["repReportedPeriod"];
                lstExportReport.Items.Add(t);
            }

        }

        private void OnExportStart(object sender, EventArgs e)
        {
            if (lstExportReport.SelectedItems.Count != 1)
            {
                MessageBox.Show(this, "Please select a Report-Period to export", "Export CSV", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                return;
            }

            if (!Directory.Exists(currentOutput))
            {
                if (MessageBox.Show(this, "The specified directory does not exist. Do you want to create it?", "Export CSV", MessageBoxButtons.OKCancel, MessageBoxIcon.Question) != DialogResult.OK)
                {
                    return;
                }
                Directory.CreateDirectory(currentOutput);
            }
            else
            {
                if (Directory.GetFiles(currentOutput).Length > 0)
                {
                    if (MessageBox.Show(this, "The specified directory is not empty. All Data will be deleted. Continue?", "Export CSV", MessageBoxButtons.OKCancel, MessageBoxIcon.Question) != DialogResult.OK)
                    {
                        return;
                    }
                    foreach (string file in files)
                    {
                        File.Delete(currentOutput + "\\" + file);
                    }
                } // if
            }
            object[] reps = new object[lstExportReport.SelectedItems.Count];
            for (int i = 0; i < reps.Length; i++)
            {
                reps[i] = (lstExportReport.SelectedItems[i] as CheckBoxDisplayable).key;
            }
            currentLogger = new LoggingClass();
            cmdClose.Enabled = false;
            cmdExport.Enabled = false;
            cmbBrowse.Enabled = false;
            backgroundWorker.RunWorkerAsync(reps);
        }

        private void OnProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            progressExport.Value = e.ProgressPercentage;
        }

        private void OnExportComplete(object sender, RunWorkerCompletedEventArgs e)
        {
            progressExport.Value = progressExport.Maximum;
            cmdClose.Enabled = true;
            cmdExport.Enabled = true;
            cmbBrowse.Enabled = true;
            ImportExportSummary dlg = new ImportExportSummary();
            dlg.Logger = currentLogger;
            dlg.Show();
        }

        private void DoExporting(object sender, DoWorkEventArgs e)
        {

            currentLogger.Init();
            try
            {
                object[] sel = e.Argument as object[];
                if (sel == null)
                {
                    e.Result = null;
                    return;
                }

                bool enforceConstraints = false;
                String currentFile = "";
                IDbConnection conn = dataFactory.GetConnection();
                try
                {
                    IDbTransaction trans = null;//conn.BeginTransaction();
                    try
                    {
                        DataSet ds = dataFactory.getReporterDataSet();
                        enforceConstraints = ds.EnforceConstraints;
                        ds.EnforceConstraints = false;

                        DataRow reporterRow = null;
                        DataRow reportperiodRow = null;
                        //DataRow contactRow = null;
                        string[] parts = null;
                        backgroundWorker.ReportProgress(0);
                        //if (Directory.Exists(currentOutput))
                        //{
                        //    Directory.Delete(currentOutput, true);
                        //}
                        //Directory.CreateDirectory(currentOutput);
                        bool hasError = false;
                        reportperiodRow = dataFactory.getRepPeridodforRepID(sel[0]);
                        reporterRow = reportperiodRow.GetParentRow("T_ReporterT_ReportPeriod");
                        String rptMemberState = "";

                        DataFormatConverter conv = null;
                        currentLogger.log(LoggingLevel.INFO, "CSVExport", "========================================================================");
                        currentLogger.log(LoggingLevel.INFO, "CSVExport", "      Exporting Reporter ");
                        currentLogger.log(LoggingLevel.INFO, "CSVExport", "========================================================================");
                        // Reporter ...
                        currentFile = currentOutput + "\\" + files[0];
                        StreamWriter wr = new StreamWriter(currentFile);
                        try
                        {
                            hasError = false;
                            parts = new string[5];

                            if ((reporterRow["rptCulture"] != null) && !(reporterRow["rptCulture"] is DBNull))
                                parts[2] = reporterRow["rptCulture"] as string;
                            else
                                parts[2] = System.Globalization.CultureInfo.CurrentCulture.Name;

                            conv = new DataFormatConverter(parts[2]);

                            DataRow repLov = dataFactory.getLovRowByID(reporterRow["rptMState"]);

                            if (reporterRow["rptMState"] is DBNull)
                            {
                                hasError = true;
                                currentLogger.log(LoggingLevel.ERROR, "Reporter", "rptMState is Master Data");
                            }

                            parts[0] = repLov["lovKey"] as string;
                            if (parts[0] == null)
                            {
                                hasError = true;
                                currentLogger.log(LoggingLevel.ERROR, "Reporter", "The Memberstate can not be found in LOV-Tables");
                            }
                            else 
                                rptMemberState = repLov["lovKey"] as string;
                            parts[1] = repLov["lovWert"] as string;
                            if (!(reportperiodRow["repRATypes"] is DBNull))
                                parts[3] = DataAccessFactory.newRATypToOld((int)reportperiodRow["repRATypes"]).ToString();
                            else
                                parts[3] = "";
                            parts[4] = conv.formatDateLong(DateTime.Now);

                            wr.WriteLine(string.Join("\t", parts));
                        }
                        finally
                        {
                            wr.Close();
                        }

                        backgroundWorker.ReportProgress(10);
                        if (hasError)
                            currentLogger.reportFail("Reporter");
                        else
                            currentLogger.reportSuccess("Reporter");

                        currentLogger.log(LoggingLevel.INFO, "CSVExport", "========================================================================");
                        currentLogger.log(LoggingLevel.INFO, "CSVExport", "      Exporting ReportPeriod ");
                        currentLogger.log(LoggingLevel.INFO, "CSVExport", "========================================================================");
                        
                        // ReportPeriod
                        currentFile = currentOutput + "\\" + files[1];
                        wr = new StreamWriter(currentFile);
                        try
                        {
                            hasError = false;
                            parts = new string[6];
                            parts[0] = reporterRow["rptMStateKey"] as string;
                            parts[1] = reportperiodRow["repCode"] as string;
                            if ((parts[1] == null) || (parts[1].Length<1))
                            {
                                hasError = true;
                                currentLogger.log(LoggingLevel.ERROR, "ReportPeriod", "The repCode Parameter is not Set");
                            }
                            else if (!parts[1].StartsWith(rptMemberState))
                            {
                                hasError = true;
                                currentLogger.log(LoggingLevel.ERROR, "ReportPeriod", "The repCode (" + parts[1] + ") has to start with " + rptMemberState);
                            }
                            if (reportperiodRow["repVersion"] is DBNull)
                                parts[2] = "";
                            else
                                parts[2] = conv.formatDateShort((DateTime)reportperiodRow["repVersion"]);
                            parts[3] = conv.formatDateShort((DateTime)reportperiodRow["repSituationAt"]);
                            if (parts[3] == null)
                            {
                                hasError = true;
                                currentLogger.log(LoggingLevel.ERROR, "ReportPeriod", "repSituationAt is Master Data");
                            }
                            parts[4] = (string)reportperiodRow["repReportedPeriod"];
                            parts[5] = conv.formatDateLong(DateTime.Now);

                            wr.WriteLine(string.Join("\t", parts));
                            currentLogger.addReportPeriod((int)reportperiodRow["repID"]);
                        }
                        finally
                        {
                            wr.Close();
                        }

                        if (hasError)
                            currentLogger.reportFail("ReportPeriod");
                        else
                            currentLogger.reportSuccess("ReportPeriod");

                        backgroundWorker.ReportProgress(20);
                        currentLogger.log(LoggingLevel.INFO, "CSVExport", "========================================================================");
                        currentLogger.log(LoggingLevel.INFO, "CSVExport", "      Exporting Contact ");
                        currentLogger.log(LoggingLevel.INFO, "CSVExport", "========================================================================");
                        currentFile = currentOutput + "\\" + files[2];
                        wr = new StreamWriter(currentFile);
                        try
                        {
                            IDbDataAdapter contactTAdapter = dataFactory.CreateFullDataAdapter(dataFactory.CreateCommand("select * from T_Contact", conn, trans));
                            dataFactory.FillDataAdapter(contactTAdapter, ds, "T_Contact"); // in case - anything could have been changed in the meantime
                            DataTable contact = ds.Tables["T_Contact"];

                            int count = 0; ;
                            foreach (DataRow row in ds.Tables["T_Contact"].Select("repID = " + reportperiodRow["repID"]))
                            {
                                parts = new string[12];
                                try
                                {
                                    parts[0] = (string)reportperiodRow["repCode"];
                                    parts[1] = (string)row["conCode"].ToString();
                                    parts[2] = (string)row["conName"].ToString();
                                    parts[3] = (string)row["conInstitution"].ToString();
                                    parts[4] = (string)row["conStreet"].ToString();
                                    parts[5] = (string)row["conZIP"].ToString();
                                    parts[6] = (string)row["conCity"].ToString();
                                    parts[7] = (string)row["conPhone"].ToString();
                                    parts[8] = (string)row["conFax"].ToString();
                                    parts[9] = (string)row["conEmail"].ToString();
                                    parts[10] = (string)row["conRemarks"].ToString();
                                    parts[11] = conv.formatDateLong(DateTime.Now);
                                    if ((parts[2] == null) || (parts[2].Length < 1))
                                    {
                                        hasError = true;
                                        currentLogger.log(LoggingLevel.ERROR, "Contact", "The Name Parameter is Master Data");
                                    }
                                    if ((parts[3] == null) || (parts[3].Length < 1))
                                    {
                                        hasError = true;
                                        currentLogger.log(LoggingLevel.ERROR, "Contact", "The Institution Parameter is Master Data");
                                    }
                                    if ((parts[7] == null) || (parts[7].Length < 1))
                                    {
                                        hasError = true;
                                        currentLogger.log(LoggingLevel.ERROR, "Contact", "The Phone Parameter is Master Data");
                                    }
                                    wr.WriteLine(string.Join("\t", parts));
                                    count++;
                                    currentLogger.reportSuccess("Contact");                                
                                }
                                catch (Exception ex) {
                                    currentLogger.log(LoggingLevel.ERROR, "Contact", "Export Contact resulted in Exception: " + ex.Message + "\n" + ex.StackTrace);
                                    currentLogger.reportFail("Contact");                                
                                }
                            }

                        }
                        finally
                        {
                            wr.Close();
                        }

                        backgroundWorker.ReportProgress(30);
                        InterfaceBase intf = null;
                        backgroundWorker.ReportProgress(35);
                        // ReceivingAreas
                        currentFile = currentOutput + "\\" + files[3];
                        wr = new StreamWriter(currentFile);
                        try
                        {
                            intf = new InterfaceReceivingAreas(conn, trans, conv);
                            intf.Logger = currentLogger;
                            intf.exportData(wr, reportperiodRow);
                        }
                        finally
                        {
                            wr.Close();
                        }
                        backgroundWorker.ReportProgress(50);

                        // Agglomerations
                        currentFile = currentOutput + "\\" + files[4];
                        wr = new StreamWriter(currentFile);
                        try
                        {
                            intf = new InterfaceAgglomerations(conn, trans, conv);
                            intf.Logger = currentLogger;
                            intf.exportData(wr, reportperiodRow);
                            backgroundWorker.ReportProgress(50);
                        }
                        finally
                        {
                            wr.Close();
                        }
                        // UWWTPs
                        currentFile = currentOutput + "\\" + files[5];
                        wr = new StreamWriter(currentFile);
                        try
                        {
                            intf = new InterfaceUWWTP(conn, trans, conv);
                            intf.Logger = currentLogger;
                            intf.exportData(wr, reportperiodRow);
                        }
                        finally
                        {
                            wr.Close();
                        }
                        backgroundWorker.ReportProgress(60);
                        // UWWTP_AGGLO
                        currentFile = currentOutput + "\\" + files[6];
                        wr = new StreamWriter(currentFile);
                        try
                        {
                            intf = new InterfaceUwwAgglo(conn, trans, conv);
                            intf.Logger = currentLogger;
                            intf.exportData(wr, reportperiodRow);
                        }
                        finally
                        {
                            wr.Close();
                        }
                        backgroundWorker.ReportProgress(70);
                        // Dischargepoints
                        currentFile = currentOutput + "\\" + files[7];
                        wr = new StreamWriter(currentFile);
                        try
                        {
                            intf = new InterfaceDischargePoint(conn, trans, conv);
                            intf.Logger = currentLogger;
                            intf.exportData(wr, reportperiodRow);
                        }
                        finally
                        {
                            wr.Close();
                        }
                        backgroundWorker.ReportProgress(80);
                        // MSLevel
                        currentFile = currentOutput + "\\" + files[8];
                        wr = new StreamWriter(currentFile);
                        try
                        {
                            intf = new InterfaceMemberState(conn, trans, conv, reporterRow);
                            intf.Logger = currentLogger;
                            intf.exportData(wr, reportperiodRow);
                        }
                        finally
                        {
                            wr.Close();
                        }
                        backgroundWorker.ReportProgress(90);
                        // Industries
                        currentFile = currentOutput + "\\" + files[9];
                        wr = new StreamWriter(currentFile);
                        try
                        {
                            intf = new InterfaceIndustries(conn, trans, conv, reporterRow);
                            intf.Logger = currentLogger;
                            intf.exportData(wr, reportperiodRow);
                        }
                        finally
                        {
                            wr.Close();
                        }
                        backgroundWorker.ReportProgress(95);

                        if (trans != null) trans.Commit();
                        trans = null;
                    }
                    finally
                    {
                        if (trans != null)
                            trans.Rollback();
                        trans = null;
                    }
                }
                finally
                {
                    conn.Close();
                }
            }
            catch (Exception ex)
            {
                currentLogger.log(LoggingLevel.ERROR, "CSVExport", "Unhandled Exception during Export: " + ex.Message + "\n" + ex.StackTrace);
            }
            finally
            {
                currentLogger.Dispose();
            }
        }
    }
}
