﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Xml.Serialization;
using System.IO;
using System.Windows.Forms;
using System.Collections;
using at.uba.uwwtd2008.DataAccess;
using at.uba.uwwtd2008.xml;
using at.uba.uwwtd2008.Properties;
using at.uba.uwwtd2008.Items;

namespace at.uba.uwwtd2008.Dialogs
{
    public partial class CSVImportDialog : Form
    {
        DataAccessFactory dataFactory = DataAccessFactory.Instance;
        string currentInput = null;
        Settings settings = Settings.Default;
        LoggingClass currentLogger;

        public CSVImportDialog()
        {
            InitializeComponent();
            if ((settings.Last_Import_Dir != null) && (settings.Last_Import_Dir.Length > 0))
                currentInput = settings.Last_Import_Dir;
            else
                currentInput = ".";
            textBox1.Text = currentInput;
        }

        private void OnBrowseClick(object sender, EventArgs e)
        {
            dlgImportFolder.SelectedPath = currentInput;
            if (dlgImportFolder.ShowDialog(this) == DialogResult.OK)
            {
                textBox1.Text = dlgImportFolder.SelectedPath;
                settings.Last_Import_Dir = textBox1.Text;
                settings.Save();
            }
        }

        private void OnImportFileChanged(object sender, EventArgs e)
        {
            currentInput = ((TextBox)sender).Text;
        }

        private void OnClose(object sender, EventArgs e)
        {
            this.Close();

        }

        private void OnImportStart(object sender, EventArgs e)
        {
            if (!Directory.Exists(currentInput))
            {
                MessageBox.Show(this, "The specified directory does not exist. Please select a valid Import Directory", "Import CSV", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                {
                    return;
                }
            }
            currentLogger = new LoggingClass();
            cmdClose.Enabled = false;
            cmdImport.Enabled = false;
            cmdImportLog.Enabled = false;
            progressImport.Value = 0;
            backgroundWorker.RunWorkerAsync(null);
        }

        private void OnProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            progressImport.Value = e.ProgressPercentage;
        }

        private void OnImportComplete(object sender, RunWorkerCompletedEventArgs e)
        {
            progressImport.Value = progressImport.Maximum;
            cmdClose.Enabled = true;
            cmdImport.Enabled = true;
            cmdImportLog.Enabled = true;

            ImportExportSummary dlg = new ImportExportSummary();
            dlg.Logger = currentLogger;
            dlg.Show();
        }

        private void DoImporting(object sender, DoWorkEventArgs e)
        {
            currentLogger.Init();
            try
            {
                // list of files
                string[] files = { "Reporter.txt", "ReportPeriod.txt", "Contact.txt", "ReceivingAreas.txt", "Agglomerations.txt",
                             "Uwwtps.txt", "UwwtpAgglo.txt", "DischargePoints.txt", "MSLevel.txt", "Industries.txt"};
                string rptMemberState = "";
                bool hasError = false;
                bool enforceConstraints = false;
                String currentFile = "";
                IDbConnection conn = dataFactory.GetConnection();
                try
                {
                    IDbTransaction trans = null;//conn.BeginTransaction();
                    try
                    {
                        DataSet ds = dataFactory.getReporterDataSet();
                        enforceConstraints = ds.EnforceConstraints;
                        ds.EnforceConstraints = false;

                        DataRow reporterRow = null;
                        DataRow reportperiodRow = null;
                        DataRow contactRow = null;
                        backgroundWorker.ReportProgress(0);
                        // Reporter ...
                        currentFile = currentInput + "\\" + files[0];
                        currentLogger.log(LoggingLevel.INFO, "CSVImport", "========================================================================");
                        currentLogger.log(LoggingLevel.INFO, "CSVImport", "      Importing Reporter ");
                        currentLogger.log(LoggingLevel.INFO, "CSVImport", "========================================================================");

                        if (!File.Exists(currentFile))
                        {
                            currentLogger.log(LoggingLevel.ERROR, "CSVImport", "The file " + files[0] + " does not exist - import aborted");
                            return;
                        }

                        StreamReader rdr = new StreamReader(currentFile, true);
                        string line = rdr.ReadLine();
                        string[] parts = line.Split(new string[] { "\t" }, StringSplitOptions.None);
                        if (parts.Length < 3)
                        {
                            hasError = true;
                            currentLogger.log(LoggingLevel.ERROR, "Reporter", "Too less elements in line");
                        }

                        

                        IDbDataAdapter reporterTAdapter = dataFactory.CreateFullDataAdapter(dataFactory.CreateCommand("Select * from T_Reporter", conn, trans));
                        dataFactory.FillDataAdapter(reporterTAdapter, ds, "T_Reporter"); // in case - anything could have been changed in the meantime
                        int iFormRA = -1;
                        DataTable reporter = ds.Tables["T_Reporter"];
                        //reporter.Select("rptMState = ''");
                        DataRow state = dataFactory.getLovRowByKey("MS", parts[0]);

                        if (state["lovKey"] is DBNull)
                        {
                            hasError = true;
                            currentLogger.log(LoggingLevel.ERROR, "Reporter", "The memberstate " + parts[0] + " cannot be found in LOV-Table");
                        }
                        else
                            rptMemberState = state["lovKey"] as string;

                        if (reporter.Select("rptMState = " + state["lovID"]).Length > 0) 
                        {
                            reporterRow = reporter.Select("rptMState = " + state["lovID"])[0];
                            if (!(reporterRow["rptFormRA"] is DBNull))
                                iFormRA = int.Parse(reporterRow["rptFormRA"].ToString());
                        } 
                        else
                        {
                            reporterRow = reporter.NewRow();
                            reporterRow["rptMState"] = state["lovID"];
                            reporterRow["rptMStateKey"] = parts[0];
                            reporterRow["rptMStateWert"] = parts[1];
                            reporterRow["rptCulture"] = parts[2];
                            reporterRow["rptFormRA"] = -1;
                            try
                            {
                                if ((parts.Length > 3) && (parts[3].Length > 0))
                                    iFormRA = int.Parse(parts[3]);
                                else
                                    currentLogger.log(LoggingLevel.ERROR, "Reporter", "rptFormRA is Master Data");
                            }
                            catch (Exception ex)
                            {
                                currentLogger.log(LoggingLevel.ERROR, "Reporter", "FormRA reading caused an Error: " + ex.Message);
                            }
                            reporter.Rows.Add(reporterRow);
                            dataFactory.UpdateDataAdapter(reporterTAdapter, ds, "T_Reporter");
                            dataFactory.FillDataAdapter(reporterTAdapter, ds, "T_Reporter"); // refresh the internal state (ids)
                            reporterRow = reporter.Select("rptMState = " + state["lovID"])[0];
                        }
                        if (hasError)
                            currentLogger.reportFail("Reporter");
                        else
                            currentLogger.reportSuccess("Reporter");

                        hasError = false;
                        backgroundWorker.ReportProgress(10);
                        DataFormatConverter conv = new DataFormatConverter(parts[2]);

                        
                        // ReportPeriod
                        currentFile = currentInput + "\\" + files[1];
                        currentLogger.log(LoggingLevel.INFO, "CSVImport", "========================================================================");
                        currentLogger.log(LoggingLevel.INFO, "CSVImport", "      Importing ReportPeriod ");
                        currentLogger.log(LoggingLevel.INFO, "CSVImport", "========================================================================");
                        if (!File.Exists(currentFile))
                        {
                            currentLogger.log(LoggingLevel.ERROR, "CSVImport", "The file " + files[1] + " does not exist - import aborted");
                            return;
                        }
                        hasError = false;
                        rdr = new StreamReader(currentFile, true);
                        line = rdr.ReadLine();
                        parts = line.Split(new string[] { "\t" }, StringSplitOptions.None);
                        if (parts.Length < 5)
                        {
                            hasError = true;
                            currentLogger.log(LoggingLevel.ERROR, "ReportPeriod", "Too less elements in line");
                        }

                        IDbDataAdapter repperiodTAdapter = dataFactory.CreateFullDataAdapter(dataFactory.CreateCommand("select * from T_ReportPeriod", conn, trans));
                        dataFactory.FillDataAdapter(repperiodTAdapter, ds, "T_ReportPeriod"); // in case - anything could have been changed in the meantime

                        DataTable reportperiod = ds.Tables["T_ReportPeriod"];
                        if (reportperiod.Select("rptID = " + reporterRow["rptID"] + " and repCode = '" + parts[1] + "'").Length > 0)
                        {
                            reportperiodRow = reportperiod.Select("rptID = " + reporterRow["rptID"] + " and repCode = '" + parts[1] + "'")[0];
                        }
                        else
                        {
                            reportperiodRow = reportperiod.NewRow();
                            reportperiodRow["rptID"] = reporterRow["rptID"];
                            reportperiodRow["repCode"] = parts[1];
                            reportperiodRow["repVersion"] = conv.parseDate(parts[2]);
                            reportperiodRow["repSituationAt"] = conv.parseDate(parts[3]);
                            reportperiodRow["repReportedPeriod"] = parts[4];
                            reportperiodRow["repRATypes"] = DataAccessFactory.oldRATypToNew(iFormRA);

                            if ((parts[1] == null) || (parts[1].Length < 1))
                            {
                                hasError = true;
                                currentLogger.log(LoggingLevel.ERROR, "ReportPeriod", "The repCode Parameter is not Set");
                            }
                            else if (!parts[1].StartsWith(rptMemberState))
                            {
                                hasError = true;
                                currentLogger.log(LoggingLevel.ERROR, "ReportPeriod", "The repCode (" + parts[1] + ") has to start with " + rptMemberState);
                            }
                            if ((parts[3] == null) || (parts[3].Length < 1))
                            {
                                hasError = true;
                                currentLogger.log(LoggingLevel.ERROR, "ReportPeriod", "repSituationAt is Master Data");
                            }

                            reportperiod.Rows.Add(reportperiodRow);
                            dataFactory.UpdateDataAdapter(repperiodTAdapter, ds, "T_ReportPeriod");
                            dataFactory.FillDataAdapter(repperiodTAdapter, ds, "T_ReportPeriod");
                            reportperiodRow = reportperiod.Select("rptID = " + reporterRow["rptID"] + " and repCode = '" + parts[1] + "'")[0]; // refresh reportperiod
                            //if (hasError)
                            //    currentLogger.reportFail("ReportPeriod");
                            //else
                                currentLogger.reportSuccess("ReportPeriod");

                        }
                        currentLogger.addReportPeriod((int)reportperiodRow["repID"]);

                        IDbCommand cmdRemoveAllPrevAgglos = dataFactory.CreateCommand("DELETE FROM T_Agglomerations WHERE repID = " + reportperiodRow["repID"], conn, trans);
                        cmdRemoveAllPrevAgglos.ExecuteNonQuery();

                        backgroundWorker.ReportProgress(20);
                        currentFile = currentInput + "\\" + files[2];
                        currentLogger.log(LoggingLevel.INFO, "CSVImport", "========================================================================");
                        currentLogger.log(LoggingLevel.INFO, "CSVImport", "      Importing Contacts ");
                        currentLogger.log(LoggingLevel.INFO, "CSVImport", "========================================================================");
                        if (!File.Exists(currentFile))
                        {
                            currentLogger.log(LoggingLevel.ERROR, "CSVImport", "The file " + files[2] + " does not exist - import aborted");
                            return;
                        }


                        if (!File.Exists(currentFile)) return;
                        rdr = new StreamReader(currentFile, true);
                        IDbDataAdapter contactTAdapter = dataFactory.CreateFullDataAdapter(dataFactory.CreateCommand("select * from T_Contact", conn, trans));
                        dataFactory.FillDataAdapter(contactTAdapter, ds, "T_Contact"); // in case - anything could have been changed in the meantime
                        DataTable contact = ds.Tables["T_Contact"];

                        // delete all current Contacts
                        foreach (DataRow row in contact.Select("repID = " + reportperiodRow["repID"]))
                        {
                            row.Delete();
                        }
                        dataFactory.UpdateDataAdapter(contactTAdapter, ds, "T_Contact");
                        //int count = 0; ;
                        while ((line = rdr.ReadLine()) != null)
                        {
                            if (line.Length < 1) continue;
                            parts = line.Split(new string[] { "\t" }, StringSplitOptions.None);

                            // check, if there are only empty elements (bug report from 2009-06-04
                            bool allEmpty = true;
                            foreach (string part in parts)
                            {
                                if (part.Trim().Length > 0)
                                {
                                    allEmpty = false;
                                    break;
                                }
                            }

                            if (allEmpty) continue;

                            contactRow = contact.NewRow();
                            try
                            {
                                contactRow["repID"] = reportperiod.Select("repCode = '" + parts[0] + "'")[0]["repID"];
                                contactRow["conCode"] = parts[1];
                                contactRow["conName"] = parts[2];
                                contactRow["conInstitution"] = parts[3];
                                contactRow["conStreet"] = parts[4];
                                contactRow["conZIP"] = parts[5];
                                contactRow["conCity"] = parts[6];
                                contactRow["conPhone"] = parts[7];
                                contactRow["conFax"] = parts[8];
                                contactRow["conEmail"] = parts[9];
                                contactRow["conRemarks"] = parts[10];
                                if ((parts[2] == null) || (parts[2].Length < 1))
                                {
                                    hasError = true;
                                    currentLogger.log(LoggingLevel.ERROR, "Contact", "The Name Parameter is Master Data");
                                }
                                if ((parts[3] == null))
                                {
                                    hasError = true;
                                    currentLogger.log(LoggingLevel.ERROR, "Contact", "The Institution Parameter is Master Data");
                                }
                                if ((parts[7] == null)) {
                                    hasError = true;
                                    currentLogger.log(LoggingLevel.ERROR, "Contact", "The Phone Parameter is Master Data");
                                }
                                contact.Rows.Add(contactRow);
                                dataFactory.UpdateDataAdapter(contactTAdapter, ds, "T_Contact");
                                //if (hasError)
                                //    currentLogger.reportFail("Contact");
                                //else
                                    currentLogger.reportSuccess("Contact");
                            }
                            catch (Exception)
                            {
                                currentLogger.reportFail("Contact");
                            }
                        }
                        dataFactory.FillDataAdapter(contactTAdapter, ds, "T_Contact");

                        backgroundWorker.ReportProgress(30);
                        InterfaceBase intf = null;
                        IDbCommand cmd = dataFactory.CreateCommand("DELETE from T_ReceivingAreas WHERE repID = " + reportperiodRow["repID"], conn, trans);
                        cmd.ExecuteNonQuery();
                        cmd = dataFactory.CreateCommand("DELETE from T_Agglomerations WHERE repID = " + reportperiodRow["repID"], conn, trans);
                        cmd.ExecuteNonQuery();
                        cmd = dataFactory.CreateCommand("DELETE from T_UWWTPS WHERE repID = " + reportperiodRow["repID"], conn, trans);
                        cmd.ExecuteNonQuery();
                        cmd = dataFactory.CreateCommand("DELETE from T_UWWTP_AGGLO WHERE repID = " + reportperiodRow["repID"], conn, trans);
                        cmd.ExecuteNonQuery();
                        cmd = dataFactory.CreateCommand("DELETE from T_DischargePoints WHERE repID = " + reportperiodRow["repID"], conn, trans);
                        cmd.ExecuteNonQuery();
                        cmd = dataFactory.CreateCommand("DELETE from T_MSLevel WHERE repID = " + reportperiodRow["repID"], conn, trans);
                        cmd.ExecuteNonQuery();
                        cmd = dataFactory.CreateCommand("DELETE from T_Industries WHERE repID = " + reportperiodRow["repID"], conn, trans);
                        cmd.ExecuteNonQuery();

                        

                        backgroundWorker.ReportProgress(35);
                        // ReceivingAreas
                        currentFile = currentInput + "\\" + files[3];
                        if (!File.Exists(currentFile)) return;
                        rdr = new StreamReader(currentFile, true);
                        intf = new InterfaceReceivingAreas(conn, trans, conv);
                        intf.Logger = currentLogger;
                        intf.importData(rdr, reportperiodRow);
                        backgroundWorker.ReportProgress(50);

                        // Agglomerations
                        currentFile = currentInput + "\\" + files[4];
                        if (!File.Exists(currentFile)) return;
                        rdr = new StreamReader(currentFile, true);
                        intf = new InterfaceAgglomerations(conn, trans, conv);
                        intf.Logger = currentLogger;
                        intf.importData(rdr, reportperiodRow);
                        backgroundWorker.ReportProgress(50);
                        // UWWTPs
                        currentFile = currentInput + "\\" + files[5];
                        if (!File.Exists(currentFile)) return;
                        rdr = new StreamReader(currentFile, true);
                        intf = new InterfaceUWWTP(conn, trans, conv);
                        intf.Logger = currentLogger;
                        intf.importData(rdr, reportperiodRow);
                        backgroundWorker.ReportProgress(60);
                        // UWWTP_AGGLO
                        currentFile = currentInput + "\\" + files[6];
                        if (!File.Exists(currentFile)) return;
                        rdr = new StreamReader(currentFile, true);
                        intf = new InterfaceUwwAgglo(conn, trans, conv);
                        intf.Logger = currentLogger;
                        intf.importData(rdr, reportperiodRow);
                        backgroundWorker.ReportProgress(70);
                        // Dischargepoints
                        currentFile = currentInput + "\\" + files[7];
                        if (!File.Exists(currentFile)) return;
                        rdr = new StreamReader(currentFile, true);
                        intf = new InterfaceDischargePoint(conn, trans, conv);
                        intf.Logger = currentLogger;
                        intf.importData(rdr, reportperiodRow);
                        backgroundWorker.ReportProgress(80);
                        // MSLevel
                        currentFile = currentInput + "\\" + files[8];
                        if (!File.Exists(currentFile)) return;
                        rdr = new StreamReader(currentFile, conv.FileEncoding);
                        intf = new InterfaceMemberState(conn, trans, conv, reporterRow);
                        intf.Logger = currentLogger;
                        intf.importData(rdr, reportperiodRow);
                        backgroundWorker.ReportProgress(90);
                        // Industries
                        currentFile = currentInput + "\\" + files[9];
                        if (!File.Exists(currentFile)) return;
                        rdr = new StreamReader(currentFile, conv.FileEncoding);
                        intf = new InterfaceIndustries(conn, trans, conv, reporterRow);
                        intf.Logger = currentLogger;
                        intf.importData(rdr, reportperiodRow);
                        backgroundWorker.ReportProgress(95);

                        if (trans != null) trans.Commit();
                        trans = null;
                        dataFactory.initReporters();
                    }
                    finally
                    {
                        if (trans != null)
                            trans.Rollback();
                        trans = null;
                        dataFactory.initReporters();
                    }
                }
                finally
                {
                    conn.Close();
                }
            }
            catch (Exception ex)
            {
                currentLogger.log(LoggingLevel.ERROR, "CSVImport", "Uncaught while importing : " + ex.Message + "\n" + ex.StackTrace);
            }
            finally
            {
                currentLogger.log(LoggingLevel.INFO, "CSVImport", "Finished Import ...");
                currentLogger.Dispose();
            }
        }
    }
}
