﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Xml.Serialization;
using System.IO;
using System.Windows.Forms;
using System.Collections;
using at.uba.uwwtd2008.DataAccess;
using at.uba.uwwtd2008.xml;
using at.uba.uwwtd2008.Properties;
using at.uba.uwwtd2008.Items;

namespace at.uba.uwwtd2008.Dialogs
{
    public partial class XMLExportDialog : Form
    {
        DataAccessFactory dataFactory = DataAccessFactory.Instance;
        string currentOutput = null;
        Settings settings = Settings.Default;
        LoggingClass currentLogger = null;
        private DataSet dataSet = null;

        public XMLExportDialog()
        {
            InitializeComponent();
            if ((settings.LastXMLExportFile != null) && (settings.LastXMLExportFile.Length>0))
                currentOutput = settings.LastXMLExportFile;
            else
                currentOutput = settings.Last_Export_Dir + "\\report.xml";
            textBox1.Text = currentOutput;
        }

        private void OnBrowseClick(object sender, EventArgs e)
        {
            saveFileDialog.FileName = currentOutput;
            if (saveFileDialog.ShowDialog(this) == DialogResult.OK)
            {
                textBox1.Text = saveFileDialog.FileName;
                settings.LastXMLExportFile = textBox1.Text;
                settings.Save();
            }
        }

        private void OnExportFileChanged(object sender, EventArgs e)
        {
            currentOutput = ((TextBox)sender).Text;
        }

        private void OnClose(object sender, EventArgs e)
        {
            this.Close();

        }

        private void OnFormLoad(object sender, EventArgs e)
        {
            dataSet = dataFactory.getReporterDataSet();
            //IDbConnection conn = dataFactory.GetConnection();
            //try
            //{
            //    // load all the reporter relevant data
            //    dataFactory.FillDataAdapter(
            //        dataFactory.CreateDataAdapter(
            //            dataFactory.CreateCommand("SELECT * FROM T_LOV", conn)),
            //        dataSet, "T_LOV");
            //    dataFactory.FillDataAdapter(
            //        dataFactory.CreateDataAdapter(
            //            dataFactory.CreateCommand("SELECT * FROM T_REPORTPERIOD", conn)),
            //        dataSet, "T_REPORTPERIOD");
            //    dataFactory.FillDataAdapter(
            //        dataFactory.CreateDataAdapter(
            //            dataFactory.CreateCommand("SELECT * FROM T_REPORTER", conn)),
            //        dataSet, "T_REPORTER");
            //    dataFactory.FillDataAdapter(
            //        dataFactory.CreateDataAdapter(
            //            dataFactory.CreateCommand("SELECT * FROM T_CONTACT", conn)),
            //        dataSet, "T_CONTACT");
            //    dataFactory.FillDataAdapter(
            //        dataFactory.CreateDataAdapter(
            //            dataFactory.CreateCommand("SELECT * FROM T_NUTS", conn)),
            //        dataSet, "T_NUTS");
            //    dataSet.Relations.Add("T_ReporterT_ReportPeriod",
            //        dataSet.Tables["T_REPORTER"].Columns["rptID"],
            //        dataSet.Tables["T_REPORTPERIOD"].Columns["rptID"]);
            //    dataSet.Relations.Add("T_LOV_T_REPORTER",
            //        dataSet.Tables["T_LOV"].Columns["lovID"],
            //        dataSet.Tables["T_REPORTER"].Columns["rptMState"]);
            //    dataSet.Relations.Add("T_CONTACT_T_REPORTPERIOD",
            //        dataSet.Tables["T_REPORTPERIOD"].Columns["repID"],
            //        dataSet.Tables["T_CONTACT"].Columns["repID"]);
            //}
            //finally
            //{
            //    conn.Close();
            //}

            DataTable tab = new DataTable();
            tab.Columns.Add("repID", dataSet.Tables["T_REPORTPERIOD"].Columns["repID"].DataType);
            tab.Columns.Add("display", typeof(string));

            foreach (DataRow reportperiod in dataSet.Tables["T_REPORTPERIOD"].Rows)
            {
                CheckBoxDisplayable t = new CheckBoxDisplayable();
                t.key=reportperiod["repID"];
                DataRow reporter = reportperiod.GetParentRow("T_ReporterT_ReportPeriod");
                t.display = reporter.GetParentRow("T_Lov_T_Reporter")["lovWert"] + " - " + reportperiod["repReportedPeriod"];
                chkListReports.Items.Add(t, true);
            }

        }

        private void OnExportStart(object sender, EventArgs e)
        {
            object[] reps = new object[chkListReports.CheckedItems.Count];
            for (int i = 0; i < reps.Length; i++)
            {
                reps[i] = (chkListReports.CheckedItems[i] as CheckBoxDisplayable).key;
            }
            currentLogger = new LoggingClass();
            
            backgroundWorker.RunWorkerAsync(reps);
        }

        private void OnProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            progressExport.Value = e.ProgressPercentage;
        }

        private void OnExportComplete(object sender, RunWorkerCompletedEventArgs e)
        {
            progressExport.Value = progressExport.Maximum;
            ImportExportSummary dlg = new ImportExportSummary();
            dlg.Logger = currentLogger;
            dlg.ShowDialog();
        }

        private void DoExporting(object sender, DoWorkEventArgs e)
        {
            currentLogger.Init();
            try
            {
                object[] sel = e.Argument as object[];
                if (sel == null)
                {
                    e.Result = null;
                    return;
                }

                UWWTD_Data data = new UWWTD_Data();
                data.UWWTD_Report = new UWWTD_DataUWWTD_Report[sel.Length];
                for (int iReport = 0; iReport < sel.Length; iReport++)
                {
                    data.UWWTD_Report[iReport] = new UWWTD_DataUWWTD_Report();
                    data.UWWTD_Report[iReport].rptDateexported = DateTime.Now;
                    bool hasError = false;


                    DataRow reportperiod = dataSet.Tables["T_REPORTPERIOD"].Rows.Find(sel[iReport]);

                    currentLogger.log(LoggingLevel.INFO, "XMLExport", "========================================================================");
                    currentLogger.log(LoggingLevel.INFO, "XMLExport", "      Exporting Reporter ");
                    currentLogger.log(LoggingLevel.INFO, "XMLExport", "========================================================================");
                    hasError = false;
                    DataRow reporter = reportperiod.GetParentRow("T_ReporterT_ReportPeriod");
                    data.UWWTD_Report[iReport].Reporter = new UWWTD_DataUWWTD_ReportReporter();
                    data.UWWTD_Report[iReport].Reporter.rptMStateKey = reporter.GetParentRow("T_Lov_T_Reporter")["lovKey"] as string;
                    data.UWWTD_Report[iReport].Reporter.rptMStateValue = reporter.GetParentRow("T_Lov_T_Reporter")["lovWert"] as string;
                    currentLogger.reportSuccess("Reporter");
                    data.UWWTD_Report[iReport].Reporter.rptCulture = reporter["rptCulture"] as string;
                    if (reportperiod["repRATypes"] is DBNull)
                        data.UWWTD_Report[iReport].Reporter.rptFormRASpecified = false;
                    else
                    {
                        data.UWWTD_Report[iReport].Reporter.rptFormRA = DataAccessFactory.newRATypToOld((int)reportperiod["repRATypes"]);
                        data.UWWTD_Report[iReport].Reporter.rptFormRASpecified = true;
                    }

                    currentLogger.log(LoggingLevel.INFO, "XMLExport", "========================================================================");
                    currentLogger.log(LoggingLevel.INFO, "XMLExport", "      Exporting ReportPeriod ");
                    currentLogger.log(LoggingLevel.INFO, "XMLExport", "========================================================================");
                    hasError = false;
                    data.UWWTD_Report[iReport].ReportPeriod = new UWWTD_DataUWWTD_ReportReportPeriod();
                    data.UWWTD_Report[iReport].ReportPeriod.repCode = reportperiod["repCode"] as string;
                    if (!data.UWWTD_Report[iReport].ReportPeriod.repCode.StartsWith(data.UWWTD_Report[iReport].Reporter.rptMStateKey)) {
                        hasError = true;
                        currentLogger.log(LoggingLevel.ERROR, "ReportPeriod", "repCode (" + reportperiod["repCode"] + ") should start with " + data.UWWTD_Report[iReport].Reporter.rptMStateKey);
                    }
                    data.UWWTD_Report[iReport].ReportPeriod.repReportedPeriod = reportperiod["repReportedPeriod"] as string;
                    if (reportperiod["repSituationAt"] is DateTime)
                        data.UWWTD_Report[iReport].ReportPeriod.repSituationAt = (DateTime)reportperiod["repSituationAt"];
                    else {
                        currentLogger.log(LoggingLevel.ERROR, "ReportPeriod", "repSituationAt is Master Data");
                        hasError = true;
                    }
                    if (reportperiod["repVersion"] is DateTime) 
                        data.UWWTD_Report[iReport].ReportPeriod.repVersion = (DateTime)reportperiod["repVersion"];
                    data.UWWTD_Report[iReport].ReportPeriod.repVersionSpecified = (reportperiod["repVersion"] is DateTime);
                    currentLogger.addReportPeriod((int)reportperiod["repID"]);
                    currentLogger.reportSuccess("ReportPeriod");

                    currentLogger.log(LoggingLevel.INFO, "XMLExport", "========================================================================");
                    currentLogger.log(LoggingLevel.INFO, "XMLExport", "      Exporting Contacts ");
                    currentLogger.log(LoggingLevel.INFO, "XMLExport", "========================================================================");

                    DataRow[] contacts = reportperiod.GetChildRows("T_ReportPeriodT_Contact");
                    data.UWWTD_Report[iReport].Contacts = new UWWTD_DataUWWTD_ReportContact[contacts.Length];
                    for (int iContact = 0; iContact < contacts.Length; iContact++)
                    {
                        hasError = false;
                        data.UWWTD_Report[iReport].Contacts[iContact] = new UWWTD_DataUWWTD_ReportContact();
                        data.UWWTD_Report[iReport].Contacts[iContact].conCity = contacts[iContact]["conCity"] as string;
                        data.UWWTD_Report[iReport].Contacts[iContact].conEmail = contacts[iContact]["conEmail"] as string;
                        data.UWWTD_Report[iReport].Contacts[iContact].conFax = contacts[iContact]["conFax"] as string;
                        data.UWWTD_Report[iReport].Contacts[iContact].conInstitution = contacts[iContact]["conInstitution"] as string;
                        data.UWWTD_Report[iReport].Contacts[iContact].conName = contacts[iContact]["conName"] as string;
                        data.UWWTD_Report[iReport].Contacts[iContact].conPhone = contacts[iContact]["conPhone"] as string;
                        data.UWWTD_Report[iReport].Contacts[iContact].conRemarks = contacts[iContact]["conRemarks"] as string;
                        data.UWWTD_Report[iReport].Contacts[iContact].conStreet = contacts[iContact]["conStreet"] as string;
                        data.UWWTD_Report[iReport].Contacts[iContact].conZIP = contacts[iContact]["conZIP"] as string;

                        if ((data.UWWTD_Report[iReport].Contacts[iContact].conName == null) || (data.UWWTD_Report[iReport].Contacts[iContact].conName.Length < 1))
                        {
                            hasError = true;
                            currentLogger.log(LoggingLevel.ERROR, "Contact", "Name is Master Data");
                        }
                        if ((data.UWWTD_Report[iReport].Contacts[iContact].conInstitution == null) || (data.UWWTD_Report[iReport].Contacts[iContact].conInstitution.Length < 1))
                        {
                            hasError = true;
                            currentLogger.log(LoggingLevel.ERROR, "Contact", "Institution is Master Data");
                        }
                        if ((data.UWWTD_Report[iReport].Contacts[iContact].conPhone == null) || (data.UWWTD_Report[iReport].Contacts[iContact].conPhone.Length < 1))
                        {
                            hasError = true;
                            currentLogger.log(LoggingLevel.ERROR, "Contact", "Phone is Master Data");
                        }

                        if (hasError)
                            currentLogger.reportFail("Contact");
                        else
                            currentLogger.reportSuccess("Contact");
                    }

                    // do the actual work
                    ArrayList lstTmp = new ArrayList();
                    IDbConnection conn = dataFactory.GetConnection();
                    try
                    {
                        IDbTransaction trans = null;
                        DataFormatConverter conv = new DataFormatConverter(null);
                        DataRow repPeriod = dataFactory.getRepPeridodforRepID(reportperiod["repID"]);

                        backgroundWorker.ReportProgress(20);
                        // AGGLOMERATIONS
                        InterfaceBase intf = new InterfaceAgglomerations(conn, trans, conv);
                        intf.Logger = currentLogger;
                        intf.exportXML(data.UWWTD_Report[iReport], repPeriod);

                        backgroundWorker.ReportProgress(30);
                        // BigCities
                        intf = new InterfaceBigCities(conn, trans, conv);
                        intf.Logger = currentLogger;
                        intf.exportXML(data.UWWTD_Report[iReport], repPeriod);

                        backgroundWorker.ReportProgress(40);
                        // DischargePoints
                        intf = new InterfaceDischargePoint(conn, trans, conv);
                        intf.Logger = currentLogger;
                        intf.exportXML(data.UWWTD_Report[iReport], repPeriod);

                        backgroundWorker.ReportProgress(50);
                        // Industries
                        intf = new InterfaceIndustries(conn, trans, conv, reporter);
                        intf.Logger = currentLogger;
                        intf.exportXML(data.UWWTD_Report[iReport], repPeriod);

                        backgroundWorker.ReportProgress(55);
                        // MSLevel
                        intf = new InterfaceMemberState(conn, trans, conv, reporter);
                        intf.Logger = currentLogger;
                        intf.exportXML(data.UWWTD_Report[iReport], repPeriod);

                        backgroundWorker.ReportProgress(60);
                        // ReceivingAreas
                        intf = new InterfaceReceivingAreas(conn, trans, conv);
                        intf.Logger = currentLogger;
                        intf.exportXML(data.UWWTD_Report[iReport], repPeriod);

                        backgroundWorker.ReportProgress(70);
                        // Uwwtp_Agglo
                        intf = new InterfaceUwwAgglo(conn, trans, conv);
                        intf.Logger = currentLogger;
                        intf.exportXML(data.UWWTD_Report[iReport], repPeriod);

                        backgroundWorker.ReportProgress(80);
                        // UWWTP
                        intf = new InterfaceUWWTP(conn, trans, conv);
                        intf.Logger = currentLogger;
                        intf.exportXML(data.UWWTD_Report[iReport], repPeriod);

                    }
                    finally
                    {
                        conn.Close();
                    }
                }

                backgroundWorker.ReportProgress(90);
                XmlSerializer ser = new XmlSerializer(typeof(UWWTD_Data));
                System.IO.FileStream strm = new FileStream(currentOutput, FileMode.Create);
                try
                {
                    ser.Serialize(strm, data);
                }
                finally
                {
                    strm.Close();
                }
            }
            finally
            {
                currentLogger.Dispose();
            }
        }
    }
}
