﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Xml.Serialization;
using System.Xml.Schema;
using System.Xml;
using System.IO;
using System.Windows.Forms;
using System.Collections;
using at.uba.uwwtd2008.DataAccess;
using at.uba.uwwtd2008.xml;
using at.uba.uwwtd2008.Properties;
using at.uba.uwwtd2008.Items;

namespace at.uba.uwwtd2008.Dialogs
{
    public partial class XMLImportDialog : Form
    {
        DataAccessFactory dataFactory = DataAccessFactory.Instance;
        string currentInput = null;
        Settings settings = Settings.Default;
        string schema = "";
        //bool schemaErrors = false;
        ArrayList schemaErrorList = new ArrayList();
        LoggingClass currentLogger;
        

        public XMLImportDialog()
        {
            InitializeComponent();
            if ((settings.Last_Import_Dir != null) && (settings.LastXMLExportFile.Length > 0))
                currentInput = settings.LastXMLExportFile;
            else
                currentInput = "";
            textBox1.Text = currentInput;

            // read schema
            schema = ".\\UWWTD2009.xsd";
            if (schema.StartsWith(".\\"))
            {
                schema = Directory.GetParent(Application.ExecutablePath) + schema.Substring(1);
            }
        }

        private void SchemaValidationEvents(object target, ValidationEventArgs args)
        {
            //schemaErrors = true;
            LoggingLevel level = LoggingLevel.INFO;
            switch (args.Severity) {
                case XmlSeverityType.Error:
                    level = LoggingLevel.ERROR;
                    break;
                case XmlSeverityType.Warning:
                    level = LoggingLevel.WARNING;
                    break;
                default:
                    level = LoggingLevel.INFO;
                    break;
            }

            currentLogger.log(level, "XMLSchema", args.Message);
        }

        private void OnBrowseClick(object sender, EventArgs e)
        {
            dlgImportFile.FileName = currentInput;
            if (dlgImportFile.ShowDialog(this) == DialogResult.OK)
            {
                textBox1.Text = dlgImportFile.FileName;
                settings.LastXMLExportFile = textBox1.Text;
                settings.Save();
            }
        }

        private void OnImportFileChanged(object sender, EventArgs e)
        {
            currentInput = ((TextBox)sender).Text;
        }

        private void OnClose(object sender, EventArgs e)
        {
            this.Close();

        }

        private void OnImportStart(object sender, EventArgs e)
        {
            if (!File.Exists(currentInput))
            {
                MessageBox.Show("The specified File does not exist. Please select a valid Import File", "Import XML", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                return;
            }

            currentLogger = new LoggingClass();
            cmdClose.Enabled = false;
            cmdImport.Enabled = false;
            cmdImportLog.Enabled = false;
            progressImport.Value = 0;
            backgroundWorker.RunWorkerAsync(null);
        }

        private void OnProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            progressImport.Value = e.ProgressPercentage;
        }

        private void OnImportComplete(object sender, RunWorkerCompletedEventArgs e)
        {
            progressImport.Value = progressImport.Maximum;
            cmdClose.Enabled = true;
            cmdImport.Enabled = true;
            cmdImportLog.Enabled = true;

            ImportExportSummary dlg = new ImportExportSummary();
            dlg.Logger = currentLogger;
            dlg.ShowDialog();

        }


        private void DoImporting(object sender, DoWorkEventArgs e)
        {
            try
            {
                //schemaErrors = false;
                schemaErrorList.Clear();
                currentLogger.Init();
                try
                {
                    currentLogger.log(LoggingLevel.INFO, "XMLImport", "===================================================================");
                    currentLogger.log(LoggingLevel.INFO, "XMLImport", "Importing File : " + currentInput);
                    currentLogger.log(LoggingLevel.INFO, "XMLImport", "===================================================================");
                    // Create the XmlSchemaSet class.
                    XmlSchemaSet sc = new XmlSchemaSet();

                    // Add the schema to the collection.
                    sc.Add("", schema);
                    backgroundWorker.ReportProgress(5);
                    // Set the validation settings.
                    XmlReaderSettings settings = new XmlReaderSettings();
                    settings.ValidationType = ValidationType.Schema;
                    settings.Schemas = sc;
                    settings.ValidationEventHandler += new ValidationEventHandler(SchemaValidationEvents);

                    UWWTD_Data data = null;

                    using (System.IO.FileStream strm = new FileStream(currentInput, FileMode.Open)) {
                        XmlSerializer ser = new XmlSerializer(typeof(UWWTD_Data));
                        XmlReader rdr = XmlReader.Create(strm, settings);

                        data = (UWWTD_Data)ser.Deserialize(rdr);
                    }

                    //if (schemaErrors)
                    //{
                    //    return;
                    //}

                    backgroundWorker.ReportProgress(20);
                    if (data.UWWTD_Report.Length < 1) return;
                    int step = 80 / data.UWWTD_Report.Length;
                    int i = 0;
                    foreach (UWWTD_DataUWWTD_Report report in data.UWWTD_Report)
                    {
                        int curProgress = 20 + (step * i);
                        backgroundWorker.ReportProgress(curProgress);
                        importReport(report, step, curProgress);

                        i++;
                    }
                }
                catch (Exception ex)
                {
                    currentLogger.log(LoggingLevel.ERROR, "XMLImport", "Fatal Error: " + ex.Message);
                    currentLogger.log(LoggingLevel.ERROR, "XMLImport", ex.StackTrace);
                }
                finally
                {
                    currentLogger.Dispose();
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(null, ex.Message, "Exception", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            //backgroundWorker.ReportProgress(90);
            //XmlSerializer ser = new XmlSerializer(typeof(UWWTD_Data));
            //System.IO.FileStream strm = new FileStream(currentInput, FileMode.Create);
            //ser.Serialize(strm, data);
        }

        private void importReport(UWWTD_DataUWWTD_Report report, int step, int curProgress)
        {
            bool enforceConstraints = false;
            IDbConnection conn = dataFactory.GetConnection();
            try
            {
                bool hasError = false;
                string rptMemberState = "";
                IDbTransaction trans = null;//conn.BeginTransaction();
                try
                {
                    DataSet ds = dataFactory.getReporterDataSet();
                    enforceConstraints = ds.EnforceConstraints;
                    ds.EnforceConstraints = false;

                    DataRow reporterRow = null;
                    DataRow reportperiodRow = null;
                    DataRow contactRow = null;

                    currentLogger.log(LoggingLevel.INFO, "XMLImport", "========================================================================");
                    currentLogger.log(LoggingLevel.INFO, "XMLImport", "      Importing Reporter ");
                    currentLogger.log(LoggingLevel.INFO, "XMLImport", "========================================================================");

                    // Reporter ...
                    IDbDataAdapter reporterTAdapter = dataFactory.CreateFullDataAdapter(dataFactory.CreateCommand("Select * from T_Reporter", conn, trans));
                    dataFactory.FillDataAdapter(reporterTAdapter, ds, "T_Reporter"); // in case - anything could have been changed in the meantime

                    DataTable reporter = ds.Tables["T_Reporter"];
                    //reporter.Select("rptMState = ''");
                    DataRow state = dataFactory.getLovRowByKey("MS", report.Reporter.rptMStateKey);
                    if (state["lovKey"] is DBNull)
                    {
                        hasError = true;
                        currentLogger.log(LoggingLevel.ERROR, "Reporter", "The Memberstate-Key in the reporter ('" + report.Reporter.rptMStateKey + "')is not present or invalid");
                    }
                    else
                    {
                        rptMemberState = state["lovKey"] as string;
                    }

                    if (reporter.Select("rptMState = " + state["lovID"]).Length > 0)
                    {
                        reporterRow = reporter.Select("rptMState = " + state["lovID"])[0];
                        currentLogger.log(LoggingLevel.INFO, "XMLImport", "Reporter " + report.Reporter.rptMStateKey + " existed - using existing one");
                    }
                    else
                    {
                        reporterRow = reporter.NewRow();
                        reporterRow["rptMState"] = state["lovID"];
                        reporterRow["rptMStateKey"] = report.Reporter.rptMStateKey;
                        reporterRow["rptMStateWert"] = report.Reporter.rptMStateValue;
                        reporterRow["rptCulture"] = report.Reporter.rptCulture;
                        reporterRow["rptFormRA"] = report.Reporter.rptFormRA;
                        reporter.Rows.Add(reporterRow);
                        dataFactory.UpdateDataAdapter(reporterTAdapter, ds, "T_Reporter");
                        dataFactory.FillDataAdapter(reporterTAdapter, ds, "T_Reporter"); // refresh the internal state (ids)
                        reporterRow = reporter.Select("rptMState = " + state["lovID"])[0];
                        currentLogger.log(LoggingLevel.INFO, "XMLImport", "New Reporter " + report.Reporter.rptMStateKey + " created");
                    }
                    if (hasError)
                        currentLogger.reportFail("Reporter");
                    else
                        currentLogger.reportSuccess("Reporter");

                    backgroundWorker.ReportProgress(curProgress + (step * 10 / 100));
                    DataFormatConverter conv = new DataFormatConverter(report.Reporter.rptCulture);

                    currentLogger.log(LoggingLevel.INFO, "XMLImport", "========================================================================");
                    currentLogger.log(LoggingLevel.INFO, "XMLImport", "      Importing ReportPeriod ");
                    currentLogger.log(LoggingLevel.INFO, "XMLImport", "========================================================================");
                    hasError = false;

                    IDbDataAdapter repperiodTAdapter = dataFactory.CreateFullDataAdapter(dataFactory.CreateCommand("select * from T_ReportPeriod", conn, trans));
                    dataFactory.FillDataAdapter(repperiodTAdapter, ds, "T_ReportPeriod"); // in case - anything could have been changed in the meantime

                    if ((report.ReportPeriod.repCode == null) || (report.ReportPeriod.repCode.Length<1))
                    {
                        hasError = true;
                        currentLogger.log(LoggingLevel.ERROR, "ReportPeriod", "repCode is not present");
                    }
                    else if (!report.ReportPeriod.repCode.StartsWith(rptMemberState))
                    {
                        hasError = true;
                        currentLogger.log(LoggingLevel.ERROR, "ReportPeriod", "repCode (" + report.ReportPeriod.repCode + ") does not start with " + rptMemberState);
                    }

                    DataTable reportperiod = ds.Tables["T_ReportPeriod"];
                    if (reportperiod.Select("rptID = " + reporterRow["rptID"] + " and repCode = '" + report.ReportPeriod.repCode + "'").Length > 0)
                    {
                        reportperiodRow = reportperiod.Select("rptID = " + reporterRow["rptID"] + " and repCode = '" + report.ReportPeriod.repCode + "'")[0];
                        currentLogger.log(LoggingLevel.INFO, "XMLImport", "ReportPeriod " + report.ReportPeriod.repCode + " existed - using existing one");
                    }
                    else
                    {
                        reportperiodRow = reportperiod.NewRow();
                        reportperiodRow["rptID"] = reporterRow["rptID"];
                        reportperiodRow["repCode"] = report.ReportPeriod.repCode;
                        if (report.ReportPeriod.repVersionSpecified)
                            reportperiodRow["repVersion"] = report.ReportPeriod.repVersion;
                        else
                            reportperiodRow["repVersion"] = DBNull.Value;
                        reportperiodRow["repSituationAt"] = report.ReportPeriod.repSituationAt;
                        reportperiodRow["repReportedPeriod"] = report.ReportPeriod.repReportedPeriod;
                        reportperiodRow["repRATypes"] = DataAccessFactory.oldRATypToNew((int) report.Reporter.rptFormRA);

                        reportperiod.Rows.Add(reportperiodRow);
                        dataFactory.UpdateDataAdapter(repperiodTAdapter, ds, "T_ReportPeriod");
                        dataFactory.FillDataAdapter(repperiodTAdapter, ds, "T_ReportPeriod");
                        reportperiodRow = reportperiod.Select("rptID = " + reporterRow["rptID"] + " and repCode = '" + report.ReportPeriod.repCode + "'")[0]; // refresh reportperiod
                        currentLogger.log(LoggingLevel.INFO, "XMLImport", "ReportPeriod " + report.ReportPeriod.repCode + " imported");
                        currentLogger.addReportPeriod((int)reportperiodRow["repID"]);
                    }

                    //if (hasError)
                    //    currentLogger.reportFail("ReportPeriod");
                    //else
                        currentLogger.reportSuccess("ReportPeriod");

                    IDbCommand cmdRemoveAllPrevAgglos = dataFactory.CreateCommand("DELETE FROM T_Agglomerations WHERE repID = " + reportperiodRow["repID"], conn, trans);
                    cmdRemoveAllPrevAgglos.ExecuteNonQuery();

                    backgroundWorker.ReportProgress(curProgress + (step * 20 / 100));
                    currentLogger.log(LoggingLevel.INFO, "XMLImport", "========================================================================");
                    currentLogger.log(LoggingLevel.INFO, "XMLImport", "      Importing Contacts ");
                    currentLogger.log(LoggingLevel.INFO, "XMLImport", "========================================================================");
                    hasError = false;
                    // contacts
                    IDbDataAdapter contactTAdapter = dataFactory.CreateFullDataAdapter(dataFactory.CreateCommand("select * from T_Contact", conn, trans));
                    dataFactory.FillDataAdapter(contactTAdapter, ds, "T_Contact"); // in case - anything could have been changed in the meantime
                    DataTable contact = ds.Tables["T_Contact"];

                    // delete all current Contacts
                    foreach (DataRow row in contact.Select("repID = " + reportperiodRow["repID"]))
                    {
                        row.Delete();
                    }
                    dataFactory.UpdateDataAdapter(contactTAdapter, ds, "T_Contact");
                    foreach (UWWTD_DataUWWTD_ReportContact xmlContact in report.Contacts)
                    {
                        hasError = false;
                        contactRow = contact.NewRow();
                        try
                        {
                            if ((xmlContact.conName == null) || (xmlContact.conName.Length < 1))
                            {
                                hasError = true;
                                currentLogger.log(LoggingLevel.ERROR, "Contact", "Name was not provided");
                            }
                            if ((xmlContact.conInstitution == null) || (xmlContact.conInstitution.Length < 1))
                            {
                                hasError = true;
                                currentLogger.log(LoggingLevel.ERROR, "Contact", "Institution was not provided");
                            }
                            if ((xmlContact.conPhone == null) || (xmlContact.conPhone.Length < 1))
                            {
                                hasError = true;
                                currentLogger.log(LoggingLevel.ERROR, "Contact", "Phone is Master Data");
                            }
                            contactRow["repID"] = reportperiodRow["repID"];
                            contactRow["conCode"] = DBNull.Value;
                            contactRow["conName"] = xmlContact.conName;
                            contactRow["conInstitution"] = xmlContact.conInstitution;
                            contactRow["conStreet"] = xmlContact.conStreet;
                            contactRow["conZIP"] = xmlContact.conZIP;
                            contactRow["conCity"] = xmlContact.conCity;
                            contactRow["conPhone"] = xmlContact.conPhone;
                            contactRow["conFax"] = xmlContact.conFax;
                            contactRow["conEmail"] = xmlContact.conEmail;
                            contactRow["conRemarks"] = xmlContact.conRemarks;
                            contact.Rows.Add(contactRow);
                            dataFactory.UpdateDataAdapter(contactTAdapter, ds, "T_Contact");
                            //if (hasError)
                            //    currentLogger.reportFail("Contact");
                            //else
                            currentLogger.reportSuccess("Contact");
                        }
                        catch (Exception ex)
                        {
                            currentLogger.log(LoggingLevel.ERROR, "Contact", "Failed to import Contact " + xmlContact.conName + " - " + ex.Message);
                            currentLogger.reportFail("Contact");
                        }

                    }
                    dataFactory.FillDataAdapter(contactTAdapter, ds, "T_Contact");
                    //currentLogger.log(LoggingLevel.INFO, "Contact", "Imported " + count + " Contacts");

                    backgroundWorker.ReportProgress(curProgress + (step * 30 / 100));
                    InterfaceBase intf = null;
                    IDbCommand cmd = dataFactory.CreateCommand("DELETE from T_ReceivingAreas WHERE repID = " + reportperiodRow["repID"], conn, trans);
                    cmd.ExecuteNonQuery();
                    cmd = dataFactory.CreateCommand("DELETE from T_Agglomerations WHERE repID = " + reportperiodRow["repID"], conn, trans);
                    cmd.ExecuteNonQuery();
                    cmd = dataFactory.CreateCommand("DELETE from T_UWWTPS WHERE repID = " + reportperiodRow["repID"], conn, trans);
                    cmd.ExecuteNonQuery();
                    cmd = dataFactory.CreateCommand("DELETE from T_UWWTP_AGGLO WHERE repID = " + reportperiodRow["repID"], conn, trans);
                    cmd.ExecuteNonQuery();
                    cmd = dataFactory.CreateCommand("DELETE from T_DischargePoints WHERE repID = " + reportperiodRow["repID"], conn, trans);
                    cmd.ExecuteNonQuery();
                    cmd = dataFactory.CreateCommand("DELETE from T_MSLevel WHERE repID = " + reportperiodRow["repID"], conn, trans);
                    cmd.ExecuteNonQuery();
                    cmd = dataFactory.CreateCommand("DELETE from T_Industries WHERE repID = " + reportperiodRow["repID"], conn, trans);
                    cmd.ExecuteNonQuery();

                    backgroundWorker.ReportProgress(curProgress + (step * 35 / 100));
                    // ReceivingAreas
                    intf = new InterfaceReceivingAreas(conn, trans, conv);
                    intf.Logger = currentLogger;
                    intf.importXML(report.ReceivingAreas, reportperiodRow);
                    backgroundWorker.ReportProgress(curProgress + (step * 40 / 100));

                    // BigCities
                    intf = new InterfaceBigCities(conn, trans, conv);
                    intf.Logger = currentLogger;
                    intf.importXML(report.BigCityDischargers, reportperiodRow);
                    backgroundWorker.ReportProgress(curProgress + (step * 50 / 100));

                    // Agglomerations
                    intf = new InterfaceAgglomerations(conn, trans, conv);
                    intf.Logger = currentLogger;
                    intf.importXML(report.Agglomerations, reportperiodRow);
                    backgroundWorker.ReportProgress(curProgress + (step * 60 / 100));
                    // UWWTPs
                    intf = new InterfaceUWWTP(conn, trans, conv);
                    intf.Logger = currentLogger;
                    intf.importXML(report.UWWTPs, reportperiodRow);
                    backgroundWorker.ReportProgress(curProgress + (step * 70 / 100));
                    // UWWTP_AGGLO
                    intf = new InterfaceUwwAgglo(conn, trans, conv);
                    intf.Logger = currentLogger;
                    intf.importXML(report.UwwtpAgglos, reportperiodRow);
                    backgroundWorker.ReportProgress(curProgress + (step * 80 / 100));
                    // Dischargepoints
                    intf = new InterfaceDischargePoint(conn, trans, conv);
                    intf.Logger = currentLogger;
                    intf.importXML(report.DischargePoints, reportperiodRow);
                    backgroundWorker.ReportProgress(curProgress + (step * 85 / 100));
                    // MSLevel
                    intf = new InterfaceMemberState(conn, trans, conv, reporterRow);
                    intf.Logger = currentLogger;
                    intf.importXML(report.MSLevel, reportperiodRow);
                    backgroundWorker.ReportProgress(curProgress + (step * 90 / 100));
                    // Industries
                    intf = new InterfaceIndustries(conn, trans, conv, reporterRow);
                    intf.Logger = currentLogger;
                    intf.importXML(report.Industries, reportperiodRow);
                    backgroundWorker.ReportProgress(curProgress + (step * 95 / 100));

                    if (trans != null) trans.Commit();
                    trans = null;
                    dataFactory.initReporters();
                }
                finally
                {
                    if (trans != null)
                        trans.Rollback();
                    trans = null;
                    dataFactory.initReporters();
                }
            }
            catch (Exception ex)
            {
                currentLogger.log(LoggingLevel.ERROR, "XMLImport", "Fatal Error while importing: " + ex.Message + "\n" + ex.StackTrace);
            }
            finally
            {
                conn.Close();
            }
        }
    }
}
