﻿using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Windows.Forms;
using at.uba.uwwtd2008.Properties;

namespace at.uba.uwwtd2008.Items
{
    public enum LoggingLevel { ERROR, WARNING, INFO, DEBUG };

    public class StatisticCounter
    {
        private long failedCount;
        private long successCount;

        public StatisticCounter()
        {
            failedCount = 0;
            successCount = 0;
        }

        public long FailedCount
        {
            get { return failedCount; }
        }

        public long SuccessCount
        {
            get { return successCount; }
        }

        public void SetFailedCount(long val)
        {
            failedCount = val;
        }

        public void reportFail()
        {
            failedCount++;
        }

        public void reportSuccess()
        {
            successCount++;
        }
    }

    public class LoggingClass : IDisposable
    {
        Dictionary<string, StatisticCounter> statistics = new Dictionary<string, StatisticCounter>();

        private System.IO.FileStream logFile;
        private string logFileName;
        private System.Collections.ArrayList lstRepCodes = new System.Collections.ArrayList();

        public StatisticCounter getCounter(string key)
        {
            if (statistics.ContainsKey(key))
                return statistics[key];

            StatisticCounter counter = new StatisticCounter();
            statistics[key] = counter;
            return counter;
        }

        public void reportFail(string iface)
        {
            getCounter(iface).reportFail();
        }

        public void reportSuccess(string iface)
        {
            getCounter(iface).reportSuccess();
        }

        public string LogFileName
        {
            get { return logFileName; }
        }

        public void Init()
        {
            string logFile = Settings.Default.LogDir;
            if (logFile.StartsWith(".\\")) logFile = Directory.GetParent(Application.ExecutablePath) + logFile.Substring(1);
            logFile = logFile + "\\log_" + DateTime.Now.ToString("yyyyMMddHHmmss") + ".log";
            InitLogging(logFile);
            lstRepCodes.Clear();
            statistics.Clear();
        }

        public void InitLogging(string file)
        {
            lock (this)
            {
                logFileName = file;
                logFile = new System.IO.FileStream(file, System.IO.FileMode.Create);
            }
        }

        public void log(LoggingLevel level, string part, string message)
        {
            log(level.ToString(), part, message);
        }

        public void log(string level, string part, string message)
        {
            StringBuilder builder = new StringBuilder();
            builder.Append(DateTime.Now.ToString("dd-MM-yyyy HH:mm:ss"));
            builder.Append("  ");
            if (level == null) level = "INFO";
            if (level.Length > 7) level = level.Substring(0, 7);
            builder.Append("[");
            builder.Append(level);
            builder.Append("]");
            for (int i = level.Length; i < 9; i++) builder.Append(' ');
            if (part == null) part = "unknown";
            if (part.Length > 10) part = part.Substring(0, 10);
            builder.Append("(");
            builder.Append(part);
            builder.Append(")");
            for (int i = part.Length; i < 12; i++) builder.Append(' ');
            builder.Append(message);
            builder.Append("\r\n");
            lock (this)
            {
                if (logFile != null)
                    logFile.Write(Encoding.UTF8.GetBytes(builder.ToString()), 0, Encoding.UTF8.GetByteCount(builder.ToString()));
                else
                    Console.Out.Write(builder.ToString());
            }

        }

        public void addReportPeriod(long reportPeriod) {
            lstRepCodes.Add(reportPeriod);
        }

        public System.Collections.IList getReportPeriods() {
            return lstRepCodes;
        }

        public void Dispose() {
            try
            {
                if (logFile != null)
                {
                    lock (this)
                    {
                        logFile.Close();
                        logFile = null;
                    }
                }
            }
            catch (Exception) { }
        }

    }
}
