using System;
using System.Collections.Generic;
using System.Windows.Forms;
using System.Data;
using at.uba.uwwtd2008.DataAccess;
using at.uba.uwwtd2008.Properties;
using System.Configuration;
using System.Runtime.InteropServices;

namespace at.uba.uwwtd2008
{
    public static class UWWTD2008
    {
        public static System.Drawing.Color ColNorm = System.Drawing.SystemColors.Window;
        public static System.Drawing.Color ColError = System.Drawing.Color.Red;
        public static System.Drawing.Color ColMandatory = System.Drawing.Color.Yellow;

        public static DateTime MIN_VALID_DATE = new DateTime(1991, 1, 1);
        public static DateTime MIN_VALID_DATE_REP = new DateTime(2006, 12, 31);
        public static DateTime MAX_VALID_DATE = new DateTime(2020, 12, 31);

        [DllImport("oleaut32.dll")]
        public static extern int SetVarConversionLocaleSetting(ulong flags);

        public static DataTable iniDataTable = null;
        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main()
        {
            Form mainForm = null;
            try
            {
                // Correct OleDB behavior
                SetVarConversionLocaleSetting(1);
                Application.EnableVisualStyles();
                Application.SetCompatibleTextRenderingDefault(false);
                InitializeDBConnection();
                mainForm = new UWWTD2008MainForm();
                Application.Run(mainForm);
            }
            catch (Exception e)
            {
                FatalErrorDialog dlg = new FatalErrorDialog();
                dlg.setErrorMessage(e.ToString());
                dlg.ShowDialog();
                //MessageBox.Show("Fatal Error! Terminating Application\n\nDetails:\n" + e.ToString(), "Fatal Error", MessageBoxButtons.OK, MessageBoxIcon.Error);

            }
        }

        private static void InitializeDBConnection() {
            DataAccessFactory factory = DataAccessFactory.Instance;
            IDbConnection conn = factory.GetConnection();
            try
            {
                long lastReportPeriod = Settings.Default.LastReportPeriod;
                if (lastReportPeriod < 0)
                {
                    factory.setCurrentReportPeriod(null);
                    return;
                } 
                    try
                    {
                        uwwtd2008.components.UWWTDBaseCtrl.CurrentRep = at.uba.uwwtd2008.Items.ReportPeriod.LoadReportPeriod(lastReportPeriod);
                    }
                    catch (ArgumentOutOfRangeException)
                    {
                        Settings.Default.LastReportPeriod = -1;
                    }
            }
            finally
            {
                conn.Close();
            }
            
            //string strConnection = System.Configuration.ConfigurationSettings.AppSettings.Get("Uwwtd2009ConnectionString");
        }
    }
}