﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

namespace at.uba.uwwtd2008.components
{
    public partial class CtrlAgglomerationsEdit : UWWTDBaseCtrl
    {
        public const String SelectString = "SELECT * FROM T_AGGLOMERATIONS WHERE  repID = ";
        public const String TNAME_AGGLO = "agglo";
        private DataSet dataSet;
        private DataRowView currentRow = null;
        private DataTable bigCities;
        private EventHandler currentRowEventHandler = null;


        public CtrlAgglomerationsEdit()
        {
            currentRowEventHandler = new EventHandler(CurrentRowChanged);
            InitializeComponent();
        }

        public long CurrentAggID {
            get { if (currentRow == null) return -1;
                return (long.Parse(currentRow["aggID"].ToString())); }
        }

        public int CurrentAggloBindingContextPosition
        {
            get { return BindingContext[dataSet, TNAME_AGGLO].Position; }
        }

        public DataSet CurrentDataSet {
            get { return dataSet; }
        }

        public override void OnBackButtonClick()
        {
            if (currentRow != null) currentRow.EndEdit();
            if (dataSet.HasChanges())
            {
                if (!checkExitWithoutSave("Agglomerations"))
                {
                    if (currentRow != null) currentRow.BeginEdit();
                    return;
                }
                dataSet.RejectChanges();
            }

            UWWTD2008MainForm mf = UWWTD2008MainForm.getMainForm(this);
            if (mf == null) return;

            BindingContext[dataSet, TNAME_AGGLO].CurrentChanged -= currentRowEventHandler;

            if (PrevControl != null) mf.getSplitterPanel().setContentPane(PrevControl);
            else mf.OpenDataEditMain();
        }

        private void ConnectUWWTD_Click(object sender, EventArgs e)
        {
            ((UWWTD2008MainForm)(this.Parent.Parent.Parent.Parent)).OpenUWWTDEdit(this);
        }

        private void EditBigCity(object sender, EventArgs e)
        {
            ((UWWTD2008MainForm)(this.Parent.Parent.Parent.Parent)).OpenEditBigCity(this);
        }

        private void OnCollectionEdit(object sender, LinkLabelLinkClickedEventArgs e)
        {
            ((UWWTD2008MainForm)(this.Parent.Parent.Parent.Parent)).OpenEditCollection(this);
        }

        public override void OpenForm() {
            //DataSet dataSet = new DataSet();
            //PerformDataBinding(dataSet);
            if ((dataSet == null) || (CurrentRep == null))
            {
                MessageBox.Show(UWWTD2008MainForm.getMainForm(this), "Please select a valid ReportPeriod first", "No ReportPeriod selected", MessageBoxButtons.OK, MessageBoxIcon.Error);
                UWWTD2008MainForm.getMainForm(this).OpenDataEditMain();
                return;
            }

            BindingContext[dataSet, TNAME_AGGLO].CurrentChanged += currentRowEventHandler;
            BindingContext[dataSet].Position = BindingContext[dataSet].Position;

            if ((currentRow != null) &&
                (BindingContext[dataSet, TNAME_AGGLO].Position >= 0) &&
                (BindingContext[dataSet, TNAME_AGGLO].Current is DataRowView) &&
                (((DataRowView)BindingContext[dataSet, TNAME_AGGLO].Current).Row.Equals(currentRow)))
            {
                // There is already a row that is open - we simply reuse it
                checkBigCityYN(currentRow);
            }
            else
            {
                CurrentRowChanged(this, null);
            }
            updateComponentView();
        }

        public void UpdateUwwAgglo(DataSet dataSet, IDbConnection conn)
        {
            IDbCommand cmdUWWAGGLO = mDataFactory.CreateCommand("Select t2.uwwCode, t2.uwwName, t1.aggID FROM T_UWWTP_AGGLO t1 INNER JOIN T_UWWTPS t2 ON t1.uwwID = t2.uwwID WHERE t1.repID = " + CurrentRep.RepID + " AND t2.repID = " + CurrentRep.RepID, conn);
            IDbDataAdapter adapter = mDataFactory.CreateDataAdapter(cmdUWWAGGLO);
            mDataFactory.FillDataAdapter(adapter, dataSet, "uwwagglo");
        }

        public override void PerformDataBinding(DataSet dataSet) {
            bool hasNewItems = false;
            IDbConnection conn = mDataFactory.GetConnection();
            try
            {
                this.dataSet = dataSet;

                if (!dataSet.Tables.Contains(TNAME_AGGLO))
                {
                    IDbCommand cmd = mDataFactory.CreateCommand(SelectString + CurrentRep.RepID, conn);
                    mDataFactory.FillDataAdapter(mDataFactory.CreateDataAdapter(cmd), dataSet, TNAME_AGGLO);
                    dataSet.Tables[TNAME_AGGLO].Columns["aggID"].AutoIncrement = true;
                    //dataSet.Tables[TNAME_AGGLO].Columns["aggLongitude"]
                    hasNewItems = true;
                }
                if (!dataSet.Tables.Contains("uwwagglo"))
                {
                    UpdateUwwAgglo(dataSet, conn);
                    hasNewItems = true;
                }
                if (!dataSet.Relations.Contains("UWWTDAgglos"))
                {
                    DataColumn dc1, dc2;
                    DataRelation rel;
                    dc1 = dataSet.Tables["agglo"].Columns["aggID"];
                    dc2 = dataSet.Tables["uwwagglo"].Columns["aggID"];
                    rel = new DataRelation("UWWTDAgglos", dc1, dc2);
                    dataSet.Relations.Add(rel);
                    hasNewItems = true;
                }

                bigCities = dataSet.Tables[CtrlBigCityEdit.TNAME_BIGCITY];

            }
            finally
            {
                conn.Close();
            }

            if (hasNewItems)
            {
                currentRow = null;

                mcomboAggloRegion.DataSource = mDataFactory.getNUTSTableForCurrentReporter();
                mcomboAggloRegion.DisplayMember = "nutRegion";
                mcomboAggloRegion.ValueMember = "nutID";

                gridAgglos.AutoGenerateColumns = false;
                gridAgglos.DataSource = dataSet;
                gridAgglos.DataMember = TNAME_AGGLO;
//                gridAgglos.CurrentCellChanged += new EventHandler(CurrentAggloChanged);

                mgridAggloUWWTD.AutoGenerateColumns = false;
                mgridAggloUWWTD.DataSource = dataSet; // TODO HPW: Find a way to populate this list correctly with a subset of the dataTable
                mgridAggloUWWTD.DataMember = TNAME_AGGLO + ".UWWTDAgglos";

                ((DataGridViewComboBoxColumn)gridAgglos.Columns[3]).DataSource = bigCities;
                ((DataGridViewComboBoxColumn)gridAgglos.Columns[3]).DisplayMember = "bigCity";
                ((DataGridViewComboBoxColumn)gridAgglos.Columns[3]).ValueMember = "bigID";

                DataGridViewComboBoxColumn statusCol = (DataGridViewComboBoxColumn)gridAgglos.Columns["status"];
                statusCol.DataSource = mDataFactory.getLovViewByKey("STAT");
                statusCol.DisplayMember = "lovWert";
                statusCol.ValueMember = "lovID";

                mtxtAggloID.DataBindings.Clear();
                mtxtAggloID.DataBindings.Add("Text", dataSet, "agglo.aggCode");
                mtxtAggloName.DataBindings.Clear();
                mtxtAggloName.DataBindings.Add("Text", dataSet, "agglo.aggName");
                mtxtAggloLoad.DataBindings.Clear();
                mtxtAggloLoad.DataBindings.Add("Text", dataSet, "agglo.aggGenerated");
                mtxtAggloLongitude.DataBindings.Clear();
                Binding tmp = new Binding("Text", dataSet, "agglo.aggLongitude", true, DataSourceUpdateMode.OnValidation, "");
                tmp.Format += new ConvertEventHandler(formatStringDecimal);
                tmp.Parse += new ConvertEventHandler(parseStringDecimal);
                mtxtAggloLongitude.DataBindings.Add(tmp);

                mtxtAggloLatitude.DataBindings.Clear();
                tmp = new Binding("Text", dataSet, "agglo.aggLatitude", true, DataSourceUpdateMode.OnValidation, "");
                tmp.Format += new ConvertEventHandler(formatStringDecimal);
                tmp.Parse += new ConvertEventHandler(parseStringDecimal);
                mtxtAggloLatitude.DataBindings.Add(tmp);

                mtxtAggloCalc.DataBindings.Clear();
                mtxtAggloCalc.DataBindings.Add("Text", dataSet, "agglo.aggCalculation");
                mtxtAggloDate.DataBindings.Clear();
                mtxtAggloDate.DataBindings.Add("Text", dataSet, "agglo.aggPeriodOver", true, DataSourceUpdateMode.OnPropertyChanged, "", "d");

                mcomboAggloRegion.DataBindings.Clear();
                mcomboAggloRegion.DataBindings.Add("SelectedValue", dataSet, "agglo.aggNUTS");

                mcomboAggloChanges.DataBindings.Clear();
                mcomboAggloChanges.DataSource = mDataFactory.getYNLovValues();
                mcomboAggloChanges.DisplayMember = "lovWert";
                mcomboAggloChanges.ValueMember = "lovID";
                mcomboAggloChanges.DataBindings.Add("SelectedValue", dataSet, "agglo.aggChanges");

                mtxtAggloComments.DataBindings.Clear();
                mtxtAggloComments.DataBindings.Add("Text", dataSet, "agglo.aggChangesComment");

                mcomboBoxCity.DataSource = mDataFactory.getYNLovValues();
                mcomboBoxCity.DisplayMember = "lovWert";
                mcomboBoxCity.ValueMember = "lovKey";

                mcomboBoxSelectCity.DataBindings.Clear();
                mcomboBoxSelectCity.DataSource = bigCities;
                mcomboBoxSelectCity.DisplayMember = "bigCity";
                mcomboBoxSelectCity.ValueMember = "bigID";
                mcomboBoxSelectCity.DataBindings.Add("SelectedValue", dataSet, "agglo.bigID");

                comboState.DataSource = mDataFactory.getLovViewByKey("STAT");
                comboState.DisplayMember = "lovWert";
                comboState.ValueMember = "lovID";
                comboState.DataBindings.Clear();
                comboState.DataBindings.Add("SelectedValue", dataSet, "agglo.aggState");

                dataSet.Tables[TNAME_AGGLO].ColumnChanged += new DataColumnChangeEventHandler(CtrlAgglomerationsEdit_ColumnChanged);
            }
        }

        private void InsertAgglo_Click(object sender, EventArgs e)
        {
            if (currentRow != null) currentRow.CancelEdit();
            currentRow = null;
            DataRow row = dataSet.Tables[TNAME_AGGLO].NewRow();
            row["repID"] = CurrentRep.RepID;
            // set state to active
            row["aggState"] = mDataFactory.getLovRowByKey("STAT", "1")["lovID"];
            row["aggCode"] = CurrentRep.Reporter.State.Key;
            // set the default values for the table
            row["aggPressureTest"] = false;
            row["aggVideoInspections"] = false;
            row["aggOtherMeasures"] = false;
            row["aggDilutionRates"] = false;
            row["aggCapacity"] = false;
            row["aggAccOverflows"] = false;
            row["aggChanges"] = mDataFactory.getLovRowByKey("YN", "0")["lovID"];
            dataSet.Tables[TNAME_AGGLO].Rows.Add(row);
            //currentRow = row;
            //currentRow.BeginEdit();
            BindingContext[dataSet, TNAME_AGGLO].Position =
                BindingContext[dataSet, TNAME_AGGLO].Count - 1;
        }

        private void DeleteAgglo_Click(object sender, EventArgs e)
        {
            if (currentRow == null) return;
            if (ShowConfirmDelete())
            {
                currentRow.CancelEdit();

                currentRow.Delete();
                this.OnSave(null, null);
            }
        }

        void CurrentRowChanged(object sender, EventArgs e)
        {
            if (currentRow != null)
            {
                if ((currentRow.Row.RowState != DataRowState.Unchanged) &&
                    (currentRow.Row.RowState != DataRowState.Detached))
                {
                    currentRow.EndEdit();
                    currentRow = null;
                }
            }

            if (BindingContext[dataSet, TNAME_AGGLO].Position >= 0)
            {
                DataRowView view = BindingContext[dataSet, TNAME_AGGLO].Current as DataRowView;
                if ((view != null) && (view.Row.RowState != DataRowState.Deleted))
                {
                    currentRow = view;
                    checkBigCityYN(currentRow);
                    currentRow.BeginEdit();
                }
            }

            updateComponentView();
            string errorText = "";
            ValidateForm(ref errorText, currentRow != null ? currentRow.Row.RowState : DataRowState.Detached);
        }

        private void updateComponentView()
        {
            base.updateComponentView(BindingContext[dataSet, TNAME_AGGLO]);
            if (currentRow != null)
            {
                cmdDeleteAgglo.Enabled = true;
                mtxtAggloID.ReadOnly = currentRow.Row.RowState != DataRowState.Added;
            }
            else
            {
                // Do not delete the following line - otherwise you'll get an error when cmdDeleteAgglo is disabled
                // and the update was triggered by a delete.
                // This is because the next focused item would be the Agglo-Datagrid that has no more (valid) data
                // to display causing an Exception
                cmdInsertAgglo.Focus();
                cmdDeleteAgglo.Enabled = false;
            }
        }

        void CtrlAgglomerationsEdit_ColumnChanged(object sender, DataColumnChangeEventArgs e) {
            if (e.Column.ColumnName.Equals("bigID"))
            {
                if (currentRow != null)
                    checkBigCityYN(currentRow);
            }
        }

        void checkBigCityYN(DataRowView row) {
            if ((row["bigID"] is DBNull) || (row["bigID"] == null))
            {
                mcomboBoxCity.SelectedValue = 0;
            }
            else
            {
                mcomboBoxCity.SelectedValue = 1;
            }
        }

        //void CurrentAggloChanged(object sender, EventArgs e)
        //{
        //    DataRowView newRow = null;
        //    if (gridAgglos.CurrentRow != null)
        //        newRow = gridAgglos.CurrentRow.DataBoundItem as DataRowView;

        //    // check, if the new row equals the old one - in this case only the cell has changed.
        //    if ((newRow != null) && (currentRow == newRow.Row))
        //    {
        //            return;
        //    }

        //    if (currentRow != null) currentRow.EndEdit();
        //    currentRow = null;

        //    if (newRow != null)
        //    {
        //        currentRow = newRow.Row;
        //        checkBigCityYN();
        //        currentRow.BeginEdit();
        //    }

        //}

        void AggloChanges_SelectedValueChanged(object sender, EventArgs e) {
            DataTable dt = mDataFactory.getYNLovValues();
            if ((mcomboAggloChanges.SelectedValue != null) && mcomboAggloChanges.SelectedValue.Equals(dt.Select("lovKey = 1")[0]["lovID"]))
            {
                mtxtAggloComments.Visible = true;
                labChangesComment.Visible = true;
            }
            else
            {
                mtxtAggloComments.Visible = false;
                labChangesComment.Visible = false;
            }
        }

        void HasBigCity_SelectedValueChanged(object sender, EventArgs e) {
            if (mcomboBoxCity.SelectedValue == null || mcomboBoxCity.SelectedValue is DBNull) return;
            if (mcomboBoxCity.SelectedValue is DataRowView) return; // we do not handle full rows
            if (int.Parse(mcomboBoxCity.SelectedValue.ToString()) == 1)
            {
                mcomboBoxSelectCity.Visible = true;
                labSelectBigCity.Visible = true;
                butEditBigCity.Visible = true;
                mcomboBoxSelectCity.Refresh();
            }
            else
            {
                if ((currentRow != null) && !(currentRow["bigID"] is DBNull))
                {
                    currentRow["bigID"] = DBNull.Value;
                    mcomboBoxSelectCity.SelectedValue = DBNull.Value;
                    gridAgglos.Refresh();
                }
                mcomboBoxSelectCity.Visible = false;
                labSelectBigCity.Visible = false;
                butEditBigCity.Visible = false;
            }
        }

        private void updateEditAggloItems() {
            if (currentRow != null)
            {
                currentRow.BeginEdit();
            }
            else
            {
            }
            mgridAggloUWWTD.Refresh();
        }

        private void OnSave(object sender, LinkLabelLinkClickedEventArgs e) {
            string errorText = "";
            if (!ValidateForm(ref errorText, currentRow != null ? currentRow.Row.RowState : DataRowState.Detached))
            {
                MessageBox.Show(this, "Please correct Form Errors before save:\r\n" + errorText, "Form not valid", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                return;
            }

            if (currentRow != null) currentRow.EndEdit();
            currentRow = null;
            if (dataSet.HasChanges())
            {
                UWWTD2008.SetVarConversionLocaleSetting(1);
                IDbConnection conn = mDataFactory.GetConnection();
                try
                {
                    bool enforceConstraints = dataSet.EnforceConstraints;
                    IDbCommand cmd = mDataFactory.CreateCommand(SelectString + CurrentRep.RepID, conn);
                    IDbDataAdapter adapter = mDataFactory.CreateFullDataAdapter(cmd);
                    dataSet.EnforceConstraints = false;
                    try
                    {
                        string curAggID = null;
                        if (BindingContext[dataSet, TNAME_AGGLO].Position >= 0) 
                            curAggID = ((DataRowView)BindingContext[dataSet, TNAME_AGGLO].Current)["aggCode"] as string;
                        mDataFactory.UpdateDataAdapter(adapter, dataSet, TNAME_AGGLO);
                        mDataFactory.FillDataAdapter(adapter, dataSet, TNAME_AGGLO);
                        if (curAggID != null)
                        {
                            DataRow[] aggWithCodes = dataSet.Tables[TNAME_AGGLO].Select("repID = " + CurrentRep.RepID + " AND aggCode='" + curAggID + "'");
                            int curPos = int.MaxValue;
                            if (aggWithCodes.Length > 0)
                            {
                                curPos = dataSet.Tables[TNAME_AGGLO].Rows.IndexOf(aggWithCodes[0]);
                            }
                            if (BindingContext[dataSet, TNAME_AGGLO].Count > curPos)
                            {
                                BindingContext[dataSet, TNAME_AGGLO].Position = curPos;
                            }
                        }
                    }
                    finally
                    {
                        dataSet.EnforceConstraints = enforceConstraints;
                    }
                }
                finally
                {
                    conn.Close();
                }
            }
            UWWTD2008MainForm mf = UWWTD2008MainForm.getMainForm(this);
            if (mf != null) mf.getSplitterPanel().updateUWWAggloTable(dataSet);

            CurrentRowChanged(this, null);

            if (e != null)
                ShowSaveConfirmation();

        }

        protected override void validateStringDecimal(object sender, CancelEventArgs e)
        {
            base.validateStringDecimal(sender, e);
        }

        protected override bool ValidateForm(ref string errorText, DataRowState state)
        {
            if (BindingContext[dataSet, TNAME_AGGLO].Position < 0) return true;
            DataRowView view = BindingContext[dataSet, TNAME_AGGLO].Current as DataRowView;
            if (view == null) return false;

            string strCode = currentRow["aggCode"] as string;
            if (strCode == null) strCode = "";
            string strName = currentRow["aggName"] as string;
            if (strName == null) strName = "";
            
            bool retVal = true;
            if (!strCode.StartsWith(CurrentRep.Reporter.State.Key))
            {
                if (mtxtAggloID.Enabled)
                    mtxtAggloID.BackColor = state == DataRowState.Added ? UWWTD2008.ColMandatory : UWWTD2008.ColError;
                if (errorText.Length > 1) errorText += "\r\n";
                errorText += "Code for this Agglomeration has to start with " + CurrentRep.Reporter.State.Key;
                retVal = false;
            }
            else
            {
                if ((dataSet.Tables[TNAME_AGGLO].Select("aggCode = '" + strCode + "'").Length > 0) &&
                    (dataSet.Tables[TNAME_AGGLO].Select("aggCode = '" + strCode + "'")[0] != view.Row))
                {
                    if (mtxtAggloID.Enabled)
                        mtxtAggloID.BackColor = UWWTD2008.ColError;
                    if (errorText.Length > 1) errorText += "\r\n";
                    errorText += "Code for Agglomeration already used";
                    retVal = false;
                }
                else
                {
                    if (strCode.Length < 3)
                    {
                        if (mtxtAggloID.Enabled)
                            mtxtAggloID.BackColor = state == DataRowState.Added ? UWWTD2008.ColMandatory : UWWTD2008.ColError;
                        if (errorText.Length > 1) errorText += "\r\n";
                        errorText += "Please specify a valid Agglomeration Code";
                        retVal = false;
                    }
                    else
                        mtxtAggloID.BackColor = UWWTD2008.ColNorm;
                }
            }

            if (strName.Length < 3)
            {
                if (mtxtAggloName.Enabled)
                    mtxtAggloName.BackColor = state == DataRowState.Added ? UWWTD2008.ColMandatory : UWWTD2008.ColError;
                if (errorText.Length > 1) errorText += "\r\n";
                errorText += "Please provide a name for this Agglomeration";
                retVal = false;
            }
            else
                mtxtAggloName.BackColor = UWWTD2008.ColNorm;

            if ((currentRow["aggGenerated"] == null) || (currentRow["aggGenerated"] is DBNull)) {
                if (mtxtAggloLoad.Enabled)
                    mtxtAggloLoad.BackColor = state == DataRowState.Added ? UWWTD2008.ColMandatory : UWWTD2008.ColError;
                if (errorText.Length > 1) errorText += "\r\n";
                errorText += "Generated Load is Master Data";
                retVal = false;
            }
            else
                mtxtAggloLoad.BackColor = UWWTD2008.ColNorm;

            if ((currentRow["aggLongitude"] == null) || (currentRow["aggLongitude"] is DBNull)) {
                if (mtxtAggloLongitude.Enabled)
                    mtxtAggloLongitude.BackColor = state == DataRowState.Added ? UWWTD2008.ColMandatory : UWWTD2008.ColError;
                if (errorText.Length > 1) errorText += "\r\n";
                errorText += "Longitude is Master Data";
                retVal = false;
            }
            else
                mtxtAggloLongitude.BackColor = UWWTD2008.ColNorm;

            if ((currentRow["aggLatitude"] == null) || (currentRow["aggLatitude"] is DBNull)) {
                if (mtxtAggloLatitude.Enabled)
                    mtxtAggloLatitude.BackColor = state == DataRowState.Added ? UWWTD2008.ColMandatory : UWWTD2008.ColError;
                if (errorText.Length > 1) errorText += "\r\n";
                errorText += "Latitude is Master Data";
                retVal = false;
            }
            else
                mtxtAggloLatitude.BackColor = UWWTD2008.ColNorm;

            if (currentRow["aggPeriodOver"] is DateTime)
            {
                DateTime aggPeriodOver = (DateTime)currentRow["aggPeriodOver"];
                if ((aggPeriodOver != null) && (aggPeriodOver.CompareTo(UWWTD2008.MIN_VALID_DATE) < 0))
                {
                    mtxtAggloDate.BackColor = UWWTD2008.ColError;
                    if (errorText.Length > 1) errorText += "\r\n";
                    errorText += "Period Date is too early";
                    retVal = false;
                }
                else if ((aggPeriodOver != null) && (aggPeriodOver.CompareTo(UWWTD2008.MAX_VALID_DATE) > 0))
                {
                    mtxtAggloDate.BackColor = UWWTD2008.ColError;
                    if (errorText.Length > 1) errorText += "\r\n";
                    errorText += "Period Date is too late";
                    retVal = false;
                }
                else
                    mtxtAggloDate.BackColor = UWWTD2008.ColNorm;
            }
            else
            {
                mtxtAggloDate.BackColor = UWWTD2008.ColMandatory;
                if (errorText.Length > 1) errorText += "\r\n";
                errorText += "Date of relevant deadline of UWWTD is Master Data";
                retVal = false;
            }

            return retVal;
        }

    }
}
