﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.IO;
using at.uba.uwwtd2008.Properties;

namespace at.uba.uwwtd2008.components {
    public partial class CtrlArt17Data : UWWTDBaseCtrl {
        string art17FormsDir = Settings.Default.Art17Data;
        string currentDataDir = null;
        public CtrlArt17Data() {
            InitializeComponent();
            if (art17FormsDir.StartsWith(".\\")) art17FormsDir = Directory.GetParent(Application.ExecutablePath) + art17FormsDir.Substring(1);
        }

        public override void OpenForm() {
            if (CurrentRep == null)
            {
                MessageBox.Show(UWWTD2008MainForm.getMainForm(this), "Please select a valid ReportPeriod first", "No ReportPeriod selected", MessageBoxButtons.OK, MessageBoxIcon.Error);
                UWWTD2008MainForm.getMainForm(this).OpenDataEditMain();
                return;
            }
            if (!Directory.Exists(art17FormsDir)) Directory.CreateDirectory(art17FormsDir);
            
            if (File.Exists(art17FormsDir + "\\" + Settings.Default.Art17RawFile))
                linkGetForms.Enabled = true;
            else
                linkGetForms.Enabled = false;

            currentDataDir = art17FormsDir + "\\Art17impl_" + CurrentRep.RepCode;

            checkStatusLabel();
        }

        private void checkStatusLabel() {
            if (CurrentRep == null)
            {
                labStatus.Text = "Please select a Reporter!";
                return;
            }

            string url = getURLText();
            if (url != null)
                txtURL.Text = url;
            else
                txtURL.Text = "";

            if (Directory.Exists(currentDataDir))
            {
                string [] files = Directory.GetFiles(currentDataDir);
                string text = "No data has been provided";
                if (files.Length > 0)
                {
                    string file = files[0];
                    if (file.IndexOf("\\") >= 0) file = file.Substring(file.LastIndexOf("\\") + 1);

                    if (file.Equals("link.url"))
                        text = "Link has been provided";
                    else
                        text = "File has been provided (" + file + ")";
                }
                labStatus.Text = text;
            }
            else
            {
                labStatus.Text = "No data has been provided";
            }

            checkURLButton();
        }

        private void checkURLButton()
        {
            string url = txtURL.Text;
            if (url != null)
            {
                if ((url.StartsWith("http://") || (url.StartsWith("https://"))) && url.LastIndexOf("/") > 8)
                    cmdSetURL.Enabled = true;
                else
                    cmdSetURL.Enabled = false;
            }
        }

        private void OnStoreForms(object sender, LinkLabelLinkClickedEventArgs e) {
            if (Directory.Exists(currentDataDir))
            {
                if (MessageBox.Show(this, "Data has already been provided. Do you want to overwrite?", "Confirm", MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2) != DialogResult.Yes)
                {
                    return;
                }
            }
            DialogResult res = dlgUploadForm.ShowDialog(this);
            if (res == DialogResult.OK)
            {
                if (Directory.Exists(currentDataDir))
                    Directory.Delete(currentDataDir, true);
                Directory.CreateDirectory(currentDataDir);
                string file = dlgUploadForm.FileName;
                if (file.IndexOf("\\") >= 0) file = file.Substring(file.LastIndexOf("\\") + 1);

                try
                {
                    File.Copy(dlgUploadForm.FileName, currentDataDir + "\\" + file, true);
                }
                catch (IOException ex)
                {
                    MessageBox.Show(this, ex.Message, "File Upload failed", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                }
            }

            checkStatusLabel();
        }

        private void OnDownloadForms(object sender, LinkLabelLinkClickedEventArgs e) {
            string file = getFileName();
            if (file == null)
                file = art17FormsDir + "\\" + Settings.Default.Art17RawFile;
            else
                file = getFileName();

            DialogResult res = dlgDownloadForm.ShowDialog(this);
            if (res == DialogResult.OK)
            {
                File.Copy(file, dlgDownloadForm.FileName, true);
            }

            checkStatusLabel();
        }

        private string getFileName()
        {
            if (!Directory.Exists(currentDataDir))
                return null;
            string[] files = Directory.GetFiles(currentDataDir);
            if (files.Length < 1) return null;
            string file = files[0];
            if (file.IndexOf("\\") >= 0)
            {
                file = file.Substring(file.LastIndexOf("\\") + 1);
            }

            if (file.Equals("link.url"))
            {
                return null;
            }

            return files[0];
        }

        private string getURLText()
        {
            if (!Directory.Exists(currentDataDir))
                return null;
            string[] files = Directory.GetFiles(currentDataDir);
            if (files.Length < 1) return null;
            string file = files[0];
            if (file.IndexOf("\\") >= 0)
            {
                file = file.Substring(file.LastIndexOf("\\") + 1);
            }

            if (file.Equals("link.url"))
            {
                return File.ReadAllText(files[0], Encoding.UTF8);
            }

            return null;
        }

        private void OnSetURL(object sender, EventArgs e)
        {
            if (Directory.Exists(currentDataDir))
            {
                if (MessageBox.Show(this, "Data has already been provided. Do you want to overwrite?", "Confirm", MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2) != DialogResult.Yes)
                {
                    return;
                }
            }

            if (Directory.Exists(currentDataDir))
                Directory.Delete(currentDataDir, true);
            Directory.CreateDirectory(currentDataDir);
            File.WriteAllText(currentDataDir + "\\link.url", txtURL.Text, Encoding.UTF8);

            checkStatusLabel();
        }

        private void OnURLTextChanged(object sender, EventArgs e)
        {
            checkURLButton();
        }
    }
}
