﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

namespace at.uba.uwwtd2008.components
{
    public partial class CtrlBigCityEdit : UWWTDBaseCtrl
    {
        DataSet dataSet = null;

        DataRow currentRow = null;

        public const String SELECT_ALL_BIGCITY = "SELECT * FROM T_BigCity WHERE bigCountryCode = ";
        public const String TNAME_BIGCITY = "t_bigcity";

        public CtrlBigCityEdit()
        {
            InitializeComponent();
        }

        public override void OnBackButtonClick()
        {
            if (currentRow != null) currentRow.EndEdit();
            if (dataSet.HasChanges()) {
                if (!checkExitWithoutSave("BigCity")) {
                    if (currentRow != null) currentRow.BeginEdit();
                    return;
                }
                dataSet.RejectChanges();
            }

            UWWTD2008MainForm mf = UWWTD2008MainForm.getMainForm(this);
            if (mf == null) return;

            if (PrevControl != null) mf.getSplitterPanel().setContentPane(PrevControl);
            else mf.OpenDataEditMain();
        }

        public override void PerformDataBinding(DataSet dataSet) {
            this.dataSet = dataSet;
            IDbConnection conn = mDataFactory.GetConnection();
            try
            {
                //if (dataSet.Relations.Contains(RNAME_UWWTDSAGGLO_UWWTDS))
                //    dataSet.Relations.Remove(RNAME_UWWTDSAGGLO_UWWTDS);

                if (!dataSet.Tables.Contains(TNAME_BIGCITY))
                {
                    mDataFactory.FillDataAdapter(
                        mDataFactory.CreateDataAdapter(
                            mDataFactory.CreateCommand(SELECT_ALL_BIGCITY + "'" + CurrentRep.Reporter.State.Key + "'", conn)
                        ), dataSet, TNAME_BIGCITY);

                    // refine the Table a little bit
                    // dataSet.Tables[TNAME_BIGCITY].Columns["bigCityID"].Unique = true;
                }
            }
            finally
            {
                conn.Close();
            }

            BindingContext[dataSet, TNAME_BIGCITY].CurrentChanged += new EventHandler(CurrentBigCityChanged);
            gridBigCities.AutoGenerateColumns = false;
            gridBigCities.DataSource = dataSet;
            gridBigCities.DataMember = TNAME_BIGCITY;

            txtBigCityID.DataBindings.Clear();
            txtBigCityID.DataBindings.Add("Text", dataSet, TNAME_BIGCITY + ".bigCityID");

            txtBigCityName.DataBindings.Clear();
            txtBigCityName.DataBindings.Add("Text", dataSet, TNAME_BIGCITY + ".bigCity");

            txtCountryCode.DataBindings.Clear();
            txtCountryCode.DataBindings.Add("Text", dataSet, TNAME_BIGCITY + ".bigCountryCode");
        }

        void CurrentBigCityChanged(object sender, EventArgs e) {
            if (currentRow != null)
            {
                currentRow.CancelEdit();
                currentRow = null;
            }

            if (BindingContext[dataSet, TNAME_BIGCITY].Position >= 0)
            {
                DataRowView view = BindingContext[dataSet, TNAME_BIGCITY].Current as DataRowView;
                if (view != null)
                {
                    currentRow = view.Row;
                    currentRow.BeginEdit();
                }
            }

            updateComponentView();
        }

        private void updateComponentView() {
            base.updateComponentView(BindingContext[dataSet, TNAME_BIGCITY]);
            if (currentRow != null)
            {
                butDeleteBigCity.Enabled = true;
                if (currentRow.RowState != DataRowState.Added)
                {
                    txtBigCityID.BackColor = UWWTD2008MainForm.COLOR_FIELD_NORMAL;
                }
                else
                {
                    txtBigCityID.BackColor = UWWTD2008MainForm.COLOR_FIELD_MANDATORY;
                }
                txtBigCityID.ReadOnly = currentRow.RowState != DataRowState.Added;
            }
            else
            {
                butDeleteBigCity.Enabled = false;
            }
        }

        private void OnAddBigCity(object sender, EventArgs e) {
            if (currentRow != null) currentRow.EndEdit();
            DataRow row = dataSet.Tables[TNAME_BIGCITY].NewRow();
            row["bigCountryCode"] = CurrentRep.Reporter.State.Key;
            row["bigCityID"] = CurrentRep.Reporter.State.Key;

            dataSet.Tables[TNAME_BIGCITY].Rows.Add(row);
            // this will be called afterwards in OnSelChanged of the gridBigCities - Control
            //currentRow = row;
            //row.BeginEdit();
            gridBigCities.CurrentCell = gridBigCities.Rows[gridBigCities.Rows.Count - 1].Cells[0];
        }

        private void OnRowValidating(object sender, DataGridViewCellCancelEventArgs e) {
            //if (sender == gridBigCities)
            //{
            //    DataRowView selRow = gridBigCities.Rows[e.RowIndex].DataBoundItem as DataRowView;


            //    if (selRow.Row.RowState.Equals(DataRowState.Added) ||
            //        selRow.Row.RowState.Equals(DataRowState.Modified))
            //    {
            //        // TODO HPW: Validate Row Here
            //        e.Cancel = true;
            //    }
            //}
        }

        private void OnSaveLinkClicked(object sender, LinkLabelLinkClickedEventArgs e) {
            if (currentRow != null) {
                currentRow.EndEdit();
                string errorText = "";
                if (!ValidateForm(ref errorText, currentRow != null ? currentRow.RowState : DataRowState.Detached)) {
                    MessageBox.Show(this, "Please correct Form Errors before save:\r\n" + errorText, "Form not valid", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                    return;
                }
            }
            currentRow = null;
            IDbConnection conn = mDataFactory.GetConnection();
            try
            {
                IDbDataAdapter adapter = mDataFactory.CreateFullDataAdapter(
                        mDataFactory.CreateCommand(SELECT_ALL_BIGCITY +
                                    "'" + CurrentRep.Reporter.State.Key + "'", conn));
                mDataFactory.UpdateDataAdapter(adapter,
                        dataSet, TNAME_BIGCITY);
                mDataFactory.FillDataAdapter(adapter,
                        dataSet, TNAME_BIGCITY);

                // mDataFactory.FillDataAdapter(adapter, dataSet, TNAME_BIGCITY);
            }
            finally
            {
                conn.Close();
                CurrentBigCityChanged(this, null);
            }

            if (e != null)
                ShowSaveConfirmation();
            
        }

        private void OnValidatingUID(object sender, CancelEventArgs e) {
            TextBox text = ((TextBox)sender);
            DataRow[] rows = dataSet.Tables[TNAME_BIGCITY].Select("bigCityID = '" + text.Text + "'");
            if (rows.Length > 0)
            {
                if (rows[0] != ((DataRowView)gridBigCities.CurrentRow.DataBoundItem).Row)
                {
                    e.Cancel = true;
                }
            }
            if (!e.Cancel) 
                text.BackColor = Color.White;
            else
                text.BackColor = Color.Red;

            return;
        }

        private void OnDataError(object sender, DataGridViewDataErrorEventArgs e) {
            e.Cancel = true;
            currentRow.CancelEdit();
            currentRow = null;
        }

        private void OnDeleteBigCities(object sender, EventArgs e) {
            if (ShowConfirmDelete()) {
                foreach (DataGridViewRow rowView in gridBigCities.SelectedRows) {
                    if (((DataRowView)rowView.DataBoundItem).Row == currentRow) {
                        currentRow.CancelEdit();
                        currentRow = null;
                    }
                    ((DataRowView)rowView.DataBoundItem).Delete();
                }
            }
        }
    }
}
