﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using at.uba.uwwtd2008.DataAccess;

namespace at.uba.uwwtd2008.components
{
    public partial class CtrlCollectionEdit : UWWTDBaseCtrl
    {
        private DataSet dataSet = null;
        private DataView uwwView;
        private bool isSaving = false;

        public const String TNAME_UWWAGGLO = "uwwtd_agglo_coll";
        public const String TNAME_AGGLOCONNECTEDUWWTDS = "agglo_connecteduwwtds";
        public const String RNAME_AGGLOCONNECTEDUWWTDS = "r_agglo_uwwagglo_coll";
        public const String RNAME_CONNECTEDUWWTDSUWWTD = "r_uwwagglo_uwwtd_coll";
        public const String RNAME_AGGLO_UWWAGGLO = "agglo_uwwagglo_coll";
        public const String RNAME_UWWAGGLO_UWWTD = "uwwagglo_uwwtd_coll";
        public const String TNAME_AGGLO = CtrlAgglomerationsEdit.TNAME_AGGLO;

        //private const String RNAME_AGGLO_UWWAGGLO = "uwwtd_agglo_coll";

        public CtrlCollectionEdit()
        {
            isSaving = false;
            InitializeComponent();
        }

        public override void OnBackButtonClick()
        {
            if (dataSet.HasChanges()) {
                if (!checkExitWithoutSave("Agglomerations")) {
                    return;
                }
            }
            
            UWWTD2008MainForm mf = UWWTD2008MainForm.getMainForm(this);
            if (mf == null) return;

            if (BindingContext[dataSet, CtrlAgglomerationsEdit.TNAME_AGGLO].Position >= 0)
            {
                DataRowView row = BindingContext[dataSet, CtrlAgglomerationsEdit.TNAME_AGGLO].Current as DataRowView;
                if (row != null) row.CancelEdit();
            }

            dataSet.RejectChanges();

            base.OnBackButtonClick();
        }

        public override void OpenForm()
        {
            this.BindingContext[dataSet, CtrlAgglomerationsEdit.TNAME_AGGLO].Position = ((CtrlAgglomerationsEdit)PrevControl).CurrentAggloBindingContextPosition;
            // check, what to do in here
            if (BindingContext[dataSet, CtrlAgglomerationsEdit.TNAME_AGGLO].Position < 0)
            {
                MessageBox.Show(this, "Please select a valid Agglomeration to open this Form", "No Agglomeration selected", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                OnBackButtonClick();
                return;
            }

            if (BindingContext[dataSet, CtrlAgglomerationsEdit.TNAME_AGGLO].Position >= 0)
            {
                DataRowView row = BindingContext[dataSet, CtrlAgglomerationsEdit.TNAME_AGGLO].Current as DataRowView;
                if (row != null)
                {
                    row.BeginEdit();
                    uwwView.RowFilter = "aggID = " + row["aggID"];
                }
                else
                {
                    uwwView.RowFilter = "1 = 0";
                }
            }

            OnRateWithoutTreatmentChanged(txtPercWithoutTreatmentUWWTP, null);
            updatePercWithoutTreatment();
            base.OpenForm();
        }

        public void UpdateUwwAgglo(DataSet dataSet, IDbConnection conn)
        {
            mDataFactory.FillDataAdapter(
                mDataFactory.CreateDataAdapter(
                    mDataFactory.CreateCommand(
                        "Select * FROM T_UWWTP_AGGLO WHERE repID = " + CurrentRep.RepID, conn)
                    ), dataSet, TNAME_UWWAGGLO);

            mDataFactory.FillDataAdapter(
                mDataFactory.CreateDataAdapter(
                    mDataFactory.CreateCommand("select * " +
                                "from (T_Uwwtp_Agglo t1 INNER JOIN T_UWWTPS t2 " +
                                "ON t1.uwwID = t2.uwwID) WHERE t1.repID = " + CurrentRep.RepID, conn)
                ), dataSet, TNAME_AGGLOCONNECTEDUWWTDS);

        }

        public override void PerformDataBinding(DataSet dataSet)
        {
            // the selected Agglo should already be set here
            //((CtrlAgglomerationsEdit)PrevControl).CurrentAggID;
            this.dataSet = dataSet;
            bool hasNewItems = false;

            IDbConnection conn = mDataFactory.GetConnection();
            try
            {
                if (!dataSet.Tables.Contains(TNAME_AGGLOCONNECTEDUWWTDS) ||
                    !dataSet.Tables.Contains(TNAME_UWWAGGLO)) 
                {
                    UpdateUwwAgglo(dataSet, conn);
                    hasNewItems = true;
                }
                uwwView = new DataView(dataSet.Tables[TNAME_UWWAGGLO]);
                

                // Create Data Relations
                DataColumn dc1, dc2;
                DataRelation rel;

                if (!dataSet.Relations.Contains(RNAME_AGGLO_UWWAGGLO))
                {
                    dc1 = dataSet.Tables[CtrlAgglomerationsEdit.TNAME_AGGLO].Columns["aggID"];
                    dc2 = dataSet.Tables[TNAME_UWWAGGLO].Columns["aggID"];
                    rel = new DataRelation(RNAME_AGGLO_UWWAGGLO, dc1, dc2);
                    dataSet.Relations.Add(rel);
                    hasNewItems = true;
                }
                if (!dataSet.Relations.Contains(RNAME_UWWAGGLO_UWWTD))
                {
                    dc1 = dataSet.Tables[CtrlUWWTDEdit.TNAME_UWWTDS].Columns["uwwID"];
                    dc2 = dataSet.Tables[TNAME_UWWAGGLO].Columns["uwwID"];
                    rel = new DataRelation(RNAME_UWWAGGLO_UWWTD, dc1, dc2);
                    dataSet.Relations.Add(rel);
                    dataSet.Tables[TNAME_UWWAGGLO].Columns.Add("uwwCode", typeof(string), "Parent(" + RNAME_UWWAGGLO_UWWTD + ").uwwCode");
                    dataSet.Tables[TNAME_UWWAGGLO].Columns.Add("uwwName", typeof(string), "Parent(" + RNAME_UWWAGGLO_UWWTD + ").uwwName");
                    hasNewItems = true;
                }
                if (!dataSet.Relations.Contains(RNAME_AGGLOCONNECTEDUWWTDS))
                {
                    dc1 = dataSet.Tables[CtrlAgglomerationsEdit.TNAME_AGGLO].Columns["aggID"];
                    dc2 = dataSet.Tables[TNAME_AGGLOCONNECTEDUWWTDS].Columns["t1.aggID"];
                    rel = new DataRelation(RNAME_AGGLOCONNECTEDUWWTDS, dc1, dc2);
                    dataSet.Relations.Add(rel);
                    hasNewItems = true;
                }
                if (!dataSet.Relations.Contains(RNAME_CONNECTEDUWWTDSUWWTD))
                {
                    dc1 = dataSet.Tables[CtrlUWWTDEdit.TNAME_UWWTDS].Columns["uwwID"];
                    dc2 = dataSet.Tables[TNAME_AGGLOCONNECTEDUWWTDS].Columns["t2.uwwID"];
                    rel = new DataRelation(RNAME_CONNECTEDUWWTDSUWWTD, dc1, dc2);
                    dataSet.Relations.Add(rel);
                    hasNewItems = true;
                }
            }
            finally
            {
                conn.Close();
            }

            if (hasNewItems)
            {
                txtAggloID.DataBindings.Clear();
                txtAggloID.DataBindings.Add("Text", dataSet, CtrlAgglomerationsEdit.TNAME_AGGLO + ".aggCode");
                txtAggloID.ReadOnly = true;

                txtAggloName.DataBindings.Clear();
                txtAggloName.DataBindings.Add("Text", dataSet, CtrlAgglomerationsEdit.TNAME_AGGLO + ".aggName");
                txtAggloName.ReadOnly = true;

                gridConnectedUWWTPs.AutoGenerateColumns = false;
                gridConnectedUWWTPs.DataSource = uwwView;
                ((DataGridViewComboBoxColumn)gridConnectedUWWTPs.Columns["determination"]).DataSource = mDataFactory.getCOLLDETLovValues();
                ((DataGridViewComboBoxColumn)gridConnectedUWWTPs.Columns["determination"]).DisplayMember = "lovWert";
                ((DataGridViewComboBoxColumn)gridConnectedUWWTPs.Columns["determination"]).ValueMember = "lovID";
                //gridConnectedUWWTPs.DataMember = RNAME_AGGLO_UWWAGGLO;
                gridConnectedUWWTPs.CurrentCellChanged += new EventHandler(CurrentConnectedUWWTPChanged);

                txtAggloC1.DataBindings.Clear();
                txtAggloC1.DataBindings.Add("Text", dataSet, CtrlAgglomerationsEdit.TNAME_AGGLO + ".aggC1", true, DataSourceUpdateMode.OnValidation, "", "#0.00");

                comboAggloMethodC1.DataBindings.Clear();
                comboAggloMethodC1.DataSource = mDataFactory.getCOLLDETLovValuesNullable();
                comboAggloMethodC1.DisplayMember = "lovWert";
                comboAggloMethodC1.ValueMember = "lovID";
                comboAggloMethodC1.DataBindings.Add("SelectedValue", dataSet, CtrlAgglomerationsEdit.TNAME_AGGLO + ".aggMethodC1");

                txtAggloC2.DataBindings.Clear();
                txtAggloC2.DataBindings.Add("Text", dataSet, CtrlAgglomerationsEdit.TNAME_AGGLO + ".aggC2", true, DataSourceUpdateMode.OnValidation, "", "#0.00");

                comboAggloMethodC2.DataBindings.Clear();
                comboAggloMethodC2.DataSource = mDataFactory.getCOLLDETLovValuesNullable();
                comboAggloMethodC2.DisplayMember = "lovWert";
                comboAggloMethodC2.ValueMember = "lovID";
                comboAggloMethodC2.DataBindings.Add("SelectedValue", dataSet, CtrlAgglomerationsEdit.TNAME_AGGLO + ".aggMethodC2");

                txtAggloPercWithoutTreatment.DataBindings.Clear();
                txtAggloPercWithoutTreatment.DataBindings.Add("Text", dataSet, CtrlAgglomerationsEdit.TNAME_AGGLO + ".aggPercWithoutTreatment", true, DataSourceUpdateMode.OnValidation, "", "#0.00");

                comboAggloPercWithoutTreatmentMethod.DataBindings.Clear();
                comboAggloPercWithoutTreatmentMethod.DataSource = mDataFactory.getCOLLDETLovValuesNullable();
                comboAggloPercWithoutTreatmentMethod.DisplayMember = "lovWert";
                comboAggloPercWithoutTreatmentMethod.ValueMember = "lovID";
                comboAggloPercWithoutTreatmentMethod.DataBindings.Add("SelectedValue", dataSet, CtrlAgglomerationsEdit.TNAME_AGGLO + ".aggMethodWithoutTreatment");

                txtAggloRemarks.DataBindings.Clear();
                txtAggloRemarks.DataBindings.Add("Text", dataSet, CtrlAgglomerationsEdit.TNAME_AGGLO + ".aggRemarks");

                txtForecast.DataBindings.Clear();
                txtForecast.DataBindings.Add("Text", dataSet, CtrlAgglomerationsEdit.TNAME_AGGLO + ".aggForecast", true, DataSourceUpdateMode.OnValidation, "", "d");
            }
        }

        void CurrentConnectedUWWTPChanged(object sender, EventArgs e)
        {
            if (gridConnectedUWWTPs.CurrentRow == null) return;
            DataRowView rv = gridConnectedUWWTPs.CurrentRow.DataBoundItem as DataRowView;
            if (rv == null) return;
            DataRow row = dataSet.Tables[CtrlUWWTDEdit.TNAME_UWWTDS].Rows.Find(rv.Row["uwwID"]);

            this.BindingContext[dataSet, CtrlUWWTDEdit.TNAME_UWWTDS].Position = dataSet.Tables[CtrlUWWTDEdit.TNAME_UWWTDS].Rows.IndexOf(row);
            if (!isSaving)
                updatePercWithoutTreatment();
        }

        private void OnOpenTreatment(object sender, LinkLabelLinkClickedEventArgs e)
        {
            ((UWWTD2008MainForm)(this.Parent.Parent.Parent.Parent)).OpenTreatment(this);
        }

        private void OnEditIASDataClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            ((UWWTD2008MainForm)(this.Parent.Parent.Parent.Parent)).OpenEditIAS(this);
        }

        private void OpenLeaksAndStormWater(object sender, LinkLabelLinkClickedEventArgs e)
        {
            ((UWWTD2008MainForm)(this.Parent.Parent.Parent.Parent)).OpenEditLeaksAndStormWater(this);
        }

        private void OnDischargePointsOpen(object sender, EventArgs e)
        {
            UWWTD2008MainForm.getMainForm(this).OpenDPWithoutTreatment(this);
            //((UWWTD2008MainForm)(this.Parent.Parent.Parent.Parent)).OpenDischargePointsEdit(this);
        }

        public void saveAndUpdateUWWTDAGGLOS()
        {
        }

        private void OnSaveCollection(object sender, LinkLabelLinkClickedEventArgs e)
        {
            DataRowView row = BindingContext[dataSet, CtrlAgglomerationsEdit.TNAME_AGGLO].Current as DataRowView;
            if (row != null) row.EndEdit();
            string errorText = "";
            if (!ValidateForm(ref errorText, row != null ? row.Row.RowState : DataRowState.Detached)) {
                MessageBox.Show(this, "Please correct Form Errors before save:\r\n" + errorText, "Form not valid", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                return;
            }

            if (dataSet.HasChanges())
            {
                isSaving = true;
                IDbConnection conn = mDataFactory.GetConnection();
                try
                {
                    bool enforceConstraints = dataSet.EnforceConstraints;
                    dataSet.EnforceConstraints = false;
                    try
                    {
                        int curPos = BindingContext[dataSet, TNAME_AGGLO].Position;
                        IDbCommand cmd = mDataFactory.CreateCommand(CtrlAgglomerationsEdit.SelectString + CurrentRep.RepID, conn);
                        IDbDataAdapter adapter = mDataFactory.CreateFullDataAdapter(cmd);
                        mDataFactory.UpdateDataAdapter(adapter, dataSet, TNAME_AGGLO);
                        mDataFactory.FillDataAdapter(adapter, dataSet, TNAME_AGGLO);
                        if (BindingContext[dataSet, TNAME_AGGLO].Count > curPos)
                        {
                            BindingContext[dataSet, TNAME_AGGLO].Position = curPos;
                            UWWTDBaseCtrl ctrl = this;
                            while (ctrl.PrevControl != null) ctrl = ctrl.PrevControl;
                            ctrl.BindingContext[dataSet, TNAME_AGGLO].Position = curPos; // re-set the position in the Agglomeration-Form
                        }
                        // now: store the aucPercC2T
                        cmd = mDataFactory.CreateCommand("Select * FROM T_UWWTP_AGGLO WHERE repID = " + CurrentRep.RepID, conn);
                        adapter = mDataFactory.CreateFullDataAdapter(cmd);
                        mDataFactory.UpdateDataAdapter(adapter, dataSet, CtrlCollectionEdit.TNAME_UWWAGGLO);
                    }
                    finally
                    {
                        dataSet.EnforceConstraints = enforceConstraints;
                    }
                }
                finally
                {
                    conn.Close();
                }
                UWWTD2008MainForm mf = UWWTD2008MainForm.getMainForm(this);
                if (mf != null) mf.getSplitterPanel().updateUWWAggloTable(dataSet);
                isSaving = false;

            }

            if (e != null)
                ShowSaveConfirmation();
        }

        public void updatePercWithoutTreatment()
        {
            if (BindingContext[dataSet, CtrlAgglomerationsEdit.TNAME_AGGLO].Position >= 0) {
                float totalPercEnteringUWWTP = 0;
                foreach (DataGridViewRow rowView in gridConnectedUWWTPs.Rows)
                {
                    if (!(rowView.Cells["loadpercent"].Value is DBNull))
                    {
                        totalPercEnteringUWWTP += float.Parse(rowView.Cells["loadpercent"].Value.ToString());
                    }
                }
                object objC1 = ((DataRowView)BindingContext[dataSet, CtrlAgglomerationsEdit.TNAME_AGGLO].Current)["aggC1"];
                if ((objC1 == null) || (objC1 is DBNull))
                {
                    objC1 = 0.0f;
                }
                float percWithoutTreatment = ((float)objC1) - totalPercEnteringUWWTP;
                if (percWithoutTreatment < 0) percWithoutTreatment = 0;
                txtPercWithoutTreatmentUWWTP.Text = percWithoutTreatment.ToString("#0.00");
            }
            else {
                txtPercWithoutTreatmentUWWTP.Text = "";
            }
        }

        private void OnRateChanged(object sender, EventArgs e)
        {
            //txtPercWithoutTreatmentUWWTP.Text = ((TextBox)sender).Text;
            //updatePercWithoutTreatment();
        }

        private bool validateCollectionSums()
        {
            bool hasError = false;
            updatePercWithoutTreatment();
            if (BindingContext[dataSet, CtrlAgglomerationsEdit.TNAME_AGGLO].Position < 0) return true; // no agglo selected
            DataRowView row = BindingContext[dataSet, CtrlAgglomerationsEdit.TNAME_AGGLO].Current as DataRowView;
            if (row == null) return true;
            float sum = 0;
            sum += DataAccessFactory.getFloatValue(row["aggC1"]);
            sum += DataAccessFactory.getFloatValue(row["aggC2"]);
            sum += DataAccessFactory.getFloatValue(row["aggPercWithoutTreatment"]);
            if (Math.Round(sum, 2) != 100f)
            {
                txtAggloC1.BackColor = UWWTD2008.ColError;
                txtAggloC2.BackColor = UWWTD2008.ColError;
                txtAggloPercWithoutTreatment.BackColor = UWWTD2008.ColError;
                hasError = true;
            }
            else
            {
                if (!hasError)
                {
                    txtAggloC1.BackColor = UWWTD2008.ColNorm;
                    txtAggloC2.BackColor = UWWTD2008.ColNorm;
                    txtAggloPercWithoutTreatment.BackColor = UWWTD2008.ColNorm;
                }
            }

            bool hasIASValue = false;
            if (!((row["aggPercPrimTreatment"] is DBNull) &&
                (row["aggPercSecTreatment"] is DBNull) &&
                (row["aggPercStringentTreatment"] is DBNull)))
                hasIASValue = true;
            else
                foreach (DataRow childRow in row.Row.GetChildRows(CtrlCollectionEdit.RNAME_AGGLO_UWWAGGLO))
                {
                    if (!(childRow["aucPercC2T"] is DBNull)) {
                        hasIASValue = true;
                        break;
                    }
                }


            if (hasIASValue)
            {
                float sumIAS = 0;
                sumIAS += DataAccessFactory.getFloatValue(row["aggPercPrimTreatment"]);
                sumIAS += DataAccessFactory.getFloatValue(row["aggPercSecTreatment"]);
                sumIAS += DataAccessFactory.getFloatValue(row["aggPercStringentTreatment"]);
                foreach (DataRow childRow in row.Row.GetChildRows(CtrlCollectionEdit.RNAME_AGGLO_UWWAGGLO))
                {
                    sumIAS += DataAccessFactory.getFloatValue(childRow["aucPercC2T"]);
                }

                if (Math.Round(sumIAS, 2) != Math.Round(DataAccessFactory.getFloatValue(row["aggC2"])))
                {
                    txtAggloC2.BackColor = UWWTD2008.ColError;
                    hasError = true;
                }
                else
                {
                    if (!hasError)
                        txtAggloC2.BackColor = UWWTD2008.ColNorm;
                }
            }

            return !hasError;
        }

        private void OnAggloSumValidated(object sender, EventArgs e)
        {
            validateCollectionSums();
        }

        public override bool validateBeforeClose()
        {
            string errorText = "";
            DataRowView row = null;
            if (BindingContext[dataSet, CtrlAgglomerationsEdit.TNAME_AGGLO].Position >= 0)
                row = (DataRowView) BindingContext[dataSet, CtrlAgglomerationsEdit.TNAME_AGGLO].Current;
            if (!ValidateForm(ref errorText, row != null ? row.Row.RowState : DataRowState.Detached))
            {

                return confirmExitWithValidationFailed("Following Form Errors found:\r\n"+errorText+"\r\nContinue anyway?", "Collection");
            }

            return true;

        }

        protected override bool ValidateForm(ref string errorText, DataRowState state)
        {
            bool retVal = true;
            if (!validateCollectionSums())
            {
                errorText = "Treatment percentage sum is incorrect";
                retVal = false;
            }
            if (BindingContext[dataSet, CtrlAgglomerationsEdit.TNAME_AGGLO].Position < 0) return true; // no agglo selected
            DataRowView row = BindingContext[dataSet, CtrlAgglomerationsEdit.TNAME_AGGLO].Current as DataRowView;
            if (row == null) return true;

            if (row["aggForecast"] is DateTime)
            {
                DateTime aggDate = (DateTime)row["aggForecast"];
                if ((aggDate != null) && (aggDate.CompareTo(UWWTD2008.MIN_VALID_DATE) < 0))
                {
                    txtForecast.BackColor = UWWTD2008.ColError;
                    if (errorText.Length > 1) errorText += "\r\n";
                    errorText += "Forecast Date is too early";
                    retVal = false;
                }
                else if ((aggDate != null) && (aggDate.CompareTo(UWWTD2008.MAX_VALID_DATE) > 0))
                {
                    txtForecast.BackColor = UWWTD2008.ColError;
                    if (errorText.Length > 1) errorText += "\r\n";
                    errorText += "Forecast Date is too late";
                    retVal = false;
                }
                else
                    txtForecast.BackColor = UWWTD2008.ColNorm;
            }
            else txtForecast.BackColor = UWWTD2008.ColNorm;
            return retVal;
        }

        private void OnDeleteClick(object sender, LinkLabelLinkClickedEventArgs e)
        {
            if (ShowConfirmDelete()) {
                if (BindingContext[dataSet, TNAME_AGGLO].Position < 0) return; // no current dataSet
                DataRowView currentAgglo = (DataRowView)BindingContext[dataSet, TNAME_AGGLO].Current;

                currentAgglo.BeginEdit(); // start changeing the row
                currentAgglo["aggC1"] = DBNull.Value;
                currentAgglo["aggMethodC1"] = DBNull.Value;
                currentAgglo["aggC2"] = DBNull.Value;
                currentAgglo["aggMethodC2"] = DBNull.Value;
                currentAgglo["aggPercWithoutTreatment"] = DBNull.Value;
                currentAgglo["aggMethodWithoutTreatment"] = DBNull.Value;
                currentAgglo["aggRemarks"] = DBNull.Value;
                currentAgglo["aggForecast"] = DBNull.Value;
                currentAgglo.EndEdit(); // finished
                OnSaveCollection(sender, null);
            }
        }

        private void OnRateWithoutTreatmentChanged(object sender, EventArgs e)
        {
            if (DataAccessFactory.getFloatValue(txtPercWithoutTreatmentUWWTP.Text) > 0)
                cmdInsertDPWithoutTreatment.Enabled = true;
            else
                cmdInsertDPWithoutTreatment.Enabled = false;
        }

    }
}
