﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

namespace at.uba.uwwtd2008.components {
    public partial class CtrlDPWithoutTreatment : UWWTDBaseCtrl {
        public const string TNAME_AGGLO = CtrlAgglomerationsEdit.TNAME_AGGLO;
        public const string TNAME_UWWTDS = CtrlUWWTDEdit.TNAME_UWWTDS;
        public const string TNAME_DISCPOINTS = CtrlDischargePointsEdit.TNAME_DISCPOINTS;
        public const string RNAME_UWWTD_DP = CtrlTreatment.RNAME_UWWTD_DP;
        private DataSet dataSet;

        private object lovid_isconn;
        private object lovid_notconn;

        public CtrlDPWithoutTreatment() {
            InitializeComponent();
        }

        public override void OnBackButtonClick()
        {
            DataRowView current = BindingContext[dataSet, TNAME_UWWTDS].Current as DataRowView;
            if (current != null)
            {
                current.EndEdit();
                if (current.Row.RowState != DataRowState.Unchanged) {
                    if (!checkExitWithoutSave("DP without Treatment")) {
                        return;
                    }
                }

                dataSet.Tables[TNAME_UWWTDS].RejectChanges();
            }
            base.OnBackButtonClick();
        }

        public override void OpenForm() {
            BindingContext[dataSet, TNAME_AGGLO].Position = PrevControl.BindingContext[dataSet, TNAME_AGGLO].Position;
            DataRowView currentAgglo = BindingContext[dataSet, TNAME_AGGLO].Current as DataRowView;
            if (currentAgglo == null) this.OnBackButtonClick(); // leave this form - nothing can be done
            // find the uwwtd ...

            DataRow []notconnuwwtps = dataSet.Tables[TNAME_UWWTDS].Select("aggID = " + currentAgglo.Row["aggID"] + " AND uwwCollectingSystem = " + lovid_notconn);
            if (notconnuwwtps.Length > 1) //this is an internal error ... there shouldnt be more than on "not-connected-uwwtp"
                this.OnBackButtonClick();
            if (notconnuwwtps.Length < 1) //there is no "not-connected-uwwtp" ... so we add one
            {
                DataRow newRow = dataSet.Tables[TNAME_UWWTDS].NewRow();
                newRow["repID"] = CurrentRep.RepID;
                newRow["aggID"] = currentAgglo["aggID"];
                newRow["uwwCode"] = CurrentRep.Reporter.State.Key;
                newRow["uwwState"] = mDataFactory.getLovRowByKey("STAT", "1")["lovID"];
                newRow["uwwCollectingSystem"] = lovid_notconn;
                dataSet.Tables[TNAME_UWWTDS].Rows.Add(newRow);
                BindingContext[dataSet, TNAME_UWWTDS].Position = dataSet.Tables[TNAME_UWWTDS].Rows.IndexOf(newRow);
            } // if
            else
            {
                BindingContext[dataSet, TNAME_UWWTDS].Position = dataSet.Tables[TNAME_UWWTDS].Rows.IndexOf(notconnuwwtps[0]);
            }

            (BindingContext[dataSet, TNAME_UWWTDS].Current as DataRowView).BeginEdit();
            updateComponentView(BindingContext[dataSet, TNAME_UWWTDS]);
        }

        public override void PerformDataBinding(DataSet dataSet) {
            this.dataSet = dataSet;

            DataView dv = new DataView(dataSet.Tables[TNAME_UWWTDS]);
            DataTable lovValuesByKey = mDataFactory.getUWWTPCONNValues();
            lovid_isconn = lovValuesByKey.Select("lovKey='ISCON'")[0]["lovID"];
            lovid_notconn = lovValuesByKey.Select("lovKey='NOTCON'")[0]["lovID"];

            txtAggCode.DataBindings.Clear();
            txtAggCode.DataBindings.Add("Text", dataSet, TNAME_AGGLO + ".aggCode");
            txtAggName.DataBindings.Clear();
            txtAggName.DataBindings.Add("Text", dataSet, TNAME_AGGLO + ".aggName");

            txtUwwCode.DataBindings.Clear();
            txtUwwCode.DataBindings.Add("Text", dataSet, TNAME_UWWTDS + ".uwwCode");
            txtUwwName.DataBindings.Clear();
            txtUwwName.DataBindings.Add("Text", dataSet, TNAME_UWWTDS + ".uwwName");

            gridDischargePoints.AutoGenerateColumns = false;
            gridDischargePoints.DataSource = dataSet;
            gridDischargePoints.DataMember = TNAME_UWWTDS + "." + RNAME_UWWTD_DP;
        }

        public override void updateComponentView(BindingManagerBase context)
        {
            base.updateComponentView(context);

            if ((context.Current as DataRowView) != null)
            {
                DataRow row = (context.Current as DataRowView).Row;
                cmdEditDP.Enabled = (row.RowState == DataRowState.Unchanged);
            }
            else
            {
                cmdEditDP.Enabled = false;
            }
        }

        private void OnEditDischargePoints(object sender, EventArgs e)
        {
            UWWTD2008MainForm.getMainForm(this).OpenDischargePointsEdit(this);
        }

        private void LinkSaveClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            string errorText = "";
            (BindingContext[dataSet, TNAME_UWWTDS].Current as DataRowView).EndEdit();
            if (!ValidateForm(ref errorText, (BindingContext[dataSet, TNAME_UWWTDS].Current as DataRowView).Row.RowState)) {
                MessageBox.Show(this, "Please correct the following Form errors:\r\n" + errorText + "\r\nDo you want to save anyway?", "Form Validation",
                    MessageBoxButtons.OK, MessageBoxIcon.Exclamation);

                return;
            }
            saveData(false);
            ShowSaveConfirmation();
            //(BindingContext[dataSet, TNAME_UWWTDS].Current as DataRowView).BeginEdit();
            updateComponentView(BindingContext[dataSet, TNAME_UWWTDS]);
        }

        private void LinkDeleteClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            (BindingContext[dataSet, TNAME_UWWTDS].Current as DataRowView).EndEdit();
            // first: delete all the attached DPs
            foreach (DataRow row in (BindingContext[dataSet, TNAME_UWWTDS].Current as DataRowView).Row.GetChildRows(RNAME_UWWTD_DP))
            {
                row.BeginEdit();
                row.Delete();
                row.EndEdit();
            }
            // and then delete this row
            (BindingContext[dataSet, TNAME_UWWTDS].Current as DataRowView).Delete();
            if ((BindingContext[dataSet, TNAME_UWWTDS].Current as DataRowView).IsNew)
                dataSet.Tables[TNAME_UWWTDS].Rows.Remove((BindingContext[dataSet, TNAME_UWWTDS].Current as DataRowView).Row);
            else
                saveData(true);

            OnBackButtonClick();
        }
        private void saveData(bool isDeleting)
        {
            bool isEnforceConstraints = dataSet.EnforceConstraints;
            IDbConnection conn = mDataFactory.GetConnection();
            try
            {
                dataSet.EnforceConstraints = false;
                IDbDataAdapter adapterU = mDataFactory.CreateFullDataAdapter(
                            mDataFactory.CreateCommand(CtrlUWWTDEdit.SELECT_ALL_UWWTD + CurrentRep.RepID, conn)
                        );
                IDbDataAdapter adapterD = mDataFactory.CreateFullDataAdapter(
                            mDataFactory.CreateCommand(CtrlDischargePointsEdit.SELECT_DISCPOINTS + CurrentRep.RepID, conn)
                        );

                mDataFactory.UpdateDataAdapter(adapterU, dataSet, TNAME_UWWTDS);
                mDataFactory.UpdateDataAdapter(adapterD, dataSet, TNAME_DISCPOINTS);
                
                mDataFactory.FillDataAdapter(adapterU, dataSet, TNAME_UWWTDS);
                mDataFactory.FillDataAdapter(adapterD, dataSet, TNAME_DISCPOINTS);

                if (!isDeleting)
                {
                    BindingContext[dataSet, TNAME_AGGLO].Position = PrevControl.BindingContext[dataSet, TNAME_AGGLO].Position;
                    DataRowView currentAgglo = BindingContext[dataSet, TNAME_AGGLO].Current as DataRowView;

                    DataRow[] notconnuwwtps = dataSet.Tables[TNAME_UWWTDS].Select("aggID = " + currentAgglo.Row["aggID"] + " AND uwwCollectingSystem = " + lovid_notconn);

                    if (notconnuwwtps.Length < 1) //there is no "not-connected-uwwtp" ... so we add one
                    {
                        DataRow newRow = dataSet.Tables[TNAME_UWWTDS].NewRow();
                        newRow["repID"] = CurrentRep.RepID;
                        newRow["aggID"] = currentAgglo["aggID"];
                        newRow["uwwCode"] = CurrentRep.Reporter.State.Key;
                        newRow["uwwState"] = mDataFactory.getLovRowByKey("STAT", "1")["lovID"];
                        newRow["uwwCollectingSystem"] = lovid_notconn;
                        dataSet.Tables[TNAME_UWWTDS].Rows.Add(newRow);
                        BindingContext[dataSet, TNAME_UWWTDS].Position = dataSet.Tables[TNAME_UWWTDS].Rows.IndexOf(newRow);
                    } // if

                    BindingContext[dataSet, TNAME_UWWTDS].Position = dataSet.Tables[TNAME_UWWTDS].Rows.IndexOf(notconnuwwtps[0]);
                }
            }
            finally
            {
                conn.Close();
                dataSet.EnforceConstraints = isEnforceConstraints;
            }
        }
    }
}
