﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

namespace at.uba.uwwtd2008.components
{
    public partial class CtrlDischargePointsEdit : UWWTDBaseCtrl
    {
        private DataSet dataSet;
        private DataRowView currentRow;
        public const string TNAME_DISCPOINTS = "t_dischargepoints";
        public const string TNAME_RA = CtrlReceivingAreasEdit.TNAME_RA;
        public const string TNAME_UWWTDS = CtrlDPWithoutTreatment.TNAME_UWWTDS;
        public const string RNAME_UWWTD_DP = CtrlDPWithoutTreatment.RNAME_UWWTD_DP;
        public const string SELECT_DISCPOINTS = "select * FROM T_DischargePoints WHERE repID = ";

        //private DataView raView;
        private DataView tabDiscPoints;

        public CtrlDischargePointsEdit()
        {
            InitializeComponent();
        }

        public override void OnBackButtonClick()
        {
            UWWTD2008MainForm mf = UWWTD2008MainForm.getMainForm(this);
            if (mf == null) return;
            if (currentRow != null) {
                currentRow.EndEdit();

                if (currentRow.Row.RowState != DataRowState.Unchanged) {
                    if (!checkExitWithoutSave("Dischargepoints")) {
                        if (currentRow != null) currentRow.BeginEdit();
                        return;
                    }
                }
            }

            if (dataSet.HasChanges())
            {
                //foreach (DataTable table in dataSet.Tables)
                //{
                //    bool hasChanges = false;
                //    foreach (DataRow row in table.Rows)
                //    {
                //        if (row.RowState != DataRowState.Unchanged)
                //            hasChanges = true;
                //    }

                //    if (hasChanges)
                //    {
                //        Console.WriteLine("Test " + table.TableName);
                //    }
                //}

                //if (!checkExitWithoutSave("Discharge Points"))
                //{
                //    currentRow.BeginEdit();
                //    return;
                //}
                //else
                //{
                //    dataSet.RejectChanges();
                //}
                dataSet.Tables[TNAME_DISCPOINTS].RejectChanges();
            }

            if (PrevControl != null) mf.getSplitterPanel().setContentPane(PrevControl);
            else mf.OpenDataEditMain();
        }

        public override void OpenForm()
        {
            if (BindingContext[dataSet, TNAME_UWWTDS].Position < 0)
            {
                MessageBox.Show(this, "Please select a valid UWWTD to open this Form", "No UWWTD selected", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                OnBackButtonClick();
                return;
            }

            BindingContext[dataSet, TNAME_UWWTDS].Position = PrevControl.BindingContext[dataSet, TNAME_UWWTDS].Position;
            BindingContext[dataSet, TNAME_UWWTDS + "." + RNAME_UWWTD_DP].Position = PrevControl.BindingContext[dataSet, TNAME_UWWTDS + "." + RNAME_UWWTD_DP].Position;
            tabDiscPoints.RowFilter = "uwwID = " + (BindingContext[dataSet, TNAME_UWWTDS].Current as DataRowView).Row["uwwID"];
            CurrentDataRowChanged(this, null);
            OnSurfaceWatersChanged(null, null);
        }

        public override void PerformDataBinding(DataSet dataSet)
        {
            this.dataSet = dataSet;
            //bool hasNewItems = false;
            IDbConnection conn = mDataFactory.GetConnection();
            try
            {
                if (!dataSet.Tables.Contains(TNAME_DISCPOINTS))
                {
                    mDataFactory.FillDataAdapter(
                        mDataFactory.CreateDataAdapter(
                            mDataFactory.CreateCommand(SELECT_DISCPOINTS + CurrentRep.RepID, conn)
                        ), dataSet, TNAME_DISCPOINTS);

                    //hasNewItems = true;
                }
            }
            finally
            {
                conn.Close();
            }

            tabDiscPoints = new DataView(dataSet.Tables[TNAME_DISCPOINTS]);

            gridDischargePoints.AutoGenerateColumns = false;
            gridDischargePoints.DataSource = tabDiscPoints;

            BindingContext[tabDiscPoints].CurrentChanged += new EventHandler(CurrentDataRowChanged);
            txtCode.DataBindings.Clear();
            txtCode.DataBindings.Add("Text", tabDiscPoints, "dcpCode", true, DataSourceUpdateMode.OnValidation, "");
            txtName.DataBindings.Clear();
            txtName.DataBindings.Add("Text", tabDiscPoints, "dcpName", true, DataSourceUpdateMode.OnValidation, "");
            comboNuts.DataBindings.Clear();
            comboNuts.DataSource = mDataFactory.getNUTSTableForCurrentReporter();
            comboNuts.DisplayMember = "nutRegion";
            comboNuts.ValueMember = "nutID";
            comboNuts.DataBindings.Add("SelectedValue", tabDiscPoints, "dcpNuts", true, DataSourceUpdateMode.OnValidation, -1);
            txtLatitude.DataBindings.Clear();
            Binding tmp = new Binding("Text", tabDiscPoints, "dcpLatitude", true, DataSourceUpdateMode.OnValidation);
            tmp.Format += new ConvertEventHandler(formatStringDecimal);
            tmp.Parse += new ConvertEventHandler(parseStringDecimal);
            txtLatitude.DataBindings.Add(tmp);
            txtLongitude.DataBindings.Clear();
            tmp = new Binding("Text", tabDiscPoints, "dcpLongitude", true, DataSourceUpdateMode.OnValidation);
            tmp.Format += new ConvertEventHandler(formatStringDecimal);
            tmp.Parse += new ConvertEventHandler(parseStringDecimal);
            txtLongitude.DataBindings.Add(tmp);
            comboState.DataBindings.Clear();
            comboState.DataSource = mDataFactory.getLovSTAT();
            comboState.DisplayMember = "lovWert";
            comboState.ValueMember = "lovID";
            comboState.DataBindings.Add("SelectedValue", tabDiscPoints, "dcpState", true, DataSourceUpdateMode.OnValidation, -1);

            // fill the RA-Combobox;
            //raView = new DataView(dataSet.Tables[TNAME_RA]);
            //raView.RowFilter = "";
            comboRcaID.DataBindings.Clear();
            comboRcaID.DataSource = dataSet.Tables[TNAME_RA];
            comboRcaID.DisplayMember = "rcaName";
            comboRcaID.ValueMember = "rcaID";
            comboRcaID.DataBindings.Add("SelectedValue", tabDiscPoints, "rcaID", true, DataSourceUpdateMode.OnValidation, -1); //, true, DataSourceUpdateMode.OnValidation, "");

            DataTable lovYNValues = mDataFactory.getYNLovValues();
            comboSurfaceWaters.DataBindings.Clear();
            comboSurfaceWaters.DataSource = mDataFactory.getYNLovValues();
            comboSurfaceWaters.DisplayMember = "lovWert";
            comboSurfaceWaters.ValueMember = "lovID";
            comboSurfaceWaters.DataBindings.Add("SelectedValue", tabDiscPoints, "dcpSurfaceWaters", true, DataSourceUpdateMode.OnValidation, -1);

            comboWaterBodyType.DataBindings.Clear();
            comboWaterBodyType.DataSource = null;  //mDataFactory.getLovViewByKey("WATTYP1");
            //comboWaterBodyType.DisplayMember = "lovWert";
            //comboWaterBodyType.ValueMember = "lovID";
            //comboWaterBodyType.DataBindings.Add("SelectedValue", tabDiscPoints, "dcpWaterbodyType", true, DataSourceUpdateMode.OnValidation, -1);

            comboIrrigation.DataBindings.Clear();
            comboIrrigation.DataSource = mDataFactory.getLovViewByKey("LAND");
            comboIrrigation.DisplayMember = "lovWert";
            comboIrrigation.ValueMember = "lovID";
            comboIrrigation.DataBindings.Add("SelectedValue", tabDiscPoints, "dcpIrrigation", true, DataSourceUpdateMode.OnValidation, -1);

            txtWaterBodyType.DataBindings.Clear();
            txtWaterBodyType.DataBindings.Add("Text", tabDiscPoints, "dcpWaterBodyID");
            txtGroundWater.DataBindings.Clear();
            txtGroundWater.DataBindings.Add("Text", tabDiscPoints, "dcpGroundWater");
            txtReceivingWater.DataBindings.Clear();
            txtReceivingWater.DataBindings.Add("Text", tabDiscPoints, "dcpReceivingWater");
            txtWFDSubUnit.DataBindings.Clear();
            txtWFDSubUnit.DataBindings.Add("Text", tabDiscPoints, "dcpWFDSubUnit");
            txtWFDRBD.DataBindings.Clear();
            txtWFDRBD.DataBindings.Add("Text", tabDiscPoints, "dcpWFDRBD");
            txtRemarks.DataBindings.Clear();
            txtRemarks.DataBindings.Add("Text", tabDiscPoints, "dcpRemarks");
            txtReceivingWaterReferenceDate.DataBindings.Clear();
            txtReceivingWaterReferenceDate.DataBindings.Add("Text", tabDiscPoints, "dcpReceivingWaterReferenceDate", true, DataSourceUpdateMode.OnValidation, "", "d");
            txtWFDSubUnitReferenceDate.DataBindings.Clear();
            txtWFDSubUnitReferenceDate.DataBindings.Add("Text", tabDiscPoints, "dcpWFDSubUnitReferenceDate", true, DataSourceUpdateMode.OnValidation, "", "d");
            txtWFDRBDReferenceDate.DataBindings.Clear();
            txtWFDRBDReferenceDate.DataBindings.Add("Text", tabDiscPoints, "dcpWFDRBDReferenceDate", true, DataSourceUpdateMode.OnValidation, "", "d");
            txtWaterBodyTypeReferenceDate.DataBindings.Clear();
            txtWaterBodyTypeReferenceDate.DataBindings.Add("Text", tabDiscPoints, "dcpWaterBodyReferenceDate", true, DataSourceUpdateMode.OnValidation, "", "d");
            txtGroundWaterReferenceDate.DataBindings.Clear();
            txtGroundWaterReferenceDate.DataBindings.Add("Text", tabDiscPoints, "dcpGroundWaterReferenceDate", true, DataSourceUpdateMode.OnValidation, "", "d");

            comboNotAffect.DataBindings.Clear();
            comboNotAffect.DataSource = mDataFactory.getYNLovValues();
            comboNotAffect.DisplayMember = "lovWert";
            comboNotAffect.ValueMember = "lovID";
            comboNotAffect.DataBindings.Add("SelectedValue", tabDiscPoints, "dcpNotAffect", true, DataSourceUpdateMode.OnValidation, -1);

            comboMSProvide.DataBindings.Clear();
            comboMSProvide.DataSource = mDataFactory.getYNLovValues();
            comboMSProvide.DisplayMember = "lovWert";
            comboMSProvide.ValueMember = "lovID";
            comboMSProvide.DataBindings.Add("SelectedValue", tabDiscPoints, "dcpMSProvide", true, DataSourceUpdateMode.OnValidation, -1);

            comboCOMAccept.DataBindings.Clear();
            comboCOMAccept.DataSource = mDataFactory.getYNLovValues();
            comboCOMAccept.DisplayMember = "lovWert";
            comboCOMAccept.ValueMember = "lovID";
            comboCOMAccept.DataBindings.Add("SelectedValue", tabDiscPoints, "dcpCOMAccept", true, DataSourceUpdateMode.OnValidation, -1);
        }

        private void RadioSelected(object sender, EventArgs e)
        {
            if (!(sender is RadioButton) || !((RadioButton)sender).Checked) return; // do nothing if a check was released
            DataRow []row = null;

            if (currentRow == null) return;
            DataTable tab = mDataFactory.getRATYPLovValues();

            if (sender == optNormal)
            {
                row = tab.Select("lovKey = 'NA'");
                panelLSASpecific.Visible = false;
                comboRcaID.Visible = false;
            }
            else if (sender == optSensitive)
            {
                row = tab.Select("lovKey = 'SA'");
                panelLSASpecific.Visible = false;
                comboRcaID.Visible = true;
            }
            else if (sender == optLessSensitive)
            {
                row = tab.Select("lovKey = 'LSA'");
                panelLSASpecific.Visible = true;
                comboRcaID.Visible = true;
            }
            else if (sender == optArt58)
            {
                row = tab.Select("lovKey = 'A58'");
                panelLSASpecific.Visible = false;
                comboRcaID.Visible = false;
            }

            if ((row == null) || (row.Length != 1))
            {
                panelLSASpecific.Visible = false;
                comboRcaID.Visible = false;
                // some errorhandling wouldn't be bad here
                return;
            }

            // only set this, if it is a new value - otherwise leave as is ...
            if (!row[0]["lovID"].Equals(currentRow["dcpTypeOfReceivingArea"]))
                currentRow["dcpTypeOfReceivingArea"] = row[0]["lovID"];
        }

        public void refreshRARadios()
        {
            DataRow []row = null;
            if (currentRow == null)
            {
                row = new DataRow[0];
            }
            else
            {
                DataTable tab = mDataFactory.getRATYPLovValues();
                if (currentRow["dcpTypeOfReceivingArea"] is DBNull)
                {
                    row = new DataRow[0];
                }
                else
                {
                    row = tab.Select("lovID = " + currentRow["dcpTypeOfReceivingArea"]);
                }
            }

            if (row.Length==1) {
                optArt58.Checked = true;
                lblReceivingArea.Visible = true;
                comboRcaID.Visible = true;
                grpReceivingWaters.Visible = true;
                string key = row[0]["lovKey"] as string;
                switch(key) {
                    case "NA":
                        optNormal.Checked = true;
                        panelLSASpecific.Visible = false;
                        comboRcaID.Visible = false;
                        lblReceivingArea.Visible = false;
                        break;
                    case "SA":
                        optSensitive.Checked=true;
                        panelLSASpecific.Visible = false;
                        comboRcaID.Visible = true;
                        lblReceivingArea.Visible = true;
                        break;
                    case "LSA":
                        optLessSensitive.Checked=true;
                        panelLSASpecific.Visible = true;
                        comboRcaID.Visible = true;
                        lblReceivingArea.Visible = true;
                        break;
                    case "A58":
                        optArt58.Checked=true;
                        panelLSASpecific.Visible = false;
                        comboRcaID.Visible = false;
                        lblReceivingArea.Visible = false;
                        break;
                }
            }
            else {
                optNormal.Checked = false;
                optSensitive.Checked = false;
                optLessSensitive.Checked = false;
                optArt58.Checked = false;
                lblReceivingArea.Visible = false;
                comboRcaID.Visible = false;
                grpReceivingWaters.Visible = false;
                panelLSASpecific.Visible = false;
                comboRcaID.Visible = false;
            }
        }

        void CurrentDataRowChanged(object sender, EventArgs e)
        {
            if (currentRow != null)
            {
                currentRow.CancelEdit();
                currentRow = null;
            }

            if (BindingContext[tabDiscPoints].Position >= 0)
            {
                DataRowView view = BindingContext[tabDiscPoints].Current as DataRowView;
                if (view != null)
                {
                    currentRow = view;
                    currentRow.BeginEdit();
                }
            }

            updateComponentView();
            refreshRARadios();
        }

        private void updateComponentView()
        {
            string text = "";
            base.updateComponentView(BindingContext[tabDiscPoints]);
            if (currentRow == null)
            {
                groupRA.Enabled = false;
                panelLSASpecific.Visible = false;
                ValidateForm(ref text, DataRowState.Detached);
            } // if
            else
            {
                groupRA.Enabled = true;

                //cmdDelete.Enabled = true;
                txtCode.ReadOnly = currentRow.Row.RowState != DataRowState.Added;
                //txtName.ReadOnly = currentRow.Row.RowState != DataRowState.Added;
                txtName.ReadOnly = false;
                ValidateForm(ref text, currentRow.Row.RowState);
            }
        }

        private void LinkSaveClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            string errorText = "";
            if (!ValidateForm(ref errorText, currentRow.Row.RowState))
            {
                MessageBox.Show(this, "Please correct the following Form errors:\r\n" + errorText, "Form Validation",
                    MessageBoxButtons.OK, MessageBoxIcon.Exclamation);

                return;
            }

            currentRow.EndEdit();
            saveData();
            //currentRow.BeginEdit();
            
            ShowSaveConfirmation();
            updateComponentView();
        }

        private void saveData()
        {
            IDbConnection conn = mDataFactory.GetConnection();
            try
            {
                bool enforceConstraints = dataSet.EnforceConstraints;
                IDbCommand cmd = mDataFactory.CreateCommand(SELECT_DISCPOINTS + CurrentRep.RepID, conn);
                IDbDataAdapter adapter = mDataFactory.CreateFullDataAdapter(cmd);
                try
                {
                    dataSet.EnforceConstraints = false;
                    int curPos = BindingContext[tabDiscPoints].Position;
                    mDataFactory.UpdateDataAdapter(adapter, dataSet, TNAME_DISCPOINTS);
                    mDataFactory.FillDataAdapter(adapter, dataSet, TNAME_DISCPOINTS);
                    if (curPos < BindingContext[tabDiscPoints].Count)
                    {
                        BindingContext[tabDiscPoints].Position = curPos;
                    }

                    // TODO: HPW - correct this here to enable refreshing the DP-View in UWWTP
                    if (Parent is CtrlUWWTDEdit)
                        ((CtrlUWWTDEdit)Parent).UpdateDiscPoints(dataSet, conn);
                }
                finally
                {
                    dataSet.EnforceConstraints = enforceConstraints;
                }
            }
            finally
            {
                conn.Close();
            }
        }

        private void InsertNewRow(object sender, EventArgs e)
        {
            DataRow newRow = dataSet.Tables[TNAME_DISCPOINTS].NewRow();
            newRow["repID"] = CurrentRep.RepID;
            newRow["uwwID"] = (BindingContext[dataSet, TNAME_UWWTDS].Current as DataRowView)["uwwID"];
            newRow["dcpState"] = mDataFactory.getLovRowByKey("STAT", "1")["lovID"];
            newRow["dcpTypeOfReceivingArea"] = mDataFactory.getLovRowByKey("RATYP", "NA")["lovID"];
            newRow["dcpCode"] = CurrentRep.Reporter.State.Key;
            dataSet.Tables[TNAME_DISCPOINTS].Rows.Add(newRow);
            BindingContext[tabDiscPoints].Position = tabDiscPoints.Count - 1;
        }

        private void DeleteRow(object sender, EventArgs e)
        {
            if (currentRow != null) {
                if (ShowConfirmDelete()) {
                    if (currentRow.Row.RowState == DataRowState.Added) {
                        currentRow.CancelEdit();
                        dataSet.Tables[TNAME_DISCPOINTS].Rows.Remove(currentRow.Row);
                        // the bindingcontext corrects its position by itself following two lines are not needed
                        //int pos =
                        //BindingContext[dataSet.Tables[TNAME_DISCPOINTS]].Position = BindingContext[dataSet.Tables[TNAME_DISCPOINTS]].Position - 1;
                    }
                    else {
                        currentRow.CancelEdit();
                        currentRow.Delete();
                        saveData();
                    }
                }
            }
        }

        override protected void validateStringDecimal(object sender, CancelEventArgs e)
        {
            base.validateStringDecimal(sender, e);
        }

        private void OnSurfaceWatersChanged(object sender, EventArgs e)
        {
            if (currentRow == null) {
                comboWaterBodyType.DataSource = null;
                comboWaterBodyType.Enabled = false;
                return;
            }
            if (mDataFactory.getLovRowByID(comboSurfaceWaters.SelectedValue)["lovKey"].Equals("1"))
            {
                DataView view = mDataFactory.getLovViewByKey("WATTYP1");
                view.RowFilter += " AND (lovKey = 'CW' OR lovKey = 'ES' OR lovKey = 'FW')";
                comboWaterBodyType.DisplayMember = "lovWert";
                comboWaterBodyType.ValueMember = "lovID";

                comboWaterBodyType.DataBindings.Clear();
                comboWaterBodyType.Enabled = true;
                comboWaterBodyType.DataSource = view;
                comboWaterBodyType.DataBindings.Add("SelectedValue", tabDiscPoints, "dcpWaterbodyType", true, DataSourceUpdateMode.OnValidation, -1);
            }
            else if (mDataFactory.getLovRowByID(comboSurfaceWaters.SelectedValue)["lovKey"].Equals("0")) {
                DataView view = mDataFactory.getLovViewByKey("WATTYP2");
                view.RowFilter += " AND (lovKey = 'LC' OR lovKey = 'LF')";
                comboWaterBodyType.DisplayMember = "lovWert";
                comboWaterBodyType.ValueMember = "lovID";

                comboWaterBodyType.DataBindings.Clear();
                comboWaterBodyType.Enabled = true;
                comboWaterBodyType.DataSource = view;
                comboWaterBodyType.DataBindings.Add("SelectedValue", tabDiscPoints, "dcpWaterbodyType", true, DataSourceUpdateMode.OnValidation, -1);
            }
            else {
                //DataView view = mDataFactory.getLovViewByKey("WATTYP2");
                //view.RowFilter += " AND 1=0";

                comboWaterBodyType.DataSource = null;
                comboWaterBodyType.Enabled = false;
            }
        }

        protected override bool ValidateForm(ref string errorText, DataRowState state)
        {
            bool retVal = true;
            errorText = "";
            if (currentRow == null)
            {
                txtCode.BackColor = UWWTD2008.ColNorm;
                txtCode.ReadOnly = true;
                txtName.BackColor = UWWTD2008.ColNorm;
                txtCode.ReadOnly = true;
                txtLatitude.BackColor = UWWTD2008.ColNorm;
                txtLatitude.ReadOnly = true;
                txtLongitude.BackColor = UWWTD2008.ColNorm;
                txtLongitude.ReadOnly = true;
                return true;
            }

            //if (state == DataRowState.Detached) state = currentRow.Row.RowState;
            txtLatitude.ReadOnly = false;
            txtLongitude.ReadOnly = false;


            txtCode.BackColor = state == DataRowState.Added ? UWWTD2008.ColMandatory : UWWTD2008.ColNorm;
            txtCode.ReadOnly = state != DataRowState.Added;
            txtName.BackColor = UWWTD2008.ColNorm;
            txtLatitude.BackColor = state == DataRowState.Added ? UWWTD2008.ColMandatory : UWWTD2008.ColNorm;
            txtLongitude.BackColor = state == DataRowState.Added ? UWWTD2008.ColMandatory : UWWTD2008.ColNorm;
            string strCode = currentRow["dcpCode"] as string;
            if (strCode == null) strCode = "";
            string strName = currentRow["dcpName"] as string;
            if (strName == null) strName = "";
            string strLongitude = currentRow["dcpLongitude"] as string;
            string strLatitude = currentRow["dcpLatitude"] as string;

            if (strName.Length < 3)
            {
                errorText = "Please specify a Name for this Dischargepoint";
                txtName.BackColor = state == DataRowState.Added ? UWWTD2008.ColMandatory : UWWTD2008.ColError;
                retVal = false;
            }

            if (strCode.Length < 3)
            {
                if (errorText.Length > 1) errorText += "\r\n";
                errorText += "Please specify a Valid ID for this Dischargepoint";
                txtCode.BackColor = state == DataRowState.Added ? UWWTD2008.ColMandatory : UWWTD2008.ColError;
                retVal = false;
            }
            else if (dataSet.Tables[TNAME_DISCPOINTS].Select("dcpCode = '" + strCode + "'").Length > ((state == DataRowState.Added) ? 0 : 1))
            {
                if (errorText.Length > 1) errorText += "\r\n";
                errorText += "The rcaCode is already used";
                txtCode.BackColor = UWWTD2008.ColError;
                retVal = false;
            }
            else if (!strCode.StartsWith(CurrentRep.Reporter.State.Key))
            {
                if (errorText.Length > 1) errorText += "\r\n";
                errorText += "The rcaCode should start with " + CurrentRep.Reporter.State.Key;
                txtCode.BackColor = state == DataRowState.Added ? UWWTD2008.ColMandatory : UWWTD2008.ColError;
                retVal = false;
            }

            if (strLongitude == null) strLongitude = "";
            if (strLatitude == null) strLatitude = "";
            if (strLatitude.Length < 1) {
                txtLatitude.BackColor = UWWTD2008.ColMandatory;
                if (errorText.Length > 1) errorText += "\r\n";
                errorText += "Latitiude is Master Data";
                retVal = false;
            }
            if (strLongitude.Length < 1)
            {
                txtLongitude.BackColor = UWWTD2008.ColMandatory;
                if (errorText.Length > 1) errorText += "\r\n";
                errorText += "Longitude is Master Data";
                retVal = false;
            }

            if (!comboWaterBodyType.Enabled || comboWaterBodyType.SelectedValue == null) {
                comboWaterBodyType.BackColor = UWWTD2008.ColMandatory;
                if (errorText.Length > 1) errorText += "\r\n";
                errorText += "Discharge is Master Data";
                retVal = false;
            }
            else
                comboWaterBodyType.BackColor = UWWTD2008.ColNorm;

            if (!checkID(txtWaterBodyType, "WFD waterbody ID", ref errorText)) retVal = false;
            if (!checkID(txtGroundWater, "groundwaterbody ID", ref errorText)) retVal = false;
            if (!checkID(txtReceivingWater, "receiving water ID", ref errorText)) retVal = false;
            if (!checkID(txtWFDSubUnit, "WFD sub unit ID", ref errorText)) retVal = false;
            if (!checkID(txtWFDRBD, "WFD river basin district ID", ref errorText)) retVal = false;
            return retVal;
        }

        private static bool checkID(TextBox control, string desc, ref string errorText) {
            if ((control.Text != null) && (control.Text.Length > 0) && (!control.Text.StartsWith(CurrentRep.Reporter.State.Key)))
            {
                control.BackColor = UWWTD2008.ColError;
                if (errorText.Length > 1) errorText += "\r\n";
                errorText += desc + " has to start with " + CurrentRep.Reporter.State.Key;
                return false;
            }
            else control.BackColor = UWWTD2008.ColNorm;

            return true;
        }

    }
}
