﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.IO;
using at.uba.uwwtd2008.Properties;

namespace at.uba.uwwtd2008.components
{
    public partial class CtrlFileControl : UWWTDBaseCtrl
    {
        string commonDir = null;
        string myDir = null;

        public CtrlFileControl()
        {
            InitializeComponent();
        }

        public override void OnBackButtonClick()
        {
            UWWTD2008MainForm mf = UWWTD2008MainForm.getMainForm(this);
            if (mf != null) mf.OpenOpenerWindow();
        }

        public override void OpenForm()
        {
            string dir = Settings.Default.FileStore;
            if (dir.StartsWith(".\\")) dir = Directory.GetParent(Application.ExecutablePath) + dir.Substring(1);

            // populate the list for display
            if (!Directory.Exists(dir))
            {
                Directory.CreateDirectory(dir);
            }

            commonDir = dir + "\\Common";
            if (!Directory.Exists(commonDir))
            {
                Directory.CreateDirectory(commonDir);
            }
            if (CurrentRep != null)
            {
                myDir = dir + "\\Files_" + CurrentRep.Reporter.State.Key;
                if (!Directory.Exists(myDir))
                {
                    Directory.CreateDirectory(myDir);
                }
            }
            else
            {
                myDir = null;
            }

            refreshList();
            OnItemSelectionChanged(lstFiles, null);
        }

        private void refreshList()
        {
            lstFiles.Items.Clear();
            if (commonDir != null)
                populateList(commonDir);

            if (myDir != null)
                populateList(myDir);

        }

        private void populateList(String dir)
        {
            string file;

            foreach (string diritem in Directory.GetFiles(dir, "*.*", SearchOption.TopDirectoryOnly))
            {
                int imageIndex = 0;
                file = diritem;

                if (file.IndexOf("\\") >= 0)
                    file = file.Substring(file.LastIndexOf("\\") + 1);

                if (file.ToLower().EndsWith(".pdf"))
                {
                    imageIndex = 1;
                }
                if (file.ToLower().EndsWith(".doc"))
                {
                    imageIndex = 2;
                }
                if (file.ToLower().EndsWith(".xls"))
                {
                    imageIndex = 3;
                }

                ListViewItem item = new ListViewItem(file, imageIndex);
                item.Tag = diritem;
                lstFiles.Items.Add(item);
            }
        }

        private void OnRemoveItem(object sender, EventArgs e)
        {
            string msg = "Please confirm to delete ";
            if (lstFiles.SelectedItems.Count > 1)
                msg += lstFiles.SelectedItems.Count + " files!";
            else
                msg += lstFiles.SelectedItems.Count + " file!";

            DialogResult res = MessageBox.Show(this, msg, "Please confirm", MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2);
            if (res != DialogResult.Yes) return;

            foreach (ListViewItem item in lstFiles.SelectedItems)
            {
                string file = item.Tag as string;
                if (file == null) continue;
                if (file.StartsWith(commonDir)) continue; // do not delete packaged files
                File.Delete(file);
            }
            refreshList();
        }

        private void OnItemSelectionChanged(object sender, ListViewItemSelectionChangedEventArgs e)
        {
            if (sender == lstFiles)
            {
                if (lstFiles.SelectedItems.Count < 1)
                {
                    cmdDownload.Enabled = false;
                    cmdRemove.Enabled = false;
                    cmdUpload.Enabled = true;
                }
                else if (lstFiles.SelectedItems.Count > 1)
                {
                    cmdDownload.Enabled = false;
                    cmdRemove.Enabled = checkRemoveableItems(lstFiles.SelectedItems);
                    cmdUpload.Enabled = true;
                }
                else
                {
                    cmdDownload.Enabled = true;
                    cmdRemove.Enabled = checkRemoveableItems(lstFiles.SelectedItems);
                    cmdUpload.Enabled = true;
                }

            }
        }

        private bool checkRemoveableItems(System.Collections.IList lst)
        {
            foreach (ListViewItem item in lst)
            {
                string file = item.Tag as string;
                if (file == null) continue;
                if (file.StartsWith(commonDir)) return false;
            }
            return true;
        }

        private void OnDownloadFile(object sender, EventArgs e)
        {
            if (lstFiles.SelectedItems.Count < 1) return;
            
            string file = lstFiles.SelectedItems[0].Tag as string;

            string newfile = file;
            if (newfile.IndexOf("\\") >= 0)
                newfile = newfile.Substring(newfile.LastIndexOf("\\") + 1);
            dlgFileSave.FileName = newfile;
            DialogResult res = dlgFileSave.ShowDialog(this);
            if (res == DialogResult.OK)
            {
                // already done by file-save-dialog
                //if (File.Exists(dlgFileSave.FileName)) {
                //    if (MessageBox.Show(this, "Do you want to overwrite this File","Please confirm",MessageBoxButtons.YesNo,MessageBoxIcon.Question,MessageBoxDefaultButton.Button2) != DialogResult.Yes)
                //        return;
                //}

                if (file.StartsWith(".\\")) file = Directory.GetParent(Application.ExecutablePath) + file.Substring(1);
                File.Copy(file, dlgFileSave.FileName, true);
            }
        }

        private void OnUploadFile(object sender, EventArgs e)
        {
            DialogResult res = dlgFileOpen.ShowDialog(this);
            if (res == DialogResult.OK)
            {
                foreach (string source in dlgFileOpen.FileNames)
                {
                    string newfile = source;
                    if (newfile.IndexOf("\\") >= 0)
                        newfile = newfile.Substring(newfile.LastIndexOf("\\")+1);
                    string newpath = myDir + "\\" + newfile;

                    if (File.Exists(newpath))
                    {
                        if (MessageBox.Show(this, "Do you want to overwrite \"" + newfile + "\"", "Please confirm", MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2) != DialogResult.Yes)
                            continue;
                    }
                    File.Copy(source, newpath, true);
                }
            }
            refreshList();
        }

    }
}
