﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using at.uba.uwwtd2008.DataAccess;

namespace at.uba.uwwtd2008.components {
    public partial class CtrlIASEdit : UWWTDBaseCtrl {
        private DataSet dataSet;
        public const String TNAME_AGGLO = CtrlAgglomerationsEdit.TNAME_AGGLO; 

        public CtrlIASEdit()
        {
            InitializeComponent();
        }

        public override void OnBackButtonClick() {
            UWWTD2008MainForm mf = UWWTD2008MainForm.getMainForm(this);
            if (mf == null) return;

            DataRowView row = BindingContext[dataSet, CtrlAgglomerationsEdit.TNAME_AGGLO].Current as DataRowView;
            if (row != null) row.CancelEdit(); // could be that there were changes from the last form - recheck this
            
            if (PrevControl != null) mf.getSplitterPanel().setContentPane(PrevControl);
            else mf.OpenDataEditMain();
        }

        public override void OpenForm()
        {
            this.BindingContext[dataSet, CtrlAgglomerationsEdit.TNAME_AGGLO].Position = ((CtrlCollectionEdit)PrevControl).BindingContext[dataSet, CtrlAgglomerationsEdit.TNAME_AGGLO].Position;
            DataRowView row = BindingContext[dataSet, CtrlAgglomerationsEdit.TNAME_AGGLO].Current as DataRowView;
            if (row != null)
            {
                row.BeginEdit();
                checkTreatmentValues();
            }

        }

        public override void PerformDataBinding(DataSet dataSet)
        {
            this.dataSet = dataSet;

            txtAggloID.DataBindings.Clear();
            txtAggloID.DataBindings.Add("Text", dataSet, CtrlAgglomerationsEdit.TNAME_AGGLO + ".aggCode");

            txtAgglomerationName.DataBindings.Clear();
            txtAgglomerationName.DataBindings.Add("Text", dataSet, CtrlAgglomerationsEdit.TNAME_AGGLO + ".aggName");

            gridUwwtpList.AutoGenerateColumns = false;
            gridUwwtpList.DataSource = dataSet;
            gridUwwtpList.DataMember = CtrlAgglomerationsEdit.TNAME_AGGLO + "." + CtrlCollectionEdit.RNAME_AGGLO_UWWAGGLO;
            gridUwwtpList.CurrentCellChanged += new EventHandler(CurrentConnectedUwwtpChanged);

            txtPrimTreatment.DataBindings.Clear();
            txtPrimTreatment.DataBindings.Add("Text", dataSet, CtrlAgglomerationsEdit.TNAME_AGGLO + ".aggPercPrimTreatment");
            txtSecTreatment.DataBindings.Clear();
            txtSecTreatment.DataBindings.Add("Text", dataSet, CtrlAgglomerationsEdit.TNAME_AGGLO + ".aggPercSecTreatment");
            txtStringentTreatment.DataBindings.Clear();
            txtStringentTreatment.DataBindings.Add("Text", dataSet, CtrlAgglomerationsEdit.TNAME_AGGLO + ".aggPercStringentTreatment");
            //txtStringentTreatment.DataBindings.Clear();
            //txtStringentTreatment.DataBindings.Add("Text", dataSet, CtrlUWWTDEdit.TNAME_UWWTDS + ".uwwCode");
        }

        void CurrentConnectedUwwtpChanged(object sender, EventArgs e)
        {
            if (gridUwwtpList.CurrentRow == null) return;
            DataRowView rv = gridUwwtpList.CurrentRow.DataBoundItem as DataRowView;
            if (rv == null) return;
            DataRow row = dataSet.Tables[CtrlUWWTDEdit.TNAME_UWWTDS].Rows.Find(rv.Row["uwwID"]);

            this.BindingContext[dataSet, CtrlUWWTDEdit.TNAME_UWWTDS].Position = dataSet.Tables[CtrlUWWTDEdit.TNAME_UWWTDS].Rows.IndexOf(row);
        }

        private void OnSaveLinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            DataRowView row = BindingContext[dataSet, CtrlAgglomerationsEdit.TNAME_AGGLO].Current as DataRowView;
            if (row != null) row.EndEdit();
            string errorText = "";
            if ((e != null) && (!ValidateForm(ref errorText, row != null ? row.Row.RowState : DataRowState.Detached))) {
                MessageBox.Show(this, "Please correct Form Errors before save:\r\n" + errorText, "Form not valid", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                return;
            }

            if (dataSet.HasChanges())
            {
                IDbConnection conn = mDataFactory.GetConnection();
                try
                {
                    bool enforceConstraints = dataSet.EnforceConstraints;
                    dataSet.EnforceConstraints = false;
                    try
                    {
                        int curPos = BindingContext[dataSet, TNAME_AGGLO].Position;
                        IDbCommand cmd = mDataFactory.CreateCommand(CtrlAgglomerationsEdit.SelectString + CurrentRep.RepID, conn);
                        IDbDataAdapter adapter = mDataFactory.CreateFullDataAdapter(cmd);
                        mDataFactory.UpdateDataAdapter(adapter, dataSet, TNAME_AGGLO);
                        mDataFactory.FillDataAdapter(adapter, dataSet, TNAME_AGGLO);
                        if (BindingContext[dataSet, TNAME_AGGLO].Count > curPos)
                        {
                            BindingContext[dataSet, TNAME_AGGLO].Position = curPos;
                            UWWTDBaseCtrl ctrl = this;
                            while (ctrl.PrevControl != null) ctrl = ctrl.PrevControl;
                            ctrl.BindingContext[dataSet, TNAME_AGGLO].Position = curPos; // re-set the position in the Agglomeration-Form
                        }
                        // now: store the aucPercC2T
                        cmd = mDataFactory.CreateCommand("Select * FROM T_UWWTP_AGGLO WHERE repID = " + CurrentRep.RepID, conn);
                        adapter = mDataFactory.CreateFullDataAdapter(cmd);
                        mDataFactory.UpdateDataAdapter(adapter, dataSet, CtrlCollectionEdit.TNAME_UWWAGGLO);
                    }
                    finally
                    {
                        dataSet.EnforceConstraints = enforceConstraints;
                    }
                }
                finally
                {
                    conn.Close();
                }
            }
            row = BindingContext[dataSet, CtrlAgglomerationsEdit.TNAME_AGGLO].Current as DataRowView;
            //if (row != null) row.BeginEdit();
            if (e != null) ShowSaveConfirmation();
        }

        private void OnDelete(object sender, LinkLabelLinkClickedEventArgs e)
        {
            if (ShowConfirmDelete()) {
                DataRowView row = BindingContext[dataSet, CtrlAgglomerationsEdit.TNAME_AGGLO].Current as DataRowView;
                if (row != null) {
                    row.BeginEdit();
                    row["aggPercPrimTreatment"] = dataSet.Tables[TNAME_AGGLO].Columns["aggPercPrimTreatment"].DefaultValue;
                    row["aggPercSecTreatment"] = dataSet.Tables[TNAME_AGGLO].Columns["aggPercSecTreatment"].DefaultValue;
                    row["aggPercStringentTreatment"] = dataSet.Tables[TNAME_AGGLO].Columns["aggPercStringentTreatment"].DefaultValue;
                    row.EndEdit();
                }
                if (dataSet.HasChanges()) {
                    OnSaveLinkClicked(sender, null);
                }
                row = BindingContext[dataSet, CtrlAgglomerationsEdit.TNAME_AGGLO].Current as DataRowView;
                //if (row != null) row.BeginEdit();
            }
        }

        private bool checkTreatmentValues()
        {
            DataRowView row = BindingContext[dataSet, CtrlAgglomerationsEdit.TNAME_AGGLO].Current as DataRowView;
            if (row == null) return true;
            bool hasError = false;

            bool hasIASValue = false;
            if (!((row["aggPercPrimTreatment"] is DBNull) &&
                (row["aggPercSecTreatment"] is DBNull) &&
                (row["aggPercStringentTreatment"] is DBNull)))
                hasIASValue = true;
            else
                foreach (DataRow childRow in row.Row.GetChildRows(CtrlCollectionEdit.RNAME_AGGLO_UWWAGGLO))
                {
                    if (!(childRow["aucPercC2T"] is DBNull))
                    {
                        hasIASValue = true;
                        break;
                    }
                }


            if (hasIASValue)
            {
                float sumIAS = 0;
                sumIAS += DataAccessFactory.getFloatValue(row["aggPercPrimTreatment"]);
                sumIAS += DataAccessFactory.getFloatValue(row["aggPercSecTreatment"]);
                sumIAS += DataAccessFactory.getFloatValue(row["aggPercStringentTreatment"]);
                foreach (DataRow childRow in row.Row.GetChildRows(CtrlCollectionEdit.RNAME_AGGLO_UWWAGGLO))
                {
                    sumIAS += DataAccessFactory.getFloatValue(childRow["aucPercC2T"]);
                }

                if (Math.Round(sumIAS, 2) != Math.Round(DataAccessFactory.getFloatValue(row["aggC2"]), 2))
                {
                    txtPrimTreatment.BackColor = UWWTD2008.ColError;
                    txtSecTreatment.BackColor = UWWTD2008.ColError;
                    txtStringentTreatment.BackColor = UWWTD2008.ColError;
                    hasError = true;
                }
                else
                {
                    if (!hasError)
                    {
                        txtPrimTreatment.BackColor = UWWTD2008.ColNorm;
                        txtSecTreatment.BackColor = UWWTD2008.ColNorm;
                        txtStringentTreatment.BackColor = UWWTD2008.ColNorm;
                    }
                }
            }

            return !hasError;
        }

        private void OnValidateCollectionSums(object sender, EventArgs e)
        {
            checkTreatmentValues();
        }

        protected override bool ValidateForm(ref string errorText, DataRowState state)
        {
            bool retVal = true;
            if (!checkTreatmentValues())
            {
                //DataRowView row = BindingContext[dataSet, CtrlAgglomerationsEdit.TNAME_AGGLO].Current as DataRowView;
                //if (row != null)
                //{
                    errorText += "Treatment percentage sum is incorrect";
                    retVal = false;
                //}
            }
            return retVal;
        }
    }
}
