﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

namespace at.uba.uwwtd2008.components
{
    public partial class CtrlIndustryEdit : UWWTDBaseCtrl
    {
        public const string TNAME_IND = "t_industries";

        private const string SELECT_IND = "select * from T_Industries where repId=";

        EventHandler positionChangeHandler = null;

        private DataSet dataSet;
        private DataRowView currentRow;

        public CtrlIndustryEdit()
        {
            positionChangeHandler = new EventHandler(updateComponentView);
            InitializeComponent();
        }

        public override void OnBackButtonClick()
        {
            UWWTD2008MainForm mf = UWWTD2008MainForm.getMainForm(this);
            if (mf == null) return;

            if (currentRow != null) currentRow.EndEdit();
            if (dataSet.HasChanges()) 
            {
                if (!checkExitWithoutSave("Industries"))
                {
                    currentRow.BeginEdit();
                    return;
                }
                else
                {
                    dataSet.RejectChanges();
                }
            }

            BindingContext[dataSet, TNAME_IND].PositionChanged -= positionChangeHandler;
            if (PrevControl != null) mf.getSplitterPanel().setContentPane(PrevControl);
            else mf.OpenDataEditMain();
        }

        public override void OpenForm()
        {
            updateComponentView(BindingContext[dataSet, TNAME_IND], null);
            BindingContext[dataSet, TNAME_IND].PositionChanged += positionChangeHandler;
        }

        public override void PerformDataBinding(DataSet dataSet)
        {
            this.dataSet = dataSet;

            IDbConnection conn = mDataFactory.GetConnection();
            try
            {
                if (!dataSet.Tables.Contains(TNAME_IND)) 
                    mDataFactory.FillDataAdapter(
                        mDataFactory.CreateDataAdapter(
                            mDataFactory.CreateCommand(SELECT_IND + CurrentRep.RepID, conn)),
                            dataSet, TNAME_IND);
            }
            finally
            {
                conn.Close();
            }

            DataView dvBranches = mDataFactory.getLovViewByKey("BRANCH");
            dgvIndustries.AutoGenerateColumns = false;
            dgvIndustries.DataSource = dataSet;
            dgvIndustries.DataMember = TNAME_IND;
            ((DataGridViewComboBoxColumn)dgvIndustries.Columns["indBranch"]).DataSource = dvBranches;
            ((DataGridViewComboBoxColumn)dgvIndustries.Columns["indBranch"]).DisplayMember = "lovWert";
            ((DataGridViewComboBoxColumn)dgvIndustries.Columns["indBranch"]).ValueMember = "lovID";

            cmbBranch.DataBindings.Clear();
            cmbBranch.DataSource = dvBranches;
            cmbBranch.DisplayMember = "lovWert";
            cmbBranch.ValueMember = "lovID";
            cmbBranch.DataBindings.Add("SelectedValue", dataSet, TNAME_IND + ".indBranch", true);

            cmbState.DataBindings.Clear();
            cmbState.DataSource = mDataFactory.getLovViewByKey("STAT");
            cmbState.DisplayMember = "lovWert";
            cmbState.ValueMember = "lovID";
            cmbState.DataBindings.Add("SelectedValue", dataSet, TNAME_IND + ".indState", true);

            txtCodePlant.DataBindings.Clear();
            txtCodePlant.DataBindings.Add("Text", dataSet, TNAME_IND + ".indCodePlant", true);
            txtNamePlant.DataBindings.Clear();
            txtNamePlant.DataBindings.Add("Text", dataSet, TNAME_IND + ".indNamePlant", true);
            txtOrganicLoad.DataBindings.Clear();
            txtOrganicLoad.DataBindings.Add("Text", dataSet, TNAME_IND + ".indOrganicLoad", true);
            txtDateCompiliance.DataBindings.Clear();
            txtDateCompiliance.DataBindings.Add("Text", dataSet, TNAME_IND + ".indDateCompiliance", true, DataSourceUpdateMode.OnValidation, "", "d");

            ckbConditions.DataBindings.Clear();
            ckbConditions.DataBindings.Add("Checked", dataSet, TNAME_IND + ".indConditions", true, DataSourceUpdateMode.OnPropertyChanged, false);
        }

        private void updateComponentView(object sender, EventArgs e)
        {
            if (!(sender is BindingManagerBase)) return;
            BindingManagerBase binder = sender as BindingManagerBase;

            if (currentRow != null) currentRow.EndEdit();

            currentRow = null;
            if (((BindingManagerBase)sender).Position >= 0)
            {
                DataRowView view = binder.Current as DataRowView;
                if (view != null)
                {
                    currentRow = view;
                }
            }

            if (currentRow == null)
            {
                txtCodePlant.ReadOnly = true;
                txtNamePlant.ReadOnly = true;
                txtOrganicLoad.ReadOnly = true;
                txtDateCompiliance.ReadOnly = true;
                cmbBranch.Enabled = false;
                cmbState.Enabled = false;
                ckbConditions.Enabled = false;

                cmdDelete.Enabled = false;
            } // if
            else
            {
                txtCodePlant.ReadOnly = currentRow.Row.RowState != DataRowState.Added;
                txtNamePlant.ReadOnly = false;
                txtOrganicLoad.ReadOnly = false;
                txtDateCompiliance.ReadOnly = false;
                cmbBranch.Enabled = true;
                cmbState.Enabled = true;
                ckbConditions.Enabled = true;

                cmdDelete.Enabled = true;

                string errorText = "";
                ValidateForm(ref errorText, currentRow != null ? currentRow.Row.RowState : DataRowState.Detached);

                currentRow.BeginEdit();
            }
        }

        private void OnInsert(object sender, EventArgs e)
        {
            DataRow newRow = dataSet.Tables[TNAME_IND].NewRow();
            newRow["repID"] = CurrentRep.RepID;
            newRow["mslID"] = ((CtrlMemberStateLevelEdit)PrevControl).CurrentMslID;
            newRow["indState"] = mDataFactory.getLovRowByKey("STAT", "1")["lovID"];
            newRow["indCodePlant"] = CurrentRep.Reporter.State.Key;
            dataSet.Tables[TNAME_IND].Rows.Add(newRow);
            BindingContext[dataSet, TNAME_IND].Position = dataSet.Tables[TNAME_IND].Rows.IndexOf(newRow);
        }

        private void OnDelete(object sender, EventArgs e)
        {
            if (currentRow != null)
            {
                currentRow.CancelEdit();
                currentRow.Delete();
                currentRow = null;
            }
        }

        public override bool validateBeforeClose()
        {
            string errorText = "";
            if (!ValidateForm(ref errorText, currentRow != null ? currentRow.Row.RowState : DataRowState.Detached)) {
                MessageBox.Show(this, "Please correct Form Errors before save:\r\n" + errorText, "Form not valid", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                return false;
            }
            return true;
        }

        protected override bool ValidateForm(ref string errorText, DataRowState state) {
            if (currentRow == null) return true;
            bool isValidated = true;
            txtCodePlant.BackColor = UWWTD2008.ColNorm;
            if ((currentRow["indCodePlant"] is DBNull) || ((string)currentRow["indCodePlant"]).Equals(CurrentRep.Reporter.State.Key)) {
                txtCodePlant.BackColor = UWWTD2008.ColMandatory;
                txtCodePlant.ReadOnly = false;
                isValidated = false;
            }
            else {
                if (!(((string)currentRow["indCodePlant"]).StartsWith(CurrentRep.Reporter.State.Key))) {
                    txtCodePlant.BackColor = UWWTD2008.ColError;
                    txtCodePlant.ReadOnly = false;
                    isValidated = false;
                }
            }

            txtNamePlant.BackColor = UWWTD2008.ColNorm;
            if ((currentRow["indNamePlant"] is DBNull) || ((currentRow["indNamePlant"] as string).Length < 1)) {
                txtNamePlant.BackColor = UWWTD2008.ColMandatory;
                isValidated = false;
            }
            return isValidated;
        }

        private void OnSave(object sender, LinkLabelLinkClickedEventArgs e)
        {
            if (currentRow != null)
            {
                currentRow.EndEdit();
            }

            string errorText = "";
            if ((e != null) && (!ValidateForm(ref errorText, currentRow != null ? currentRow.Row.RowState : DataRowState.Detached))) {
                MessageBox.Show(this, "Please correct Form Errors before save:\r\n" + errorText, "Form not valid", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                return;
            }

            IDbConnection conn = mDataFactory.GetConnection();
            try
            {
                int pos = BindingContext[dataSet, TNAME_IND].Position;

                IDbCommand cmd = mDataFactory.CreateCommand(SELECT_IND + CurrentRep.RepID, conn);
                IDbDataAdapter adapter = mDataFactory.CreateFullDataAdapter(cmd);
                mDataFactory.UpdateDataAdapter(adapter, dataSet, TNAME_IND);
                mDataFactory.FillDataAdapter(adapter, dataSet, TNAME_IND);

                if ((BindingContext[dataSet, TNAME_IND].Count > pos) && (pos >= 0))
                    BindingContext[dataSet, TNAME_IND].Position = pos;
            }
            finally
            {
                conn.Close();
            }

            if (e != null)
                ShowSaveConfirmation();

        }

        private void OnValidation(object sender, EventArgs e)
        {
            string errorText = "";
            ValidateForm(ref errorText, currentRow != null ? currentRow.Row.RowState : DataRowState.Detached);
        }

    }
}
