﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

namespace at.uba.uwwtd2008.components
{
    public partial class CtrlRaArt23Edit : UWWTDBaseCtrl
    {
        private DataSet dataSet;
        private DataRowView currentRow;

        public const string TNAME_RA = CtrlReceivingAreasEdit.TNAME_RA;
        public CtrlRaArt23Edit()
        {
            InitializeComponent();
        }

        public override void OnBackButtonClick()
        {
            if (currentRow != null) currentRow.EndEdit();
            if (dataSet.HasChanges()) {
                if (!checkExitWithoutSave("Receiving Area")) return;
                dataSet.RejectChanges();
            }

            if (currentRow.Row.RowState == DataRowState.Added) dataSet.Tables[TNAME_RA].Rows.Remove(currentRow.Row);

            UWWTD2008MainForm mf = UWWTD2008MainForm.getMainForm(this);
            if (mf == null) return;

            if (PrevControl != null) mf.getSplitterPanel().setContentPane(PrevControl);
            else mf.OpenDataEditMain();
        }

        public override void OpenForm()
        {
            BindingContext[dataSet, TNAME_RA].Position = PrevControl.BindingContext[dataSet, TNAME_RA].Position;

            // fill the type-box per hand
            DataRowView rowView = BindingContext[dataSet, TNAME_RA].Current as DataRowView;
            if (rowView != null)
            {
                currentRow = rowView;
                txtRAType.Text = mDataFactory.getLOVTable().Rows.Find(rowView["rcaType"])["lovWert"] as string;
                txtRcaName.Text = CurrentRep.Reporter.State.Value;
                txtRcaCode.ReadOnly = rowView.Row.RowState != DataRowState.Added;
            }
            else
            {
                // we should never end up in here
                currentRow = null;
                txtRAType.Text = "";
            }
        }

        public override void PerformDataBinding(DataSet dataSet)
        {
            this.dataSet = dataSet;

            // Do DataBindings
            txtRcaCode.DataBindings.Clear();
            txtRcaCode.DataBindings.Add("Text", dataSet, TNAME_RA + ".rcaCode");

            txtRcaDate58.DataBindings.Clear();
            txtRcaDate58.DataBindings.Add("Text", dataSet, TNAME_RA + ".rcaDateArt58", true, DataSourceUpdateMode.OnValidation, "", "d");
            txtRcaDateDesignation.DataBindings.Clear();
            txtRcaDateDesignation.DataBindings.Add("Text", dataSet, TNAME_RA + ".rcaDateDesignation", true, DataSourceUpdateMode.OnValidation, "", "d");

            chkParameterN.DataBindings.Clear();
            chkParameterN.DataBindings.Add("Checked", dataSet, TNAME_RA + ".rcaParameterN", true, DataSourceUpdateMode.OnValidation, false);
            chkParameterP.DataBindings.Clear();
            chkParameterP.DataBindings.Add("Checked", dataSet, TNAME_RA + ".rcaParameterP", true, DataSourceUpdateMode.OnValidation, false);
            chkParameterOther.DataBindings.Clear();
            chkParameterOther.DataBindings.Add("Checked", dataSet, TNAME_RA + ".rcaParameterOther", true, DataSourceUpdateMode.OnValidation, false);

            txtRcaRemarks.DataBindings.Clear();
            txtRcaRemarks.DataBindings.Add("Text", dataSet, TNAME_RA + ".rcaRemarks");
            txtRcaCRelevantDirective.DataBindings.Clear();
            txtRcaCRelevantDirective.DataBindings.Add("Text", dataSet, TNAME_RA + ".rcaCRelevantDirective");
        }

        private void OnSaveLinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            string errorText = "";
            if (!ValidateForm(ref errorText, currentRow != null ? currentRow.Row.RowState : DataRowState.Detached)) {
                MessageBox.Show(this, "Please correct Form Errors before save:\r\n" + errorText, "Form not valid", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                return;
            }

            currentRow.EndEdit();
            saveData();
            ShowSaveConfirmation();
        }

        private void OnDeleteLinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            if (ShowConfirmDelete()) {
                if ((currentRow.Row.RowState == DataRowState.Added) ||
                    (currentRow.Row.RowState == DataRowState.Detached)) {
                    dataSet.Tables[TNAME_RA].Rows.Remove(currentRow.Row);
                }
                else {
                    currentRow.Delete();
                    currentRow.EndEdit();
                    saveData();
                }
                UWWTD2008MainForm mf = UWWTD2008MainForm.getMainForm(this);
                if (mf == null) return;

                if (PrevControl != null) mf.getSplitterPanel().setContentPane(PrevControl);
                else mf.OpenDataEditMain();
            }
        }

        private void saveData()
        {
            IDbConnection conn = mDataFactory.GetConnection();
            try
            {
                IDbDataAdapter adapter = mDataFactory.CreateFullDataAdapter(
                            mDataFactory.CreateCommand("select * from T_ReceivingAreas WHERE repID = " + CurrentRep.RepID, conn)
                        );

                mDataFactory.UpdateDataAdapter(adapter, dataSet, TNAME_RA);
                mDataFactory.FillDataAdapter(adapter, dataSet, TNAME_RA);
                // check, if this was a delete operation ....
                if (dataSet.Tables[TNAME_RA].Rows.Count > 0)
                {
                    DataRowView rowView = BindingContext[dataSet, TNAME_RA].Current as DataRowView;
                    if (rowView != null)
                    {
                        currentRow = rowView;
                        txtRAType.Text = mDataFactory.getLOVTable().Rows.Find(rowView["rcaType"])["lovWert"] as string;
                        txtRcaName.Text = CurrentRep.Reporter.State.Value;
                        txtRcaCode.ReadOnly = rowView.Row.RowState != DataRowState.Added;
                    }
                }
            }
            finally
            {
                conn.Close();
            }
        }

        protected override bool ValidateForm(ref string errorText, DataRowState state) {
            bool retVal = true;
            if ((txtRcaCode.Text == null) || (txtRcaCode.Text.Length < 2) || !(txtRcaCode.Text.StartsWith(CurrentRep.Reporter.State.Key))) {
                errorText += "ID has to start with " + CurrentRep.Reporter.State.Key;
                txtRcaCode.BackColor = UWWTD2008.ColError;
                retVal = false;
            }
            else {
                txtRcaCode.BackColor = UWWTD2008.ColNorm;
            }

            return retVal;
        }
    }
}
