﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

namespace at.uba.uwwtd2008.components
{
    public partial class CtrlRaLSA : UWWTDBaseCtrl
    {
        public const string TNAME_RA = CtrlReceivingAreasEdit.TNAME_RA;
        private DataSet dataSet;
        DataView dv;
        private DataRowView currentRow;

        public CtrlRaLSA()
        {
            InitializeComponent();
        }

        public override void OnBackButtonClick()
        {
            if (currentRow != null) currentRow.EndEdit();
            if (dataSet.HasChanges()) {
                if (!checkExitWithoutSave("Receiving Area")) return;
                dataSet.RejectChanges();
            }

            UWWTD2008MainForm mf = UWWTD2008MainForm.getMainForm(this);
            if (mf == null) return;

            if (PrevControl != null) mf.getSplitterPanel().setContentPane(PrevControl);
            else mf.OpenDataEditMain();
        }

        public override void OpenForm()
        {
            if (BindingContext[dv].Count > 0)
                currentRow = (DataRowView)BindingContext[dv].Current;
            updateComponentView();
        }

        public override void PerformDataBinding(DataSet dataSet)
        {
            this.dataSet = dataSet;

            dv = new DataView(dataSet.Tables[TNAME_RA]);
            dv.RowFilter = "rcaType = " + mDataFactory.getRATYPLovValues().Select("lovKey='LSA'")[0]["lovID"];

            DataView dvlov = new DataView(mDataFactory.getLOVTable());
            string rowFilter = "";

            foreach (DataRow row in mDataFactory.getRATYPLovValues().Select("lovKey='LSA'"))
            {
                if (rowFilter.Length > 5)
                    rowFilter += " OR ";
                rowFilter += "lovID = " + row["lovID"];
            }
            dvlov.RowFilter = rowFilter;
            dv.RowFilter = rowFilter.Replace("lovID", "rcaType");
            dv.AllowEdit = true;
            dv.AllowDelete = true;
            dv.AllowNew = true;

            BindingContext[dv].CurrentChanged += new EventHandler(OnCurrentLSAChanged);

            DataView dvStates = mDataFactory.getLovSTAT();

            gridLessSensitiveArea.AutoGenerateColumns = false;
            gridLessSensitiveArea.DataSource = dv;
            //((DataGridViewComboBoxColumn)gridLessSensitiveArea.Columns["rcaState"]).DataSource = dvStates;
            //((DataGridViewComboBoxColumn)gridLessSensitiveArea.Columns["rcaState"]).ValueMember = "lovID";
            //((DataGridViewComboBoxColumn)gridLessSensitiveArea.Columns["rcaState"]).DisplayMember = "lovWert";

            cmbState.DataBindings.Clear();
            cmbState.DataSource = dvStates;
            cmbState.ValueMember = "lovID";
            cmbState.DisplayMember = "lovWert";
            cmbState.DataBindings.Add("SelectedValue", dv, "rcaState");

            cmbRaType.DataBindings.Clear();
            cmbRaType.DataSource = dvlov;
            cmbRaType.DisplayMember = "lovWert";
            cmbRaType.ValueMember = "lovID";
            cmbRaType.DataBindings.Add("SelectedValue", dv, "rcaType");

            txtCode.DataBindings.Clear();
            txtCode.DataBindings.Add("Text", dv, "rcaCode", true, DataSourceUpdateMode.OnValidation, "");

            txtName.DataBindings.Clear();
            txtName.DataBindings.Add("Text", dv, "rcaName", true, DataSourceUpdateMode.OnValidation, "");

            txtDateDesignation.DataBindings.Clear();
            txtDateDesignation.DataBindings.Add("Text", dv, "rcaDateDesignation", true, DataSourceUpdateMode.OnValidation, "", "d");

            ckbMorphology.DataBindings.Clear();
            ckbMorphology.DataBindings.Add("Checked", dv, "rcaMorphology", true, DataSourceUpdateMode.OnValidation, false);
            ckbHydrologie.DataBindings.Clear();
            ckbHydrologie.DataBindings.Add("Checked", dv, "rcaHydrologie", true, DataSourceUpdateMode.OnValidation, false);
            ckbHydraulic.DataBindings.Clear();
            ckbHydraulic.DataBindings.Add("Checked", dv, "rcaHydraulic", true, DataSourceUpdateMode.OnValidation, false);
            ckbAbsenceRisk.DataBindings.Clear();
            ckbAbsenceRisk.DataBindings.Add("Checked", dv, "rcaAbsenceRisk", true, DataSourceUpdateMode.OnValidation, false);

            txtGIS.DataBindings.Clear();
            txtGIS.DataBindings.Add("Text", dv, "rcaGIS", true, DataSourceUpdateMode.OnValidation, "");

            txtRemarks.DataBindings.Clear();
            txtRemarks.DataBindings.Add("Text", dv, "rcaRemarks", true, DataSourceUpdateMode.OnValidation, "");
        }

        private void OnNewLSAClicked(object sender, EventArgs e)
        {
            DataRow newRow = dataSet.Tables[TNAME_RA].NewRow();
            newRow["repID"] = CurrentRep.RepID;
            newRow["rcaState"] = mDataFactory.getLovRowByKey("STAT", "1")["lovID"];
            newRow["rcaType"] = mDataFactory.getRATYPLovValues().Select("lovKey='LSA'")[0]["lovID"];
            newRow["rcaCode"] = CurrentRep.Reporter.State.Key;
            dataSet.Tables[TNAME_RA].Rows.Add(newRow);
            BindingContext[dv].Position = BindingContext[dv].Count - 1;
        }

        private void updateComponentView()
        {
            if (currentRow == null)
            {
                foreach (Binding b in BindingContext[dv].Bindings)
                {
                    Control ctrl = b.BindableComponent as Control;
                    if (ctrl != null) ctrl.Enabled = false;
                }

                cmdDelete.Enabled = false;
            } // if
            else
            {
                foreach (Binding b in BindingContext[dv].Bindings)
                {
                    Control ctrl = b.BindableComponent as Control;
                    if (ctrl != null) ctrl.Enabled = true;
                }

                cmdDelete.Enabled = true;
                txtCode.ReadOnly = currentRow.Row.RowState != DataRowState.Added;
                txtName.ReadOnly = false;
                cmbRaType.Enabled = false;//currentRow.RowState == DataRowState.Added;
            }
        }

        private void OnCurrentLSAChanged(object sender, EventArgs e)
        {
            if (currentRow != null)
            {
                currentRow.CancelEdit();
                currentRow = null;
            }

            if (BindingContext[dv].Position >= 0)
            {
                DataRowView view = BindingContext[dv].Current as DataRowView;
                if (view != null)
                {
                    currentRow = view;
                    currentRow.BeginEdit();
                    if (currentRow.Row.RowState == DataRowState.Added)
                    {
                        txtCode.BackColor = UWWTD2008.ColMandatory;
                        txtName.BackColor = UWWTD2008.ColMandatory;
                        txtDateDesignation.BackColor = UWWTD2008.ColMandatory;
                        txtGIS.BackColor = UWWTD2008.ColMandatory;
                    }
                    else
                    {
                        txtCode.BackColor = UWWTD2008.ColNorm;
                        txtName.BackColor = UWWTD2008.ColNorm;
                        txtDateDesignation.BackColor = UWWTD2008.ColNorm;
                        txtGIS.BackColor = UWWTD2008.ColNorm;
                    }
                }
            }

            updateComponentView();
        }

        private void LinkSaveClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {

            if (currentRow != null)
                currentRow.EndEdit();

            string errorText = "";
            if (!ValidateForm(ref errorText, currentRow != null ? currentRow.Row.RowState : DataRowState.Detached)) {
                MessageBox.Show(this, "Please correct Form Errors before save:\r\n" + errorText, "Form not valid", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                return;
            }
            
            saveData();
            //if (currentRow != null)
            //    currentRow.BeginEdit();
            updateComponentView();

            ShowSaveConfirmation();
        }

        private void saveData()
        {
            IDbConnection conn = mDataFactory.GetConnection();
            try
            {
                mDataFactory.UpdateDataAdapter(
                        mDataFactory.CreateFullDataAdapter(
                            mDataFactory.CreateCommand("select * from T_ReceivingAreas WHERE repID = " + CurrentRep.RepID, conn)
                        ), dataSet, TNAME_RA
                    );
            }
            finally
            {
                conn.Close();
            }
        }

        private void OnDGVDataError(object sender, DataGridViewDataErrorEventArgs e)
        {
            DataRowView v = gridLessSensitiveArea.Rows[e.RowIndex].DataBoundItem as DataRowView;
            if (v != null)
            {
                if (e.ColumnIndex == 2)
                { // this is the "state" - column - set to default 
                    v.Row["rcaState"] = mDataFactory.getLovRowByKey("STAT", "1")["lovID"];
                    return;
                }
            }

            e.Cancel = true;
        }

        private void OnDeleteRA(object sender, EventArgs e)
        {
            if (BindingContext[dv].Count < 1) return;
            if (ShowConfirmDelete()) {
                DataRowView row = (DataRowView)BindingContext[dv].Current;
                if (row != null) {
                    row.Delete();
                }
            }
        }

        protected override bool ValidateForm(ref string errorText, DataRowState state) {
            bool retVal = true;
            if ((txtCode.Text == null) || (txtCode.Text.Length < 2) || !(txtCode.Text.StartsWith(CurrentRep.Reporter.State.Key))) {
                errorText += "ID has to start with " + CurrentRep.Reporter.State.Key;
                txtCode.BackColor = UWWTD2008.ColError;
                retVal = false;
            }
            else {
                if ((dataSet.Tables[TNAME_RA].Select("rcaCode = '" + txtCode.Text + "'").Length > 0) &&
                    (dataSet.Tables[TNAME_RA].Select("rcaCode = '" + txtCode.Text + "'")[0] != currentRow.Row)) {
                    txtCode.BackColor = UWWTD2008.ColError;
                    if (errorText.Length > 1) errorText += "\r\n";
                    errorText += "Code for ID already used";
                    retVal = false;
                }
                else {
                    txtCode.BackColor = UWWTD2008.ColNorm;
                }
            }

            if ((txtName.Text == null) || (txtName.Text.Length < 1)) {
                if (errorText.Length > 0) errorText += "\r\n";
                errorText += "Name is Master Data";
                retVal = false;
                txtName.BackColor = UWWTD2008.ColError;
            }
            else {
                txtName.BackColor = UWWTD2008.ColNorm;
            }

            if ((txtDateDesignation.Text == null) || (txtDateDesignation.Text.Length < 1)) {
                if (errorText.Length > 0) errorText += "\r\n";
                errorText += "Designation Date is Master Data";
                retVal = false;
                txtDateDesignation.BackColor = UWWTD2008.ColError;
            }
            else {
                if ((currentRow["rcaDateDesignation"] != null) && !(currentRow["rcaDateDesignation"] is DBNull))
                {
                    DateTime date = (DateTime) currentRow["rcaDateDesignation"];
                    if (date.CompareTo(UWWTD2008.MIN_VALID_DATE) < 0)
                    {
                        if (errorText.Length > 0) errorText += "\r\n";
                        errorText += "Designation Date is too far in the past";
                        retVal = false;
                        txtDateDesignation.BackColor = UWWTD2008.ColError;
                    }
                    else if (date.CompareTo(UWWTD2008.MAX_VALID_DATE) > 0)
                    {
                        if (errorText.Length > 0) errorText += "\r\n";
                        errorText += "Designation Date is too far in the future";
                        retVal = false;
                        txtDateDesignation.BackColor = UWWTD2008.ColError;
                    }
                    else
                        txtDateDesignation.BackColor = UWWTD2008.ColNorm;

                }
                else
                    txtDateDesignation.BackColor = UWWTD2008.ColNorm;
            }

            if ((txtGIS.Text == null) || (txtGIS.Text.Length < 1)) {
                if (errorText.Length > 0) errorText += "\r\n";
                errorText += "GIS Data is Master Data";
                retVal = false;
                txtGIS.BackColor = UWWTD2008.ColError;
            }
            else {
                txtGIS.BackColor = UWWTD2008.ColNorm;
            }

            //if (!ckbMorphology.Checked && !ckbHydrologie.Checked && !ckbHydraulic.Checked && !ckbAbsenceRisk.Checked) {
            //    if (errorText.Length > 0) errorText += "\r\n";
            //    errorText += "Designation Criteria is Master Data";
            //    retVal = false;
            //    ckbMorphology.BackColor = UWWTD2008.ColError;
            //    ckbHydrologie.BackColor = UWWTD2008.ColError;
            //    ckbHydraulic.BackColor = UWWTD2008.ColError;
            //    ckbAbsenceRisk.BackColor = UWWTD2008.ColError;
            //}
            //else {
            //    ckbMorphology.BackColor = UWWTD2008.ColNorm;
            //    ckbHydrologie.BackColor = UWWTD2008.ColNorm;
            //    ckbHydraulic.BackColor = UWWTD2008.ColNorm;
            //    ckbAbsenceRisk.BackColor = UWWTD2008.ColNorm;
            //}

            return retVal;
        }
    }
}
