﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

namespace at.uba.uwwtd2008.components
{
    public partial class CtrlReceivingAreasEdit : UWWTDBaseCtrl
    {
        private DataSet dataSet;
        public const string TNAME_RA = "t_receivingareas";
        public const string TNAME_REP = "t_reportperiod";

        public const int RATYP_ART5253 = 1;
        public const int RATYP_ART54 = 2;
        public const int RATYP_CSASALSA = 4;
        public const int RATYP_ART51 = 8;
        public const int RATYP_ART61 = 16;
        public const int RATYP_NORMAL = 0;

        private bool isCBUpdating = false;

        public CtrlReceivingAreasEdit()
        {
            InitializeComponent();
        }

        public override void OnBackButtonClick()
        {
            UWWTD2008MainForm mf = UWWTD2008MainForm.getMainForm(this);
            if (mf == null) return;

            if (dataSet.HasChanges()) {
                if (!checkExitWithoutSave("Receiving Areas")) return;
                dataSet.RejectChanges();
            }

            if (PrevControl != null) mf.getSplitterPanel().setContentPane(PrevControl);
            else mf.OpenDataEditMain();
        }

        public override void OpenForm()
        {
            if ((dataSet == null) || (CurrentRep == null))
            {
                MessageBox.Show(UWWTD2008MainForm.getMainForm(this), "Please select a valid ReportPeriod first", "No ReportPeriod selected", MessageBoxButtons.OK, MessageBoxIcon.Error);
                UWWTD2008MainForm.getMainForm(this).OpenDataEditMain();
                return;
            }
            updateCheckBoxes();
        }

        public static void updateTNAME_RATable(DataSet dataSet, IDbConnection conn)
        {
            DataAccess.DataAccessFactory dataFactory = DataAccess.DataAccessFactory.Instance;
            dataFactory.FillDataAdapter(
                dataFactory.CreateDataAdapter(
                    dataFactory.CreateCommand("select * from T_ReceivingAreas WHERE repID = " + CurrentRep.RepID, conn)
                ), dataSet, TNAME_RA);

            dataSet.Tables[TNAME_RA].PrimaryKey = new DataColumn[] { dataSet.Tables[TNAME_RA].Columns["rcaID"] };
        }

        public override void PerformDataBinding(DataSet dataSet)
        {
            this.dataSet = dataSet;
            IDbConnection conn = mDataFactory.GetConnection();
            try
            {
                if (!dataSet.Tables.Contains(TNAME_RA))
                {
                    updateTNAME_RATable(dataSet, conn);
                }
                if (!dataSet.Tables.Contains(TNAME_REP))
                {
                    mDataFactory.FillDataAdapter(
                        mDataFactory.CreateDataAdapter(
                            mDataFactory.CreateCommand("select * from T_ReportPeriod WHERE repID = " + CurrentRep.RepID, conn)
                        ), dataSet, TNAME_REP);
                }
            }
            finally
            {
                conn.Close();
            }
        }

        private void Art5854Click(object sender, EventArgs e)
        {
            DataRow row = null;
            if ((dataSet.Tables[TNAME_RA].Rows.Count > 1) || (dataSet.Tables[TNAME_RA].Rows.Count < 1))
            {
                row = dataSet.Tables[TNAME_RA].NewRow();
                row["repID"] = CurrentRep.RepID;
                row["rcaCode"] = CurrentRep.Reporter.State.Key;
                row["rcaName"] = CurrentRep.Reporter.State.Value;
                dataSet.Tables[TNAME_RA].Rows.Add(row);
            }

            BindingContext[dataSet, TNAME_RA].Position = 0;
            row = (BindingContext[dataSet, TNAME_RA].Current as DataRowView).Row;
            row["rcaType"] = mDataFactory.getRATYPLovValues().Select("lovKey='A54'")[0]["lovID"];
            row["rcaState"] = mDataFactory.getLovRowByKey("STAT", "1")["lovID"];
            ((UWWTD2008MainForm)(this.Parent.Parent.Parent.Parent)).OpenRaArt5854Edit(this);
        }

        private void Art23Click(object sender, EventArgs e)
        {
            DataRow row = null;
            if (dataSet.Tables[TNAME_RA].Rows.Count < 1)
            {
                row = dataSet.Tables[TNAME_RA].NewRow();
                row["repID"] = CurrentRep.RepID;
                row["rcaCode"] = CurrentRep.Reporter.State.Key;
                row["rcaName"] = CurrentRep.Reporter.State.Value;
                dataSet.Tables[TNAME_RA].Rows.Add(row);
            }

            BindingContext[dataSet, TNAME_RA].Position = 0;
            row = (BindingContext[dataSet, TNAME_RA].Current as DataRowView).Row;
            row.BeginEdit();
            row["rcaType"] = mDataFactory.getRATYPLovValues().Select("lovKey='A58'")[0]["lovID"];
            row["rcaState"] = mDataFactory.getLovRowByKey("STAT", "1")["lovID"];
            row.EndEdit();
            ((UWWTD2008MainForm)(this.Parent.Parent.Parent.Parent)).OpenRaArt23Edit(this);
        }

        private void SAEditClick(object sender, EventArgs e)
        {
            ((UWWTD2008MainForm)(this.Parent.Parent.Parent.Parent)).OpenSAEdit(this);
        }

        private void LSAEditClick(object sender, EventArgs e)
        {
            ((UWWTD2008MainForm)(this.Parent.Parent.Parent.Parent)).OpenLSAEdit(this);
        }

        private void updateCheckBoxes()
        {
            isCBUpdating = true;
            try {
                mbutArt4.Enabled = false;
                mbutLSA.Enabled = false;
                mbutRAArt23.Enabled = false;
                mbutSA.Enabled = false;

                DataRowView rowView = BindingContext[dataSet, TNAME_REP].Current as DataRowView;
                if (rowView == null) return; // nothing we can do here
                DataRow row = rowView.Row;

                if (row["repRATypes"] == null || row["repRATypes"] is DBNull) {
                    // uncheck all
                    mradioRA_58.Checked = false;
                    mradioRA_NoSaLsa.Checked = false;
                    mradioRA5253.Checked = false;
                    mradioRA54.Checked = false;
                    mradioRASaLsa.Checked = false;
                    mcheckRA51.Checked = false;
                    mcheckRa61.Checked = false;
                }
                else {
                    int iType = (int)row["repRATypes"];
                    switch (iType) {
                        case RATYP_ART5253:
                            mradioRA_58.Checked = true;
                            mradioRA5253.Checked = true;
                            mbutRAArt23.Enabled = true;
                            break;
                        case RATYP_ART54:
                            mradioRA_58.Checked = true;
                            mradioRA54.Checked = true;
                            mbutArt4.Enabled = true;
                            break;
                        case RATYP_NORMAL:
                            mradioRA_NoSaLsa.Checked = true;
                            break;
                    }

                    if ((iType & RATYP_CSASALSA) != 0) {
                        mradioRASaLsa.Checked = true;
                        if ((iType & RATYP_ART51) != 0) {
                            mcheckRA51.Checked = true;
                            mbutSA.Enabled = true;
                        }
                        else
                            mcheckRA51.Checked = false;

                        if ((iType & RATYP_ART61) != 0) {
                            mcheckRa61.Checked = true;
                            mbutLSA.Enabled = true;
                        }
                        else
                            mcheckRa61.Checked = false;
                    }
                }

                grpArt58.Enabled = mradioRA_58.Checked;
                grpSALSA.Enabled = mradioRASaLsa.Checked;
            }
            finally {
                isCBUpdating = false;
            }
        }

        private void OnCheckChanged(object sender, EventArgs e)
        {
            DataRowView rowView = BindingContext[dataSet, TNAME_REP].Current as DataRowView;
            if (rowView == null) return;
            bool disableButtons = !isCBUpdating;

            rowView.BeginEdit();
            try
            {
                if (sender == mradioRA_58)
                {
                    grpArt58.Enabled = mradioRA_58.Checked;
                    return; // we do nothing more her ... rest is done in Enable-Event-Handler
                }
                if (sender == mradioRASaLsa)
                {
                    grpSALSA.Enabled = mradioRASaLsa.Checked;
                    if (!isCBUpdating && mradioRASaLsa.Checked) { // the radio was unchecked before - so we need to clear anything
                        if (!deleteAllRAs()) {
                            updateCheckBoxes();
                            disableButtons = false;
                            return;
                        }
                    }
                }

                if (sender == mradioRA_NoSaLsa)
                {
                    if (((RadioButton)sender).Checked)
                    {
                        if (!isCBUpdating) {
                            if (deleteAllRAs()) {
                                if (mradioRA_NoSaLsa.Checked)
                                    rowView["repRATypes"] = RATYP_NORMAL;
                            }
                            else {
                                updateCheckBoxes();
                                disableButtons = false;
                                return;
                            }
                        }
                    }
                    return;
                }
                if (sender == mradioRA5253)
                {
                    if (mradioRA5253.Checked) {
                        if (!isCBUpdating) {
                            if (!deleteAllRAs()) {
                                updateCheckBoxes();
                                disableButtons = false;
                                return;
                            }
                            rowView["repRATypes"] = RATYP_ART5253;
                        }
                    }
                    return;
                }
                if (sender == mradioRA54)
                {
                    if (mradioRA54.Checked) {
                        if (!isCBUpdating) {
                            if (!deleteAllRAs()) {
                                updateCheckBoxes();
                                disableButtons = false;
                                return;
                            }
                            rowView["repRATypes"] = RATYP_ART54;
                        }
                    }
                    return;
                }
                if ((sender == mradioRASaLsa) || (sender == mcheckRA51) || (sender == mcheckRa61))
                {
                    if (mradioRASaLsa.Checked)
                    {
                        int iType = RATYP_CSASALSA;
                        if (mcheckRA51.Checked) iType |= RATYP_ART51;
                        if (mcheckRa61.Checked) iType |= RATYP_ART61;
                        rowView["repRATypes"] = iType;
                    }
                    return;
                }
                rowView["repRATypes"] = DBNull.Value;
            }
            finally
            {
                rowView.EndEdit();
                if (disableButtons)
                {
                    mbutArt4.Enabled = false;
                    mbutLSA.Enabled = false;
                    mbutRAArt23.Enabled = false;
                    mbutSA.Enabled = false;
                }
            }
        }

        private bool deleteAllRAs()
        {
            DataRowView rowView = BindingContext[dataSet, TNAME_REP].Current as DataRowView;
            if (rowView["repRATypes"] is DBNull) return true; //nothing to do if there was no data yet
            if (MessageBox.Show(this, "This is a crucial change! All existing data entries on receiving areas and links to discharge points will be deleted. Are you sure?", "Please Confirm", MessageBoxButtons.YesNo, MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button2) == DialogResult.No)
                return false;
            foreach (DataRow row in dataSet.Tables[TNAME_RA].Rows)
            {
                if (row.RowState != DataRowState.Deleted)
                    row.Delete();
            }
            return true;
        }

        private void OnGroupArt58Enabled(object sender, EventArgs e)
        {
            if (grpArt58.Enabled) {
                DataRowView rowView = BindingContext[dataSet, TNAME_REP].Current as DataRowView;
                if (rowView == null) return; // nothing we can do here
                DataRow row = rowView.Row;
                if (!(mradioRA5253.Checked || mradioRA54.Checked)) {
                    mradioRA5253.Checked = true;
                    //row["repRATypes"] = RATYP_ART5253;
                } // if
                //else {
                //    if (mradioRA5253.Checked) row["repRATypes"] = RATYP_ART5253;
                //    if (mradioRA54.Checked) row["repRATypes"] = RATYP_ART54;
                //}
            }
            else {
                mradioRA5253.Checked = false;
                mradioRA54.Checked = false;
            }
        }

        private void SaveLinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            DataRowView rowView = BindingContext[dataSet, TNAME_REP].Current as DataRowView;
            if (rowView == null) return;
            DataRow row = rowView.Row;

            IDbConnection conn = mDataFactory.GetConnection();
            try
            {
                if (row.RowState == DataRowState.Modified)
                {
                    mDataFactory.UpdateDataAdapter(
                        mDataFactory.CreateFullDataAdapter(
                            mDataFactory.CreateCommand("select * from T_ReportPeriod WHERE repID = " + CurrentRep.RepID, conn)
                        ), dataSet, TNAME_REP
                    );
                }

                mDataFactory.UpdateDataAdapter(
                        mDataFactory.CreateFullDataAdapter(
                            mDataFactory.CreateCommand("select * from T_ReceivingAreas WHERE repID = " + CurrentRep.RepID, conn)
                        ), dataSet, TNAME_RA
                    );
            }
            finally
            {
                conn.Close();
            }


            updateCheckBoxes();
        }
    }
}
