using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using at.uba.uwwtd2008.DataAccess;
using at.uba.uwwtd2008.Items;

namespace at.uba.uwwtd2008.components
{
    public partial class CtrlReporterEdit : UWWTDBaseCtrl
    {
        DataSet dataSet = null;
        DataRowView currentReporter = null;
        DataRowView currentReportPeriod = null;
        DataRowView currentContact = null;
        DataColumnChangeEventHandler changeHandler = null;
        DataRowChangeEventHandler deleteHandler = null;

        public CtrlReporterEdit()
        {
            changeHandler = new DataColumnChangeEventHandler(DataValueChanging);
            deleteHandler = new DataRowChangeEventHandler(DeletingRow);
            InitializeComponent();
        }


        public override void OnBackButtonClick()
        {
            UWWTD2008MainForm mf = UWWTD2008MainForm.getMainForm(this);
            if (mf == null) return;

            if (dataSet.HasChanges()) {
                if (!checkExitWithoutSave("Reporter")) {
                    return;
                }
            }

            ReporterBindingSource.CancelEdit();
            ReportPeriodBindingSource.CancelEdit();
            ContactBindingSource.CancelEdit();

            if (ContactBindingSource.Count < 1) {
                MessageBox.Show(this, "Please insert at least one contact person.", "Form Validation Error", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                return;
            }

            if (PrevControl != null) mf.getSplitterPanel().setContentPane(PrevControl);
            else mf.OpenOpenerWindow();

            dataSet.Tables["T_Reporter"].ColumnChanged -= changeHandler;
            dataSet.Tables["T_ReportPeriod"].ColumnChanged -= changeHandler;
            dataSet.Tables["T_Contact"].ColumnChanged -= changeHandler;

            dataSet.Tables["T_Reporter"].RowDeleting -= deleteHandler;
            dataSet.Tables["T_ReportPeriod"].RowDeleting -= deleteHandler;
            dataSet.Tables["T_Contact"].RowDeleting -= deleteHandler;

            dataSet.RejectChanges();
            
            try
            {
                saveAllData();
            }
            catch (Exception) { }

            DataRowView rv = ReportPeriodBindingSource.Current as DataRowView;
            if (rv != null)
            {
                CurrentRep = ReportPeriod.LoadReportPeriod(long.Parse(rv["repID"].ToString()));
                Properties.Settings.Default.LastReportPeriod = long.Parse(rv["repID"].ToString());
                Properties.Settings.Default.Save();
                mf.getSplitterPanel().UpdateDataSet();
            }
            else
            {
                CurrentRep = null;
            }
        }

        public override void OpenForm()
        {
            DataAccessFactory factory = DataAccessFactory.Instance;

            comboMemberState.DataSource = factory.getLovViewByKey("MS");
            comboMemberState.DisplayMember = "lovWert";
            comboMemberState.ValueMember = "lovID";

            // some major trouble prevented me from doing the changes for bug#9232 here
            dataSet = mDataFactory.getReporterDataSet();
            ReporterBindingSource.DataSource = dataSet;
            ReportPeriodBindingSource.DataSource = dataSet;
            ContactBindingSource.DataSource = dataSet;

            DataRow current = dataSet.Tables["T_ReportPeriod"].Rows.Find(Properties.Settings.Default.LastReportPeriod);
            if (current != null)
            {
                ReporterBindingSource.Position = dataSet.Tables["T_Reporter"].Rows.IndexOf(dataSet.Tables["T_Reporter"].Rows.Find(current["rptID"]));
                ReportPeriodBindingSource.Position = dataSet.Tables["T_ReportPeriod"].Rows.IndexOf(current);
            }

            CurrentReporterChanged(ReporterBindingSource, null);

            //comboMemberState.Enabled = true;
            comboMemberState.DataBindings.Clear();
            comboMemberState.DataBindings.Add("SelectedValue", ReporterBindingSource, "rptMState");
            
            reportedYear.DataBindings.Clear();
            reportedYear.DataBindings.Add("Text", ReportPeriodBindingSource, "repReportedPeriod");
            reportID.DataBindings.Clear();
            reportID.DataBindings.Add("Text", ReportPeriodBindingSource, "repCode");
            reportVersion.DataBindings.Clear();
            reportVersion.DataBindings.Add("Text", ReportPeriodBindingSource, "repVersion", true, DataSourceUpdateMode.OnValidation, "", "d");
            situationAt.DataBindings.Clear();
            situationAt.DataBindings.Add("Text", ReportPeriodBindingSource, "repSituationAt", true, DataSourceUpdateMode.OnValidation, "", "d");

            txtConCity.DataBindings.Clear();
            txtConCity.DataBindings.Add("Text", ContactBindingSource, "conCity");
            txtConEmail.DataBindings.Clear();
            txtConEmail.DataBindings.Add("Text", ContactBindingSource, "conEmail");
            txtConFax.DataBindings.Clear();
            txtConFax.DataBindings.Add("Text", ContactBindingSource, "conFax");
            txtConInstitution.DataBindings.Clear();
            txtConInstitution.DataBindings.Add("Text", ContactBindingSource, "conInstitution");
            txtConName.DataBindings.Clear();
            txtConName.DataBindings.Add("Text", ContactBindingSource, "conName");
            txtConPhone.DataBindings.Clear();
            txtConPhone.DataBindings.Add("Text", ContactBindingSource, "conPhone");
            txtConRemarks.DataBindings.Clear();
            txtConRemarks.DataBindings.Add("Text", ContactBindingSource, "conRemarks");
            txtConStreet.DataBindings.Clear();
            txtConStreet.DataBindings.Add("Text", ContactBindingSource, "conStreet");
            txtConZIP.DataBindings.Clear();
            txtConZIP.DataBindings.Add("Text", ContactBindingSource, "conZIP");

            dataSet.Tables["T_Contact"].ColumnChanged += changeHandler;
            dataSet.Tables["T_ReportPeriod"].ColumnChanged += changeHandler;
            dataSet.Tables["T_Reporter"].ColumnChanged += changeHandler;

            dataSet.Tables["T_Contact"].RowDeleting += deleteHandler;
            dataSet.Tables["T_ReportPeriod"].RowDeleting += deleteHandler;
            dataSet.Tables["T_Reporter"].RowDeleting += deleteHandler;
        }


        private void OnLoadForm(object sender, EventArgs e) {
        }

        void DataValueChanging(object sender, DataColumnChangeEventArgs e)
        {
            if (sender is DataTable)
            {
                DataTable table = sender as DataTable;
                switch (table.TableName)
                {
                    case "T_Reporter":
                        saveToolStripButton.Visible = ValidateReporter(e.Row);
                        break;
                    case "T_ReportPeriod":
                        saveReportIDButton.Visible = ValidateReportID(e.Row, e.Column.ColumnName);
                        break;
                    case "T_Contact":
                        saveContactButton.Visible = ValidateContact(e.Row, e.Column.ColumnName);
                        break;
                }
            }
        }

        bool ValidateReporter(DataRow row)
        {
            bool retVal = true;

            if ((row == null) || (row.RowState == DataRowState.Detached))
            {
                if (row == null) {
                    comboMemberState.Enabled = false;
                }
                else {
                    comboMemberState.Enabled = true;
                    comboMemberState.BackColor = UWWTD2008.ColMandatory;
                }
                reportedYear.Enabled = false;
                reportID.Enabled = false;
                situationAt.Enabled = false;
                txtConCity.Enabled = false;
                txtConEmail.Enabled = false;
                txtConFax.Enabled = false;
                txtConInstitution.Enabled = false;
                txtConName.Enabled = false;
                txtConPhone.Enabled = false;
                txtConRemarks.Enabled = false;
                txtConStreet.Enabled = false;
                txtConZIP.Enabled = false;
                return true;
            }

            if ((comboMemberState.SelectedValue == null) || (comboMemberState.SelectedValue is DBNull))
            {
                comboMemberState.BackColor = UWWTD2008.ColError;
                retVal = false;
            }
            else
            {
                comboMemberState.BackColor = UWWTD2008.ColNorm;
                DataRow []rows = dataSet.Tables["T_Reporter"].Select("rptMState = " + comboMemberState.SelectedValue);
                if (rows.Length > 0)
                {
                    if (rows[0] != row)
                    {
                        retVal = false;
                        comboMemberState.BackColor = UWWTD2008.ColError;
                    }
                }
            }

            return retVal;
        }

        bool ValidateReportID(DataRow row, string column)
        {
            bool retVal = true;

            if ((row == null) || (row.RowState == DataRowState.Detached))
            {
                txtConCity.Enabled = false;
                txtConEmail.Enabled = false;
                txtConFax.Enabled = false;
                txtConInstitution.Enabled = false;
                txtConName.Enabled = false;
                txtConPhone.Enabled = false;
                txtConRemarks.Enabled = false;
                txtConStreet.Enabled = false;
                txtConZIP.Enabled = false;

                if (row != null) 
                    addReportPeriod.Visible = false;
                else
                    addReportPeriod.Visible = true;
            }
            else
                addReportPeriod.Visible = true;

            if (column == null || column.Equals("repCode"))
            {
                if (!reportID.Text.StartsWith((string)currentReporter["rptMStateKey"]))
                {
                    reportID.BackColor = UWWTD2008.ColError;
                    retVal = false;
                }
                else
                    reportID.BackColor = UWWTD2008.ColNorm;
            }

            if ((column == null) || (column.Equals("repSituationAt")))
            {
                if ((row != null) && ((row.RowState == DataRowState.Detached) || (row.RowState == DataRowState.Added)))
                {
                    if (situationAt.Text.Length < 1)
                    {
                        situationAt.BackColor = UWWTD2008.ColError;
                        retVal = false;
                    }
                    else
                    {
                        if (row["repSituationAt"] is DateTime)
                        {
                            DateTime aggPeriodOver = (DateTime)row["repSituationAt"];
                            if ((aggPeriodOver != null) && (aggPeriodOver.CompareTo(UWWTD2008.MIN_VALID_DATE_REP) < 0))
                            {
                                situationAt.BackColor = UWWTD2008.ColError;
                                retVal = false;
                            }
                            else if ((aggPeriodOver != null) && (aggPeriodOver.CompareTo(UWWTD2008.MAX_VALID_DATE) > 0))
                            {
                                situationAt.BackColor = UWWTD2008.ColError;
                                retVal = false;
                            }
                            else
                            {
                                row["repReportedPeriod"] = ((DateTime)row["repSituationAt"]).Year.ToString();
                                reportedYear.Text = ((DateTime)row["repSituationAt"]).Year.ToString();
                                situationAt.BackColor = UWWTD2008.ColNorm;
                            }
                        }
                        else situationAt.BackColor = UWWTD2008.ColNorm;

                    }
                }
            }

            // This functionality is obsolete -- the reported period will be taken from "Situation At"
            //if (column == null || column.Equals("repReportedPeriod"))
            //{
            //    if (reportedYear.Text.Length != 4)
            //    {
            //        reportedYear.BackColor = UWWTD2008.ColError;
            //        retVal = false;
            //    }
            //    else
            //    {
            //        try
            //        {
            //            if (!long.Parse(reportedYear.Text).ToString().Equals(reportedYear.Text))
            //            {
            //                reportedYear.BackColor = UWWTD2008.ColError;
            //                retVal = false;
            //            }
            //            reportedYear.BackColor = UWWTD2008.ColNorm;
            //        }
            //        catch (Exception)
            //        {
            //            reportedYear.BackColor = UWWTD2008.ColError;
            //            retVal = false;
            //        }
            //    }
            //}

            return retVal;
        }

        bool ValidateContact(DataRow row, string column)
        {
            bool retVal = true;
            if (column == null || column.Equals("conName"))
            {
                if (txtConName.Text.Length<4)
                {
                    txtConName.BackColor = UWWTD2008.ColError;
                    retVal = false;
                }
                else
                    txtConName.BackColor = UWWTD2008.ColNorm;
            }

            if (column == null || column.Equals("conInstitution"))
            {
                if (txtConInstitution.Text.Length < 1)
                {
                    txtConInstitution.BackColor = UWWTD2008.ColError;
                    retVal = false;
                }
                else
                    txtConInstitution.BackColor = UWWTD2008.ColNorm;
            }

            //if (column == null || column.Equals("conStreet"))
            //{
            //    if (txtConStreet.Text.Length < 4)
            //    {
            //        txtConStreet.BackColor = UWWTD2008.ColError;
            //        retVal = false;
            //    }
            //    else
            //        txtConStreet.BackColor = UWWTD2008.ColNorm;
            //}

            if (column == null || column.Equals("conPhone"))
            {
                if (txtConPhone.Text.Length < 4)
                {
                    txtConPhone.BackColor = UWWTD2008.ColError;
                    retVal = false;
                }
                else
                    txtConPhone.BackColor = UWWTD2008.ColNorm;
            }

            return retVal;
        }

        void CurrentReporterChanged(object sender, EventArgs e)
        {
            DataRowView view = ((BindingSource) sender).Current as DataRowView;
            currentReporter = view;
            if ((view == null) || (view.Row == null))
            {
                // make everything unreadable
                comboMemberState.Enabled = false;
                ReportPeriodBindingSource.Filter = "0=1";
                saveToolStripButton.Visible = false;
                addReportPeriod.Visible = false;
                addContactButton.Visible = false;
                // update field visibility
                ValidateReporter(null);
                return;
            }

            ReportPeriodBindingSource.Filter = "rptID = " + view.Row["rptID"];
            if ((view.Row.RowState == DataRowState.Added) || (view.Row.RowState == DataRowState.Detached))
            {
                comboMemberState.Enabled = true;
                ReportPeriodBindingSource.Filter = "0=1";
                saveToolStripButton.Visible = true;
                addReportPeriod.Visible = false;
                addContactButton.Visible = false;
            }
            else
            {
                comboMemberState.Enabled = false;
                ReportPeriodBindingSource.Filter = "rptID = " + view.Row["rptID"];
                addReportPeriod.Visible = true;
                if (view.Row.RowState == DataRowState.Modified)
                    saveToolStripButton.Visible = true;
                else
                    saveToolStripButton.Visible = false;
            }

        }

        private void OnCurrentReportPeriodChanged(object sender, EventArgs e)
        {
            DataRowView view = ((BindingSource)sender).Current as DataRowView;
            currentReportPeriod = view;
            if ((view == null) || (view.Row == null))
            {
                // make everything unreadable
                reportedYear.Enabled = false;
                reportID.Enabled = false;
                reportVersion.Enabled = false;
                situationAt.Enabled = false;
                ContactBindingSource.Filter = "0=1";
                saveReportIDButton.Visible = false;
                addContactButton.Visible = false;

                return;
            }
            ContactBindingSource.Filter = "repID = " + view.Row["repID"];
            if ((view.Row.RowState == DataRowState.Added) || (view.Row.RowState == DataRowState.Detached))
            {
                DataRowView reporter = ReporterBindingSource.Current as DataRowView;
                if (reporter == null) return;
                view.Row["rptID"] = reporter["rptID"];
                reportedYear.Enabled = true;
                reportID.Enabled = true;
                view["repCode"] = reporter["rptMStateKey"] as string;
                reportID.Text = reporter["rptMStateKey"] as string; // dont know why - but in some cases the framework uses the above code and in some cases it does not
                reportVersion.Enabled = true;
                situationAt.Enabled = true;
                saveReportIDButton.Visible = true;
                addContactButton.Visible = false;
                addReportPeriod.Visible = false;
                reportID.BackColor = UWWTD2008.ColMandatory;
                //reportedYear.BackColor = UWWTD2008.ColMandatory;
                situationAt.BackColor = UWWTD2008.ColMandatory;
            }
            else
            {
                reportedYear.Enabled = false;
                reportID.BackColor = UWWTD2008.ColNorm;
                reportID.Enabled = false;
                reportVersion.Enabled = false;
                situationAt.BackColor = UWWTD2008.ColNorm;
                situationAt.Enabled = false;
                addContactButton.Visible = true;
                addReportPeriod.Visible = true;
                if (view.Row.RowState == DataRowState.Modified)
                    saveReportIDButton.Visible = true;
                else
                    saveReportIDButton.Visible = false;
            }

        }

        private void OnCurrentContactChanged(object sender, EventArgs e)
        {
            DataRowView view = ((BindingSource)sender).Current as DataRowView;
            currentContact = view;
            if ((view == null) || (view.Row == null))
            {
                // make everything unreadable
                txtConCity.Enabled = false;
                txtConEmail.Enabled = false;
                txtConFax.Enabled = false;
                txtConInstitution.Enabled = false;
                txtConName.Enabled = false;
                txtConPhone.Enabled = false;
                txtConRemarks.Enabled = false;
                txtConStreet.Enabled = false;
                txtConZIP.Enabled = false;
                saveContactButton.Visible = false;
                return;
            }
            if ((view.Row.RowState == DataRowState.Added) || (view.Row.RowState == DataRowState.Detached))
            {
                DataRowView reportPeriod = ReportPeriodBindingSource.Current as DataRowView;
                if (reportPeriod == null) return;
                view.Row["repID"] = reportPeriod["repID"];
                txtConCity.Enabled = true;
                txtConEmail.Enabled = true;
                txtConFax.Enabled = true;
                txtConInstitution.Enabled = true;
                txtConName.Enabled = true;
                txtConPhone.Enabled = true;
                txtConRemarks.Enabled = true;
                txtConStreet.Enabled = true;
                txtConZIP.Enabled = true;
                saveContactButton.Visible = true;
                txtConName.BackColor = UWWTD2008.ColMandatory;
                //txtConStreet.BackColor = UWWTD2008.ColMandatory;
                txtConInstitution.BackColor = UWWTD2008.ColMandatory;
                txtConPhone.BackColor = UWWTD2008.ColMandatory;
            }
            else
            {
                txtConCity.Enabled = true;
                txtConEmail.Enabled = true;
                txtConFax.Enabled = true;
                txtConInstitution.Enabled = true;
                txtConName.Enabled = true;
                txtConPhone.Enabled = true;
                txtConRemarks.Enabled = true;
                txtConStreet.Enabled = true;
                txtConZIP.Enabled = true;
                if (view.Row.RowState == DataRowState.Modified)
                    saveContactButton.Visible = true;
                else
                    saveContactButton.Visible = false;
            }

        }

        private void OnSaveNewReporter(object sender, EventArgs e)
        {
            DataAccessFactory factory = DataAccessFactory.Instance;
            ReporterBindingSource.EndEdit();
            DataRowView view = ReporterBindingSource.Current as DataRowView;
            if (view == null) return;
            if (view.Row["rptMState"] == null || view.Row["rptMState"] is DBNull) {
                MessageBox.Show(this, "Please select a Member State", "Form Validation Error", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                return;
            }

            DataRow [] rows = factory.getLOVTable().Select("lovID = " + view.Row["rptMState"]);
            view.Row["rptMStateKey"] = rows[0]["lovKey"];
            view.Row["rptMStateWert"] = rows[0]["lovWert"];
            view.Row["rptFormRA"] = -1;

            if (!ValidateReporter(view.Row)) return;

            IDbConnection conn = factory.GetConnection();
            try
            {
                int curPos = ReporterBindingSource.Position;
                IDbDataAdapter adapter = factory.CreateFullDataAdapter(
                    factory.CreateCommand("select * from T_Reporter", conn));
                try {
                    factory.UpdateDataAdapter(adapter, dataSet, "T_Reporter");
                }
                catch (Exception ex) {
                    MessageBox.Show(this, "Saving the data was not possible.\r\n\"" + ex.Message + "\"\r\nPlease try again.", "Save Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                factory.FillDataAdapter(adapter, dataSet, "T_Reporter");
                ReporterBindingSource.Position = curPos;
                CurrentReporterChanged(ReporterBindingSource, null);
                //ReporterBindingSource.Position = ReporterBindingSource.Position;
            }
            finally
            {
                conn.Close();
            }

        }

        private void OnSaveReportID(object sender, EventArgs e)
        {
            DataAccessFactory factory = DataAccessFactory.Instance;
            DataRowView view = ReportPeriodBindingSource.Current as DataRowView;
            if (view == null) return;

            ReportPeriodBindingSource.EndEdit();
            if (!ValidateReportID(view.Row, null))
            {
                if (view.Row["repSituationAt"] == null || view.Row["repSituationAt"] is DBNull)
                    MessageBox.Show(this, "Situation is missing", "Form Validation Error", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                else if (view.Row["repCode"] == null || view.Row["repCode"] is DBNull || !reportID.Text.StartsWith((ReporterBindingSource.Current as DataRowView)["rptMStateKey"] as string))
                    MessageBox.Show(this, "Wrong Report ID", "Form Validation Error", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);

                return;
            }
            DataRow[] existingRowsWithID=dataSet.Tables["T_ReportPeriod"].Select("repID = "+view["repID"]);
            if (existingRowsWithID.Length!=0) 
                if((existingRowsWithID.Length > 1) || (existingRowsWithID[0] != view.Row)) {
                    reportID.BackColor = UWWTD2008.ColError;
                    MessageBox.Show(this, "The ReportPeriodID must be uniqe", "Validation Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }

            IDbConnection conn = factory.GetConnection();
            try
            {
                int curPos = ReportPeriodBindingSource.Position;
                IDbDataAdapter adapter = factory.CreateFullDataAdapter(
                    factory.CreateCommand("select * from T_ReportPeriod", conn));
                factory.UpdateDataAdapter(adapter, dataSet, "T_ReportPeriod");
                factory.FillDataAdapter(adapter, dataSet, "T_ReportPeriod");
                ReportPeriodBindingSource.Position = curPos;
                DataRowView currentRow = ReportPeriodBindingSource.Current as DataRowView;
                ValidateReportID(currentRow == null ? null : currentRow.Row, null);
                //ReporterBindingSource.Position = ReporterBindingSource.Position;
            }
            finally
            {
                conn.Close();
            }
        }

        private void OnSaveContact(object sender, EventArgs e)
        {
            DataAccessFactory factory = DataAccessFactory.Instance;
            ContactBindingSource.EndEdit();
            DataRowView view = ContactBindingSource.Current as DataRowView;
            if (view == null) return;

            if (!ValidateContact(view.Row, null)) {
                string strErrorText = "Please correct the followin Form Errors:";
                if (txtConName.Text.Length < 4)
                    strErrorText += "\r\nContact Name is Master Data";
                if (txtConInstitution.Text.Length < 1)
                    strErrorText += "\r\nContact Institution is Master Data";
                if (txtConPhone.Text.Length < 4)
                    strErrorText += "\r\nContact Phone is Master Data";

                MessageBox.Show(this, strErrorText, "Form Validation failed", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                return;
            }

            IDbConnection conn = factory.GetConnection();
            try
            {
                IDbDataAdapter adapter = factory.CreateFullDataAdapter(
                    factory.CreateCommand("select * from T_Contact", conn));
                factory.UpdateDataAdapter(adapter, dataSet, "T_Contact");
                factory.FillDataAdapter(adapter, dataSet, "T_Contact");
                //ReporterBindingSource.Position = ReporterBindingSource.Position;
            }
            finally
            {
                conn.Close();
            }
        }

        private void saveAllData()
        {
            DataAccessFactory factory = DataAccessFactory.Instance;
            IDbConnection conn = factory.GetConnection();
            try
            {
                int repPos = ReporterBindingSource.Position;
                int rptPos = ReportPeriodBindingSource.Position;
                IDbDataAdapter adapter = factory.CreateFullDataAdapter(
                    factory.CreateCommand("select * from T_Reporter", conn));
                factory.UpdateDataAdapter(adapter, dataSet, "T_Reporter");
                factory.FillDataAdapter(adapter, dataSet, "T_Reporter");

                adapter = factory.CreateFullDataAdapter(
                    factory.CreateCommand("select * from T_ReportPeriod", conn));
                factory.UpdateDataAdapter(adapter, dataSet, "T_ReportPeriod");
                factory.FillDataAdapter(adapter, dataSet, "T_ReportPeriod");

                adapter = factory.CreateFullDataAdapter(
                    factory.CreateCommand("select * from T_Contact", conn));
                factory.UpdateDataAdapter(adapter, dataSet, "T_Contact");
                factory.FillDataAdapter(adapter, dataSet, "T_Contact");
                //ReporterBindingSource.Position = ReporterBindingSource.Position;
                ReporterBindingSource.Position = repPos;
                ReportPeriodBindingSource.Position = rptPos;
            }
            finally
            {
                conn.Close();
            }
        }

        void DeletingRow(object sender, DataRowChangeEventArgs e) {
            DialogResult result = MessageBox.Show(this, "This will delete ALL data in the database relevant to deleted country. Are you sure?", "Please Confirm", MessageBoxButtons.YesNo, MessageBoxIcon.Asterisk, MessageBoxDefaultButton.Button2);
            if (result != DialogResult.Yes) throw new Exception("Prevent Delete");
        }

        private void OnDeleteReporter(object sender, EventArgs e) {
            try {
                ReporterBindingSource.RemoveCurrent();
            }
            catch (Exception) { }
        }

        private void OnDeleteReportPeriod(object sender, EventArgs e) {
            try {
                ReportPeriodBindingSource.RemoveCurrent();
            }
            catch (Exception) { }
        }

        private void OnDeleteContact(object sender, EventArgs e) {
            try {
                ContactBindingSource.RemoveCurrent();
            }
            catch (Exception) { }
        }

    }
}
