﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.IO;
using at.uba.uwwtd2008.Properties;

namespace at.uba.uwwtd2008.components
{
    public partial class CtrlSituationReport : UWWTDBaseCtrl
    {
        string reportDir = Settings.Default.SituationReportData;
        string currentDataDir = null;

        public CtrlSituationReport()
        {
            InitializeComponent();
            if (reportDir.StartsWith(".\\")) reportDir = Directory.GetParent(Application.ExecutablePath) + reportDir.Substring(1);
        }
        public override void OpenForm() {
            if (CurrentRep == null)
            {
                MessageBox.Show(UWWTD2008MainForm.getMainForm(this), "Please select a valid ReportPeriod first", "No ReportPeriod selected", MessageBoxButtons.OK, MessageBoxIcon.Error);
                UWWTD2008MainForm.getMainForm(this).OpenDataEditMain();
                return;
            }

            if (!Directory.Exists(reportDir)) Directory.CreateDirectory(reportDir);
            
            currentDataDir = reportDir + "\\Report_" + CurrentRep.RepCode;
            if (!Directory.Exists(currentDataDir)) Directory.CreateDirectory(currentDataDir);

            string url = getURLText();
            if (url != null)
                txtURL.Text = url;
            else
                txtURL.Text = "";

            populateList();
            checkURLButton();
        }

        private void populateList()
        {
            string file;
            txtURL.Text = "";

            lstFiles.Items.Clear();
            foreach (string diritem in Directory.GetFiles(currentDataDir, "*.*", SearchOption.TopDirectoryOnly))
            {
                int imageIndex = 0;
                file = diritem;

                if (file.IndexOf("\\") >= 0)
                    file = file.Substring(file.LastIndexOf("\\") + 1);

                if (file.Equals("link.url"))
                {
                    txtURL.Text = getURLText();
                    continue;
                }

                if (file.ToLower().EndsWith(".pdf"))
                {
                    imageIndex = 1;
                }
                if (file.ToLower().EndsWith(".doc"))
                {
                    imageIndex = 2;
                }
                if (file.ToLower().EndsWith(".xls"))
                {
                    imageIndex = 3;
                }

                ListViewItem item = new ListViewItem(file, imageIndex);
                item.Tag = diritem;
                lstFiles.Items.Add(item);
            }
        }

        private void checkURLButton()
        {
            string url = txtURL.Text;
            if (url != null)
            {
                if ((url.StartsWith("http://") || (url.StartsWith("https://")) || (url.StartsWith("ftp://"))) && url.LastIndexOf("/") > 8)
                    cmdSetURL.Enabled = true;
                else
                    cmdSetURL.Enabled = false;
            }
        }

        private void OnRemoveItem(object sender, EventArgs e)
        {
            string msg = "Please confirm to delete ";
            if (lstFiles.SelectedItems.Count > 1)
                msg += lstFiles.SelectedItems.Count + " files!";
            else
                msg += lstFiles.SelectedItems.Count + " file!";

            DialogResult res = MessageBox.Show(this, msg, "Please confirm", MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2);
            if (res != DialogResult.Yes) return;

            foreach (ListViewItem item in lstFiles.SelectedItems)
            {
                string file = item.Tag as string;
                if (file == null) continue;
                File.Delete(file);
            }
            populateList();
        }

        private void OnItemSelectionChanged(object sender, ListViewItemSelectionChangedEventArgs e)
        {
            if (sender == lstFiles)
            {
                if (lstFiles.SelectedItems.Count < 1)
                {
                    cmdDownload.Enabled = false;
                    cmdRemove.Enabled = false;
                    cmdUpload.Enabled = true;
                }
                else if (lstFiles.SelectedItems.Count > 1)
                {
                    cmdDownload.Enabled = false;
                    cmdRemove.Enabled = checkRemoveableItems(lstFiles.SelectedItems);
                    cmdUpload.Enabled = true;
                }
                else
                {
                    cmdDownload.Enabled = true;
                    cmdRemove.Enabled = checkRemoveableItems(lstFiles.SelectedItems);
                    cmdUpload.Enabled = true;
                }

            }
        }

        private bool checkRemoveableItems(System.Collections.IList lst)
        {
            foreach (ListViewItem item in lst)
            {
                string file = item.Tag as string;
                if (file == null) continue;
            }
            return true;
        }

        private void OnDownloadFile(object sender, EventArgs e)
        {
            if (lstFiles.SelectedItems.Count < 1) return;

            string file = lstFiles.SelectedItems[0].Tag as string;

            string newfile = file;
            if (newfile.IndexOf("\\") >= 0)
                newfile = newfile.Substring(newfile.LastIndexOf("\\") + 1);
            dlgFileSave.FileName = newfile;
            DialogResult res = dlgFileSave.ShowDialog(this);
            if (res == DialogResult.OK)
            {
                // already done by file-save-dialog
                //if (File.Exists(dlgFileSave.FileName)) {
                //    if (MessageBox.Show(this, "Do you want to overwrite this File","Please confirm",MessageBoxButtons.YesNo,MessageBoxIcon.Question,MessageBoxDefaultButton.Button2) != DialogResult.Yes)
                //        return;
                //}

                if (file.StartsWith(".\\")) file = Directory.GetParent(Application.ExecutablePath) + file.Substring(1);
                File.Copy(file, dlgFileSave.FileName, true);
            }
        }

        private void OnUploadFile(object sender, EventArgs e)
        {
            DialogResult res = dlgFileOpen.ShowDialog(this);
            if (res == DialogResult.OK)
            {
                foreach (string source in dlgFileOpen.FileNames)
                {
                    string newfile = source;
                    if (newfile.IndexOf("\\") >= 0)
                        newfile = newfile.Substring(newfile.LastIndexOf("\\") + 1);
                    string newpath = currentDataDir + "\\" + newfile;
                    if (newpath.StartsWith(".\\")) newpath = Directory.GetParent(Application.ExecutablePath) + newpath.Substring(1);

                    if (File.Exists(newpath))
                    {
                        if (MessageBox.Show(this, "Do you want to overwrite \"" + newfile + "\"", "Please confirm", MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2) != DialogResult.Yes)
                            continue;
                    }
                    File.Copy(source, newpath, true);
                }
            }
            populateList();
        }

        private string getURLText()
        {
            if (!Directory.Exists(currentDataDir))
                return null;
            string[] files = Directory.GetFiles(currentDataDir);
            if (files.Length < 1) return null;
            foreach (string item in files)
            {
                string file = item;
                if (file.IndexOf("\\") >= 0)
                {
                    file = file.Substring(file.LastIndexOf("\\") + 1);
                }

                if (file.Equals("link.url"))
                {
                    return File.ReadAllText(item, Encoding.UTF8);
                }
            }

            return null;
        }

        private void OnSetURL(object sender, EventArgs e)
        {
            if (txtURL.Text.Length < 1)
                File.Delete(currentDataDir + "\\link.url");
            else
                File.WriteAllText(currentDataDir + "\\link.url", txtURL.Text, Encoding.UTF8);
        }

        private void OnURLTextChanged(object sender, EventArgs e)
        {
            checkURLButton();
        }
    }
}
