﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

namespace at.uba.uwwtd2008.components
{
    public partial class CtrlUWWTDEdit : UWWTDBaseCtrl
    {
        DataSet dataSet = null;
        DataTable availableAgglos = null;
        DataTable selectedAgglos = null;
        DataRowView currentRow = null;
        DataView connectedAgglos = null;

        public const String TNAME_UWWTDAGGLO = "uwwtdagglo";
        public const String SELECT_ALL_UWWTDAGGLO = "SELECT * FROM T_UWWTP_AGGLO WHERE repID = ";
        public const String TNAME_UWWTDS = "uwwtds";
        public const String SELECT_ALL_DISCHARGEPOINTS = "SELECT t1.*, t2.rcaCode FROM T_DischargePoints t1 LEFT JOIN T_ReceivingAreas t2 ON t1.rcaID = t2.rcaID WHERE t1.repID = ";
        public const String TNAME_DISCHARGEPOINTS = CtrlDischargePointsEdit.TNAME_DISCPOINTS;
        public const String SELECT_ALL_UWWTD = "SELECT * FROM T_UWWTPS WHERE repID = ";
        public const String RNAME_UWWTDSAGGLO_UWWTDS = "uwwtdsagglo_uwwtds";
        public const String RNAME_AGGLO_UWWTDSAGGLO = "agglo_uwwtdsagglo";
        public const String RNAME_DP_UWWTD = "r_dp_uwwtd_pos";

        public const String TNAME_AGGLO = CtrlAgglomerationsEdit.TNAME_AGGLO;

        public CtrlUWWTDEdit()
        {
            InitializeComponent();
        }

        public override void OnBackButtonClick()
        {
            UWWTD2008MainForm mf = UWWTD2008MainForm.getMainForm(this);
            if (mf == null) return;

            if (BindingContext[connectedAgglos].Position >= 0)
            {
                DataRowView row = BindingContext[connectedAgglos].Current as DataRowView;
                if (row != null) row.CancelEdit();
            }

            if (PrevControl != null) mf.getSplitterPanel().setContentPane(PrevControl);
            else mf.OpenDataEditMain();
        }

        public void UpdateUwwAgglo(DataSet dataSet, IDbConnection conn)
        {
            IDbDataAdapter adapter = mDataFactory.CreateDataAdapter(mDataFactory.CreateCommand(SELECT_ALL_UWWTDAGGLO + CurrentRep.RepID, conn));
            mDataFactory.FillDataAdapter(adapter, dataSet, TNAME_UWWTDAGGLO);
        }

        public void UpdateDiscPoints(DataSet dataSet, IDbConnection conn)
        {
            mDataFactory.FillDataAdapter(
                mDataFactory.CreateDataAdapter(
                    mDataFactory.CreateCommand(SELECT_ALL_DISCHARGEPOINTS + CurrentRep.RepID, conn)
                ), dataSet, TNAME_DISCHARGEPOINTS);
        }

        public override void PerformDataBinding(DataSet dataSet) {
            this.dataSet = dataSet;
            bool hasNewItems = false;

            //if (PrevControl is CtrlAgglomerationsEdit)
            //{
            //    dataSet = ((CtrlAgglomerationsEdit)PrevControl).CurrentDataSet;
            //}
            //else
            //{
            //    dataSet = new DataSet();
            //}

            // fill the correct Data
            IDbConnection conn = mDataFactory.GetConnection();
            try
            {
                if (!dataSet.Tables.Contains(TNAME_UWWTDS))
                {
                    mDataFactory.FillDataAdapter(
                        mDataFactory.CreateDataAdapter(
                            mDataFactory.CreateCommand(SELECT_ALL_UWWTD + CurrentRep.RepID, conn)
                        ), dataSet, TNAME_UWWTDS);
                    hasNewItems = true;
                }

                if (!dataSet.Tables.Contains(TNAME_UWWTDAGGLO))
                {
                    UpdateUwwAgglo(dataSet, conn);
                    hasNewItems = true;
                }

                if (!dataSet.Tables.Contains(TNAME_DISCHARGEPOINTS))
                {
                    UpdateDiscPoints(dataSet, conn);
                    hasNewItems = true;
                }

                // Create Data Relations
                DataColumn dc1, dc2;
                DataRelation rel;

                if (!dataSet.Relations.Contains(RNAME_UWWTDSAGGLO_UWWTDS))
                {
                    dc1 = dataSet.Tables[TNAME_UWWTDS].Columns["uwwID"];
                    dc2 = dataSet.Tables[TNAME_UWWTDAGGLO].Columns["uwwID"];
                    rel = new DataRelation(RNAME_UWWTDSAGGLO_UWWTDS, dc1, dc2);
                    dataSet.Relations.Add(rel);
                    hasNewItems = true;
                }
                if (!dataSet.Relations.Contains(RNAME_AGGLO_UWWTDSAGGLO))
                {
                    dc1 = dataSet.Tables[CtrlAgglomerationsEdit.TNAME_AGGLO].Columns["aggID"];
                    dc2 = dataSet.Tables[TNAME_UWWTDAGGLO].Columns["aggID"];
                    rel = new DataRelation(RNAME_AGGLO_UWWTDSAGGLO, dc1, dc2);
                    dataSet.Relations.Add(rel);
                    dataSet.Tables[TNAME_AGGLO].Columns.Add("connected", typeof(int), "Count(Child(" + RNAME_AGGLO_UWWTDSAGGLO + ").uwwID)");
                    hasNewItems = true;
                }
                if (!dataSet.Relations.Contains(RNAME_DP_UWWTD))
                {
                    dc1 = dataSet.Tables[TNAME_UWWTDS].Columns["uwwID"];
                    dc2 = dataSet.Tables[TNAME_DISCHARGEPOINTS].Columns["uwwID"];
                    rel = new DataRelation(RNAME_DP_UWWTD, dc1, dc2);
                    dataSet.Relations.Add(rel);
                    hasNewItems = true;
                }

            }
            finally
            {
                conn.Close();
            }

            if (hasNewItems)
            {
                connectedAgglos = new DataView(dataSet.Tables[TNAME_UWWTDS]);
                connectedAgglos.RowFilter = "uwwCollectingSystem = "+mDataFactory.getLovRowByKey("UWWTP", "ISCON")["lovID"];

                availableAgglos = createAggloViewTable("availableAgglos");
                selectedAgglos = createAggloViewTable("selectedAgglos");

                BindingContext[connectedAgglos].CurrentChanged += new EventHandler(CurrentUWWTDChanged);
                comboNUTS.DataSource = mDataFactory.getNUTSTableForCurrentReporter();
                comboNUTS.DisplayMember = "nutRegion";
                comboNUTS.ValueMember = "nutID";

                gridAvailableAgglos.AutoGenerateColumns = false;
                // this is a view only generated by the change_uwwtp_event
                gridAvailableAgglos.DataSource = availableAgglos;

                gridUWWTDMaster.AutoGenerateColumns = false;
                gridUWWTDMaster.DataSource = connectedAgglos;
                //gridUWWTDMaster.DataMember = TNAME_UWWTDS;
                DataGridViewComboBoxColumn statusCol = (DataGridViewComboBoxColumn)gridUWWTDMaster.Columns["status"];
                statusCol.DataSource = mDataFactory.getLovViewByKey("STAT");
                statusCol.DisplayMember = "lovWert";
                statusCol.ValueMember = "lovID";

                gridSelectedAgglos.AutoGenerateColumns = false;
                // this is a view only generated by the change_uwwtp_event
                gridSelectedAgglos.DataSource = selectedAgglos;

                txtUwwID.DataBindings.Clear();
                txtUwwID.DataBindings.Add("Text", connectedAgglos, "uwwCode");

                txtUwwName.DataBindings.Clear();
                txtUwwName.DataBindings.Add("Text", connectedAgglos, "uwwName");

                txtUwwLongitude.DataBindings.Clear();
                Binding tmp = new Binding("Text", connectedAgglos, "uwwLongitude");
                tmp.Format += new ConvertEventHandler(formatStringDecimal);
                tmp.Parse += new ConvertEventHandler(parseStringDecimal);
                txtUwwLongitude.DataBindings.Add(tmp);

                txtUwwLatitude.DataBindings.Clear();
                tmp = new Binding("Text", connectedAgglos, "uwwLatitude");
                tmp.Format += new ConvertEventHandler(formatStringDecimal);
                tmp.Parse += new ConvertEventHandler(parseStringDecimal);
                txtUwwLatitude.DataBindings.Add(tmp);

                comboNUTS.DataBindings.Clear();
                comboNUTS.DataBindings.Add("SelectedValue", connectedAgglos, "uwwNUTS");

                gridDPUWWTDS.AutoGenerateColumns = false;
                gridDPUWWTDS.DataSource = connectedAgglos;
                gridDPUWWTDS.DataMember = RNAME_DP_UWWTD;

                comboState.DataSource = mDataFactory.getLovViewByKey("STAT");
                comboState.DisplayMember = "lovWert";
                comboState.ValueMember = "lovID";
                comboState.DataBindings.Clear();
                comboState.DataBindings.Add("SelectedValue", connectedAgglos, "uwwState");

                //TODO HPW: Make this work here
            }
        }

        void CurrentUWWTDChanged(object sender, EventArgs e)
        {
            if (currentRow != null) currentRow.EndEdit();
            currentRow = null;
            if (BindingContext[connectedAgglos].Position < 0) return;
            DataRowView view = BindingContext[connectedAgglos].Current as DataRowView;
            if (view != null)
            {
                object uwwID = view["uwwID"];
                fillDataTableAvail(availableAgglos, uwwID);
                fillDataTableSelected(selectedAgglos, uwwID);
            }
            currentRow = view;

            txtUwwID.ReadOnly = currentRow.Row.RowState != DataRowState.Added;
            if ((currentRow != null) && (currentRow.Row.RowState == DataRowState.Added))
            {
                txtUwwID.BackColor = UWWTD2008.ColMandatory;
                txtUwwName.BackColor = UWWTD2008.ColMandatory;
                txtUwwLongitude.BackColor = UWWTD2008.ColMandatory;
                txtUwwLatitude.BackColor = UWWTD2008.ColMandatory;
            }
            else
            {
                txtUwwID.BackColor = UWWTD2008.ColNorm;
                txtUwwName.BackColor = UWWTD2008.ColNorm;
                txtUwwLongitude.BackColor = UWWTD2008.ColNorm;
                txtUwwLatitude.BackColor = UWWTD2008.ColNorm;
            }

            if (currentRow == null)
            {
                linkConnectAgglo.Enabled = false;
                linkDisconnectAgglo.Enabled = false;
            }
            else
            {
                if (currentRow.Row.RowState == DataRowState.Added)
                {
                    linkConnectAgglo.Enabled = false;
                    linkDisconnectAgglo.Enabled = false;
                }
                else
                {
                    linkConnectAgglo.Enabled = true;
                    linkDisconnectAgglo.Enabled = true;
                }
            }
            //if (currentRow != null)
            //{
            //    if (!txtUwwID.Text.StartsWith(CurrentRep.Reporter.State.Key))
            //        txtUwwID.BackColor = UWWTD2008.ColError;
            //    if ((currentRow.Row.RowState != DataRowState.Added) && (txtUwwName.Text.Length<1))
            //        txtUwwName.BackColor = UWWTD2008.ColError;
            //}
        }

        public override void OpenForm() {
            CurrentUWWTDChanged(null, null);
        }

        private void groupBox1_Enter(object sender, EventArgs e)
        {

        }

        private void mbutEditDP_Click(object sender, EventArgs e)
        {
            if (BindingContext[connectedAgglos].Position >= 0)
            {
                DataRowView currentRow = (DataRowView) BindingContext[connectedAgglos].Current;
                int row = dataSet.Tables[TNAME_UWWTDS].Rows.IndexOf(dataSet.Tables[TNAME_UWWTDS].Rows.Find(currentRow["uwwID"]));

                BindingContext[dataSet, TNAME_UWWTDS].Position = row;
            }
            ((UWWTD2008MainForm)(this.Parent.Parent.Parent.Parent)).OpenDischargePointsEdit(this);
        }

        private static DataTable createAggloViewTable(string name)
        {
            DataTable t = new DataTable(name);
            t.Columns.Add("aggID", typeof(long));
            t.Columns.Add("aggCode", typeof(string));
            t.Columns.Add("aggName", typeof(string));
            return t;
        }

        private void fillDataTableAvail(DataTable t, object uwwID)
        {
            t.Clear();
            foreach (DataRow row in dataSet.Tables[TNAME_AGGLO].Rows)
            {
                if (row.RowState == DataRowState.Deleted) continue;
                if (dataSet.Tables[TNAME_UWWTDAGGLO].Select("uwwID = " + uwwID + " AND aggID = " + row["aggID"]).Length > 0)
                    continue;
                DataRow newrow = t.NewRow();
                foreach (DataColumn col in t.Columns)
                {
                    newrow[col.ColumnName] = row[col.ColumnName];
                }
                t.Rows.Add(newrow);
            }
        }
        private void fillDataTableSelected(DataTable t, object uwwID)
        {
            t.Clear();
            foreach (DataRow row in dataSet.Tables[TNAME_AGGLO].Rows)
            {
                if (row.RowState == DataRowState.Deleted) continue;
                if (dataSet.Tables[TNAME_UWWTDAGGLO].Select("uwwID = " + uwwID + " AND aggID = " + row["aggID"]).Length == 0)
                    continue;
                DataRow newrow = t.NewRow();
                foreach (DataColumn col in t.Columns)
                {
                    newrow[col.ColumnName] = row[col.ColumnName];
                }
                t.Rows.Add(newrow);
            }
        }

        private void OnAddAggloSelected(object sender, LinkLabelLinkClickedEventArgs e)
        {
            if (BindingContext[connectedAgglos].Position < 0) return;
            foreach (DataGridViewRow dgvRow in gridAvailableAgglos.SelectedRows)
            {
                DataRowView view = dgvRow.DataBoundItem as DataRowView;
                if (view != null)
                {
                    DataRowView uwwView = BindingContext[connectedAgglos].Current as DataRowView;
                    if (uwwView != null)
                    {
                        object uwwID = uwwView["uwwID"];
                        DataRow newRow = dataSet.Tables[TNAME_UWWTDAGGLO].NewRow();
                        newRow["repID"] = CurrentRep.RepID;
                        newRow["aggID"] = view["aggID"];
                        newRow["aucAggCode"] = view["aggCode"];
                        newRow["aucAggName"] = view["aggName"];
                        newRow["uwwID"] = uwwID;
                        newRow["aucUwwCode"] = uwwView["uwwCode"];
                        newRow["aucUwwName"] = uwwView["uwwName"];
                        dataSet.Tables[TNAME_UWWTDAGGLO].Rows.Add(newRow);

                        fillDataTableAvail(availableAgglos, uwwID);
                        fillDataTableSelected(selectedAgglos, uwwID);
                    }
                }
            }
        }

        private void OnRemoveAggloSelected(object sender, LinkLabelLinkClickedEventArgs e)
        {
            if (BindingContext[connectedAgglos].Position < 0) return;
            foreach (DataGridViewRow dgvRow in gridSelectedAgglos.SelectedRows)
            {
                DataRowView view = dgvRow.DataBoundItem as DataRowView;
                if (view != null)
                {
                    DataRowView uwwView = BindingContext[connectedAgglos].Current as DataRowView;
                    if (uwwView != null)
                    {
                        object uwwID = uwwView["uwwID"];
                        foreach (DataRow oldrow in dataSet.Tables[TNAME_UWWTDAGGLO].Select("uwwID = " + uwwID + " AND aggID = " + view["aggID"]))
                        {
                            oldrow.Delete();
                        }

                        fillDataTableAvail(availableAgglos, uwwID);
                        fillDataTableSelected(selectedAgglos, uwwID);
                    }
                }
            }
        }

        protected override bool ValidateForm(ref string errorText, DataRowState state) {
            bool retVal = true;
            bool justAdded = false;
            if ((state == DataRowState.Added) || (state == DataRowState.Detached))
                justAdded = true;

            txtUwwName.BackColor = UWWTD2008.ColNorm;
            txtUwwID.BackColor = UWWTD2008.ColNorm;
            txtUwwLongitude.BackColor = UWWTD2008.ColNorm;
            txtUwwLatitude.BackColor = UWWTD2008.ColNorm;

            if ((BindingContext[connectedAgglos].Position < 0) || ((BindingContext[connectedAgglos].Current as DataRowView) == null)) {
                if (errorText.Length > 0) errorText += "\r\n";
                errorText += "No Agglomerations are connected to this UWWTD";
                retVal = false;
            }

            if (!txtUwwID.Text.StartsWith(CurrentRep.Reporter.State.Key)) {
                txtUwwID.BackColor = justAdded ? UWWTD2008.ColMandatory : UWWTD2008.ColError;
                if (errorText.Length > 0) errorText += "\r\n";
                errorText += "The UwwtdID has to start with " + CurrentRep.Reporter.State.Key;
                retVal = false;
            }
            if (txtUwwID.Text.Length < 3) {
                txtUwwID.BackColor = justAdded ? UWWTD2008.ColMandatory : UWWTD2008.ColError;
                if (errorText.Length > 0) errorText += "\r\n";
                errorText += "Please set the UwwtdID to a correct Value";
                retVal = false;
            }
            if (((dataSet.Tables[TNAME_UWWTDS].Select("uwwCode = '" + txtUwwID.Text + "'").Length == 1) &&
                (dataSet.Tables[TNAME_UWWTDS].Select("uwwCode = '" + txtUwwID.Text + "'")[0] != currentRow.Row)) ||
                (dataSet.Tables[TNAME_UWWTDS].Select("uwwCode = '" + txtUwwID.Text + "'").Length > 1)) {
                txtUwwID.BackColor = UWWTD2008.ColError;
                if (errorText.Length > 0) errorText += "\r\n";
                errorText += "The UwwtdID is already used";
                retVal = false;
            }

            if (txtUwwName.Text.Length < 2) {
                txtUwwName.BackColor = justAdded ? UWWTD2008.ColMandatory : UWWTD2008.ColError;
                if (errorText.Length > 0) errorText += "\r\n";
                errorText += "Please specify a name for this UWWTD";
                retVal = false;
            }

            if (txtUwwLatitude.Text.Length < 1) {
                txtUwwLatitude.BackColor = justAdded ? UWWTD2008.ColMandatory : UWWTD2008.ColError;
                if (errorText.Length > 0) errorText += "\r\n";
                errorText += "Latitude is Master Data";
                retVal = false;
            }

            if (txtUwwLongitude.Text.Length < 1) {
                txtUwwLongitude.BackColor = justAdded ? UWWTD2008.ColMandatory : UWWTD2008.ColError;
                if (errorText.Length > 0) errorText += "\r\n";
                errorText += "Longitude is Master Data";
                retVal = false;
            }

            return retVal;
        }

        private void OnSaveClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            string strErrorText = "";

            if (!ValidateForm(ref strErrorText, currentRow.Row.RowState))
            {
                MessageBox.Show(this, "Please correct Form Errors before save:\r\n" + strErrorText, "Form not valid", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                return;
            }

            DataRowView row = BindingContext[connectedAgglos].Current as DataRowView;
            if (row != null) row.EndEdit();

            IDbConnection conn = mDataFactory.GetConnection();
            try
            {
                int curPos = BindingContext[connectedAgglos].Position;
                bool enforceConstraints = dataSet.EnforceConstraints;
                IDbDataAdapter adapter = null;
                dataSet.EnforceConstraints = false;
                try
                {
                    adapter = mDataFactory.CreateFullDataAdapter(mDataFactory.CreateCommand(SELECT_ALL_UWWTDAGGLO + CurrentRep.RepID, conn));
                    mDataFactory.UpdateDataAdapter(adapter, dataSet, TNAME_UWWTDAGGLO);
                    mDataFactory.FillDataAdapter(adapter, dataSet, TNAME_UWWTDAGGLO);

                    adapter = mDataFactory.CreateFullDataAdapter(mDataFactory.CreateCommand(SELECT_ALL_UWWTD + CurrentRep.RepID, conn));
                    mDataFactory.UpdateDataAdapter(adapter, dataSet, TNAME_UWWTDS);
                    mDataFactory.FillDataAdapter(adapter, dataSet, TNAME_UWWTDS);
                    if (BindingContext[connectedAgglos].Count > curPos)
                        BindingContext[connectedAgglos].Position = curPos;


                    // Update the connected Dischargepoints
                    adapter = mDataFactory.CreateFullDataAdapter(mDataFactory.CreateCommand(CtrlDischargePointsEdit.SELECT_DISCPOINTS + CurrentRep.RepID, conn));
                    mDataFactory.UpdateDataAdapter(adapter, dataSet, CtrlDischargePointsEdit.TNAME_DISCPOINTS);

                    // Update the Dischargepoint-View in this form
                    mDataFactory.FillDataAdapter(
                        mDataFactory.CreateDataAdapter(
                            mDataFactory.CreateCommand(SELECT_ALL_DISCHARGEPOINTS + CurrentRep.RepID, conn)
                        ), dataSet, TNAME_DISCHARGEPOINTS);
                }
                finally
                {
                    dataSet.EnforceConstraints = enforceConstraints;
                }
            }
            finally
            {
                conn.Close();
            }

            UWWTD2008MainForm mf = UWWTD2008MainForm.getMainForm(this);
            if (mf != null) mf.getSplitterPanel().updateUWWAggloTable(dataSet);

            if (BindingContext[connectedAgglos].Position >= 0)
            {
                row = BindingContext[connectedAgglos].Current as DataRowView;
                if (row != null) row.BeginEdit();
            }

        }

        private void OnAddUWWTPClick(object sender, EventArgs e)
        {
            DataRow row = dataSet.Tables[TNAME_UWWTDS].NewRow();
            row["uwwCode"] = CurrentRep.Reporter.State.Key;
            row["repID"] = CurrentRep.RepID;
            row["uwwState"] = mDataFactory.getLovRowByKey("STAT", "1")["lovID"];
            row["uwwCollectingSystem"] = mDataFactory.getLovRowByKey("UWWTP", "ISCON")["lovID"];
            dataSet.Tables[TNAME_UWWTDS].Rows.Add(row);
            if (BindingContext[connectedAgglos].Count > 0)
                BindingContext[connectedAgglos].Position = BindingContext[connectedAgglos].Count - 1;
        }

        private void OnDeleteUWWTP(object sender, EventArgs e)
        {
            if (BindingContext[connectedAgglos].Position >= 0)
            {
                DataRowView row = BindingContext[connectedAgglos].Current as DataRowView;
                if (row != null)
                {
                    if ((row.Row.RowState == DataRowState.Added) || (row.Row.RowState == DataRowState.Detached))
                    {
                        dataSet.Tables[TNAME_UWWTDS].Rows.Remove(row.Row);
                    }
                    else
                    {
                        foreach (DataRow dp in dataSet.Tables[CtrlDischargePointsEdit.TNAME_DISCPOINTS].Select("uwwID=" + row["uwwID"]))
                        {
                            dp.Delete();
                        }
                        row.Delete();
                    }
                }
            }
        }

        override protected void validateStringDecimal(object sender, CancelEventArgs e)
        {
            base.validateStringDecimal(sender, e);
        }
    }
}
