﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using at.uba.uwwtd2008.DataAccess;
using at.uba.uwwtd2008.Items;
using System.Data;
using System.ComponentModel;

namespace at.uba.uwwtd2008.components
{
    public class UWWTDBaseCtrl : UserControl
    {
        private static ReportPeriod currentRep = null;
        protected DataAccessFactory mDataFactory = DataAccessFactory.Instance;


        public static void UpdateAllData()
        {
            if (UWWTD2008MainForm.Instance != null)
            {
                UWWTD2008MainForm.Instance.getSplitterPanel().UpdateDataSet();
            }
        }

        public static ReportPeriod CurrentRep {
            get { return UWWTDBaseCtrl.currentRep; }
            set
            {
                DataAccessFactory dataFactory = DataAccessFactory.Instance;
                UWWTDBaseCtrl.currentRep = value;
                if (dataFactory != null) dataFactory.setCurrentReportPeriod(value);
                if (UWWTD2008MainForm.Instance != null)
                {
                    if (currentRep != null)
                    {
                        UWWTD2008MainForm.Instance.getSplitterPanel().setMemberState(currentRep.Reporter.State.Value);
                    }
                    else
                    {
                        UWWTD2008MainForm.Instance.getSplitterPanel().setMemberState(null);
                    }
                }
            }

        }

        private UWWTDBaseCtrl mPrevControl;
        protected BindingSource uwwDataBindingSource;
        private System.ComponentModel.IContainer components;

        private static DataSet dataT_Init = null;

        public static DataRow getIniDataRow() {
            if (dataT_Init == null) return null;
            return dataT_Init.Tables[0].Rows[0];
        }

        public static void updateIniTable() {
            DataAccessFactory dataFactory = DataAccessFactory.Instance;
            IDbConnection conn = dataFactory.GetConnection();
            try
            {
                IDbCommand cmd = dataFactory.CreateCommand("SELECT * FROM T_Init", conn);
                if (dataT_Init == null)
                {
                    DataSet ds = new DataSet();
                    dataFactory.CreateDataAdapter(cmd).Fill(ds);
                    dataT_Init = ds;
                }
                DataTable T_Init = dataT_Init.Tables[0];

                DataRow row;
                if (T_Init.Rows.Count < 1)
                {
                    row = T_Init.NewRow();
                    T_Init.Rows.Add(row);
                }
                else
                {
                    row = T_Init.Rows[0];
                }
                
                row["repID"] = UWWTDBaseCtrl.currentRep.RepID;
                row["rptID"] = UWWTDBaseCtrl.currentRep.Reporter.RptId;

                dataFactory.CreateFullDataAdapter(cmd).Update(dataT_Init);
            }
            finally
            {
                conn.Close();
            }
        }

        /// <summary>
        /// This should be overrided to do specific back-options
        /// </summary>
        public virtual void OnBackButtonClick()
        {
            UWWTD2008MainForm mf = UWWTD2008MainForm.getMainForm(this);
            if (mf == null) return;

            if (validateBeforeClose())
            {
                if (PrevControl != null) mf.getSplitterPanel().setContentPane(PrevControl);
                else mf.OpenDataEditMain();
            }
        }

        /// <summary>
        /// Per form - Validation. Please add coloring of the Form-Fileds here - for coloring use:
        /// <list type="bullet">
        /// <item><b>UWWTD2008.ColError</b> for Items with error</item>
        /// <item><b>UWWTD2008.ColMandatory</b> for "Master Data" items</item>
        /// <item><b>UWWTD2008.ColNorm</b> normal items - or items that passed the check</item>
        /// </list>
        /// </summary>
        /// <param name="errorText">receives the error text about the form validation</param>
        /// <param name="state">RowState of the currentRow (sometimes useful for coloring the Form-Fields)</param>
        /// <returns>true, if form has passed validation. false otherwise</returns>
        protected virtual bool ValidateForm(ref string errorText, DataRowState state)
        {
            return true;
        }

        /// <summary>
        /// Override this method to do custom form validation before close via the Back - Button.
        /// </summary>
        /// <returns>true, if validation went well, false otherwise</returns>
        public virtual bool validateBeforeClose() {
            string errorText = "";
            return ValidateForm(ref errorText, DataRowState.Detached);
        }

        protected virtual void ShowSaveConfirmation()
        {
            MessageBox.Show(this, "Save Operation completed successfully", "Save", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        protected virtual bool ShowConfirmDelete()
        {
            return MessageBox.Show(this, "You would like to delete an object and all associated data!\r\nAre you sure?", "Delete", MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2) == DialogResult.Yes;
        }

        public UWWTDBaseCtrl PrevControl
        {
            get { return mPrevControl; }
            set { mPrevControl = value; }
        }

        public virtual void OpenForm() {
            // this is to be implemented in the derived classes
        }

        public virtual void PerformDataBinding(DataSet dataSet) {
            // override this in your implementation to give the parent
            // control the possibility to do the databinding before the
            // actual form is visible
        }

        public static String handleContent(object obj) {
            if (obj == null) return "";
            if (obj is DBNull) return "";
            return obj.ToString();
        }

        private void InitializeComponent()
        {
            this.components = new System.ComponentModel.Container();
            this.ResumeLayout(false);

        }

        /// <summary>
        ///  update all the databound controls (Enables them, if current row is valid
        ///  and disables them if current row is invalid or no current row is there
        /// </summary>
        /// <param name="row">The current row in the BindingContext</param>
        /// <param name="context">The BindingContext used</param>
        public virtual void updateComponentView(BindingManagerBase context)
        {
            DataRow row = null;
            DataRowView view = null;
            if (context.Position >= 0)
                view = context.Current as DataRowView;
            if (view != null)
                row = view.Row;

            if ((row == null) || (row.RowState == DataRowState.Deleted))
            {
                foreach (Binding b in context.Bindings)
                {
                    Control ctrl = b.BindableComponent as Control;
                    if (ctrl != null) ctrl.Enabled = false;
                }
            } // if
            else
            {
                foreach (Binding b in context.Bindings)
                {
                    Control ctrl = b.BindableComponent as Control;
                    if (ctrl != null) ctrl.Enabled = true;
                }
            }
        }

        protected virtual void validateStringDecimal(object sender, CancelEventArgs e)
        {
            try
            {
                string text = ((TextBox)sender).Text;
                decimal d = decimal.Parse(text);
                ((TextBox)sender).Text = d.ToString();
                if (((TextBox)sender).Text.Length >= 20)
                    ((TextBox)sender).Text = ((TextBox)sender).Text.Substring(0, 19);
            }
            catch (Exception)
            {
                e.Cancel = true;
            }
        }

        protected void formatStringDecimal(object sender, ConvertEventArgs e)
        {
            try
            {
                object objValue = DataAccess.DataAccessFactory.parseDBDecimal(e.Value);
                if ((objValue == null) || (objValue is DBNull))
                    e.Value = DBNull.Value;
                else
                    e.Value = objValue.ToString();
            }
            catch (Exception) { }
        }

        protected void parseStringDecimal(object sender, ConvertEventArgs e)
        {
            try
            {
                if ((e.Value == null) || (e.Value is DBNull)) return;
                if (e.Value.ToString().Length < 1) {
                    e.Value = DBNull.Value;
                    return;
                }
                e.Value = DataAccess.DataAccessFactory.toDBDecimal(decimal.Parse(e.Value.ToString()));
            }
            catch (Exception) { }
        }

        public bool checkExitWithoutSave(string formName)
        {
            //return true;
            DialogResult result = MessageBox.Show(UWWTD2008MainForm.getMainForm(this), "There are unsaved Items on this Form that will be lost when you go back. Do you really want to exit?", formName, MessageBoxButtons.YesNo, MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button2);
            return result == DialogResult.Yes;
        }

        public bool confirmExitWithValidationFailed(string errorText, string formName)
        {
            DialogResult result = MessageBox.Show(UWWTD2008MainForm.getMainForm(this), errorText, formName + " - Validation failed", MessageBoxButtons.YesNo, MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button2);
            return result == DialogResult.Yes;
        }
    }
}
