﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Xml.Serialization;
using System.IO;
using System.Data;
using at.uba.uwwtd2008.DataAccess;
using at.uba.uwwtd2008.components;

namespace at.uba.uwwtd2008.xml
{
    class UWWXMLWriter
    {
        private static DataAccessFactory dataFactory = DataAccessFactory.Instance;

        public static UWWTD_DataUWWTD_ReportAgglomeration readAgglomeration(IDataReader rdr, DataSet dataSet, IDbConnection conn)
        {
            UWWTD_DataUWWTD_ReportAgglomeration agglo = new UWWTD_DataUWWTD_ReportAgglomeration();
            agglo.aggAccOverflowNumberSpecified = !(rdr["aggAccOverflowNumber"] is DBNull);
            if (agglo.aggAccOverflowNumberSpecified)
                agglo.aggAccOverflowNumber = long.Parse(rdr["aggAccOverflowNumber"].ToString());
            agglo.aggAccOverflowsSpecified = !(rdr["aggAccOverflows"] is DBNull);
            if (agglo.aggAccOverflowsSpecified)
                agglo.aggAccOverflows = (bool)rdr["aggAccOverflows"];
            agglo.aggBestTechnicalKnowledge = rdr["aggBestTechnicalKnowledge"] as string;

            agglo.aggC1Specified = !(rdr["aggC1"] is DBNull);
            if (agglo.aggC1Specified)
                agglo.aggC1 = (long)double.Parse(rdr["aggC1"].ToString());

            agglo.aggC2Specified = !(rdr["aggC2"] is DBNull);
            if (agglo.aggC2Specified)
                agglo.aggC2 = (long)double.Parse(rdr["aggC2"].ToString());

            agglo.aggCalculation = rdr["aggCalculation"] as string;

            agglo.aggCapacitySpecified = !(rdr["aggCapacity"] is DBNull);
            if (agglo.aggCapacitySpecified)
                agglo.aggCapacity = (bool)rdr["aggCapacity"];

            agglo.aggChanges = rdr["aggChanges"] as string;
            agglo.aggChangesComment = rdr["aggChangesComment"] as string;
            agglo.aggCode = rdr["aggCode"] as string;

            agglo.aggDilutionRatesSpecified = !(rdr["aggDilutionRates"] is DBNull);
            if (agglo.aggDilutionRatesSpecified)
                agglo.aggDilutionRates = (bool)rdr["aggDilutionRates"];

            agglo.aggExistMaintenancePlan = rdr["aggExistMaintenancePlan"] as string;
            agglo.aggExplanationOther = rdr["aggExplanationOther"] as string;
            agglo.aggForecastSpecified = !(rdr["aggForecast"] is DBNull);
            if (agglo.aggForecastSpecified)
                agglo.aggForecast = (DateTime)rdr["aggForecast"];

            if (rdr["aggGenerated"] is DBNull)
                agglo.aggGenerated = 0;
            else
                agglo.aggGenerated = long.Parse(rdr["aggGenerated"].ToString());
            agglo.aggHaveRegistrationSystem = rdr["aggHaveRegistrationSystem"] as string;
            agglo.aggLatitudeSpecified = !(rdr["aggLatitude"] is DBNull);
            if (agglo.aggLatitudeSpecified)
                agglo.aggLatitude = convertDecimal(rdr["aggLatitude"]);
            agglo.aggLongitudeSpecified = !((rdr["aggLongitude"] is DBNull));
            if (agglo.aggLongitudeSpecified)
                agglo.aggLongitude = convertDecimal(rdr["aggLongitude"]);
            if (rdr["aggMethodC1"] is DBNull)
            {
                // error - this must be specified
            }
            else
            {
                agglo.aggMethodC1 = dataSet.Tables["T_LOV"].Rows.Find(rdr["aggMethodC1"])["lovKey"] as string;
            }
            if (rdr["aggMethodC2"] is DBNull)
            {
                // error - this must be specified
            }
            else
            {
                agglo.aggMethodC2 = dataSet.Tables["T_LOV"].Rows.Find(rdr["aggMethodC2"])["lovKey"] as string;
            }
            if (rdr["aggMethodWithoutTreatment"] is DBNull)
            {
                // error - this must be specified
            }
            else
            {
                agglo.aggMethodWithoutTreatment = dataSet.Tables["T_LOV"].Rows.Find(rdr["aggMethodWithoutTreatment"])["lovKey"] as string;
            }
            agglo.aggName = rdr["aggName"] as string;
            if (rdr["aggNUTS"] is DBNull)
                agglo.aggNUTS = null;
            else
                agglo.aggNUTS = (dataSet.Tables["T_NUTS"].Rows.Find(rdr["aggNUTS"])["nutNUTS"]) as string;

            agglo.aggOtherMeasuresSpecified = !(rdr["aggOtherMeasures"] is DBNull);
            if (agglo.aggOtherMeasuresSpecified)
                agglo.aggOtherMeasures = (bool)rdr["aggOtherMeasures"];
            agglo.aggPercPrimTreatmentSpecified = !(rdr["aggPercPrimTreatment"] is DBNull);
            if (agglo.aggPercPrimTreatmentSpecified)
                agglo.aggPercPrimTreatment = convertDecimal(rdr["aggPercPrimTreatment"]);
            agglo.aggPercSecTreatmentSpecified = !(rdr["aggPercSecTreatment"] is DBNull);
            if (agglo.aggPercSecTreatmentSpecified)
                agglo.aggPercSecTreatment = convertDecimal(rdr["aggPercSecTreatment"]);
            agglo.aggPercStringentTreatmentSpecified = !(rdr["aggPercStringentTreatment"] is DBNull);
            if (agglo.aggPercStringentTreatmentSpecified)
                agglo.aggPercStringentTreatment = convertDecimal(rdr["aggPercStringentTreatment"]);
            agglo.aggPercWithoutTreatmentSpecified = !(rdr["aggPercWithoutTreatment"] is DBNull);
            if (agglo.aggPercWithoutTreatmentSpecified)
                agglo.aggPercWithoutTreatment = convertDecimal(rdr["aggPercWithoutTreatment"]);
            if (rdr["aggPeriodOver"] is DBNull)
                agglo.aggPeriodOver = DateTime.MinValue; // TODO: This is an error - should be filled
            else
                agglo.aggPeriodOver = (DateTime)rdr["aggPeriodOver"];
            agglo.aggPercWithoutTreatmentSpecified = !(rdr["aggPercWithoutTreatment"] is DBNull);
            if (agglo.aggPercWithoutTreatmentSpecified)
                agglo.aggPercWithoutTreatment = convertDecimal(rdr["aggPercWithoutTreatment"]);
            agglo.aggPressureTest = (bool)rdr["aggPressureTest"];
            agglo.aggPressureTestSpecified = !(rdr["aggPressureTest"] is DBNull);
            if (agglo.aggPressureTestSpecified)
                agglo.aggPressureTest = (bool)rdr["aggPressureTest"];
            agglo.aggRemarks = rdr["aggRemarks"] as string;
            agglo.aggSewageNetwork = rdr["aggSewageNetwork"] as string;
            agglo.aggSewerOverflows_m3Specified = !(rdr["aggSewerOverflows_m3"] is DBNull);
            if (agglo.aggSewerOverflows_m3Specified)
                agglo.aggSewerOverflows_m3 = long.Parse(rdr["aggSewerOverflows_m3"].ToString());
            agglo.aggSewerOverflows_peSpecified = !(rdr["aggSewerOverflows_pe"] is DBNull);
            if (agglo.aggSewerOverflows_peSpecified)
                agglo.aggSewerOverflows_pe = long.Parse(rdr["aggSewerOverflows_pe"].ToString());

            agglo.aggState = dataSet.Tables["T_LOV"].Rows.Find(rdr["aggState"])["lovKey"] as string;
            agglo.aggVideoInspectionsSpecified = !(rdr["aggVideoInspections"] is DBNull);
            if (agglo.aggVideoInspectionsSpecified)
                agglo.aggVideoInspections = (bool)rdr["aggVideoInspections"];
            if (rdr["bigID"] is DBNull)
                agglo.bigID = null;
            else
                agglo.bigID = (dataFactory.CreateCommand("SELECT bigCity FROM T_BIGCITY WHERE bigID = " + rdr["bigID"], conn).ExecuteScalar()) as string;

            return agglo;
        }

        public static UWWTD_DataUWWTD_ReportBigCityDischarger readBigCities(IDataReader rdr, DataSet dataSet, IDbConnection conn)
        {
            UWWTD_DataUWWTD_ReportBigCityDischarger city = new UWWTD_DataUWWTD_ReportBigCityDischarger();
            city.bigCityID = rdr["bigCityID"] as string;
            city.bigCountryCode = rdr["bigCountryCode"] as string;
            city.bigCity = rdr["bigCity"] as string;
            return city;
        }

        public static UWWTD_DataUWWTD_ReportDischargePoint readDischargePoints(IDataReader rdr, DataSet dataSet, IDbConnection conn)
        {
            UWWTD_DataUWWTD_ReportDischargePoint dp = new UWWTD_DataUWWTD_ReportDischargePoint();
            dp.dcpState = dataSet.Tables["T_LOV"].Rows.Find(rdr["dcpState"])["lovKey"] as string;
            if (!(rdr["uwwID"] is DBNull))
            {
                dp.uwwID = (dataFactory.CreateCommand("SELECT uwwCode FROM T_UWWTPS WHERE uwwID = " + rdr["uwwID"], conn).ExecuteScalar()) as string;
            }
            dp.dcpCode = rdr["dcpCode"] as string;
            dp.dcpName = rdr["dcpName"] as string;
            if (rdr["dcpNuts"] is DBNull)
                dp.dcpNUTS = null;
            else
                dp.dcpNUTS = (dataSet.Tables["T_NUTS"].Rows.Find(rdr["dcpNuts"])["nutNUTS"]) as string;
            dp.dcpLatitudeSpecified = !(rdr["dcpLatitude"] is DBNull);
            if (dp.dcpLatitudeSpecified)
                dp.dcpLatitude = convertDecimal(rdr["dcpLatitude"]);
            dp.dcpLongitudeSpecified = !((rdr["dcpLongitude"] is DBNull));
            if (dp.dcpLongitudeSpecified)
                dp.dcpLongitude = convertDecimal(rdr["dcpLongitude"]);

            if (rdr["dcpWaterBodyType"] is DBNull)
                dp.dcpWaterBodyType = null;
            else
                dp.dcpWaterBodyType = dataSet.Tables["T_LOV"].Rows.Find(rdr["dcpWaterBodyType"])["lovKey"] as string;

            if (rdr["dcpIrrigation"] is DBNull)
                dp.dcpIrrigation = null;
            else
                dp.dcpIrrigation = dataSet.Tables["T_LOV"].Rows.Find(rdr["dcpIrrigation"])["lovKey"] as string;
            if (rdr["dcpTypeOfReceivingArea"] is DBNull)
                dp.dcpTypeOfReceivingArea = null;
            else
                dp.dcpTypeOfReceivingArea = dataSet.Tables["T_LOV"].Rows.Find(rdr["dcpTypeOfReceivingArea"])["lovKey"] as string;

            if (rdr["rcaID"] is DBNull)
                dp.rcaID = null;
            else
                dp.rcaID = (dataFactory.CreateCommand("SELECT rcaCode FROM T_ReceivingAreas WHERE rcaID = " + rdr["rcaID"], conn).ExecuteScalar()) as string;

            if (rdr["dcpWaterbodyID"] is DBNull)
                dp.dcpWaterbodyID = null;
            else
                dp.dcpWaterbodyID = rdr["dcpWaterbodyID"] as string;
            if (rdr["dcpSurfaceWaters"] is DBNull)
                dp.dcpSurfaceWaters = null;
            else
                dp.dcpSurfaceWaters = dataSet.Tables["T_LOV"].Rows.Find(rdr["dcpSurfaceWaters"])["lovKey"] as string;
            if (rdr["dcpNotAffect"] is DBNull)
                dp.dcpNotAffect = null;
            else
                dp.dcpNotAffect = dataSet.Tables["T_LOV"].Rows.Find(rdr["dcpNotAffect"])["lovKey"] as string;
            if (rdr["dcpMSProvide"] is DBNull)
                dp.dcpMSProvide = null;
            else
                dp.dcpMSProvide = dataSet.Tables["T_LOV"].Rows.Find(rdr["dcpMSProvide"])["lovKey"] as string;
            if (rdr["dcpCOMAccept"] is DBNull)
                dp.dcpCOMAccept = null;
            else
                dp.dcpCOMAccept = dataSet.Tables["T_LOV"].Rows.Find(rdr["dcpCOMAccept"])["lovKey"] as string;
            dp.dcpGroundWater = rdr["dcpGroundWater"] as string;
            dp.dcpReceivingWater = rdr["dcpReceivingWater"] as string;
            dp.dcpWFDSubUnit = rdr["dcpWFDSubUnit"] as string;
            dp.dcpWFDRBD = rdr["dcpWFDRBD"] as string;
            dp.dcpRemarks = rdr["dcpRemarks"] as string;
            dp.dcpWFDRBDReferenceDateSpecified = rdr["dcpWFDRBDReferenceDate"] is DateTime;
            if (dp.dcpWFDRBDReferenceDateSpecified)
            {
                dp.dcpWFDRBDReferenceDate = (DateTime) rdr["dcpWFDRBDReferenceDate"];
            }
            return dp;
        }

        public static UWWTD_DataUWWTD_ReportIndustry readIndustry(IDataReader rdr, DataSet dataSet, IDbConnection conn) {
            UWWTD_DataUWWTD_ReportIndustry data = new UWWTD_DataUWWTD_ReportIndustry();
            data.indState = dataSet.Tables["T_LOV"].Rows.Find(rdr["indState"])["lovKey"] as string;
            data.indCodePlant = rdr["indCodePlant"] as string;
            data.indNamePlant = rdr["indNamePlant"] as string;
            if (rdr["indBranch"] is DBNull)
                data.indBranch = null;
            else
                data.indBranch = dataSet.Tables["T_LOV"].Rows.Find(rdr["indBranch"])["lovKey"] as string;
            if (rdr["indOrganicLoad"] is DBNull)
                data.indOrganicLoadSpecified = false;
            else
            {
                data.indOrganicLoadSpecified = true;
                data.indOrganicLoad = long.Parse(rdr["indOrganicLoad"].ToString());
            }
            if (rdr["indConditions"] is DBNull)
                data.indConditionsSpecified = false;
            else
            {
                data.indConditionsSpecified = true;
                data.indConditions = (bool)rdr["indConditions"];
            }
            if (rdr["indDateCompiliance"] is DBNull)
                data.indDateCompilianceSpecified = false;
            else
            {
                data.indDateCompilianceSpecified = true;
                data.indDateCompiliance = (DateTime)rdr["indDateCompiliance"];
            }
            return data;
        }

        public static UWWTD_DataUWWTD_ReportMSLevel readMSLevel(IDataReader rdr, DataSet dataSet, IDbConnection conn) {
            UWWTD_DataUWWTD_ReportMSLevel data = new UWWTD_DataUWWTD_ReportMSLevel();
            if (rdr["mslSludgeProduction"] is DBNull)
                data.mslSludgeProductionSpecified = false;
            else
            {
                data.mslSludgeProductionSpecified = true;
                data.mslSludgeProduction = long.Parse(rdr["mslSludgeProduction"].ToString());
            }
            if (rdr["mslDischargePipelines"] is DBNull)
                data.mslDischargePipelinesSpecified = false;
            else
            {
                data.mslDischargePipelinesSpecified = true;
                data.mslDischargePipelines = convertDecimal(rdr["mslDischargePipelines"]);
            }
            if (rdr["mslDischargeShips"] is DBNull)
                data.mslDischargeShipsSpecified = false;
            else
            {
                data.mslDischargeShipsSpecified = true;
                data.mslDischargeShips = convertDecimal(rdr["mslDischargeShips"]);
            }
            if (rdr["mslDischargeOthers"] is DBNull)
                data.mslDischargeOthersSpecified = false;
            else
            {
                data.mslDischargeOthersSpecified = true;
                data.mslDischargeOthers = convertDecimal(rdr["mslDischargeOthers"]);
            }
            if (rdr["mslReuseSoilAgriculture"] is DBNull)
                data.mslReuseSoilAgricultureSpecified = false;
            else
            {
                data.mslReuseSoilAgricultureSpecified = true;
                data.mslReuseSoilAgriculture = convertDecimal(rdr["mslReuseSoilAgriculture"]);
            }
            if (rdr["mslReuseOthers"] is DBNull)
                data.mslReuseOthersSpecified = false;
            else
            {
                data.mslReuseOthersSpecified = true;
                data.mslReuseOthers = convertDecimal(rdr["mslReuseOthers"]);
            }
            if (rdr["mslDisposalLandfill"] is DBNull)
                data.mslDisposalLandfillSpecified = false;
            else
            {
                data.mslDisposalLandfillSpecified = true;
                data.mslDisposalLandfill = convertDecimal(rdr["mslDisposalLandfill"]);
            }
            if (rdr["mslDisposalIncineration"] is DBNull)
                data.mslDisposalIncinerationSpecified = false;
            else
            {
                data.mslDisposalIncinerationSpecified = true;
                data.mslDisposalIncineration = convertDecimal(rdr["mslDisposalIncineration"]);
            }
            if (rdr["mslDisposalOthers"] is DBNull)
                data.mslDisposalOthersSpecified = false;
            else
            {
                data.mslDisposalOthersSpecified = true;
                data.mslDisposalOthers = convertDecimal(rdr["mslDisposalOthers"]);
            }

            data.mslRemarks = rdr["mslRemarks"] as String;

            if (rdr["mslWWReusePerc"] is DBNull)
                data.mslWWReusePercSpecified = false;
            else
            {
                data.mslWWReusePercSpecified = true;
                data.mslWWReusePerc = long.Parse(rdr["mslWWReusePerc"].ToString());
            }

            if (rdr["mslWWReuseAgri"] is DBNull)
                data.mslWWReuseAgriSpecified = false;
            else
            {
                data.mslWWReuseAgriSpecified = true;
                data.mslWWReuseAgri = (bool)rdr["mslWWReuseAgri"];
            }
            if (rdr["mslWWReuseInd"] is DBNull)
                data.mslWWReuseIndSpecified = false;
            else
            {
                data.mslWWReuseIndSpecified = true;
                data.mslWWReuseInd = (bool)rdr["mslWWReuseInd"];
            }
            if (rdr["mslWWReuseOther"] is DBNull)
                data.mslWWReuseOtherSpecified = false;
            else
            {
                data.mslWWReuseOtherSpecified = true;
                data.mslWWReuseOther = (bool)rdr["mslWWReuseOther"];
            }

            data.mslWWReuseExplain = rdr["mslWWReuseExplain"] as String;

            return data;
        }

        public static UWWTD_DataUWWTD_ReportReceivingArea readReceivingAreas(IDataReader rdr, DataSet dataSet, IDbConnection conn) {
            UWWTD_DataUWWTD_ReportReceivingArea data = new UWWTD_DataUWWTD_ReportReceivingArea();
            data.rcaState = dataSet.Tables["T_LOV"].Rows.Find(rdr["rcaState"])["lovKey"] as string;
            data.rcaCode = rdr["rcaCode"] as string;
            data.rcaName = rdr["rcaName"] as string;
            if (rdr["rcaType"] is DBNull)
                data.rcaType = null;
            else
                data.rcaType = dataSet.Tables["T_LOV"].Rows.Find(rdr["rcaType"])["lovKey"] as string;

            if (rdr["rcaSensitiveArea"] is DBNull)
                data.rcaSensitiveArea = null;
            else
            {
                //data.rcaSensitiveArea = dataSet.Tables["T_LOV"].Rows.Find(rdr["rcaSensitiveArea"])["lovKey"] as string;
                IDbCommand cmd = dataFactory.CreateCommand("select rcaCode from T_ReceivingAreas where rcaID = ?", conn);
                cmd.Parameters.Add(dataFactory.CreateParameter("@rcaID", DbType.Int32, 0, "rcaID", rdr["rcaSensitiveArea"]));
                data.rcaSensitiveArea = cmd.ExecuteScalar() as string;
                cmd.Dispose();
            }

            data.rcaDateArt58Specified = true;
            if (rdr["rcaDateArt58"] is DBNull)
                data.rcaDateArt58Specified = false;
            else
                data.rcaDateArt58 = ((DateTime)rdr["rcaDateArt58"]);
            data.rcaParameterPSpecified = true;
            if (rdr["rcaParameterP"] is DBNull)
                data.rcaParameterPSpecified = false;
            else
                data.rcaParameterP = ((bool)rdr["rcaParameterP"]);
            data.rcaParameterNSpecified = true;
            if (rdr["rcaParameterN"] is DBNull)
                data.rcaParameterNSpecified = false;
            else
                data.rcaParameterN = ((bool)rdr["rcaParameterN"]);
            data.rcaParameterOtherSpecified = true;
            if (rdr["rcaParameterOther"] is DBNull)
                data.rcaParameterOtherSpecified = false;
            else
                data.rcaParameterOther = ((bool)rdr["rcaParameterOther"]);
            data.rcaDateDesignationSpecified = true;
            if (rdr["rcaDateDesignation"] is DBNull)
                data.rcaDateDesignationSpecified = false;
            else
                data.rcaDateDesignation = ((DateTime)rdr["rcaDateDesignation"]);
            data.rcaANitroSpecified = true;
            if (rdr["rcaANitro"] is DBNull)
                data.rcaANitroSpecified = false;
            else
                data.rcaANitro = ((bool)rdr["rcaANitro"]);
            data.rcaAPhosSpecified = true;
            if (rdr["rcaAPhos"] is DBNull)
                data.rcaAPhosSpecified = false;
            else
                data.rcaAPhos = ((bool)rdr["rcaAPhos"]);
            data.rcaBSpecified = true;
            if (rdr["rcaB"] is DBNull)
                data.rcaBSpecified = false;
            else
                data.rcaB = ((bool)rdr["rcaB"]);
            data.rcaCSpecified = true;
            if (rdr["rcaC"] is DBNull)
                data.rcaCSpecified = false;
            else
                data.rcaC = ((bool)rdr["rcaC"]);
            data.rcaCRelevantDirective = rdr["rcaCRelevantDirective"] as string;
            data.rcaMorphologySpecified = true;
            if (rdr["rcaMorphology"] is DBNull)
                data.rcaMorphologySpecified = false;
            else
                data.rcaMorphology = ((bool)rdr["rcaMorphology"]);
            data.rcaHydrologieSpecified = true;
            if (rdr["rcaHydrologie"] is DBNull)
                data.rcaHydrologieSpecified = false;
            else
                data.rcaHydrologie = ((bool)rdr["rcaHydrologie"]);
            data.rcaHydraulicSpecified = true;
            if (rdr["rcaHydraulic"] is DBNull)
                data.rcaHydraulicSpecified = false;
            else
                data.rcaHydraulic = ((bool)rdr["rcaHydraulic"]);
            data.rcaAbsenceRiskSpecified = true;
            if (rdr["rcaAbsenceRisk"] is DBNull)
                data.rcaAbsenceRiskSpecified = false;
            else
                data.rcaAbsenceRisk = ((bool)rdr["rcaAbsenceRisk"]);
            data.rcaDateArt54Specified = true;
            if (rdr["rcaDateArt54"] is DBNull)
                data.rcaDateArt54Specified = false;
            else
                data.rcaDateArt54 = ((DateTime)rdr["rcaDateArt54"]);

            data.rcaPlantsSpecified = true;
            if (rdr["rcaPlants"] is DBNull)
                data.rcaPlantsSpecified = false;
            else
                data.rcaPlants = long.Parse(rdr["rcaPlants"].ToString());
            data.rcaPlantsCapacitySpecified = true;
            if (rdr["rcaPlantsCapacity"] is DBNull)
                data.rcaPlantsCapacitySpecified = false;
            else
                data.rcaPlantsCapacity = long.Parse(rdr["rcaPlantsCapacity"].ToString());

            data.rcaNIncomingMeasuredSpecified = true;
            if (rdr["rcaNIncomingMeasured"] is DBNull)
                data.rcaNIncomingMeasuredSpecified = false;
            else
                data.rcaNIncomingMeasured = convertDecimal(rdr["rcaNIncomingMeasured"]);
            data.rcaNIncomingEstimatedSpecified = true;
            if (rdr["rcaNIncomingEstimated"] is DBNull)
                data.rcaNIncomingEstimatedSpecified = false;
            else
                data.rcaNIncomingEstimated = convertDecimal(rdr["rcaNIncomingEstimated"]);
            data.rcaNIncomingCalculatedSpecified = true;
            if (rdr["rcaNIncomingCalculated"] is DBNull)
                data.rcaNIncomingCalculatedSpecified = false;
            else
                data.rcaNIncomingCalculated = convertDecimal(rdr["rcaNIncomingCalculated"]);
            data.rcaPIncomingMeasuredSpecified = true;
            if (rdr["rcaPIncomingMeasured"] is DBNull)
                data.rcaPIncomingMeasuredSpecified = false;
            else
                data.rcaPIncomingMeasured = convertDecimal(rdr["rcaPIncomingMeasured"]);
            data.rcaPIncomingEstimatedSpecified = true;
            if (rdr["rcaPIncomingEstimated"] is DBNull)
                data.rcaPIncomingEstimatedSpecified = false;
            else
                data.rcaPIncomingEstimated = convertDecimal(rdr["rcaPIncomingEstimated"]);
            data.rcaPIncomingCalculatedSpecified = true;
            if (rdr["rcaPIncomingCalculated"] is DBNull)
                data.rcaPIncomingCalculatedSpecified = false;
            else
                data.rcaPIncomingCalculated = convertDecimal(rdr["rcaPIncomingCalculated"]);
            data.rcaNDischargedMeasuredSpecified = true;
            if (rdr["rcaNDischargedMeasured"] is DBNull)
                data.rcaNDischargedMeasuredSpecified = false;
            else
                data.rcaNDischargedMeasured = convertDecimal(rdr["rcaNDischargedMeasured"]);
            data.rcaNDischargedEstimatedSpecified = true;
            if (rdr["rcaNDischargedEstimated"] is DBNull)
                data.rcaNDischargedEstimatedSpecified = false;
            else
                data.rcaNDischargedEstimated = convertDecimal(rdr["rcaNDischargedEstimated"]);
            data.rcaNDischargedCalculatedSpecified = true;
            if (rdr["rcaNDischargedCalculated"] is DBNull)
                data.rcaNDischargedCalculatedSpecified = false;
            else
                data.rcaNDischargedCalculated = convertDecimal(rdr["rcaNDischargedCalculated"]);
            data.rcaPDischargedMeasuredSpecified = true;
            if (rdr["rcaPDischargedMeasured"] is DBNull)
                data.rcaPDischargedMeasuredSpecified = false;
            else
                data.rcaPDischargedMeasured = convertDecimal(rdr["rcaPDischargedMeasured"]);
            data.rcaPDischargedEstimatedSpecified = true;
            if (rdr["rcaPDischargedEstimated"] is DBNull)
                data.rcaPDischargedEstimatedSpecified = false;
            else
                data.rcaPDischargedEstimated = convertDecimal(rdr["rcaPDischargedEstimated"]);
            data.rcaPDischargedCalculatedSpecified = true;
            if (rdr["rcaPDischargedCalculated"] is DBNull)
                data.rcaPDischargedCalculatedSpecified = false;
            else
                data.rcaPDischargedCalculated = convertDecimal(rdr["rcaPDischargedCalculated"]);

            data.rcaGIS = rdr["rcaGIS"] as string;
            data.rcaCIDOtherDirective = rdr["rcaCIDOtherDirective"] as string;

            data.rcaCDateOtherDirectiveSpecified = true;
            if (rdr["rcaCDateOtherDirective"] is DBNull)
                data.rcaCDateOtherDirectiveSpecified = false;
            else
                data.rcaCDateOtherDirective = ((DateTime)rdr["rcaCDateOtherDirective"]);

            data.rcaRemarks = rdr["rcaRemarks"] as string;

            data.rcaArt54AppliedSpecified = true;
            if (rdr["rcaArt54Applied"] is DBNull)
                data.rcaArt54AppliedSpecified = false;
            else
                data.rcaArt54Applied = ((bool)rdr["rcaArt54Applied"]);
            return data;
        }

        public static UWWTD_DataUWWTD_ReportUwwtpAgglo readUwwtpAgglo(IDataReader rdr, DataSet dataSet, IDbConnection conn) {
            UWWTD_DataUWWTD_ReportUwwtpAgglo data = new UWWTD_DataUWWTD_ReportUwwtpAgglo();
            data.aucAggCode = rdr["aucAggCode"] as string;
            data.aucAggName = rdr["aucAggName"] as string;
            data.aucUwwCode = rdr["aucUwwCode"] as string;
            data.aucUwwName = rdr["aucUwwName"] as string;

            if (rdr["aucMethodPercEnteringUWWTP"] is DBNull)
                data.aucMethodPercEnteringUWWTP = null;
            else
                data.aucMethodPercEnteringUWWTP = dataSet.Tables["T_LOV"].Rows.Find(rdr["aucMethodPercEnteringUWWTP"])["lovKey"] as string;

            data.aucPercEnteringUWWTPSpecified = true;
            if (rdr["aucPercEnteringUWWTP"] is DBNull)
                data.aucPercEnteringUWWTPSpecified = false;
            else
                data.aucPercEnteringUWWTP = convertDecimal(rdr["aucPercEnteringUWWTP"]);

            data.aucPercC2TSpecified = true;
            if (rdr["aucPercC2T"] is DBNull)
                data.aucPercC2TSpecified = false;
            else
                data.aucPercC2T = convertDecimal(rdr["aucPercC2T"]);
            return data;
        }

        public static UWWTD_DataUWWTD_ReportUWWTP readUWWTP(IDataReader rdr, DataSet dataSet, IDbConnection conn) {
            UWWTD_DataUWWTD_ReportUWWTP data = new UWWTD_DataUWWTD_ReportUWWTP();
            data.uwwState = dataSet.Tables["T_LOV"].Rows.Find(rdr["uwwState"])["lovKey"] as string;
            data.uwwCode = rdr["uwwCode"] as string;
            data.uwwName = rdr["uwwName"] as string;
            if (!(rdr["aggID"] is DBNull))
            {
                data.aggID = (dataFactory.CreateCommand("SELECT aggCode FROM T_Agglomerations WHERE aggID = " + rdr["aggID"], conn).ExecuteScalar()) as string;
            }

            if (rdr["uwwCollectingSystem"] is DBNull)
                data.uwwCollectingSystem = null;
            else
                data.uwwCollectingSystem = dataSet.Tables["T_LOV"].Rows.Find(rdr["uwwCollectingSystem"])["lovKey"] as string;

            data.uwwDateClosing = rdr["uwwDateClosing"] as string;
            data.uwwHistorie = rdr["uwwHistorie"] as string;

            data.uwwLatitudeSpecified = !(rdr["uwwLatitude"] is DBNull);
            if (data.uwwLatitudeSpecified)
                data.uwwLatitude = convertDecimal(rdr["uwwLatitude"]);
            data.uwwLongitudeSpecified = !((rdr["uwwLongitude"] is DBNull));
            if (data.uwwLongitudeSpecified)
                data.uwwLongitude = convertDecimal(rdr["uwwLongitude"]);
            if (rdr["uwwNUTS"] is DBNull)
                data.uwwNUTS = null;
            else
                data.uwwNUTS = (dataSet.Tables["T_NUTS"].Rows.Find(rdr["uwwNUTS"])["nutNUTS"]) as string;
            data.uwwLoadEnteringUWWTPSpecified = true;
            if (rdr["uwwLoadEnteringUWWTP"] is DBNull)
                data.uwwLoadEnteringUWWTPSpecified = false;
            else
                data.uwwLoadEnteringUWWTP = long.Parse(rdr["uwwLoadEnteringUWWTP"].ToString());

            data.uwwCapacitySpecified = true;
            if (rdr["uwwCapacity"] is DBNull)
                data.uwwCapacitySpecified = false;
            else
                data.uwwCapacity = long.Parse(rdr["uwwCapacity"].ToString());

            data.uwwPrimaryTreatmentSpecified = true;
            if (rdr["uwwPrimaryTreatment"] is DBNull)
                data.uwwPrimaryTreatmentSpecified = false;
            else
                data.uwwPrimaryTreatment = ((bool)rdr["uwwPrimaryTreatment"]);
            data.uwwSecondaryTreatmentSpecified = true;
            if (rdr["uwwSecondaryTreatment"] is DBNull)
                data.uwwSecondaryTreatmentSpecified = false;
            else
                data.uwwSecondaryTreatment = ((bool)rdr["uwwSecondaryTreatment"]);
            data.uwwOtherTreatmentSpecified = true;
            if (rdr["uwwOtherTreatment"] is DBNull)
                data.uwwOtherTreatmentSpecified = false;
            else
                data.uwwOtherTreatment = ((bool)rdr["uwwOtherTreatment"]);
            data.uwwNRemovalSpecified = true;
            if (rdr["uwwNRemoval"] is DBNull)
                data.uwwNRemovalSpecified = false;
            else
                data.uwwNRemoval = ((bool)rdr["uwwNRemoval"]);
            data.uwwPRemovalSpecified = true;
            if (rdr["uwwPRemoval"] is DBNull)
                data.uwwPRemovalSpecified = false;
            else
                data.uwwPRemoval = ((bool)rdr["uwwPRemoval"]);
            data.uwwUVSpecified = true;
            if (rdr["uwwUV"] is DBNull)
                data.uwwUVSpecified = false;
            else
                data.uwwUV = ((bool)rdr["uwwUV"]);
            data.uwwChlorinationSpecified = true;
            if (rdr["uwwChlorination"] is DBNull)
                data.uwwChlorinationSpecified = false;
            else
                data.uwwChlorination = ((bool)rdr["uwwChlorination"]);
            data.uwwOzonationSpecified = true;
            if (rdr["uwwOzonation"] is DBNull)
                data.uwwOzonationSpecified = false;
            else
                data.uwwOzonation = ((bool)rdr["uwwOzonation"]);
            data.uwwSandFiltrationSpecified = true;
            if (rdr["uwwSandFiltration"] is DBNull)
                data.uwwSandFiltrationSpecified = false;
            else
                data.uwwSandFiltration = ((bool)rdr["uwwSandFiltration"]);
            data.uwwMicroFiltrationSpecified = true;
            if (rdr["uwwMicroFiltration"] is DBNull)
                data.uwwMicroFiltrationSpecified = false;
            else
                data.uwwMicroFiltration = ((bool)rdr["uwwMicroFiltration"]);
            data.uwwOtherSpecified = true;
            if (rdr["uwwOther"] is DBNull)
                data.uwwOtherSpecified = false;
            else
                data.uwwOther = ((bool)rdr["uwwOther"]);

            data.uwwSpecification = rdr["uwwSpecification"] as string;
            
            if (rdr["uwwBOD5Perf"] is DBNull)
                data.uwwBOD5Perf = null;
            else
                data.uwwBOD5Perf = dataSet.Tables["T_LOV"].Rows.Find(rdr["uwwBOD5Perf"])["lovKey"] as string;
            if (rdr["uwwCODPerf"] is DBNull)
                data.uwwCODPerf = null;
            else
                data.uwwCODPerf = dataSet.Tables["T_LOV"].Rows.Find(rdr["uwwCODPerf"])["lovKey"] as string;
            if (rdr["uwwTSSPerf"] is DBNull)
                data.uwwTSSPerf = null;
            else
                data.uwwTSSPerf = dataSet.Tables["T_LOV"].Rows.Find(rdr["uwwTSSPerf"])["lovKey"] as string;
            if (rdr["uwwNTotPerf"] is DBNull)
                data.uwwNTotPerf = null;
            else
                data.uwwNTotPerf = dataSet.Tables["T_LOV"].Rows.Find(rdr["uwwNTotPerf"])["lovKey"] as string;
            if (rdr["uwwPTotPerf"] is DBNull)
                data.uwwPTotPerf = null;
            else
                data.uwwPTotPerf = dataSet.Tables["T_LOV"].Rows.Find(rdr["uwwPTotPerf"])["lovKey"] as string;
            if (rdr["uwwOtherPerf"] is DBNull)
                data.uwwOtherPerf = null;
            else
                data.uwwOtherPerf = dataSet.Tables["T_LOV"].Rows.Find(rdr["uwwOtherPerf"])["lovKey"] as string;
            data.uwwBadPerformanceSpecified = true;
            if (rdr["uwwBadPerformance"] is DBNull)
                data.uwwBadPerformanceSpecified = false;
            else
                data.uwwBadPerformance = ((bool)rdr["uwwBadPerformance"]);
            data.uwwAccidentsSpecified = true;
            if (rdr["uwwAccidents"] is DBNull)
                data.uwwAccidentsSpecified = false;
            else
                data.uwwAccidents = ((bool)rdr["uwwAccidents"]);
            data.uwwBadDesignSpecified = true;
            if (rdr["uwwBadDesign"] is DBNull)
                data.uwwBadDesignSpecified = false;
            else
                data.uwwBadDesign = ((bool)rdr["uwwBadDesign"]);

            data.uwwInformation = rdr["uwwInformation"] as string;

            data.uwwBODIncomingMeasuredSpecified = true;
            if (rdr["uwwBODIncomingMeasured"] is DBNull)
                data.uwwBODIncomingMeasuredSpecified = false;
            else
                data.uwwBODIncomingMeasured = (decimal) double.Parse(rdr["uwwBODIncomingMeasured"].ToString());
            data.uwwBODIncomingEstimatedSpecified = true;
            if (rdr["uwwBODIncomingEstimated"] is DBNull)
                data.uwwBODIncomingEstimatedSpecified = false;
            else
                data.uwwBODIncomingEstimated = (decimal) double.Parse(rdr["uwwBODIncomingEstimated"].ToString());
            data.uwwBODIncomingCalculatedSpecified = true;
            if (rdr["uwwBODIncomingCalculated"] is DBNull)
                data.uwwBODIncomingCalculatedSpecified = false;
            else
                data.uwwBODIncomingCalculated = (decimal) double.Parse(rdr["uwwBODIncomingCalculated"].ToString());
            data.uwwCODIncomingMeasuredSpecified = true;
            if (rdr["uwwCODIncomingMeasured"] is DBNull)
                data.uwwCODIncomingMeasuredSpecified = false;
            else
                data.uwwCODIncomingMeasured = (decimal) double.Parse(rdr["uwwCODIncomingMeasured"].ToString());
            data.uwwCODIncomingEstimatedSpecified = true;
            if (rdr["uwwCODIncomingEstimated"] is DBNull)
                data.uwwCODIncomingEstimatedSpecified = false;
            else
                data.uwwCODIncomingEstimated = (decimal) double.Parse(rdr["uwwCODIncomingEstimated"].ToString());
            data.uwwCODIncomingCalculatedSpecified = true;
            if (rdr["uwwCODIncomingCalculated"] is DBNull)
                data.uwwCODIncomingCalculatedSpecified = false;
            else
                data.uwwCODIncomingCalculated = (decimal) double.Parse(rdr["uwwCODIncomingCalculated"].ToString());
            data.uwwNIncomingMeasuredSpecified = true;
            if (rdr["uwwNIncomingMeasured"] is DBNull)
                data.uwwNIncomingMeasuredSpecified = false;
            else
                data.uwwNIncomingMeasured = (decimal) double.Parse(rdr["uwwNIncomingMeasured"].ToString());
            data.uwwNIncomingEstimatedSpecified = true;
            if (rdr["uwwNIncomingEstimated"] is DBNull)
                data.uwwNIncomingEstimatedSpecified = false;
            else
                data.uwwNIncomingEstimated = (decimal) double.Parse(rdr["uwwNIncomingEstimated"].ToString());
            data.uwwNIncomingCalculatedSpecified = true;
            if (rdr["uwwNIncomingCalculated"] is DBNull)
                data.uwwNIncomingCalculatedSpecified = false;
            else
                data.uwwNIncomingCalculated = (decimal) double.Parse(rdr["uwwNIncomingCalculated"].ToString());
            data.uwwPIncomingMeasuredSpecified = true;
            if (rdr["uwwPIncomingMeasured"] is DBNull)
                data.uwwPIncomingMeasuredSpecified = false;
            else
                data.uwwPIncomingMeasured = (decimal) double.Parse(rdr["uwwPIncomingMeasured"].ToString());
            data.uwwPIncomingEstimatedSpecified = true;
            if (rdr["uwwPIncomingEstimated"] is DBNull)
                data.uwwPIncomingEstimatedSpecified = false;
            else
                data.uwwPIncomingEstimated = (decimal) double.Parse(rdr["uwwPIncomingEstimated"].ToString());
            data.uwwPIncomingCalculatedSpecified = true;
            if (rdr["uwwPIncomingCalculated"] is DBNull)
                data.uwwPIncomingCalculatedSpecified = false;
            else
                data.uwwPIncomingCalculated = (decimal) double.Parse(rdr["uwwPIncomingCalculated"].ToString());

            data.uwwBODDischargeMeasuredSpecified = true;
            if (rdr["uwwBODDischargedMeasured"] is DBNull)
                data.uwwBODDischargeMeasuredSpecified = false;
            else
                data.uwwBODDischargeMeasured = (decimal) double.Parse(rdr["uwwBODDischargedMeasured"].ToString());
            data.uwwBODDischargeEstimatedSpecified = true;
            if (rdr["uwwBODDischargedEstimated"] is DBNull)
                data.uwwBODDischargeEstimatedSpecified = false;
            else
                data.uwwBODDischargeEstimated = (decimal) double.Parse(rdr["uwwBODDischargedEstimated"].ToString());
            data.uwwBODDischargeCalculatedSpecified = true;
            if (rdr["uwwBODDischargedCalculated"] is DBNull)
                data.uwwBODDischargeCalculatedSpecified = false;
            else
                data.uwwBODDischargeCalculated = (decimal) double.Parse(rdr["uwwBODDischargedCalculated"].ToString());
            data.uwwCODDischargeMeasuredSpecified = true;
            if (rdr["uwwCODDischargedMeasured"] is DBNull)
                data.uwwCODDischargeMeasuredSpecified = false;
            else
                data.uwwCODDischargeMeasured = (decimal) double.Parse(rdr["uwwCODDischargedMeasured"].ToString());
            data.uwwCODDischargeEstimatedSpecified = true;
            if (rdr["uwwCODDischargedEstimated"] is DBNull)
                data.uwwCODDischargeEstimatedSpecified = false;
            else
                data.uwwCODDischargeEstimated = (decimal) double.Parse(rdr["uwwCODDischargedEstimated"].ToString());
            data.uwwCODDischargeCalculatedSpecified = true;
            if (rdr["uwwCODDischargedCalculated"] is DBNull)
                data.uwwCODDischargeCalculatedSpecified = false;
            else
                data.uwwCODDischargeCalculated = (decimal) double.Parse(rdr["uwwCODDischargedCalculated"].ToString());
            data.uwwNDischargeMeasuredSpecified = true;
            if (rdr["uwwNDischargedMeasured"] is DBNull)
                data.uwwNDischargeMeasuredSpecified = false;
            else
                data.uwwNDischargeMeasured = (decimal) double.Parse(rdr["uwwNDischargedMeasured"].ToString());
            data.uwwNDischargeEstimatedSpecified = true;
            if (rdr["uwwNDischargedEstimated"] is DBNull)
                data.uwwNDischargeEstimatedSpecified = false;
            else
                data.uwwNDischargeEstimated = (decimal) double.Parse(rdr["uwwNDischargedEstimated"].ToString());
            data.uwwNDischargeCalculatedSpecified = true;
            if (rdr["uwwNDischargedCalculated"] is DBNull)
                data.uwwNDischargeCalculatedSpecified = false;
            else
                data.uwwNDischargeCalculated = (decimal) double.Parse(rdr["uwwNDischargedCalculated"].ToString());
            data.uwwPDischargeMeasuredSpecified = true;
            if (rdr["uwwPDischargedMeasured"] is DBNull)
                data.uwwPDischargeMeasuredSpecified = false;
            else
                data.uwwPDischargeMeasured = (decimal) double.Parse(rdr["uwwPDischargedMeasured"].ToString());
            data.uwwPDischargeEstimatedSpecified = true;
            if (rdr["uwwPDischargedEstimated"] is DBNull)
                data.uwwPDischargeEstimatedSpecified = false;
            else
                data.uwwPDischargeEstimated = (decimal) double.Parse(rdr["uwwPDischargedEstimated"].ToString());
            data.uwwPDischargeCalculatedSpecified = true;
            if (rdr["uwwPDischargedCalculated"] is DBNull)
                data.uwwPDischargeCalculatedSpecified = false;
            else
                data.uwwPDischargeCalculated = (decimal) double.Parse(rdr["uwwPDischargedCalculated"].ToString());

            data.uwwWasteWaterTreatedSpecified = true;
            if (rdr["uwwWasteWaterTreated"] is DBNull)
                data.uwwWasteWaterTreatedSpecified = false;
            else
                data.uwwWasteWaterTreated = (decimal) double.Parse(rdr["uwwWasteWaterTreated"].ToString());

            if (rdr["uwwMethodWasteWaterTreated"] is DBNull)
                data.uwwMethodWasteWaterTreated = null;
            else
                data.uwwMethodWasteWaterTreated = dataSet.Tables["T_LOV"].Rows.Find(rdr["uwwMethodWasteWaterTreated"])["lovKey"] as string;

            data.uwwRemarks = rdr["uwwRemarks"] as string;
            return data;
        }

        public static decimal convertDecimal(object obj) {
            if ((obj == null) || (obj is DBNull))
            {
                return decimal.MinValue;
            }

            if (obj is float) return (decimal)((float)obj);
            if (obj is double) return (decimal)((double)obj);
            if (obj is decimal) return (decimal) obj;

            return decimal.Parse(obj.ToString());
        }

        public static long convertInt64(object obj)
        {
            if ((obj == null) || (obj is DBNull))
            {
                return long.MinValue;
            }

            if (obj is float) return (long)((float)obj);
            if (obj is double) return (long)((double)obj);
            if (obj is decimal) return (long)((decimal)obj);
            if (obj is int) return (long)((int)obj);
            if (obj is decimal) return (long)obj;

            return long.Parse(obj.ToString());
        }
    }
}
